@@ -1,8 +1,8 @@
 #!/bin/sh
 
-# Git status plugin for CliFM
+# Git status plugin for Clifm
 # Written by L. Abramovich
-# License: GPL3
+# License: GPL2+
 
 # Description: Check if the current directory is inside a git work
 # tree, and, if true, print status, i.e., branch name and non
@@ -28,10 +28,8 @@ if [ -n "$1" ] && { [ "$1" = "--help" ] || [ "$1" = "-h" ]; }; then
 	exit 0
 fi
 
-[ "$(git rev-parse --is-inside-work-tree 2>/dev/null)" != "true" ] && exit 1
-
 if [ -n "$CLIFM_COLORLESS" ]; then
-	git status -sb 2>/dev/null | cat
+	git -c color.status=false status -sb 2>/dev/null
 else
 	git status -sb 2>/dev/null
 fi
