@@ -1,17 +1,15 @@
-/* actions.c -- A few functions for the plugins system */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -20,7 +18,9 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  * MA 02110-1301, USA.
-*/
+ */
+
+/* actions.c -- A few functions for the plugins system */
 
 /* The run_action function is taken from NNN's run_selected_plugin function
  * (https://github.com/jarun/nnn/blob/master/src/nnn.c), licensed under BSD-2-Clause.
@@ -29,7 +29,6 @@
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <sys/wait.h>
 #include <unistd.h>
@@ -44,6 +43,7 @@
 #include "sanitize.h"
 #include "sort.h" /* num_to_sort_name() */
 #include "spawn.h"
+#include "term.h" /* set_term_title() */
 
 /* Get the executable's path of the action ACTION
  * Returns this path on success or NULL on error, in which case STATUS
@@ -124,12 +124,14 @@ export_status_values(void)
 		setenv("CLIFM_SELFILE", sel_file, 1);
 	if (sel_n > 0)
 		setenv("CLIFM_SEL_FILES", xitoa((long long)sel_n), 1);
-	setenv("CLIFM_SHOW_HIDDEN", conf.show_hidden == 1 ? "1" : "0", 1);
+	char buf[MAX_INT_STR + 1];
+	snprintf(buf, sizeof(buf), "%d", conf.show_hidden);
+	setenv("CLIFM_SHOW_HIDDEN", buf, 1);
 	setenv("CLIFM_SORT_REVERSE", conf.sort_reverse == 1 ? "1" : "0", 1);
-	setenv("CLIFM_SORT_STYLE", num_to_sort_name(conf.sort), 1);
+	setenv("CLIFM_SORT_STYLE", num_to_sort_name(conf.sort, 0), 1);
 	if (trash_n > 0)
 		setenv("CLIFM_TRASH_FILES", xitoa((long long)trash_n), 1);
-	setenv("CLIFM_TRIM_NAMES", conf.trim_names == 1 ? "1" : "0", 1);
+	setenv("CLIFM_TRUNCATE_NAMES", conf.trunc_names == 1 ? "1" : "0", 1);
 }
 
 static void
@@ -154,7 +156,7 @@ unset_export_values(void) {
 	unsetenv("CLIFM_SORT_REVERSE");
 	unsetenv("CLIFM_SORT_STYLE");
 	unsetenv("CLIFM_TRASH_FILES");
-	unsetenv("CLIFM_TRIM_NAMES");
+	unsetenv("CLIFM_TRUNCATE_NAMES");
 }
 
 int
@@ -185,11 +187,11 @@ run_action(char *action, char **args)
 
 	free(cmd);
 
-	/* 1.b Escape non-escaped file names */
+	/* 1.b Escape non-escaped filenames */
 	size_t i;
 	for (i = 1; args[i]; i++) {
 		struct stat a;
-		/* If the file name is already escaped, lstat(2) will fail, so that
+		/* If the filename is already escaped, lstat(2) will fail, so that
 		 * it won't be reescaped. */
 		if (!strchr(args[i], ' ') || lstat(args[i], &a) == -1)
 			continue;
@@ -395,7 +397,7 @@ edit_actions(char *app)
 	path_n = get_path_env(1);
 	get_path_programs();
 
-	print_reload_msg(_("File modified. Actions reloaded\n"));
+	print_reload_msg(NULL, NULL, _("File modified. Actions reloaded.\n"));
 	return FUNC_SUCCESS;
 }
 
@@ -423,8 +425,9 @@ actions_function(char **args)
 				"Example: enter '//' to run the rgfind plugin"));
 			size_t i, longest_name_len = get_longest_action_name();
 			for (i = 0; i < actions_n; i++) {
-				printf("%-*s %s->%s %s\n", (int)longest_name_len,
-					usr_actions[i].name, mi_c, df_c, usr_actions[i].value);
+				printf("%-*s %s%s%s %s\n", (int)longest_name_len,
+					usr_actions[i].name, mi_c, SET_MSG_PTR,
+					df_c, usr_actions[i].value);
 			}
 			return FUNC_SUCCESS;
 		}
@@ -434,7 +437,7 @@ actions_function(char **args)
 				"mode\n"), stderr);
 		} else {
 			fputs(_("actions: No actions defined. Use the 'actions edit' "
-				"command to add new actions\n"), stdout);
+				"command to add new actions.\n"), stdout);
 		}
 		return FUNC_SUCCESS;
 	}
