@@ -1,17 +1,15 @@
-/* archives.c -- archiving functions */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -21,22 +19,24 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  * MA 02110-1301, USA.
 */
+
+/* archives.c -- archiving functions */
+
 #ifndef _NO_ARCHIVING
 
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <readline/readline.h>
 
 #if defined(_NO_MAGIC)
 # if !defined(_BE_POSIX)
 #  include <paths.h>
+#  ifndef _PATH_DEVNULL
+#   define _PATH_DEVNULL "/dev/null"
+#  endif /* _PATH_DEVNULL */
 # endif /* !_BE_POSIX */
-# ifndef _PATH_DEVNULL
-#  define _PATH_DEVNULL "/dev/null"
-# endif /* _PATH_DEVNULL */
 #endif /* _NO_MAGIC */
 
 #include "aux.h"
@@ -44,28 +44,24 @@
 #include "history.h"
 #include "jump.h"
 #include "listing.h"
+#include "mime.h" /* xmagic() */
 #include "misc.h"
 #include "navigation.h"
 #include "readline.h"
 #include "spawn.h"
 
-#ifndef _NO_MAGIC
-# include "mime.h"
-#else
-# include <unistd.h> /* dup(), dup2(), unlink(), STDOUT_FILENO */
-#endif /* !_NO_MAGIC */
-
 #define OP_ISO    1
 #define OP_OTHERS 0
 
 static char *
 ask_user_for_path(void)
 {
-	const char *m = "Extraction path ('q' to quit): ";
+	const char *m = _("Extraction path ('q' to quit): ");
 
 	const int poffset_bk = prompt_offset;
 	prompt_offset = (int)strlen(m) + 1;
-	alt_prompt = rl_nohist = 1;
+	rl_nohist = 1;
+	alt_prompt = FILES_PROMPT;
 
 	char *ext_path = secondary_prompt(m, NULL);
 
@@ -98,7 +94,7 @@ get_extraction_path(void)
 		return (char *)NULL;
 	}
 
-	if (TOUPPER(*ext_path) == 'Q' && !*(ext_path + 1)) {
+	if (TOUPPER(*ext_path) == 'Q' && !ext_path[1]) {
 		free(ext_path);
 		return (char *)NULL;
 	}
@@ -111,7 +107,7 @@ get_extraction_path(void)
 		}
 	}
 
-	if (*ext_path == '.') {
+	if (*ext_path == '.' || strstr(ext_path, "../")) {
 		char *p = xrealpath(ext_path, NULL);
 		if (p) {
 			free(ext_path);
@@ -128,7 +124,7 @@ get_operation(const int mode)
 	char sel_op = 0;
 	char *op = (char *)NULL;
 	while (!op) {
-		op = rl_no_hist(_("Operation: "));
+		op = rl_no_hist(_("Operation: "), 0);
 		if (!op)
 			continue;
 		if (!*op || op[1] != '\0') {
@@ -304,7 +300,7 @@ mount_iso(char *file)
 {
 #if !defined(__linux__)
 	UNUSED(file);
-	xerror("%s\n", _("mount: This feature is for Linux only"));
+	xerror("%s\n", _("mount: This feature is available for Linux only."));
 	return FUNC_SUCCESS;
 #else
 	char *mountpoint = create_mountpoint(file);
@@ -427,7 +423,7 @@ create_iso(char *in_file, char *out_file)
 
 	/* If any other file format */
 	xerror(_("archiver: '%s': Invalid file format. File should be either "
-		"a directory or a block device\n"), in_file);
+		"a directory or a block device.\n"), in_file);
 	return FUNC_FAILURE;
 }
 
@@ -443,7 +439,6 @@ check_iso(char *file)
 
 	int is_iso = 0;
 
-#ifndef _NO_MAGIC
 	char *t = xmagic(file, TEXT_DESC);
 	if (!t) {
 		xerror("%s\n", _("Error querying file type"));
@@ -455,99 +450,18 @@ check_iso(char *file)
 
 	free(t);
 	return (is_iso == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
-
-#else
-	char *rand_ext = gen_rand_str(RAND_SUFFIX_LEN);
-	if (!rand_ext)
-		return (-1);
-
-	char tmp_file[PATH_MAX + 1];
-	snprintf(tmp_file, sizeof(tmp_file), "%s/.temp%s",
-		(xargs.stealth_mode == 1) ? P_tmpdir : tmp_dir, rand_ext);
-	free(rand_ext);
-
-	int fd;
-	FILE *fp_out = open_fwrite(tmp_file, &fd);
-	if (!fp_out) {
-		xerror("archiver: '%s': %s\n", tmp_file, strerror(errno));
-		return (-1);
-	}
-
-	FILE *fp_err = fopen(_PATH_DEVNULL, "w");
-	if (!fp_err) {
-		xerror("archiver: '%s': %s\n", _PATH_DEVNULL, strerror(errno));
-		fclose(fp_out);
-		unlink(tmp_file);
-		return (-1);
-	}
-
-	int stdout_bk = dup(STDOUT_FILENO); /* Store original stdout */
-	int stderr_bk = dup(STDERR_FILENO); /* Store original stderr */
-
-	if (stdout_bk == -1 || stderr_bk == -1)
-		goto ERROR;
-
-	/* Redirect stdout to the desired file */
-	if (dup2(fileno(fp_out), STDOUT_FILENO) == -1)
-		goto ERROR;
-
-	/* Redirect stderr to /dev/null */
-	if (dup2(fileno(fp_err), STDERR_FILENO) == -1)
-		goto ERROR;
-
-	fclose(fp_out);
-	fclose(fp_err);
-
-	char *cmd[] = {"file", "-b", file, NULL};
-	int retval = launch_execv(cmd, FOREGROUND, E_NOFLAG);
-
-	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
-	dup2(stderr_bk, STDERR_FILENO); /* Restore original stderr */
-	close(stdout_bk);
-	close(stderr_bk);
-
-	if (retval != FUNC_SUCCESS) {
-		unlink(tmp_file);
-		return (-1);
-	}
-
-	fp_out = open_fread(tmp_file, &fd);
-	if (fp_out) {
-		char line[NAME_MAX]; *line = '\0';
-		if (fgets(line, (int)sizeof(line), fp_out) == NULL) {
-			fclose(fp_out);
-			unlink(tmp_file);
-			return FUNC_FAILURE;
-		}
-
-		if (strstr(line, "ISO 9660"))
-			is_iso = 1;
-		fclose(fp_out);
-	}
-
-	unlink(tmp_file);
-
-	return (is_iso == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
-
-ERROR:
-	xerror("archiver: %s\n", strerror(errno));
-	fclose(fp_out);
-	fclose(fp_err);
-	unlink(tmp_file);
-	close(stdout_bk);
-	close(stderr_bk);
-	return (-1);
-#endif /* !_NO_MAGIC */
 }
 
 static int
 check_compressed(const char *line, const int test_iso)
 {
 	return (strstr(line, "archive") || strstr(line, "compressed")
+	|| strncmp(line, "Debian binary package ", 22) == 0
+	|| strncmp(line, "RPM ", 4) == 0
 	|| (test_iso && strstr(line, "ISO 9660")));
 }
 
-/* Check the MIME type of the file names FILE and look for "archive" and
+/* Check the MIME type of the file named FILE and look for "archive" and
  * "compressed" strings. Returns zero if compressed, one if not, and -1
  * in case of error.
  * test_iso is used to determine if ISO files should be checked as
@@ -563,7 +477,6 @@ is_compressed(char *file, const int test_iso)
 		return (-1);
 	}
 
-#ifndef _NO_MAGIC
 	char *t = xmagic(file, TEXT_DESC);
 	if (!t) {
 		xerror("%s\n", _("Error querying file type"));
@@ -574,89 +487,6 @@ is_compressed(char *file, const int test_iso)
 	free(t);
 
 	return (compressed == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
-
-#else
-	char *rand_ext = gen_rand_str(RAND_SUFFIX_LEN);
-	if (!rand_ext)
-		return (-1);
-
-	char tmp_file[PATH_MAX + 1];
-	snprintf(tmp_file, sizeof(tmp_file), "%s/.clifm%s",
-			(xargs.stealth_mode == 1) ? P_tmpdir : tmp_dir, rand_ext);
-	free(rand_ext);
-
-	int fd;
-	FILE *fp_out = open_fwrite(tmp_file, &fd);
-	if (!fp_out) {
-		xerror("archiver: '%s': %s\n", tmp_file, strerror(errno));
-		return (-1);
-	}
-
-	FILE *fp_err = fopen("/dev/null", "w");
-	if (!fp_err) {
-		xerror("archiver: '/dev/null': %s\n", strerror(errno));
-		unlink(tmp_file);
-		fclose(fp_out);
-		return (-1);
-	}
-
-	int stdout_bk = dup(STDOUT_FILENO); /* Store original stdout */
-	int stderr_bk = dup(STDERR_FILENO); /* Store original stderr */
-
-	if (stdout_bk == -1 || stderr_bk == -1)
-		goto ERROR;
-
-	/* Redirect stdout to the desired file */
-	if (dup2(fileno(fp_out), STDOUT_FILENO) == -1)
-		goto ERROR;
-
-	/* Redirect stderr to /dev/null */
-	if (dup2(fileno(fp_err), STDERR_FILENO) == -1)
-		goto ERROR;
-
-	fclose(fp_out);
-	fclose(fp_err);
-
-	char *cmd[] = {"file", "-b", file, NULL};
-	int retval = launch_execv(cmd, FOREGROUND, E_NOFLAG);
-
-	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
-	dup2(stderr_bk, STDERR_FILENO); /* Restore original stderr */
-	close(stdout_bk);
-	close(stderr_bk);
-
-	if (retval != FUNC_SUCCESS) {
-		unlink(tmp_file);
-		return (-1);
-	}
-
-	int compressed = 0;
-	fp_out = open_fread(tmp_file, &fd);
-	if (fp_out) {
-		char line[NAME_MAX + 1]; *line = '\0';
-		if (fgets(line, (int)sizeof(line), fp_out) == NULL) {
-			fclose(fp_out);
-			unlink(tmp_file);
-			return FUNC_FAILURE;
-		}
-
-		compressed = check_compressed(line, test_iso);
-		fclose(fp_out);
-	}
-
-	unlink(tmp_file);
-
-	return (compressed == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
-
-ERROR:
-	xerror("archiver: %s\n", strerror(errno));
-	unlink(tmp_file);
-	fclose(fp_out);
-	fclose(fp_err);
-	close(stdout_bk);
-	close(stderr_bk);
-	return (-1);
-#endif /* !_NO_MAGIC */
 }
 
 static char *
@@ -681,7 +511,7 @@ get_archive_filename(void)
 	char *name = (char *)NULL;
 	while (!name) {
 		flags |= NO_FIX_RL_POINT;
-		name = rl_no_hist(_("File name ('q' to quit): "));
+		name = rl_no_hist(_("Filename ('q' to quit): "), 0);
 		flags &= ~NO_FIX_RL_POINT;
 
 		if (!name || !*name) {
@@ -700,7 +530,7 @@ get_archive_filename(void)
 			return add_default_extension(name);
 
 		if (dot == name) { /* Dot is first char */
-			xerror("%s\n", _("Invalid file name"));
+			xerror("%s\n", _("Invalid filename"));
 			free(name);
 			name = (char *)NULL;
 			continue;
@@ -721,7 +551,7 @@ zstandard(char *in_file, char *out_file, const char mode, const char op)
 	int exit_status = FUNC_SUCCESS;
 	char *deq_file = unescape_str(in_file, 0);
 	if (!deq_file) {
-		xerror(_("archiver: '%s': Error unescaping file name\n"), in_file);
+		xerror(_("archiver: '%s': Error unescaping filename\n"), in_file);
 		return FUNC_FAILURE;
 	}
 
@@ -770,7 +600,7 @@ zstandard(char *in_file, char *out_file, const char mode, const char op)
 
 	char *operation = (char *)NULL;
 	while (!operation) {
-		operation = rl_no_hist(_("Operation: "));
+		operation = rl_no_hist(_("Operation: "), 0);
 		if (!operation)
 			continue;
 		if (!*operation || operation[1] != '\0') {
@@ -825,9 +655,9 @@ compress_zstandard(char *name, char **args)
 
 	/* Multiple files */
 	printf(_("\n%sNOTE%s: Zstandard does not support compression of "
-		 "multiple files into one single compressed file. Files will "
-		 "be compressed rather into multiple compressed files using the "
-		 "original file names.\n"), BOLD, df_c);
+		 "multiple files into a single compressed file. Instead, files "
+		 "will be compressed into multiple compressed files using their "
+		 "original filenames.\n"), BOLD, df_c);
 
 	size_t i;
 	for (i = 1; args[i]; i++) {
@@ -933,23 +763,15 @@ check_zstandard(char **args)
 	size_t zst = 0, i;
 
 	for (i = 1; args[i]; i++) {
-#ifndef _NO_MAGIC
 		char *mime = xmagic(args[i], MIME_TYPE);
 		if (!mime || !*mime)
 			continue;
-		if (strcmp(mime, "application/zstd") == 0) {
+		if (*mime == 'a' && strcmp(mime, "application/zstd") == 0) {
 			zst = 1;
 			free(mime);
 			break;
 		}
 		free(mime);
-#else
-		char *ret = strrchr(args[i], '.');
-		if (ret && strcmp(ret, ".zst") == 0) {
-			zst = 1;
-			break;
-		}
-#endif /* !_NO_MAGIC */
 	}
 
 	return zst;
@@ -966,7 +788,7 @@ get_zstandard_operation(void)
 	char *operation = (char *)NULL;
 
 	while (!operation) {
-		operation = rl_no_hist(_("Operation: "));
+		operation = rl_no_hist(_("Operation: "), 0);
 		if (!operation)
 			continue;
 		if (!*operation || operation[1] != '\0') {
@@ -1101,7 +923,7 @@ get_repack_format(void)
 
 	char *format = (char *)NULL;
 	while (!format) {
-		format = rl_no_hist(_("New format (e.g.: .tar.xz): "));
+		format = rl_no_hist(_("New format (e.g.: .tar.xz): "), 0);
 		if (!format)
 			continue;
 
