@@ -1,16 +1,15 @@
-/* autocmds.h
+/*
+ * This file is part of Clifm
  *
- * This file is part of CliFM
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -21,14 +20,19 @@
  * MA 02110-1301, USA.
 */
 
+/* autocmds.h */
+
 #ifndef AUTOCMDS_H
 #define AUTOCMDS_H
 
 __BEGIN_DECLS
 
-void parse_autocmd_line(char *cmd, const size_t buflen);
-void reset_opts(void);
+int  add_autocmd(char **args);
+void update_autocmd_opts(const int opt);
 int  check_autocmds(void);
+void parse_autocmd_line(char *cmd, const size_t buflen);
+void print_autocmd_msg(void);
+void reset_opts(void);
 void revert_autocmd_opts(void);
 
 __END_DECLS
