@@ -1,17 +1,15 @@
-/* aux.c -- functions that do not fit in any other file */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
  */
 
+/* aux.c -- functions that do not fit in any other file */
+
 /* These four functions: from_hex, to_hex, url_encode, and
  * url_decode, are taken from "http://www.geekhideout.com/urlcode.shtml"
  * (under Public domain) and modified to comform to RFC 2395, as recommended
@@ -34,7 +34,6 @@
 # include <attr.h> /* getattrat, nvlist_lookup_uint64_array, nvlist_free */
 #endif /* __sun && ST_BTIME */
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <termios.h>
 #include <unistd.h>
@@ -43,7 +42,7 @@
 
 #include "aux.h"
 #include "misc.h"
-#include "checks.h"
+#include "checks.h" /* is_exec_cmd() */
 #include "file_operations.h" /* open_file() */
 #ifndef _NO_HIGHLIGHT
 # include "highlight.h"
@@ -101,7 +100,7 @@ press_any_key_to_continue(const int init_newline)
 	errno = saved_errno;
 }
 
-/* Print the file name FNAME, quoted if it contains an space.
+/* Print the file named FNAME, quoted if it contains a space.
  * A slash is appended if FNAME is a directory (ISDIR >= 1). */
 void
 print_file_name(char *fname, const int isdir)
@@ -164,19 +163,20 @@ xregerror(const char *cmd_name, const char *pattern, const int errcode,
  * Based on the sdbm algorithm (see http://www.cse.yorku.ca/~oz/hash.html),
  * released under the public-domain. */
 size_t
-hashme(const char *restrict str, const int case_sensitive) {
+hashme(const char *restrict str, const int case_sensitive)
+{
 	size_t hash = 0;
 
 	/* Two while loops, so that we don't need to check CASE_SENSITIVE for
 	 * each character in STR. */
-	if (case_sensitive == 1) {
+	if (case_sensitive != 1) {
 		while (*str) {
-			hash = (size_t)*str	+ (hash << 6) + (hash << 16) - hash;
+			hash = (size_t)TOUPPER(*str) + (hash << 6) + (hash << 16) - hash;
 			str++;
 		}
 	} else {
 		while (*str) {
-			hash = (size_t)TOUPPER(*str) + (hash << 6) + (hash << 16) - hash;
+			hash = (size_t)*str + (hash << 6) + (hash << 16) - hash;
 			str++;
 		}
 	}
@@ -236,19 +236,21 @@ set_fzf_preview_border_type(void)
 
 	fzf_preview_border_type = FZF_BORDER_ROUNDED; /* fzf default */
 
-	char *p = (conf.fzftab_options && *conf.fzftab_options)
+	const char *p = (conf.fzftab_options && *conf.fzftab_options)
 		? strstr(conf.fzftab_options, "border-") : (char *)NULL;
-	if (!p || !*(p + 7)) {
-		char *q = getenv("FZF_DEFAULT_OPTS");
+	if (!p || !p[7]) {
+		const char *q = getenv("FZF_DEFAULT_OPTS");
 		p = q ? strstr(q, "border-") : (char *)NULL;
-		if (!p || !*(p + 7))
+		if (!p || !p[7])
 			return;
 	}
 
-	switch (*(p + 7)) {
+	switch (p[7]) {
 	case 'b':
-		if (*(p + 8) == 'o' && *(p + 9) == 't')
+		if (p[8] == 'o' && p[9] == 't')
 			fzf_preview_border_type = FZF_BORDER_BOTTOM;
+		else if (p[8] == 'l')
+			fzf_preview_border_type = FZF_BORDER_BLOCK;
 		else
 			fzf_preview_border_type = FZF_BORDER_BOLD;
 		break;
@@ -256,23 +258,22 @@ set_fzf_preview_border_type(void)
 	case 'h': fzf_preview_border_type = FZF_BORDER_HORIZ; break;
 	case 'l': fzf_preview_border_type = FZF_BORDER_LEFT; break;
 	case 'n': fzf_preview_border_type = FZF_BORDER_NONE; break;
-	case 'r': fzf_preview_border_type = FZF_BORDER_ROUNDED; break;
+	case 'r':
+		fzf_preview_border_type = p[8] == 'o'
+			? FZF_BORDER_ROUNDED : FZF_BORDER_RIGHT;
+		break;
 	case 's': fzf_preview_border_type = FZF_BORDER_SHARP; break;
-	case 't': fzf_preview_border_type = FZF_BORDER_TOP; break;
+	case 't':
+		if (p[8] == 'o')
+			fzf_preview_border_type = FZF_BORDER_TOP;
+		else
+			fzf_preview_border_type = FZF_BORDER_THINBLOCK;
+		break;
 	case 'v': fzf_preview_border_type = FZF_BORDER_VERT; break;
 	default: break;
 	}
 }
 
-/* Remove images printed on the kitty terminal */
-static void
-kitty_clear(void)
-{
-	char *cmd[] = {"kitty", "icat", "--clear", "--transfer-mode=stream",
-		"--stdin=no", NULL};
-	launch_execv(cmd, FOREGROUND, E_NOFLAG);
-}
-
 /* Remove any image printed by ueberzug.
  * This function assumes:
  * 1) That ueberzug was launched using the json parser (default).
@@ -290,32 +291,22 @@ ueberzug_clear(char *file)
 	fclose(fp);
 }
 
-/* Let's clear images printed on the terminal screen, either via
- * ueberzug(1) or the kitty image protocol. */
+/* Let's clear images printed on the terminal screen via ueberzug(1). */
 void
 clear_term_img(void)
 {
-	char *p = getenv("CLIFM_FIFO_UEBERZUG");
-	if (p) {
-		ueberzug_clear(p);
-		return;
+	static char fu[PATH_MAX + 1] = "";
+	char *p = (char *)NULL;
+
+	if (!*fu) {
+		p = getenv("CLIFM_FIFO_UEBERZUG");
+		if (!p || !*p)
+			return;
+
+		xstrsncpy(fu, p, sizeof(fu));
 	}
 
-	if (!(flags & KITTY_TERM))
-		return;
-
-	/* This works, but it's too slow. Replace by ACP escape codes (both for
-	 * displaying and removing the image). Take a look at how ranger does it. */
-
-	/* CLIFM_KITTY_IMG is set by the clifmrun plugin to a temp file name,
-	 * which is then created (empty) by the clifmimg plugin every time an
-	 * image is displayed via the kitty protocol. */
-	p = getenv("CLIFM_KITTY_IMG");
-	struct stat a;
-	if (p && stat(p, &a) != -1) {
-		kitty_clear();
-		unlinkat(XAT_FDCWD, p, 0);
-	}
+	ueberzug_clear(fu);
 }
 
 /* Return a pointer to the first digit found in STR or NULL if none is found. */
@@ -345,8 +336,8 @@ should_expand_eln(const char *text, char *cmd_name)
 	if (!l || !*l || *text == '0' || !is_number(text))
 		return 0;
 
-	/* Exclude 'ws' and 'st/sort' commands. */
-	if ((*l == 'w' && l[1] == 's' && !l[2])
+	/* Exclude 'ws', 'mf', and 'st/sort' commands. */
+	if ((*l == 'w' && l[1] == 's' && !l[2]) || (*l == 'm' && l[1] == 'f' && !l[2])
 	|| (*l == 's' && ((l[1] == 't' && !l[2]) || strcmp(l + 1, "ort") == 0)))
 		return 0;
 
@@ -363,25 +354,18 @@ should_expand_eln(const char *text, char *cmd_name)
 
 	char *p = strchr(l, ' ');
 	char t = ' ';
-	if (!p && (p = find_digit(l)) )
+	if (!p && (p = find_digit(l)))
 		t = *p;
 
-	if (!p)
-		return 1;
-
-	*p = '\0';
-
+	if (p) /* Either space of fused ELN. */
+		*p = '\0';
 	flags |= STATE_COMPLETING;
-	if (is_internal_c(l) && !is_internal_f(l)) {
-		*p = t;
-		flags &= ~STATE_COMPLETING;
-		return 0;
-	}
-
+	const int ret = (is_internal_cmd(l, NO_FNAME_NUM, 1, 1)) ? 0 : 1;
 	flags &= ~STATE_COMPLETING;
-	*p = t;
+	if (p)
+		*p = t;
 
-	return 1;
+	return ret;
 }
 
 #if !defined(_POSIX_C_SOURCE) || _POSIX_C_SOURCE >= 199309L
@@ -413,7 +397,7 @@ msleep(const long msec)
  * Change absolute paths into:
  * "./" if file is in CWD
  * "~" if file is in HOME
- * The reformated file name is returned if actually reformated, in which case
+ * The reformated filename is returned if actually reformated, in which case
  * the returned value should be free'd by the caller.
  * Otherwise, a pointer to the original string is returned and must not be
  * free'd by the caller.
@@ -473,17 +457,20 @@ get_cwd(char *buf, const size_t buflen, const int check_workspace)
 	if (tmp)
 		return tmp;
 
-	char *ret = getcwd(buf, buflen);
+	const char *ret = getcwd(buf, buflen);
 	tmp = (ret && *buf) ? buf : (char *)NULL;
 
 	return tmp;
 }
 
-/* OpenBSD implementation of memrchr(), licensed MIT.
- * Modified code is licensed GPL2+. */
-static const void *
+void *
 xmemrchr(const void *s, const int c, size_t n)
 {
+#ifdef HAVE_MEMRCHR
+	return memrchr(s, c, n);
+#else
+	/* OpenBSD implementation of memrchr(), licensed MIT.
+	 * Modified code is licensed GPL2+. */
 	const unsigned char *cp;
 
 	if (n != 0) {
@@ -495,6 +482,7 @@ xmemrchr(const void *s, const int c, size_t n)
 	}
 
 	return (NULL);
+#endif /* HAVE_MEMRCHR */
 }
 
 /* Canonicalize/normalize the path SRC without resolving symlinks.
@@ -519,7 +507,7 @@ normalize_path(char *src, const size_t src_len)
 		}
 
 		const size_t tlen = strlen(tmp);
-		if (tlen > 0 && tmp[tlen - 1] == '/')
+		if (tlen > 1 && tmp[tlen - 1] == '/')
 			tmp[tlen - 1] = '\0';
 
 		xstrsncpy(src, tmp, tlen + 1);
@@ -535,7 +523,7 @@ normalize_path(char *src, const size_t src_len)
 			return (char *)NULL;
 		}
 		const size_t tlen = strlen(tmp);
-		if (tlen > 0 && tmp[tlen - 1] == '/')
+		if (tlen > 1 && tmp[tlen - 1] == '/')
 			tmp[tlen - 1] = '\0';
 
 		if (!strstr(tmp, "/.."))
@@ -580,28 +568,20 @@ normalize_path(char *src, const size_t src_len)
 	const char *next;
 
 	for (ptr = s; ptr < end; ptr = next + 1) {
-		size_t len;
 		next = memchr(ptr, '/', (size_t)(end - ptr));
 		if (!next)
 			next = end;
-		len = (size_t)(next - ptr);
+		const size_t len = (size_t)(next - ptr);
 
-		switch (len) {
-		case 0: continue;
-
-		case 1:
-			if (*ptr == '.')
-				continue;
-			break;
-
-		case 2:
-			if (ptr[0] == '.' && ptr[1] == '.') {
+		if (len <= 2) {
+			if (len == 0) continue;
+			if (len == 1 && *ptr == '.') continue;
+			if (len == 2 && *ptr == '.' && ptr[1] == '.') {
 				const char *slash = xmemrchr(res, '/', res_len);
 				if (slash)
 					res_len = (size_t)(slash - res);
 				continue;
 			}
-			break;
 		}
 
 		res[res_len] = '/';
@@ -683,23 +663,52 @@ rl_ring_bell(void)
  * This function is used mostly by the trash function to generate unique
  * suffixes for trashed files. */
 char *
-gen_date_suffix(const struct tm tm)
+gen_date_suffix(const struct tm tm, const int human)
 {
-	char *suffix = xnmalloc(68, sizeof(char));
-	snprintf(suffix, 67, "%04d%02d%02d%02d%02d%02d", tm.tm_year + 1900,
-	    tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
+	char *suffix = NULL;
+
+	if (human == 0) {
+		suffix = xnmalloc(68, sizeof(char));
+		snprintf(suffix, 67, "%04d%02d%02d%02d%02d%02d", tm.tm_year + 1900,
+			tm.tm_mon + 1, tm.tm_mday, tm.tm_hour, tm.tm_min, tm.tm_sec);
+	} else {
+		char date[18] = "";
+		const size_t bytes = strftime(date, sizeof(date), "%Y%m%d-%H:%M:%S", &tm);
+		if (bytes > 0)
+			suffix = savestring(date, bytes);
+	}
 
 	return suffix;
 }
 
+char *
+gen_backup_file(const char *file, const int human)
+{
+	time_t rawtime = time(NULL);
+	struct tm t;
+	char *suffix = localtime_r(&rawtime, &t) ? gen_date_suffix(t, human) : NULL;
+	if (!suffix) {
+		xerror(_("kb: Cannot generate time suffix string for "
+			"the backup file\n"));
+		return NULL;
+	}
+
+	char backup[PATH_MAX + 1];
+	snprintf(backup, sizeof(backup), "%s-%s", file, suffix);
+
+	free(suffix);
+
+	return strdup(backup);
+}
+
 /* Create directory DIR with permissions set to MODE (this latter modified
  * by a restrictive umask value: 077). */
 int
-xmkdir(char *dir, const mode_t mode)
+xmkdir(const char *dir, const mode_t mode)
 {
-	mode_t old_mask = umask(0077); /* flawfinder: ignore */
-	int ret = mkdirat(XAT_FDCWD, dir, mode);
-	umask(old_mask);
+	const mode_t old_mask = umask(0077); /* flawfinder: ignore */
+	const int ret = mkdirat(XAT_FDCWD, dir, mode);
+	umask(old_mask); /* flawfinder: ignore */
 
 	if (ret == -1)
 		return FUNC_FAILURE;
@@ -714,7 +723,7 @@ xreadlink(const int fd, char *restrict path, char *restrict buf,
 	const size_t bufsize)
 {
 	buf[0] = '\0';
-	ssize_t buf_len = readlinkat(fd, path, buf, bufsize - 1);
+	const ssize_t buf_len = readlinkat(fd, path, buf, bufsize - 1);
 	if (buf_len == -1)
 		return (-1);
 
@@ -860,36 +869,22 @@ get_dt(const mode_t mode)
 static int
 hex2int(const char *str)
 {
-	int i, n[2] = { 0 };
-	for (i = 1; i >= 0; i--) {
-		if (str[i] >= '0' && str[i] <= '9') {
-			n[i] = str[i] - '0';
-		} else {
-			switch (str[i]) {
-			case 'A': /* fallthrough */
-			case 'a': n[i] = 10; break;
-			case 'B': /* fallthrough */
-			case 'b': n[i] = 11; break;
-			case 'C': /* fallthrough */
-			case 'c': n[i] = 12; break;
-			case 'D': /* fallthrough */
-			case 'd': n[i] = 13; break;
-			case 'E': /* fallthrough */
-			case 'e': n[i] = 14; break;
-			case 'F': /* fallthrough */
-			case 'f': n[i] = 15; break;
-			default: break;
-			}
-		}
-	}
+	static const unsigned char hex_chars[256] = {
+		['0'] = 0, ['1'] = 1, ['2'] = 2, ['3'] = 3, ['4'] = 4,
+		['5'] = 5, ['6'] = 6, ['7'] = 7, ['8'] = 8, ['9'] = 9,
+		['a'] = 10, ['b'] = 11, ['c'] = 12, ['d'] = 13, ['e'] = 14, ['f'] = 15,
+		['A'] = 10, ['B'] = 11, ['C'] = 12, ['D'] = 13, ['E'] = 14, ['F'] = 15
+	};
 
-	return ((n[0] * 16) + n[1]);
+	/* int << 4 == int * 16 */
+	return ((hex_chars[(unsigned char)str[0]] << 4)
+		+ hex_chars[(unsigned char)str[1]]);
 }
 
 /* Disassemble the hex color HEX into attribute, R, G, and B values.
  * Based on https://mprog.wordpress.com/c/miscellaneous/convert-hexcolor-to-rgb-decimal */
 int
-get_rgb(char *hex, int *attr, int *r, int *g, int *b)
+get_rgb(const char *hex, int *attr, int *r, int *g, int *b)
 {
 	if (!hex || !*hex)
 		return (-1);
@@ -900,7 +895,7 @@ get_rgb(char *hex, int *attr, int *r, int *g, int *b)
 		hex++;
 	}
 
-	char *h = hex;
+	const char *h = hex;
 
 	/* Convert 3-digits HEX to 6-digits */
 	static char buf[9];
@@ -933,8 +928,7 @@ get_rgb(char *hex, int *attr, int *r, int *g, int *b)
 	*b = hex2int(tmp);
 
 	*attr = -1; /* Attribute unset */
-	if (h[6] == '-' && h[7] && h[7] >= '0'
-	&& h[7] <= '9' && !h[8])
+	if (h[6] == '-' && h[7] >= '0' && h[7] <= '9' && !h[8])
 		*attr = h[7] - '0';
 
 	if (xargs.no_bold == 1 && *attr == 1)
@@ -960,11 +954,11 @@ get_rgb(char *hex, int *attr, int *r, int *g, int *b)
  *
  * At this point we know HEX is a valid hex color code (see is_hex_color() in
  * colors.c).
- * If using this function outside CliFM, make sure to validate HEX yourself
+ * If using this function outside Clifm, make sure to validate HEX yourself
  *
  */
 char *
-hex2rgb(char *hex)
+hex2rgb(const char *hex)
 {
 	int attr = -1, r = 0, g = 0, b = 0;
 	if (get_rgb(hex, &attr, &r, &g, &b) == -1)
@@ -980,7 +974,7 @@ hex2rgb(char *hex)
 /* Count files in the directory DIR, including self and parent. If POP is set
  * to 1, the function will just check if the directory is populated (it has at
  * least 3 files, including self and parent).
- * Returns -1 in case of error or an integer (0-INTMAX_MAX) in case of success. */
+ * Returns -1 in case of error or an integer (0-FILESN_MAX) in case of success. */
 filesn_t
 count_dir(const char *dir, const int pop)
 {
@@ -994,35 +988,29 @@ count_dir(const char *dir, const int pop)
 			exit(ENOMEM);
 		return (-1);
 	}
-/* Let's set the O_NOATIME flag (Linux only) to prevent updating atime, avoiding
- * thus unnecessary disk writes. */
-/*#ifdef O_NOATIME
-	int fd = dirfd(p);
-	if (fd != -1) {
-		int fflags = fcntl(fd, F_GETFL);
-		if (fflags != -1) {
-			fflags |= O_NOATIME;
-			fcntl(fd, F_SETFL, fflags);
+
+	size_t c = 0;
+
+	if (pop) {
+		while (readdir(p)) {
+			c++;
+			if (c > 2)
+				break;
 		}
-	}
-#endif // O_NOATIME */
-
-	filesn_t c = 0;
-
-	while (readdir(p)) {
-		if (c > FILESN_MAX - 1)
-			break;
-
-		c++;
-		if (pop && c > 2)
-			break;
+	} else {
+		while (readdir(p))
+			/* It is extremely unlikely for a directory to have more than
+			 * SIZE_MAX entries. Even if it actually happens, c would wrap
+			 * around to zero, leading thus to a wrong count (in the
+			 * the absolute worst scenario). */
+			c++;
 	}
 
 	closedir(p);
-	return c;
+	return (filesn_t)(c > FILESN_MAX ? FILESN_MAX : c);
 }
 
-/* Get the path of a the command CMD inspecting all paths in the PATH
+/* Get the path of the command CMD inspecting all paths in the PATH
  * environment variable (it basically does the same as which(1)).
  * Returns the appropriate path or NULL in case of error (in which case
  * errno is set to either EINVAL or ENOENT). */
@@ -1039,13 +1027,13 @@ get_cmd_path(const char *cmd)
 
 	if (*cmd == '~') {
 		char *p = tilde_expand(cmd);
-		if (p && access(p, X_OK) == 0)
-			cmd_path = savestring(p, strlen(p));
+		if (p && is_exec_cmd(p) == 1)
+			cmd_path = p;
 		return cmd_path;
 	}
 
 	if (*cmd == '/') {
-		if (access(cmd, X_OK) == 0)
+		if (is_exec_cmd(cmd) == 1)
 			cmd_path = savestring(cmd, strlen(cmd));
 		return cmd_path;
 	}
@@ -1063,7 +1051,7 @@ get_cmd_path(const char *cmd)
 			continue;
 
 		snprintf(cmd_path, PATH_MAX + 1, "%s/%s", paths[i].path, cmd);
-		if (access(cmd_path, X_OK) == 0)
+		if (is_exec_cmd(cmd_path) == 1)
 			return cmd_path;
 	}
 
@@ -1072,15 +1060,62 @@ get_cmd_path(const char *cmd)
 	return (char *)NULL;
 }
 
+/* Same thing as get_cmd_path(), but returns 1 in case of success or 0
+ * otherwise instead of the absolute path to CMD. Unlike get_cmd_path(),
+ * it does not allocate memory in the heap, so that it's faster. */
+int
+is_cmd_in_path(const char *cmd)
+{
+	errno = 0;
+	if (!cmd || !*cmd) {
+		errno = EINVAL;
+		return 0;
+	}
+
+	if (*cmd == '~') {
+		char *p = tilde_expand(cmd);
+		const int ret = (p && is_exec_cmd(p) == 1);
+		free(p);
+		return ret;
+	}
+
+	if (*cmd == '/')
+		return is_exec_cmd(cmd);
+
+	char cmd_path[PATH_MAX + 1];
+	const int is_secure_env =
+		(xargs.secure_env == 1 || xargs.secure_env_full == 1);
+
+	size_t i;
+	for (i = 0; i < path_n; i++) { /* Check each path in PATH. */
+		if (!paths[i].path || !*paths[i].path)
+			continue;
+
+		/* Skip '.' (CWD) if running in secure environment mode. */
+		if (is_secure_env == 1 && *paths[i].path == '.' && !paths[i].path[1])
+			continue;
+
+		snprintf(cmd_path, sizeof(cmd_path), "%s/%s", paths[i].path, cmd);
+		if (is_exec_cmd(cmd_path) == 1)
+			return 1;
+	}
+
+	errno = ENOENT;
+	return 0;
+}
+
 /* Convert SIZE to human readable form (at most 2 decimal places).
  * Returns a pointer to a string of at most MAX_UNIT_SIZE.
- * We follow here the du(1) notation: K, M, G... for powers of 1024
- * and KB, MB, KB... for power of 1000. */
+ * We use KiB, MiB, GiB... suffixes for powers of 1024
+ * and kB, MB, GB... for power of 1000. */
 char *
 construct_human_size(const off_t size)
 {
 	static char str[MAX_HUMAN_SIZE + 2];
 
+	if (size < 0)
+		return UNKNOWN_STR;
+
 	const float base = xargs.si == 1 ? 1000 : 1024;
 	static float mult_factor = 0;
 	if (mult_factor == 0)
@@ -1091,20 +1126,20 @@ construct_human_size(const off_t size)
 
 	while (s >= base) {
 		s = s * mult_factor; /* == (s = s / base), but faster */
-		++n;
+		n++;
 	}
 
-	const int x = (int)s;
-	/* If (s == 0 || s - (float)x == 0), then S has no reminder (zero)
-	 * We don't want to print the reminder when it is zero.
-	 *
-	 * R: Ronnabyte, Q: Quettabyte. It's highly unlikely to have files of
-	 * such huge sizes (and even less) in the near future, but anyway... */
 	static const char *const u = "BKMGTPEZYRQ";
+	/* R: Ronnabyte, Q: Quettabyte. It's highly unlikely to have files of
+	 * such huge sizes (and even less) in the near future. Even more, since
+	 * our original value is off_t (whose maximum value is INT64_MAX,
+	 * i.e. 2^63 - 1), we will never reach even a Zettabyte. */
+
+	/* If s == (float)(int)s, then S has no reminder (zero):
+	 * We don't want to print the reminder when it is zero. */
 	snprintf(str, sizeof(str), "%.*f %c%s",
-		(s == 0.00f || s - (float)x == 0.00f) ? 0 : 2,
-		(double)s,
-		u[n],
+		s == (float)(int)s ? 0 : 2, (double)s,
+		(u[n] == 'K' && xargs.si == 1) ? 'k' : u[n],
 		u[n] != 'B' ? (xargs.si == 1 ? "B" : "iB") : "");
 
 	return str;
@@ -1136,31 +1171,34 @@ get_link_ref(const char *link)
 	return (int)(a.st_mode & S_IFMT);
 }
 
-static char const *nums[102] = {
-	"0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
-	"10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
-	"20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
-	"30", "31", "32", "33", "34", "35", "36", "37", "38", "39",
-	"40", "41", "42", "43", "44", "45", "46", "47", "48", "49",
-	"50", "51", "52", "53", "54", "55", "56", "57", "58", "59",
-	"60", "61", "62", "63", "64", "65", "66", "67", "68", "69",
-	"70", "71", "72", "73", "74", "75", "76", "77", "78", "79",
-	"80", "81", "82", "83", "84", "85", "86", "87", "88", "89",
-	"90", "91", "92", "93", "94", "95", "96", "97", "98", "99",
-	"100", NULL
-	};
-
 /* Transform an integer (N) into a string of chars.
  * This exists because most operating systems do not provide itoa(3).
  * It handles integer numbers between 0 and LLONG_MAX. */
 const char *
 xitoa(long long n)
 {
+	static char const *nums[102] = {
+		"0", "1", "2", "3", "4", "5", "6", "7", "8", "9",
+		"10", "11", "12", "13", "14", "15", "16", "17", "18", "19",
+		"20", "21", "22", "23", "24", "25", "26", "27", "28", "29",
+		"30", "31", "32", "33", "34", "35", "36", "37", "38", "39",
+		"40", "41", "42", "43", "44", "45", "46", "47", "48", "49",
+		"50", "51", "52", "53", "54", "55", "56", "57", "58", "59",
+		"60", "61", "62", "63", "64", "65", "66", "67", "68", "69",
+		"70", "71", "72", "73", "74", "75", "76", "77", "78", "79",
+		"80", "81", "82", "83", "84", "85", "86", "87", "88", "89",
+		"90", "91", "92", "93", "94", "95", "96", "97", "98", "99",
+		"100", NULL
+	};
+
 	if (n >= 0 && n <= 100)
 		return nums[n];
 
 	static char buf[MAX_INT_STR] = {0};
-	long long i = 30;
+	/* We start writing at the end of the buffer, which has MAX_INT_STR bytes
+	 * (32), that is, from index 0 to 31. Index 31 is reserved for the NUL
+	 * terminator, so that we must start writing at index 30. */
+	long long i = MAX_INT_STR - 2;
 
 	while (n > 0 && i > 0) {
 		long long rem = n / 10;
@@ -1177,6 +1215,9 @@ xitoa(long long n)
 filesn_t
 xatof(const char *s)
 {
+	if (!s[1] && *s > '0' && *s <= '9')
+		return (filesn_t)(*s - '0');
+
 	errno = 0;
 	const long long ret = strtoll(s, NULL, 10);
 
@@ -1203,6 +1244,9 @@ xatof(const char *s)
 int
 xatoi(const char *s)
 {
+	if (!s[1] && *s >= '0' && *s <= '9')
+		return (*s - '0');
+
 	errno = 0;
 	const long ret = strtol(s, NULL, 10);
 
@@ -1239,49 +1283,53 @@ xgetchar(void)
 
 /* Converts a hex char to its integer value. */
 char
-from_hex(char c)
+from_hex(const char c)
 {
-	return (char)(isdigit(c) ? c - '0' : tolower(c) - 'a' + 10);
+	return (char)(IS_DIGIT(c) ? c - '0' : TOLOWER(c) - 'a' + 10);
 }
 
 /* Converts an integer value to its hex form. */
 static char
-to_hex(char c)
+to_hex(const char c)
 {
 	static const char hex[] = "0123456789ABCDEF";
 	return hex[c & 15];
 }
 
-/* Returns a url-encoded version of STR. */
+/* Return an URL-encoded version of STR, prefixed with "file://" if
+ * FILE_URI is set to 1. */
 char *
-url_encode(char *str)
+url_encode(const char *str, const int file_uri)
 {
 	if (!str || !*str)
 		return (char *)NULL;
 
-	char *buf = xnmalloc((strlen(str) * 3) + 1, sizeof(char));
-	/* The max lenght of our buffer is 3 times the length of STR plus
-	 * 1 extra byte for the null byte terminator: each char in STR will
-	 * be, if encoded, %XX (3 chars) */
+	const size_t len = (strlen(str) * 3) + 1 + (file_uri == 1 ? 7 : 0);
+	char *buf = xnmalloc(len, sizeof(char));
+	/* The max lenght of our buffer is 3 times the length of STR (plus
+	 * 7 (length of "file://") if FILE_URI is set to 1, plus 1 extra byte
+	 * for the null byte terminator): each byte in STR will be, if
+	 * encoded, %XX (3 chars). */
 
 	/* Copies of STR and BUF pointers to be able to increase and/or decrease
-	 * them without loosing the original memory location */
-	char *pstr = str, *pbuf = buf;
+	 * them without losing the original memory location. */
+	const char *pstr = str;
+	char *pbuf = buf;
+
+	if (file_uri == 1) {
+		buf[0] = 'f'; buf[1] = 'i'; buf[2] = 'l'; buf[3] = 'e';
+		buf[4] = ':'; buf[5] = '/'; buf[6] = '/'; buf[7] = '\0';
+		pbuf += 7;
+	}
 
 	for (; *pstr; pstr++) {
-#if defined(__CYGWIN__)
-		/* GCC on CYGWIN complains about subscript having type 'char' */
-		if (isalnum((unsigned char)*pstr) || *pstr == '-' || *pstr == '_'
-		|| *pstr == '.'
-#else
-		if (isalnum(*pstr) || *pstr == '-' || *pstr == '_' || *pstr == '.'
-#endif /* __CYGWIN__ */
+		if (IS_ALNUM(*pstr) || *pstr == '-' || *pstr == '_' || *pstr == '.'
 		|| *pstr == '~' || *pstr == '/') {
-			/* Do not encode any of the above chars */
+			/* Do not encode any of the above chars. */
 			*pbuf = *pstr;
 			pbuf++;
 		} else {
-			/* Encode char to URL format. Example: space char to %20 */
+			/* Encode char to URL format. Example: space char to "%20". */
 			*pbuf = '%';
 			pbuf++;
 			*pbuf = to_hex(*pstr >> 4);
@@ -1297,21 +1345,21 @@ url_encode(char *str)
 
 /* Returns a url-decoded version of STR. */
 char *
-url_decode(char *str)
+url_decode(const char *str)
 {
 	if (!str || !*str)
 		return (char *)NULL;
 
 	char *buf = xnmalloc(strlen(str) + 1, sizeof(char));
-	/* The decoded string will be at most as long as the encoded string */
+	/* The decoded string will be at most as long as the encoded string. */
 
-	char *pstr = str, *pbuf = buf;
+	const char *pstr = str;
+	char *pbuf = buf;
 
 	for (; *pstr; pstr++) {
 		if (*pstr == '%') {
 			if (pstr[1] && pstr[2]) {
-				/* Decode URL code. Example: %20 to space char */
-				/* Left shift and bitwise OR operations */
+				/* Decode URL code. Example: %20 to space char. */
 				*pbuf = (char)(from_hex(pstr[1]) << 4 | from_hex(pstr[2]));
 				pbuf++;
 				pstr += 2;
@@ -1326,35 +1374,20 @@ url_decode(char *str)
 	return buf;
 }
 
-/* Convert octal string into integer.
- * Based on: https://www.geeksforgeeks.org/program-octal-decimal-conversion/
- * Used by decode_prompt() to make things like this work: \033[1;34m */
+/* Convert the octal string STR into the corresponding integer value. */
 int
-read_octal(char *str)
+octal2int(const char *restrict str)
 {
 	if (!str || !*str)
 		return (-1);
 
-	int n = atoi(str);
-	if (n == INT_MIN)
-		return (-1);
-
-	int num = n;
 	int dec_value = 0;
+	while (*str) {
+		if (*str < '0' || *str > '7')
+			return (-1);
 
-	/* Initializing base value to 1, i.e 8^0 */
-	int base = 1;
-
-	int temp = num;
-	while (temp) {
-		/* Extracting last digit */
-		int last_digit = temp % 10;
-		temp = temp / 10;
-
-		/* Multiplying last digit with appropriate
-		 * base value and adding it to dec_value */
-		dec_value += last_digit * base;
-		base = base * 8;
+		dec_value = (dec_value << 3) + (*str - '0');
+		str++;
 	}
 
 	return dec_value;
