@@ -1,17 +1,15 @@
-/* bookmarks.c -- bookmarking functions */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,10 +20,11 @@
  * MA 02110-1301, USA.
 */
 
+/* bookmarks.c -- bookmarking functions */
+
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <readline/readline.h> /* tilde_expand() */
 #include <string.h>
 #include <unistd.h>
@@ -73,13 +72,20 @@ bm_prompt(const int print_header)
 	char *bm = (char *)NULL;
 	if (print_header)
 		printf(_("%s%s\nEnter '%c' to edit your bookmarks or '%c' to quit\n"
-			"Choose a bookmark (by ELN, shortcut, or name):\n"),
+			"Select a bookmark (by ELN, shortcut, or name):\n"),
 			NC, df_c, 'e', 'q');
 
+	const int prompt_offset_bk = prompt_offset;
+	prompt_offset = 0;
+	alt_prompt = BOOKMARKS_PROMPT;
+
 	char bm_str[(MAX_COLOR * 2) + 7];
 	snprintf(bm_str, sizeof(bm_str), "\001%s\002>\001%s\002 ", mi_c, tx_c);
 	while (!bm)
-		bm = rl_no_hist(bm_str);
+		bm = rl_no_hist(bm_str, 1);
+
+	alt_prompt = 0;
+	prompt_offset = prompt_offset_bk;
 
 	char **cmd = (char **)NULL;
 	/* "e/edit" is the only command that needs to split the input string:
@@ -129,32 +135,39 @@ edit_bookmarks(char *cmd, const int flag)
 
 		if (flag == NO_BM_SCREEN) {
 			reload_dirlist();
-			print_reload_msg(_("File modified. Bookmarks reloaded\n"));
+			print_reload_msg(NULL, NULL, _("File modified. Bookmarks reloaded.\n"));
 		}
 	}
 
 	return FUNC_SUCCESS;
 }
 
-static size_t
-get_longest_shortcut(void)
+/* Update LS and LN with the longest shortcut and name entries in the bookmarks
+ * database respectively. */
+static void
+get_longest_entries(size_t *ls, size_t *ln)
 {
+	*ls = 0;
+	*ln = 0;
+
 	if (bm_n == 0 || !bookmarks)
-		return 0;
+		return;
 
-	size_t l = 0;
 	int i = (int)bm_n;
 
 	while (--i >= 0) {
-		if (!bookmarks[i].shortcut || !*bookmarks[i].shortcut)
-			continue;
+		if (bookmarks[i].shortcut && *bookmarks[i].shortcut) {
+			const size_t slen = strlen(bookmarks[i].shortcut);
+			if (slen > *ls)
+				*ls = slen;
+		}
 
-		const size_t slen = strlen(bookmarks[i].shortcut);
-		if (slen > l)
-			l = slen;
+		if (bookmarks[i].name && *bookmarks[i].name) {
+			const size_t nlen = strlen(bookmarks[i].name);
+			if (nlen > *ln)
+				*ln = nlen;
+		}
 	}
-
-	return l;
 }
 
 /* Print the list of available bookmarks. */
@@ -162,13 +175,15 @@ static void
 print_bookmarks(void)
 {
 	HIDE_CURSOR;
-	printf(_("%sBookmarks Manager%s\n\n"), BOLD, df_c);
+	printf(_("%sBookmark Manager%s\n\n"), BOLD, df_c);
 
 	struct stat attr;
 	const int eln_pad = DIGINUM(bm_n);
-	const size_t ls = get_longest_shortcut();
+	size_t ls = 0, ln = 0;
 	size_t i;
 
+	get_longest_entries(&ls, &ln);
+
 	/* Print bookmarks, taking into account shortcut, name, and path. */
 	for (i = 0; i < bm_n; i++) {
 		if (!bookmarks[i].path || !*bookmarks[i].path)
@@ -187,16 +202,16 @@ print_bookmarks(void)
 		char *color = stat(bookmarks[i].path, &attr) == -1
 			? uf_c : get_entry_color(bookmarks[i].path, &attr);
 
-		printf("%s%s%-*zu%s%s%s%s%s%s%s%-*s %s%s%s\n",
+		printf("%s%s%*zu%s%s%s%s%s%s%s%-*s %s%-*s%s  %s%s%s\n",
 			NC, el_c, eln_pad, i + 1, df_c, // ELN
 			ls > 0 ? " " : "",
 
-			BOLD, sc_ok == 1 ? "[" : "", // Shortcut
-			sc_ok == 1 ? bookmarks[i].shortcut : "",
-			sc_ok == 1 ? "]" : "", df_c, sc_pad, "",
+			sc_ok == 1 ? "[" : "", sc_ok == 1 ? mi_c : bk_c, // Shortcut
+			sc_ok == 1 ? bookmarks[i].shortcut : "", df_c,
+			sc_ok == 1 ? "]" : "", sc_pad, "",
 
-			color, bookmarks[i].name ? bookmarks[i].name
-			: bookmarks[i].path, df_c);
+			mi_c, (int)ln, bookmarks[i].name ? bookmarks[i].name : "",
+			df_c, color, bookmarks[i].path, df_c);
 	}
 
 	UNHIDE_CURSOR;
@@ -233,7 +248,8 @@ get_bm_path(char *arg)
 		return bookmarks[num - 1].path;
 	}
 
-	char *name = remove_quotes(arg);
+	char *p = remove_quotes(arg);
+	char *name = p ? p : arg;
 
 	/* If string, check shortcuts and names */
 	size_t i;
@@ -304,7 +320,7 @@ open_bookmark(void)
 			continue;
 		}
 
-		struct stat a; /* If not a dir, refresh files list */
+		struct stat a; /* If not a dir, refresh the file list */
 		if (stat(tmp_path, &a) != -1 && S_ISDIR(a.st_mode))
 			is_dir = 1;
 
@@ -398,7 +414,7 @@ name_is_reserved_keyword(const char *name)
 		return 0;
 
 	if ( ((*name == 'e' || *name == 'd' || *name == 'a' || *name == 'q')
-	&& !*(name + 1)) || strcmp(name, "quit") == 0
+	&& !name[1]) || strcmp(name, "quit") == 0
 	|| strcmp(name, "edit") == 0 || strcmp(name, "del") == 0
 	|| strcmp(name, "add") == 0 || strcmp(name, "reload") == 0) {
 		xerror(_("bookmarks: '%s': Reserved bookmark keyword\n"), name);
@@ -472,7 +488,7 @@ check_bm_shortcut(const char *shortcut, const int add)
 static int
 bookmark_add(char *file, char *name, char *shortcut)
 {
-	if (check_bm_path(file) == 1 && rl_get_y_or_n("Continue? [y/n] ") == 0)
+	if (check_bm_path(file) == 1 && rl_get_y_or_n(_("Continue?"), 0) == 0)
 		return FUNC_SUCCESS;
 
 	int exit_status = FUNC_FAILURE;
@@ -520,15 +536,16 @@ bookmark_add(char *file, char *name, char *shortcut)
 
 	puts(_("File succesfully bookmarked"));
 	if (s)
-		printf("[%s]%s %s->%s %s\n", s, n, mi_c, tx_c, np ? np : file);
+		printf("[%s]%s %s%s%s %s\n", s, n, mi_c, SET_MSG_PTR,
+			tx_c, np ? np : file);
 	else
-		printf("%s %s->%s %s\n", n, mi_c, tx_c, np ? np : file);
+		printf("%s %s%s%s %s\n", n, mi_c, SET_MSG_PTR, tx_c, np ? np : file);
 
 	free(np);
 	free(p);
 	free(q);
 
-	reload_bookmarks(); /* Update bookmarks for TAB completion. */
+	reload_bookmarks(); /* Update bookmarks for tab completion. */
 
 	return FUNC_SUCCESS;
 
@@ -555,7 +572,7 @@ add_bookmark(char **cmd)
 
 	char *p = unescape_str(cmd[0], 0);
 	if (!p) {
-		xerror(_("bookmarks: '%s': Error unescaping file name\n"), cmd[0]);
+		xerror(_("bookmarks: '%s': Error unescaping filename\n"), cmd[0]);
 		return FUNC_FAILURE;
 	}
 
@@ -601,8 +618,8 @@ mark_bookmarks_for_deletion(char **args, int *exit_status)
 	return counter;
 }
 
-/* Extract name and shortcut from the bookmark line LINE and store it into
- * a bookmarks_t struct. */
+/* Extract name and shortcut from the bookmark line LINE and store it in a
+ * bookmarks_t struct. */
 static struct bookmarks_t
 extract_shortcut_and_name(char *line)
 {
@@ -729,7 +746,8 @@ del_bookmarks(char **args)
 
 	if (removed > 0) {
 		if (rename(tmp_file, bm_file) == 0) {
-			print_reload_msg(_("Removed %zu bookmark(s)\n"), removed);
+			print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+				_("Removed %zu bookmark(s)\n"), removed);
 		} else {
 			xerror(_("bookmarks: Error updating bookmarks\nCannot rename "
 				"temporary file '%s': %s\n"), tmp_file, strerror(errno));
