@@ -1,17 +1,15 @@
-/* bulk_remove.c -- bulk remove files */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,26 +20,25 @@
  * MA 02110-1301, USA.
 */
 
+/* bulk_remove.c -- bulk remove files */
+
 #include "helpers.h"
 
 #include <dirent.h> /* scandir() */
-#include <stdio.h>
-#include <stdlib.h>
 #include <string.h> /* strnlen() */
 #include <sys/stat.h> /* (l)stat() */
 #include <fcntl.h> /* unlinkat() */
 #include <unistd.h> /* unlinkat() */
 #include <errno.h>
 
-#include "aux.h" /* xnmalloc, open_fwrite(), get_cmd_path(), count_dir() */
-#include "checks.h" /* is_internal() */
+#include "aux.h" /* xnmalloc, open_fwrite(), is_cmd_in_path(), count_dir() */
 #include "file_operations.h" // open_file() */
 #include "messages.h" /* RR_USAGE */
 #include "misc.h" /* xerror() */
 #include "spawn.h" /* launch_execv() */
 
-#define BULK_RM_TMP_FILE_HEADER "# CliFM - Remove files in bulk\n\
-# Remove the file names you want to be deleted, save, and quit the\n\
+#define BULK_RM_TMP_FILE_HEADER "# Clifm - Remove files in bulk\n\
+# Remove the filenames you want to be deleted, save, and quit the\n\
 # editor (you will be asked for confirmation).\n\
 # Quit the editor without saving to cancel the operation.\n\n"
 
@@ -60,8 +57,8 @@ parse_bulk_remove_params(char *s1, char *s2, char **app, char **target)
 	int ret = 0;
 	struct stat a;
 	if ((ret = stat(s1, &a)) == -1 || !S_ISDIR(a.st_mode)) {
-		char *p = get_cmd_path(BULK_APP(s1));
-		if (!p) { /* S1 is neither a directory nor a valid application */
+		if (is_cmd_in_path(BULK_APP(s1)) == 0) {
+			/* S1 is neither a directory nor a valid application */
 			int ec = (ret == -1 && *s1 == ':') ? E_NOTFOUND : ENOTDIR;
 			if (ec == ENOTDIR)
 				xerror("rr: '%s': %s\n", s1, strerror(ec));
@@ -69,10 +66,10 @@ parse_bulk_remove_params(char *s1, char *s2, char **app, char **target)
 				xerror("rr: '%s': %s\n", BULK_APP(s1), NOTFOUND_MSG);
 			return ec;
 		}
+
 		/* S1 is an application name. TARGET defaults to CWD */
 		*target = workspaces[cur_ws].path;
 		*app = BULK_APP(s1);
-		free(p);
 		return FUNC_SUCCESS;
 	}
 
@@ -85,10 +82,9 @@ parse_bulk_remove_params(char *s1, char *s2, char **app, char **target)
 	if (!s2 || !*s2) /* No S2. APP defaults to default associated app */
 		return FUNC_SUCCESS;
 
-	char *p = get_cmd_path(BULK_APP(s2));
-	if (p) { /* S2 is a valid application name */
+	if (is_cmd_in_path(BULK_APP(s2)) == 1) {
+		/* S2 is a valid application name */
 		*app = BULK_APP(s2);
-		free(p);
 		return FUNC_SUCCESS;
 	}
 
@@ -332,7 +328,7 @@ get_remove_files(const char *target, char **tmp_files,
 
 	for (i = 0; i < (size_t)n; i++) {
 		if (remove_this_file((*a)[i]->d_name, tmp_files) == 1) {
-			char p[PATH_MAX + 1];
+			char p[PATH_MAX + NAME_MAX + 3];
 			if (*target == '/') {
 				snprintf(p, sizeof(p), "%s/%s", target, (*a)[i]->d_name);
 			} else {
@@ -418,7 +414,18 @@ bulk_remove(char *s1, char *s2)
 	int fd = 0, ret = 0, i = 0;
 	filesn_t n = 0;
 
-	if ((ret = parse_bulk_remove_params(s1, s2, &app, &target)) != FUNC_SUCCESS)
+	char dpath[PATH_MAX + 1]; *dpath = '\0';
+	if (s1 && *s1 && *s1 != ':' && strchr(s1, '\\')) {
+		char *tmp = unescape_str(s1, 0);
+		if (tmp) {
+			xstrsncpy(dpath, tmp, sizeof(dpath));
+			free(tmp);
+		}
+	}
+
+	ret = parse_bulk_remove_params(*dpath ? dpath : s1, s2, &app, &target);
+
+	if (ret != FUNC_SUCCESS)
 		return ret;
 
 	struct stat attr;
