@@ -1,17 +1,15 @@
-/* bulk_rename.h - bulk rename files */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,12 +20,14 @@
  * MA 02110-1301, USA.
 */
 
+/* bulk_rename.h */
+
 #ifndef BULK_RENAME_H
 #define BULK_RENAME_H
 
 __BEGIN_DECLS
 
-int bulk_rename(char **args);
+int bulk_rename(char **args, size_t *renamed, const size_t reload_list);
 
 __END_DECLS
 
