@@ -1,17 +1,15 @@
-/* checks.c -- misc check functions */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,10 +20,11 @@
  * MA 02110-1301, USA.
 */
 
+/* checks.c -- misc check functions */
+
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 
@@ -42,9 +41,6 @@
 #include "aux.h"
 #include "misc.h"
 #include "sanitize.h" /* sanitize_cmd() */
-#include "term_info.h"
-
-#define TRUE_COLOR 16777216
 
 /* Check whether parameter S is -f or --force.
  * Returns 1 if yes, and there is no "-f" or "--force" file in the current
@@ -68,6 +64,7 @@ check_glob_char(const char *str, const int gflag)
 {
 	if (!str || !*str)
 		return 0;
+
 	return strpbrk(str, (gflag == GLOB_ONLY)
 		? GLOB_CHARS : GLOB_REGEX_CHARS) ? 1 : 0;
 }
@@ -79,7 +76,7 @@ is_file_in_cwd(char *name)
 		return 0;
 
 	const char *s = strchr(name, '/');
-	if (!s || !*(s + 1)) /* 'name' or 'name/' */
+	if (!s || !s[1]) /* 'name' or 'name/' */
 		return 1;
 
 	char *rpath = normalize_path(name, strlen(name));
@@ -110,109 +107,12 @@ is_url(const char *url)
 	return FUNC_FAILURE;
 }
 
-/* See https://github.com/termstandard/colors#truecolor-detection */
-static int
-check_truecolor(void)
-{
-	char *c = getenv("COLORTERM");
-
-	if (c && ((*c == 't' && strcmp(c + 1, "ruecolor") == 0)
-	|| (*c == '2' && strcmp(c + 1, "4bit") == 0) ) )
-		return 1;
-
-	return 0;
-}
-
-static void
-set_term_caps(const int i)
-{
-	const int true_color = check_truecolor();
-
-	if (i == -1) { /* TERM not found in our terminfo database */
-		term_caps.color = true_color == 1 ? TRUE_COLOR : 0;
-		if (term_caps.color <= 8)
-			memset(dim_c, '\0', sizeof(dim_c));
-		term_caps.suggestions = 0;
-		term_caps.pager = 0;
-		term_caps.hide_cursor = 0;
-		term_caps.home = 0;
-		term_caps.clear = 0;
-		term_caps.del_scrollback = 0;
-		return;
-	}
-
-	term_caps.home = TERM_INFO[i].home;
-	term_caps.hide_cursor = TERM_INFO[i].hide_cursor;
-	term_caps.clear = TERM_INFO[i].ed;
-	term_caps.del_scrollback = TERM_INFO[i].del_scrollback;
-	term_caps.req_cur_pos = TERM_INFO[i].req_cur_pos;
-
-	term_caps.color = true_color == 1 ? TRUE_COLOR
-		: (TERM_INFO[i].color > 0 ? TERM_INFO[i].color : 0);
-	if (term_caps.color <= 8)
-		memset(dim_c, '\0', sizeof(dim_c));
-
-	term_caps.suggestions = (TERM_INFO[i].cub == 1 && TERM_INFO[i].ed == 1
-		&& TERM_INFO[i].el == 1) ? 1 : 0;
-
-	term_caps.pager = (TERM_INFO[i].cub == 0 || TERM_INFO[i].el == 0) ? 0 : 1;
-}
-
-/* Check whether current terminal (_TERM) supports colors and requesting
- * cursor position (needed to print suggestions). If not, disable the
- * feature accordingly. */
-static void
-check_term_support(const char *env_term)
-{
-	if (!env_term || !*env_term) {
-		set_term_caps(-1);
-		return;
-	}
-
-	size_t i;
-	const size_t len = strlen(env_term);
-	/* Color and cursor position request support. */
-	int index = -1;
-
-	if (*env_term == 'x' && strcmp(env_term, "xterm-kitty") == 0)
-		flags |= KITTY_TERM;
-
-	for (i = 0; TERM_INFO[i].name; i++) {
-		if (*env_term != *TERM_INFO[i].name || len != TERM_INFO[i].len
-		|| strcmp(env_term, TERM_INFO[i].name) != 0)
-			continue;
-
-		index = (int)i;
-		break;
-	}
-
-	if (index == -1) {
-		err('w', PRINT_PROMPT, _("%s: '%s': Terminal type not supported. "
-			"Limited functionality is expected.\n"), PROGRAM_NAME, env_term);
-	}
-
-	set_term_caps(index);
-}
-
-void
-check_term(void)
-{
-	char *t = getenv("TERM");
-	if (!t || !*t) {
-		t = "xterm";
-		err('w', PRINT_PROMPT, _("%s: The TERM environment variable is unset.\n"
-			"Running in xterm compatibility mode\n"), PROGRAM_NAME);
-	}
-
-	check_term_support(t);
-}
-
 static void
 set_mount_cmd(const int udisks2ok, const int udevilok)
 {
 	if (xargs.mount_cmd == MNT_UDISKS2 && !udisks2ok && udevilok) {
 		err('w', PRINT_PROMPT, _("%s: udisks2: Command not found. Falling "
-			"back to udevil\n"), PROGRAM_NAME);
+			"back to 'udevil'.\n"), PROGRAM_NAME);
 		xargs.mount_cmd = MNT_UDEVIL;
 		return;
 	}
@@ -226,45 +126,55 @@ set_mount_cmd(const int udisks2ok, const int udevilok)
 }
 
 #ifndef _NO_FZF
-/* fzf, fnf, and smenu are used as alternative TAB completion mechanisms.
- * fnf and smenu fallback to default if not found.
- * The default value is fzf, if found, or standard (readline). */
+static const char *
+tabmode_to_name(void)
+{
+	switch (tabmode) {
+	case FZF_TAB: return "fzf";
+	case FNF_TAB: return "fnf";
+	case SMENU_TAB: return "smenu";
+	case STD_TAB: /* fallthrough */
+	default: return "standard";
+	}
+}
+
+/* Set tab completion mode based on available binaries. */
 void
 check_completion_mode(void)
 {
-	if (fzftab == 1) { /* fzftab is zero only if running with --stdtab */
-		if (!(bin_flags & FZF_BIN_OK) && tabmode == FZF_TAB) {
-			err('w', PRINT_PROMPT, _("%s: fzf: Command not found. Falling "
-				"back to standard TAB completion\n"), PROGRAM_NAME);
-			tabmode = STD_TAB;
-			fzftab = 0;
-		}
-
-		if (!(bin_flags & FNF_BIN_OK) && tabmode == FNF_TAB) {
-			err('w', PRINT_PROMPT, _("%s: fnf: Command not found. Falling "
-				"back to the default value (fzf, if found, or standard)\n"),
-				PROGRAM_NAME);
-			tabmode = (bin_flags & FZF_BIN_OK) ? FZF_TAB : STD_TAB;
-		} else if (!(bin_flags & SMENU_BIN_OK) && tabmode == SMENU_TAB) {
-			err('w', PRINT_PROMPT, _("%s: smenu: Command not found. Falling "
-				"back to the default value (fzf, if found, or standard)\n"),
-				PROGRAM_NAME);
-			tabmode = (bin_flags & FZF_BIN_OK) ? FZF_TAB : STD_TAB;
-		}
-
-		if (tabmode == STD_TAB) {
-			if (bin_flags & FZF_BIN_OK)
-				/* We have the fzf binary, let's run in FZF mode */
-				tabmode = FZF_TAB;
-			else
-				/* Either specified mode was not found or no mode was
-				 * specified */
-				fzftab = 0;
-		}
-	} else {
+	/* fzftab is zero only if running with --stdtab */
+	if (fzftab == 0) {
 		tabmode = STD_TAB;
 		fzftab = 0;
+		return;
 	}
+
+	/* The user asked for a specific mode, but the binary wasn't found. */
+	char *err_name = (char *)NULL;
+	if (!(bin_flags & FZF_BIN_OK) && tabmode == FZF_TAB) {
+		err_name = "fzf"; tabmode = STD_TAB;
+	} else if (!(bin_flags & FNF_BIN_OK) && tabmode == FNF_TAB) {
+		err_name = "fnf"; tabmode = STD_TAB;
+	} else if (!(bin_flags & SMENU_BIN_OK) && tabmode == SMENU_TAB) {
+		err_name = "smenu"; tabmode = STD_TAB;
+	}
+
+	if (tabmode == STD_TAB) {
+		/* If a suitable binary is found, let's run in the corresponding mode. */
+		if (bin_flags & FZF_BIN_OK)
+			tabmode = FZF_TAB;
+		else if (bin_flags & FNF_BIN_OK)
+			tabmode = FNF_TAB;
+		else if (bin_flags & SMENU_BIN_OK)
+			tabmode = SMENU_TAB;
+		else
+			/* No binary found. Let's run in standard mode. */
+			fzftab = 0;
+	}
+
+	if (err_name)
+		err('w', PRINT_PROMPT, _("%s: %s: Command not found. Falling back "
+			"to '%s'.\n"), PROGRAM_NAME, err_name, tabmode_to_name());
 }
 #endif /* !_NO_FZF */
 
@@ -274,44 +184,36 @@ static void
 check_third_party_cmds_alt(void)
 {
 	int udisks2ok = 0, udevilok = 0;
-	char *p = (char *)NULL;
 
-	if ( (p = get_cmd_path("fzf")) ) {
-		free(p);
+	if (is_cmd_in_path("fzf") == 1) {
 		bin_flags |= FZF_BIN_OK;
 		if (fzftab == UNSET) fzftab = 1;
 	}
 
-	if ( (p = get_cmd_path("fnf")) ) {
-		free(p);
+	if (is_cmd_in_path("fnf") == 1) {
 		bin_flags |= FNF_BIN_OK;
 		if (fzftab == UNSET) fzftab = 1;
 	}
 
-	if ( (p = get_cmd_path("smenu")) ) {
-		free(p);
+	if (is_cmd_in_path("smenu") == 1) {
 		bin_flags |= SMENU_BIN_OK;
 		if (fzftab == UNSET) fzftab = 1;
 	}
 
-	if ( (p = get_cmd_path("udisksctl")) ) {
-		free(p); udisks2ok = 1;
-	}
+	if (is_cmd_in_path("udiskctl") == 1)
+		udisks2ok = 1;
 
-	if ( (p = get_cmd_path("udevil")) ) {
-		free(p); udevilok = 1;
-	}
+	if (is_cmd_in_path("udevil") == 1)
+		udevilok = 1;
 
 #if defined(USE_DU1) && !defined(HAVE_GNU_DU) && !defined(_BE_POSIX)
-	if ( (p = get_cmd_path("gdu")) ) {
-		free(p); bin_flags |= GNU_DU_BIN_GDU;
-	}
+	if (is_cmd_in_path("gdu") == 1)
+		bin_flags |= GNU_DU_BIN_GDU;
 #endif /* USE_DU1 && !HAVE_GNU_DU && !_BE_POSIX */
 
 #if defined(CHECK_COREUTILS)
-	if ( (p = get_cmd_path("grm")) ) {
-		free(p); bin_flags |= BSD_HAVE_COREUTILS;
-	}
+	if (is_cmd_in_path("grm") == 1)
+		bin_flags |= BSD_HAVE_COREUTILS;
 #endif /* CHECK_COREUTILS */
 
 	set_mount_cmd(udisks2ok, udevilok);
@@ -332,7 +234,6 @@ check_third_party_cmds(void)
 		return;
 	}
 
-
 	int udisks2ok = 0, udevilok = 0, check_coreutils = 0;
 #if defined(CHECK_COREUTILS)
 	check_coreutils = 1;
@@ -352,20 +253,17 @@ check_third_party_cmds(void)
 
 		if (*bin_commands[i] == 'f' && strcmp(bin_commands[i], "fzf") == 0) {
 			bin_flags |= FZF_BIN_OK;
-			if (fzftab == UNSET)
-				fzftab = 1;
+			if (fzftab == UNSET) fzftab = 1;
 		}
 
 		if (*bin_commands[i] == 'f' && strcmp(bin_commands[i], "fnf") == 0) {
 			bin_flags |= FNF_BIN_OK;
-			if (fzftab == UNSET)
-				fzftab = 1;
+			if (fzftab == UNSET) fzftab = 1;
 		}
 
 		if (*bin_commands[i] == 's' && strcmp(bin_commands[i], "smenu") == 0) {
 			bin_flags |= SMENU_BIN_OK;
-			if (fzftab == UNSET)
-				fzftab = 1;
+			if (fzftab == UNSET) fzftab = 1;
 		}
 
 		if (*bin_commands[i] == 'u'
@@ -422,35 +320,60 @@ check_file_access(const mode_t mode, const uid_t uid, const gid_t gid)
 	if (user.uid == 0) /* We are root */
 		return 1;
 
-	int f = 0; /* Hold file ownership flags */
-
 	const mode_t val = (mode & (mode_t)~S_IFMT);
-	if (val & S_IRUSR) f |= R_USR;
-	if (val & S_IXUSR) f |= X_USR;
 
-	if (val & S_IRGRP) f |= R_GRP;
-	if (val & S_IXGRP) f |= X_GRP;
+	/* Check user permissions */
+	if ((val & S_IRUSR) && uid == user.uid) {
+		if (!S_ISDIR(mode) || (val & S_IXUSR)) /* Check exec for directories */
+			return 1;
+    }
 
-	if (val & S_IROTH) f |= R_OTH;
-	if (val & S_IXOTH) f |= X_OTH;
+	/* Check group permissions */
+	if ((val & S_IRGRP) && (gid == user.gid || check_user_groups(gid) == 1)) {
+		if (!S_ISDIR(mode) || (val & S_IXGRP)) /* Check exec for directories */
+			return 1;
+    }
 
-	if (S_ISDIR(mode)) {
-		if ((f & R_USR) && (f & X_USR) && uid == user.uid)
+	/* Check other permissions */
+	if ((val & S_IROTH)) {
+		if (!S_ISDIR(mode) || (val & S_IXOTH)) /* Check exec for directories */
 			return 1;
-		if ((f & R_GRP) && (f & X_GRP) && (gid == user.gid
-		|| check_user_groups(gid) == 1))
-			return 1;
-		if ((f & R_OTH) && (f & X_OTH))
-			return 1;
-	} else {
-		if ((f & R_USR) && uid == user.uid)
-			return 1;
-		if ((f & R_GRP) && (gid == user.gid
-		|| check_user_groups(gid) == 1))
-			return 1;
-		if (f & R_OTH)
-			return 1;
-	}
+    }
+
+	return 0;
+}
+
+/* Return 1 if the command CMD exists (is found in PATH) and is executable
+ * (and readable) by the current user. Otherwise, return 0.
+ * This code is based on the file_status() function used by which(1). */
+int
+is_exec_cmd(const char *cmd)
+{
+	if (!cmd || !*cmd)
+		return 0;
+
+	struct stat a;
+	if (stat(cmd, &a) == -1 || !S_ISREG(a.st_mode))
+		return 0;
+
+	/* The file is read/exec by others (world). This is the most common case
+	 * for files in PATH. */
+	if ((a.st_mode & S_IXOTH) && (a.st_mode & S_IROTH))
+		return 1;
+
+	/* We're root: we only need the exec bit to be set for any of user,
+	 * group, or others. */
+	if (user.uid == (uid_t)0 && (a.st_mode & (S_IXUSR | S_IXGRP | S_IXOTH)))
+		return 1;
+
+	/* We're the owner of the file. */
+	if (user.uid == a.st_uid && (a.st_mode & S_IXUSR) && (a.st_mode & S_IRUSR))
+		return 1;
+
+	/* We're are in the file's group. */
+	if (check_user_groups(a.st_gid) == 1 && (a.st_mode & S_IXGRP)
+	&& (a.st_mode & S_IRGRP))
+		return 1;
 
 	return 0;
 }
@@ -487,9 +410,9 @@ is_number(const char *restrict str)
 	return 1;
 }
 
-/* Check if command STR contains a digit and this digit is not the first
- * char of STR. Used by find_cmd() (called by is_internal_c()) to check
- * for fused parameters in internal commands.
+/* Check if string STR contains a digit and this digit is not the first
+ * char in STR. Used by is_internal_cmd() to check for fused parameters in
+ * internal commands.
  * Returns the index of the digit in STR or -1 if no digit is found. */
 static int
 contains_digit(const char *str)
@@ -509,35 +432,6 @@ contains_digit(const char *str)
 	return (-1);
 }
 
-/* Return 1 if CMD is found in CMDS_LIST or zero otherwise. */
-static int
-find_cmd(const struct cmdslist_t *cmds_list, const size_t list_size, char *cmd)
-{
-	int found = 0;
-	int i = (int)list_size;
-	char c = 0;
-	const int d = contains_digit(cmd);
-
-	if (d != -1) {
-		c = cmd[d];
-		cmd[d] = '\0';
-	}
-
-	const size_t cmd_len = strlen(cmd);
-	while (--i >= 0) {
-		if (*cmd == *cmds_list[i].name && cmd_len == cmds_list[i].len
-		&& strcmp(cmd, cmds_list[i].name) == 0) {
-			found = 1;
-			break;
-		}
-	}
-
-	if (d != -1)
-		cmd[d] = c;
-
-	return found;
-}
-
 /* Check whether S is an action name.
  * Returns 1 if true or 0 otherwise. */
 int
@@ -555,182 +449,53 @@ is_action_name(const char *s)
 	return 0;
 }
 
-/* Return 1 if CMD is an internal command, or zero otherwise. */
+/* Return 1 if CMD is an internal command matching the flags in FLAG.
+ * Otherwise, return 0. */
 int
-is_internal_c(char *restrict cmd)
+is_internal_cmd(char *cmd, const int flag, const int check_hist,
+	const int check_search)
 {
-	if (find_cmd(internal_cmds, internal_cmds_n, cmd))
-		return 1;
-
-	/* Check for the search and history functions as well */
-	if ((*cmd == '/' && access(cmd, F_OK) != 0) || (*cmd == '!'
-	&& (IS_DIGIT(cmd[1]) || (cmd[1] == '-' && IS_DIGIT(cmd[2]))
-	|| cmd[1] == '!')))
-		return 1;
-
-	return 0;
-}
-
-/* Check cmd against a list of internal commands. Used by parse_input_str()
- * to know whether it should perform additional expansions, like glob, regex,
- * tilde, and so on. Only internal commands dealing with ELN/filenames
- * should be checked here. */
-int
-is_internal(char *restrict cmd)
-{
-	static struct cmdslist_t const int_cmds[] = {
-		{"ac", 2},
-		{"ad", 2},
-		{"bb", 2},
-		{"bleach", 6},
-		{"bm", 2},
-		{"bookmarks", 9},
-		{"bl", 2},
-		{"br", 2},
-		{"bulk", 4},
-		{"c", 1},
-		{"cd", 2},
-		{"d", 1},
-		{"dup", 3},
-		{"exp", 3},
-		{"export", 6},
-		{"jc", 2},
-		{"jp", 2},
-		{"l", 1},
-		{"le", 2},
-		{"m", 1},
-		{"mm", 2},
-		{"mime", 4},
-		{"n", 1},
-		{"new", 3},
-		{"o", 1},
-		{"oc", 2},
-		{"open", 4},
-		{"paste", 5},
-		{"p", 1},
-		{"pc", 2},
-		{"pp", 2},
-		{"pr", 2},
-		{"prop", 4},
-		{"pin", 3},
-		{"r", 1},
-		{"s", 1},
-		{"sel", 3},
-		{"t", 1},
-		{"tr", 2},
-		{"trash", 5},
-		{"tag", 3},
-		{"ta", 2},
-		{"te", 2},
-		{"v", 1},
-		{"vv", 2},
-		{NULL, 0}
-	};
-
-	static size_t i = 0;
-	if (i == 0)
-		i = (sizeof(int_cmds) / sizeof(struct cmdslist_t)) - 1;
-
-	if (find_cmd(int_cmds, i, cmd))
-		return 1;
-
-	/* Check for the search function as well */
-	if (*cmd == '/' && access(cmd, F_OK) != 0)
-		return 1;
-
-	return 0;
-}
-
-/* Check CMD against a list of internal commands taking ELN's or numbers
- * as parameters. Used by split_fusedcmd(). */
-int
-is_internal_f(const char *restrict cmd)
-{
-	/* If we are completing/suggesting, do not take 'ws', 'mf', and 'st/sort'
-	 * commands into account: they do not take ELN/filenames as parameters,
-	 * but just numbers, in which case no ELN-filename completion should
-	 * be made. */
-	if (flags & STATE_COMPLETING
-	&& (*cmd == 'w' || (*cmd == 'm' && *(cmd + 1) == 'f')
-	|| (*cmd == 's' && (*(cmd + 1) == 't' || *(cmd + 1) == 'o')) ) )
+	if (!cmd || !*cmd)
 		return 0;
 
-	static struct cmdslist_t const int_cmds[] = {
-		{"ac", 2},
-		{"ad", 2},
-		{"alias", 5}, /* 'alias import' takes file names */
-		{"bb", 2},
-		{"bh", 2}, // REMOVE AS SOON AS REPLACED BY DH
-		{"fh", 2}, // REMOVE AS SOON AS REPLACED BY DH
-		{"dh", 2},
-		{"bl", 2},
-		{"bleach", 6},
-		{"bm", 2},
-		{"bookmarks", 9},
-		{"br", 2},
-		{"bulk", 4},
-		{"c", 1},
-		{"cd", 2},
-		{"d", 1},
-		{"dup", 3},
-		{"ds", 2},
-		{"desel", 5},
-		{"exp", 3},
-		{"l", 1},
-		{"ln", 2},
-		{"le", 2},
-		{"m", 1},
-		{"mime", 4},
-		{"mm", 2},
-		{"md", 2},
-		{"mf", 2},
-		{"n", 1},
-		{"new", 3},
-		{"o", 1},
-		{"oc", 2},
-		{"open", 4},
-		{"ow", 2},
-		{"p", 1},
-		{"pc", 2},
-		{"pp", 2},
-		{"pr", 2},
-		{"prop", 4},
-		{"paste", 5},
-		{"pin", 3},
-		{"r", 1},
-		{"rr", 2},
-		{"s", 1},
-		{"sel", 3},
-		{"st", 2},
-		{"sort", 4},
-		{"t", 1},
-		{"tr", 2},
-		{"trash", 5},
-		{"tag", 3},
-		{"ta", 2},
-		{"te", 2},
-		{"tl", 2},
-		{"v", 1},
-		{"vv", 2},
-		{"ws", 2},
-		{"x", 1},
-		{"X", 1},
-		{NULL, 0}
-	};
+	/* Old is_internal_f() function */
+	if ((flags & STATE_COMPLETING) && (flag & PARAM_FNAME_NUM)
+	&& (*cmd == 'w' || (*cmd == 'm' && cmd[1] == 'f')
+	|| (*cmd == 's' && (cmd[1] == 't' || cmd[1] == 'o')) ) )
+		return 0;
 
-	static int n = 0;
-	if (n == 0)
-		n = (int)(sizeof(int_cmds) / sizeof(struct cmdslist_t)) - 1;
-	const size_t cmd_len = strlen(cmd);
+	char c = 0;
+	const int d = contains_digit(cmd);
+
+	if (d != -1) {
+		c = cmd[d];
+		cmd[d] = '\0';
+	}
+
+	const size_t clen = strlen(cmd);
+	ssize_t i = (ssize_t)internal_cmds_n;
+	int found = 0;
 
-	int i = n;
 	while (--i >= 0) {
-		if (*cmd == *int_cmds[i].name && cmd_len == int_cmds[i].len
-		&& strcmp(cmd, int_cmds[i].name) == 0) {
-			return 1;
+		if (((flag & ALL_CMDS) || (internal_cmds[i].flag & flag))
+		&& clen == internal_cmds[i].len && *cmd == *internal_cmds[i].name
+		&& strcmp(cmd + 1, internal_cmds[i].name + 1) == 0) {
+			found = 1;
+			break;
 		}
 	}
 
+	if (d != -1)
+		cmd[d] = c;
+
+	if (found == 1)
+		return 1;
+
+	if ((check_search == 1 && (*cmd == '/' && access(cmd, F_OK) != 0))
+	|| (check_hist == 1 && (*cmd == '!' && (IS_DIGIT(cmd[1])
+	|| (cmd[1] == '-' && IS_DIGIT(cmd[2])) || cmd[1] == '!'))))
+		return 1;
+
 	return 0;
 }
 
@@ -768,13 +533,13 @@ is_bin_cmd(char *str)
 }
 
 int
-check_regex(char *str)
+check_regex(const char *str)
 {
 	if (!str || !*str)
 		return FUNC_FAILURE;
 
 	int char_found = 0;
-	char *p = str;
+	const char *p = str;
 
 	while (*p) {
 		/* If STR contains at least one of the following chars */
@@ -787,14 +552,14 @@ check_regex(char *str)
 		p++;
 	}
 
-	/* And if STR is not a file name, take it as a possible regex */
+	/* And if STR is not a filename, take it as a possible regex */
 	if (char_found == 1 && access(str, F_OK) == -1)
 		return FUNC_SUCCESS;
 
 	return FUNC_FAILURE;
 }
 
-/* Returns the parsed aliased command in an array of strings if
+/* Returns the parsed aliased command in an array of strings if a
  * matching alias is found, or NULL if not. */
 char **
 check_for_alias(char **args)
@@ -820,31 +585,32 @@ check_for_alias(char **args)
 		&& sanitize_cmd(aliases[i].cmd, SNT_GRAL) == FUNC_FAILURE)
 			continue;
 
-		args_n = 0; /* Reset args_n to be used by parse_input_str() */
+		args_n = 0; /* Reset args_n to be used by parse_input_str(). */
 
-		char **alias_comm = parse_input_str(aliases[i].cmd);
-		if (!alias_comm) {
-			flags |= FAILED_ALIAS; /* Prevent exec_cmd() from being executed */
+		char **alias_cmd = parse_input_str(aliases[i].cmd);
+		if (!alias_cmd) {
+			flags |= FAILED_ALIAS; /* Prevent exec_cmd() from being executed. */
 			return (char **)NULL;
 		}
 
 		size_t j;
-		/* Add input parameters, if any, to alias_comm */
+		/* Add input parameters, if any, to alias_cmd. Expansions for these
+		 * parameters were already performed before calling this function. */
 		if (args[1]) {
 			for (j = 1; args[j]; j++) {
-				alias_comm = xnrealloc(alias_comm,
-				    ++args_n + 2, sizeof(char *));
-				alias_comm[args_n] = savestring(args[j], strlen(args[j]));
+				alias_cmd = xnrealloc(alias_cmd, ++args_n + 2, sizeof(char *));
+				alias_cmd[args_n] = savestring(args[j], strlen(args[j]));
 			}
 		}
 
-		alias_comm[args_n + 1] = (char *)NULL;
+		alias_cmd[args_n + 1] = (char *)NULL;
 
-		/* Free original command */
+		/* Free the original command. */
 		for (j = 0; args[j]; j++)
 			free(args[j]);
 		free(args);
-		return alias_comm;
+
+		return alias_cmd;
 	}
 
 	return (char **)NULL;
@@ -853,7 +619,7 @@ check_for_alias(char **args)
 /* Keep only the last MAX records in FILE.
  * If CHECK_DUPS is 1, skip consecutive equal entries. */
 void
-truncate_file(char *file, const int max, const int check_dups)
+truncate_file(const char *file, const int max, const int check_dups)
 {
 	if (config_ok == 0 || !file || !*file)
 		return;
@@ -885,7 +651,7 @@ truncate_file(char *file, const int max, const int check_dups)
 
 	int n = 0, c;
 
-	/* Count newline chars to get the amount of lines in the file. */
+	/* Count newline chars to get the number of lines in the file. */
 	while ((c = fgetc(orig_fp)) != EOF && n < INT_MAX) { /* Flawfinder: ignore */
 		if (c == '\n')
 			n++;
