@@ -1,17 +1,15 @@
-/* colors.h */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* colors.h */
+
 #ifndef COLORS_H
 #define COLORS_H
 
@@ -33,7 +33,7 @@
 #define SPLIT_INTERFACE_COLORS 0
 #define SPLIT_FILETYPE_COLORS  1
 
-/* Max amount of custom color variables in the color scheme file. */
+/* Max number of custom color variables in the color scheme file. */
 #define MAX_DEFS 128
 
 /* Macros for the set_shades function. */
@@ -47,6 +47,9 @@
 #define COLOR256_PREFIX  '@'
 #define IS_COLOR_PREFIX(c) ((c) == RGB_COLOR_PREFIX || (c) == COLOR256_PREFIX)
 
+#define LS_COLORS_GNU  1
+#define LS_COLORS_BSD  2
+
 __BEGIN_DECLS
 
 void color_codes(void);
@@ -55,12 +58,12 @@ int  cschemes_function(char **args);
 #ifndef CLIFM_SUCKLESS
 size_t get_colorschemes(void);
 #endif /* CLIFM_SUCKLESS */
-char *get_dir_color(const char *filename, const mode_t mode,
-	const nlink_t links, const filesn_t count);
-char *get_entry_color(char *ent, const struct stat *s);
+char *get_dir_color(const char *filename, const struct stat *a,
+	const filesn_t count);
+char *get_entry_color(char *ent, const struct stat *a);
 char *get_ext_color(const char *ext, size_t *val_len);
-char *get_file_color(const char *filename, const struct stat *attr);
-char *get_regfile_color(const char *filename, const struct stat *attr,
+char *get_file_color(const char *filename, const struct stat *a);
+char *get_regfile_color(const char *filename, const struct stat *a,
 	size_t *is_ext);
 int  import_color_scheme(const char *name);
 void update_warning_prompt_text_color(void);
