@@ -1,17 +1,15 @@
-/* config.h */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,12 +20,15 @@
  * MA 02110-1301, USA.
 */
 
+/* config.h */
+
 #ifndef CONFIG_H
 #define CONFIG_H
 
 #define DUMP_CONFIG_STR  0
 #define DUMP_CONFIG_INT  1
 #define DUMP_CONFIG_BOOL 2
+#define DUMP_CONFIG_CHR  3
 
 __BEGIN_DECLS
 
@@ -39,10 +40,12 @@ int  create_mime_file(char *file, const int new_prof);
 void create_tmp_files(void);
 void init_config(void);
 int  reload_config(void);
+int  config_reload(const char *arg);
 void set_div_line(char *line);
 void set_env(const int reload);
 #ifndef _NO_FZF
-void set_fzf_border_type(char *line);
+int  get_fzf_border_type(const char *line);
+int  get_fzf_height(char *line);
 #endif /* !_NO_FZF */
 void set_sel_file(void);
 void set_time_style(char *line, char **str, const int ptime);
