@@ -1,17 +1,15 @@
-/* file_operations.h */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* file_operations.h */
+
 #ifndef FILE_OPERATIONS_H
 #define FILE_OPERATIONS_H
 
@@ -69,7 +69,7 @@ int  open_function(char **cmd);
 int  remove_files(char **args);
 int  symlink_file(char **args);
 int  toggle_exec(const char *file, mode_t mode);
-int  umask_function(char *arg);
+int  umask_function(const char *arg);
 int  xchmod(const char *file, const char *mode_str, const int flag);
 
 __END_DECLS
