@@ -1,17 +1,15 @@
-/* fsinfo.h */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* fsinfo.h */
+
 #ifndef FSINFO_H
 #define FSINFO_H
 
@@ -29,10 +29,14 @@ __BEGIN_DECLS
 
 #if defined(LINUX_FSINFO)
 char *get_fs_type_name(const char *file, int *remote);
+#if !defined(__CYGWIN__) && !defined(__ANDROID__)
 char *get_dev_name(const dev_t dev);
+#endif /* !__CYGWIN__ && !__ANDROID__ */
 char *get_dev_name_mntent(const char *file);
 #elif defined(HAVE_STATFS)
 void get_dev_info(const char *file, char **devname, char **devtype);
+#elif defined(__sun)
+char *get_dev_mountpoint(const char *file);
 #endif /* LINUX_FSINFO */
 
 __END_DECLS
