@@ -1,17 +1,15 @@
-/* help.c -- home of clifm's help system */
-
 /*
- * This file is part of CliFM
- * 
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * This file is part of Clifm
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,11 +20,11 @@
  * MA 02110-1301, USA.
 */
 
+/* help.c -- home of clifm's help system */
+
 #include "helpers.h"
 
 #include <errno.h>  /* errno */
-#include <stdio.h>
-//#include <stdlib.h>
 #include <string.h>
 #include <unistd.h> /* unlink */
 
@@ -61,87 +59,83 @@ print_tips(const int all)
 		"Get mime information for a file. E.g.: 'mm info file'",
 		"Manage default associated applications with the 'mime' command",
 		"Customize previewing applications: 'view edit' or F7",
-		"List mountpoints: 'mp' or Alt-m",
-		"Compress and decompress files using the 'ac' and 'ad' " /* NOLINT */
-		"commands respectively. E.g.: 'ac sel' or 'ad file.zip'",
+		"List mountpoints: 'mp' or Alt+m",
+		"Compress files with 'ac' and decompress with 'ad'. E.g.: 'ac sel' or 'ad file.zip'",
 		"Disallow the use of shell commands with the -x option: 'clifm -x'",
-		"Don't like ELN's? Disable them using the -e command line switch",
-		"Disable file previews for TAB completion (fzf mode only) via --no-fzfpreview",
+		"Don't like ELNs? Disable them using the -e command line switch",
+		"Disable file previews in fzf mode with --no-fzfpreview",
 # ifdef __linux__
-		"Manage removable devices via the 'media' command",
+		"Manage removable devices with the 'media' command",
 # endif /* __linux__ */
 #endif /* _BE_POSIX */
 #ifndef _BE_POSIX
 		"Create a new profile: 'pf add PROFILE' or 'clifm -P PROFILE'",
-		"Run in incognito/private mode (-S, --stealth-mode)",
+		"Enable incognito/private mode with -S or --stealth-mode",
 		"Run in read-only mode via --readonly",
-		"Running in an untrusted environment? Try the --secure-env and "
-		"--secure-cmds command line switches",
+		"Use --secure-env and --secure-cmds for secure operation in untrusted environments",
 		"Run in disk usage analyzer mode using the -t command line switch",
 		"Fuzzy suggestions are supported. E.g.: 'dwn > Downloads'. Enable them via --fuzzy-matching, or FuzzyMatching in the configuration file",
-		"Miss LS_COLORS? Run with --lscolors (GNU ls only)",
+		"Miss LS_COLORS? Run with --lscolors",
 #else
 		"Create a new profile: 'pf add PROFILE' or 'clifm -p PROFILE'",
-		"Run in incognito/private mode via the -s flag",
+		"Enable incognito/private mode with the -s flag",
 		"Running in an untrusted environment? Try the -x, -X, and -Y flags",
 		"Run in disk usage analyzer mode using the -u flag",
-		"Fuzzy suggestions are supported. E.g.: 'dwn > Downloads'. Enable them via the -m flag (or FuzzyMatching in the configuration file)",
+		"Fuzzy suggestions are supported (e.g.: 'dwn > Downloads'). Enable them via the -m flag (or FuzzyMatching in the configuration file)",
 #endif /* _BE_POSIX */
-		"Clear the screen: 'rf', '.', Enter (on empty line), or Ctrl-l",
+		"Clear the screen: 'rf', '.', Enter (on empty line), or Ctrl+l",
 		"Try the autocd and auto-open functions: run 'FILE' instead "
 		"of 'cd FILE' or 'open FILE'",
 		"Do not forget to take a look at the manpage",
-		"Need more speed? Try the light mode ('lm' or Alt-y)",
+		"Need more speed? Try the light mode ('lm' or Alt+y)",
 		"The Selection Box is shared among different instances of Clifm",
 		"Use the 's' command to select files here and there: 's FILE...'",
 		"Use wildcards and regular expressions to select files. E.g.: "
 		"'s *.c' or 's .*\\.c$'",
 		"Operate on selected files. E.g.: 'p sel' or 'p s:'",
 		"List selected files: 'sb' or s:<TAB>",
-		"ELN's and the 'sel' keyword work for shell commands as well. E.g.: "
-		"'ls -ld 1 sel'",
+		"Use ELNs and 'sel' with shell commands, like 'ls -ld 1 sel'",
 		"Press TAB to automatically expand an ELN. E.g.: 's 2<TAB>' -> 's FILENAME'",
 		"Use ranges (ELN-ELN) to easily move multiple files. E.g.: 'm 3-12 dir/'",
 		"Trash files with a simple 't FILE'",
-		"Too many files? Run the pager: 'pg' or Alt-0",
-		"Switch to long/detail view mode: 'll' or Alt-l",
+		"Too many files? Run the pager: 'pg' or Alt+0",
+		"Toggle the long-view: 'll' or Alt+l",
 		"Search for files using the slash command. E.g.: '/*.png'",
-		"The search function allows regular expressions. E.g.: '/^c'",
+		"The search function supports regular expressions. E.g.: '/^c'",
 		"Add a new bookmark: 'bm add FILENAME BM_NAME'",
 		"Use c, l, m, md, and r instead of cp, ln, mv, mkdir, and rm",
-		"Access a remote file system using the 'net' command",
-		"Go back and forth in the directory history with Alt-j and Alt-k "
+		"Access a remote filesystem using the 'net' command",
+		"Move back and forth in the directory history with Alt+j and Alt+k "
 		"(also Shift-Left and Shift-Right)",
 		"Run a new instance of Clifm: 'x DIR'",
 		"Send a command directly to the system shell. E.g.: ';ls -l *'",
 		"Run the last executed command: '!!'",
-		"Access the commands history list: '!<TAB>'",
+		"Access the command history list: '!<TAB>'",
 		"Exclude commands from history using the HistIgnore option in the configuration file (F10)",
 		"Access the directory history list: 'dh <TAB>'",
 		"List previously used search patterns: '/*<TAB>'",
 		"Import aliases from file: 'alias import FILE'",
 		"List available aliases: 'alias'",
 		"Create aliases to easily run your preferred commands (F10)",
-		"Open and edit the configuration file: 'config' or F10",
 		"Get a brief description for each Clifm command: 'cmd<TAB>'",
 		"Preview the current color scheme: 'cs preview'",
-		"Toggle hidden files on/off: 'hh' or Alt-.",
-		"Toggle follow-links (long view only) on/off: 'k' or Alt-+",
-		"Go to the root directory: Alt-r",
-		"Go to the home directory: Alt-e (or just 'cd')",
+		"Toggle hidden files: 'hh' or Alt+.",
+		"Toggle follow-links (long view only): 'k' or Alt++",
+		"Change to the root directory: Alt+r",
+		"Change to the home directory: Alt+e (or just 'cd')",
 		"Open and edit the current color scheme file: F8 (or 'cs edit')",
 		"Open and edit the keybindings file: F9 (or 'kb edit')",
 		"Open and edit the main configuration file: F10 (or 'config')",
 		"Open and edit the bookmarks file: F11 (or 'bm edit')",
-		"Open and edit the bookmarks file: F6 (or 'mm edit')",
+		"Open and edit the MIME list file: F6 (or 'mm edit')",
 		"Set the starting path. E.g.: 'clifm ~/media'",
 		"Use the 'o' command to open files and directories. E.g.: 'o 12'",
-		"Open a file or directory by just entering its ELN or file name",
-		"Bypass the resource opener specifying an application. E.g.: '12 leafpad'",
-		"Open a file and send it to the background. E.g.: '24&'",
+		"Open a file or directory by just entering its ELN or filename",
+		"Bypass the file opener by specifying an application. E.g.: '12 leafpad'",
+		"Open a file in the background. E.g.: '24&'",
 		"Create a custom prompt by editing the prompts file ('prompt edit')",
 		"Customize your color scheme: 'cs edit' or F8",
-		"Launch the bookmarks manager: 'bm' or Alt-b",
+		"Launch the bookmark manager: 'bm' or Alt+b",
 		"Quickly list your bookmarks: 'b:<TAB>'",
 		"Change to a bookmark: 'bm NAME' or 'b:NAME'",
 		"Chain commands using ';' and '&&'. E.g.: 's 2 7-10; r sel'",
@@ -150,18 +144,18 @@ print_tips(const int all)
 		"Delete a profile: 'pf del PROFILE'",
 		"Rename a profile: 'pf rename PROFILE'",
 		"Use 'p ELN' to print file properties for ELN",
-		"Deselect all selected files: 'ds *' or Alt-d",
-		"Select all files in the current directory: 's *' or Alt-a",
-		"Jump to the Selection Box: 'sb' or Alt-s",
+		"Deselect all selected files: 'ds *' or Alt+d",
+		"Select all files in the current directory: 's *' or Alt+a",
+		"Jump to the Selection Box: 'sb' or Alt+s",
 		"Selectively restore trashed files using the 'u' command",
 		"Empty the trash can: 't empty'",
-		"Toggle list-directories-first on/off: 'ff' or Alt-g",
-		"Set the files counter on/off: 'fc'",
+		"Toggle list-directories-first: 'ff' or Alt+g",
+		"Set the file counter on/off: 'fc'",
 		"Take a look at the splash screen with the 'splash' command",
 		"Have some fun trying the 'bonus' command",
 		"Launch the default system shell in CWD using ':' or ';'",
-		"Switch files sorting order: Alt-z and Alt-x",
-		"Reverse files sorting order: 'st rev'",
+		"Cycle through file sort orders: Alt+z and Alt+x",
+		"Reverse the file sort order: 'st rev'",
 		"Rename multiple files at once. E.g.: 'br *.txt'",
 		"Need no more tips? Disable this feature in the configuration file",
 		"Need root privileges? Launch a new instance of Clifm as root "
@@ -174,11 +168,11 @@ print_tips(const int all)
 		"Pin a file via the 'pin' command and then use it with the "
 		"period keyword (,). E.g.: 'pin DIR' and then 'cd ,'",
 		"Switch color schemes using the 'cs' command",
-		"Try the 'j' command to quickly jump into any visited directory",
-		"Switch workspaces by pressing Alt-[1-4]",
+		"Try the 'j' command to quickly jump to any visited directory",
+		"Switch workspaces by pressing Alt+[1-4]",
 		"Use the 'ws' command to list available workspaces",
 		"Take a look at available plugins using the 'actions' command",
-		"Space is not needed. E.g.: 'p12' instead of 'p 12'",
+		"Space is not required. E.g.: 'p12' instead of 'p 12'",
 		"When searching or selecting files, use the exclamation mark "
 		"to negate a pattern. E.g.: 's !*.pdf'",
 		"Enable the TrashAsRm option to always send removed files to the trash can",
@@ -186,20 +180,20 @@ print_tips(const int all)
 		"Add prompt commands via the 'promptcmd' keyword: 'config' (F10)",
 		"Need git integration? Consult the manpage",
 		"Accept a given suggestion by pressing the Right arrow key",
-		"Accept only the first suggested word by pressing Alt-f or Alt-Right",
+		"Accept only the first suggested word by pressing Alt+f or Alt+Right",
 		"Enter 'c sel' to copy selected files into the current directory",
-		"Press Alt-q to delete the last entered word",
+		"Press Alt+q to delete the last entered word",
 		"Check ELN ranges by pressing TAB. E.g.: '1-12<TAB>'",
 		"Operate on specific selected files. E.g.: 'p sel<TAB>' or 'p s:<TAB>'",
-		"Use the 'ow' command to open a file with a specific application",
-		"Limit the amount of files listed on the screen via the 'mf' command",
-		"Set a maximum file name length for listed files via the MaxFilenameLen "
+		"Use the 'ow' command to open a file using a specific application",
+		"Limit the number of files listed on the screen via the 'mf' command",
+		"Set a maximum filename length for listed files via the MaxFilenameLen "
 		"option in the configuration file (F10)",
 		"Use the 'm' command to interactively rename a file. E.g.: 'm 12'",
-		"Set options on a per directory basis via the autocommands function. Try 'help autocommands'",
-		"Clean up non-ASCII file names using the 'bleach' command",
-		"Get help for any internal command via the -h or --help parameters: 'p -h'",
-		"Enable icons with 'icons on'",
+		"Set options on a per-directory basis via the autocommands function. Try 'help autocommands'",
+		"Sanitize non-ASCII filenames using the 'bleach' command",
+		"Get help for internal commands using the -h or --help parameters: 'p -h'",
+		"Enable icons with 'icons on' (or --icons)",
 		"Quickly change to any parent directory using the 'bd' command",
 		"Use 'stats' to print statistics on files in the current directory",
 		"Customize the warning prompt by setting WarningPrompt in the prompts file ('prompt edit')",
@@ -208,19 +202,20 @@ print_tips(const int all)
 		"Remove files in bulk using a text editor with 'rr'",
 		"Easily send files to a remote location with the 'cr' command",
 		"Quickly switch prompts via 'prompt NAME' (or 'prompt set <TAB>')",
-		"Press Alt-TAB to toggle the disk usage analyzer mode",
-		"Press Ctrl-Alt-l to toggle max file name length on/off",
-		"Wildcards can be expanded via TAB. E.g.: 's *.c<TAB>'",
-		"Try the help topics: 'help <TAB>'",
+		"Press Alt+Tab to toggle the disk usage analyzer mode",
+		"Press Ctrl+Alt+l to toggle max filename length",
+		"Wildcards can be expanded with the TAB key. E.g.: 's *.c<TAB>'",
+		"Try help topics: 'help <TAB>'",
 		"List clifm commands, together with a brief description: 'cmd<TAB>'",
 		"List symlinks in the current directory: '=l<TAB>'. Try 'help file-filters' for more information",
-		"Use PropFields in the configuration file to customize fields in long view mode",
+		"Use PropFields in the configuration file to customize fields in long view",
 		"Preview files in the current directory using the 'view' command (requires fzf)",
-		"Press Alt+- to launch the files previewer (requires fzf)",
+		"Press Alt+- to launch the file previewer (requires fzf)",
 		"Interactively select files (requires fzf, fnf, or smenu). E.g.: 's /dir/*<TAB>'",
-		"Change files permissions/ownership using the 'pc' and 'oc' commands respectively",
+		"Change file permissions/ownership using the 'pc' and 'oc' commands respectively",
 		"Set a custom shell to run external commands. E.g.: 'CLIFM_SHELL=/bin/dash clifm'",
 		"Print all tips: 'tips'",
+		"Create files from a template. Run 'n --help' for details.",
 		NULL};
 
 	const size_t tipsn = (sizeof(TIPS) / sizeof(TIPS[0])) - 1;
@@ -249,31 +244,31 @@ print_tips(const int all)
 }
 
 /* Retrieve pager path, first from PAGER, then try less(1), and finally
- * more(1). If none is found returns NULL. */
+ * more(1). If none is found return NULL. */
 static char *
 get_pager(void)
 {
-	char *_pager = (char *)NULL;
+	char *pager_cmd = (char *)NULL;
 	char *p = getenv("PAGER");
 	if (p) {
 		char *s = strchr(p, ' ');
 		if (s) *s = '\0';
-		_pager = savestring(p, strlen(p));
-		return _pager;
+		pager_cmd = savestring(p, strlen(p));
+		return pager_cmd;
 	}
 
 	p = get_cmd_path("less");
 	if (p) {
-		_pager = savestring(p, strlen(p));
+		pager_cmd = savestring(p, strlen(p));
 		free(p);
-		return _pager;
+		return pager_cmd;
 	}
 
 	p = get_cmd_path("more");
 	if (p) {
-		_pager = savestring(p, strlen(p));
+		pager_cmd = savestring(p, strlen(p));
 		free(p);
-		return _pager;
+		return pager_cmd;
 	}
 
 	return (char *)NULL;
@@ -305,9 +300,7 @@ print_autocmds_topic(void)
 static int
 print_basics_topic(void)
 {
-	puts(_("Run '?' and consult the BASIC FILE OPERATIONS section\n"
-		"Try also 'c --help' for more information about basic "
-		"file management commands"));
+	printf(_("Run '?' or 'help' to get started with %s\n"), PROGRAM_NAME);
 	return FUNC_SUCCESS;
 }
 
@@ -359,9 +352,9 @@ print_file_filters_topic(void)
 }
 
 static int
-print_file_previews_topic(void)
+print_file_previews_topic(const int image_prevs)
 {
-	puts(FILE_PREVIEWS);
+	puts(image_prevs == 1 ? IMAGE_PREVIEWS : FILE_PREVIEWS);
 	putchar('\n');
 	print_more_info();
 	return FUNC_SUCCESS;
@@ -463,7 +456,9 @@ run_help_topic(const char *topic)
 	if (*topic == 'f' && strcmp(topic, "file-filters") == 0)
 		return print_file_filters_topic();
 	if (*topic == 'f' && strcmp(topic, "file-previews") == 0)
-		return print_file_previews_topic();
+		return print_file_previews_topic(0);
+	if (*topic == 'i' && strcmp(topic, "image-previews") == 0)
+		return print_file_previews_topic(1);
 	if (*topic == 'f' && strcmp(topic, "file-tags") == 0)
 		return print_file_tags_topic();
 	if (*topic == 'n' && strcmp(topic, "navigation") == 0)
