@@ -1,17 +1,15 @@
-/* helpers.h -- main header file */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,14 +20,16 @@
  * MA 02110-1301, USA.
 */
 
+/* helpers.h -- main header file */
+
 #ifndef HELPERS_H
 #define HELPERS_H
 
 #define PROGRAM_NAME_UPPERCASE "Clifm"
 #define PROGRAM_NAME "clifm"
 #define PROGRAM_DESC "The command line file manager"
-#define VERSION "1.21"
-#define DATE "Sep 30, 2024"
+#define VERSION "1.26.3"
+#define DATE "Sep 28, 2025"
 #define AUTHOR "L. Abramovich"
 #define CONTACT "https://github.com/leo-arch/clifm"
 #define LICENSE "GPL2+"
@@ -128,6 +128,7 @@
 #include <glob.h>
 #include <limits.h>
 #include <regex.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>  /* S_BLKSIZE */
 #include <sys/types.h> /* ssize_t */
@@ -145,6 +146,10 @@
 # undef _DIRENT_HAVE_D_TYPE
 #endif /* _BE_POSIX && _DIRENT_HAVE_D_TYPE */
 
+#if !defined(TIGHT_COLUMNS) && !defined(NO_TIGHT_COLUMNS)
+# define TIGHT_COLUMNS
+#endif /* !TIGHT_COLUMNS && !NO_TIGHT_COLUMNS */
+
 #ifdef CLIFM_LEGACY
 /* Replace functions not available before POSIX-1.2008. More precisely,
  * let's try to be POSIX-1.2001 compliant. This is still experimental:
@@ -280,12 +285,25 @@ if (S_ISNWK(mode)) return 'n'; // HP/UX: network special file
 # include <stdint.h> /* uint8_t */
 #endif /* __linux__ */
 
+#if !defined(_BE_POSIX)
+# if defined(__GLIBC__) \
+|| (defined(__FreeBSD__) && __FreeBSD_version >= 604000) \
+|| (defined(__OpenBSD__) && OpenBSD >= 200805) \
+|| (defined(__sun) && defined(SUN_VERSION) && SUN_VERSION >= 511)
+#  define HAVE_MEMRCHR /* OpenBSD 200805 == version 4.3 */
+# elif defined(__NetBSD__)
+#  if __NetBSD_Prereq__(6,0,0)
+#   define HAVE_MEMRCHR
+#  endif
+# endif /* __GLIBC__ */
+#endif /* !_BE_POSIX */
+
 #if defined(__OpenBSD__) || defined(__NetBSD__) \
 || defined(__FreeBSD__) || defined(__APPLE__)
 # include <inttypes.h> /* uintmax_t, intmax_t */
 #endif /* BSD */
 
-/* File system event monitors (inotify and kqueue) are OS-specific.
+/* Filesystem event monitors (inotify and kqueue) are OS-specific.
  * Let's fallback to our own generic monitor. */
 #if defined(_BE_POSIX) && !defined(USE_GENERIC_FS_MONITOR)
 # define USE_GENERIC_FS_MONITOR
@@ -313,19 +331,9 @@ if (S_ISNWK(mode)) return 'n'; // HP/UX: network special file
 # endif /* MACOS_X < 10.10 */
 #endif /* __APPLE__ */
 
-/* GCC < 4.6 does not allow pragmas inside functions.
- * See https://gcc.gnu.org/gcc-4.6/changes.html */
-#if defined(__GNUC__)
-# define GCC_VERSION (__GNUC__ * 10000       \
-                      + __GNUC_MINOR__ * 100 \
-                      + __GNUC_PATCHLEVEL__)
-# if GCC_VERSION >= 40600
-#  define GCC_ALLOWS_PRAGMA_IN_FUNC
-# endif /* GCC > 4.6 */
-#endif /* __GNUC__ */
-
 #include "strings.h"
 #include "settings.h"
+#include "term.h"
 
 /* General exit codes for functions */
 #define FUNC_SUCCESS 0 /* True/success */
@@ -401,13 +409,17 @@ if (S_ISNWK(mode)) return 'n'; // HP/UX: network special file
 #ifndef _BE_POSIX
 # ifdef LINUX_STATX
 #  define ST_BTIME stx_btime
-/* OpenBSD defines the interface (see sys/stat.h), but the file system doesn't
+/* OpenBSD defines the interface (see sys/stat.h), but the filesystem doesn't
  * actually store creation times: the value of __st_birthtim is always zero.
 #elif defined(__OpenBSD__)
 # define ST_BTIME __st_birthtim */
-# elif defined(__NetBSD__) || (defined(__APPLE__) && !defined(__ppc__) \
-&& !defined(__i386__))
+# elif defined(__NetBSD__)
 #  define ST_BTIME st_birthtimespec
+# elif defined(__APPLE__)
+#  include <AvailabilityMacros.h>
+#  if MAC_OS_X_VERSION_MIN_REQUIRED >= 1060
+#   define ST_BTIME st_birthtimespec
+#  endif
 # elif defined(__FreeBSD__) || defined(__CYGWIN__)
 #  define ST_BTIME st_birthtim
 # elif defined(__HAIKU__)
@@ -427,7 +439,7 @@ if (S_ISNWK(mode)) return 'n'; // HP/UX: network special file
 # define ST_BTIME_LIGHT
 #endif /* ST_BTIME && !LINUX_STATX && !__sun */
 
-/* File system events handling */
+/* Filesystem events handling */
 #if defined(LINUX_INOTIFY)
 # define NUM_EVENT_SLOTS 32 /* Make room for 32 events */
 # define EVENT_SIZE (sizeof(struct inotify_event))
@@ -485,11 +497,13 @@ extern time_t curdir_mtime;
 # define HAVE_PLEDGE
 #endif
 
-#if !defined(_BE_POSIX) && defined(__linux__)
+#if !defined(_BE_POSIX) && (defined(__linux__) || defined(__CYGWIN__))
 # define LINUX_FSINFO
 #endif
 
-#define DEV_NO_NAME "-" /* String used when no file system name/type is found */
+#define DEV_NO_NAME "-" /* String used when no filesystem name/type is found */
+
+#define TRUECOLOR_NUM 16777216
 
 /* This is a more or less arbitrary value, but better than some huge value
  * like INT_MAX (which most likely will cause problems long before reaching
@@ -519,21 +533,20 @@ extern time_t curdir_mtime;
 
 /* Used by the refresh on resize feature */
 #define DELAYED_REFRESH     (1 << 2)
-#define PATH_PROGRAMS_ALREADY_LOADED (1 << 3)
-
-#define FIRST_WORD_IS_ELN   (1 << 4)
-#define IN_BOOKMARKS_SCREEN (1 << 5)
-#define STATE_COMPLETING    (1 << 6)
-/* Instead of a completion for the current word, a BAEJ suggestion points to
- * a possible completion as follows: WORD > COMPLETION */
-#define BAEJ_SUGGESTION     (1 << 7)
-#define STATE_SUGGESTING    (1 << 8)
-#define IN_SELBOX_SCREEN    (1 << 9)
-#define MULTI_SEL           (1 << 10)
-#define PREVIEWER           (1 << 11)
-#define KITTY_TERM          (1 << 12)
-#define NO_FIX_RL_POINT     (1 << 13)
-#define FAILED_ALIAS        (1 << 14)
+#define FIRST_WORD_IS_ELN   (1 << 3)
+#define IN_BOOKMARKS_SCREEN (1 << 4)
+#define STATE_COMPLETING    (1 << 5)
+/* Instead of a completion for the current query, a BAEJ suggestion points to
+ * a possible completion as follows: QUERY > COMPLETION */
+#define BAEJ_SUGGESTION     (1 << 6)
+#define STATE_SUGGESTING    (1 << 7)
+#define IN_SELBOX_SCREEN    (1 << 8)
+#define MULTI_SEL           (1 << 9)
+#define PREVIEWER           (1 << 10)
+#define NO_FIX_RL_POINT     (1 << 11)
+#define FAILED_ALIAS        (1 << 12)
+#define ALT_PREVIEW_FILE    (1 << 13)
+#define UEBERZUG_IMG_PREV   (1 << 14)
 
 /* Flags for third party binaries */
 #define FZF_BIN_OK     (1 << 0)
@@ -556,17 +569,19 @@ extern time_t curdir_mtime;
 # define CHECK_COREUTILS
 #endif /* (__OpenBSD__ || __NetBSD__ || __APPLE__ || __sun) && !_BE_POSIX */
 
-/* File ownership flags (used by check_file_access() in checks.c) */
-#define R_USR (1 << 1)
-#define X_USR (1 << 2)
-#define R_GRP (1 << 3)
-#define X_GRP (1 << 4)
-#define R_OTH (1 << 5)
-#define X_OTH (1 << 6)
-
 /* Flag to control the search function behavior */
 #define NO_GLOB_CHAR (1 << 0)
 
+/* Control show hidden files mode (true, false, first, last) */
+#define HIDDEN_FALSE 0
+#define HIDDEN_TRUE  1
+#define HIDDEN_FIRST 2
+#define HIDDEN_LAST  3
+
+#define DESKTOP_NOTIF_NONE   0
+#define DESKTOP_NOTIF_SYSTEM 1
+#define DESKTOP_NOTIF_KITTY  2
+
 /* Search strategy */
 #define GLOB_ONLY  0
 #define REGEX_ONLY 1
@@ -629,20 +644,41 @@ extern time_t curdir_mtime;
  * when passing color codes to readline. */
 #define RL_NC "\001\x1b[0m\002"
 
-#define UNSET  (-1)
+#define SET_MSG_PTR (term_caps.unicode == 1 ? MSG_PTR_STR_U : MSG_PTR_STR)
+#define SET_SUCCESS_PTR (term_caps.unicode == 1 ? SUCCESS_PTR_STR_U : SUCCESS_PTR_STR)
+#define SET_MISC_PTR (term_caps.unicode == 1 ? MISC_PTR_U : MISC_PTR)
+/* Unused
+#define SET_ERROR_PTR (term_caps.unicode == 1 ? ERROR_PTR_STR_U : ERROR_PTR_STR)
+*/
+
+#define UNSET (-1)
+
+/* Macros for the update_autocmd_opts() function */
+#define AC_COLOR_SCHEME  0
+#define AC_FILES_COUNTER 1
+#define AC_FILTER        2
+#define AC_FULL_DIR_SIZE 3
+#define AC_LIGHT_MODE    4
+#define AC_LONG_VIEW     5
+#define AC_MAX_FILES     6
+#define AC_MAX_NAME_LEN  7
+#define AC_ONLY_DIRS     8
+#define AC_PAGER         9
+#define AC_SHOW_HIDDEN   10
+#define AC_SORT          11
 
 /* Macros for the cp and mv cmds */
-#define CP_CP            0 /* cp -iRp */
+#define CP_CP            0 /* cp -Rp */
 #define CP_CP_FORCE      1 /* cp -Rp */
-#define CP_ADVCP         2 /* advcp -giRp */
+#define CP_ADVCP         2 /* advcp -gRp */
 #define CP_ADVCP_FORCE   3 /* advcp -gRp */
 #define CP_WCP           4 /* wcp */
 #define CP_RSYNC         5 /* rsync -avP */
 #define CP_CMD_AVAILABLE 6
 
-#define MV_MV            0 /* mv -i */
+#define MV_MV            0 /* mv */
 #define MV_MV_FORCE      1 /* mv */
-#define MV_ADVMV         2 /* advmv -gi */
+#define MV_ADVMV         2 /* advmv -g */
 #define MV_ADVMV_FORCE   3 /* advmv -g */
 #define MV_CMD_AVAILABLE 4
 
@@ -685,7 +721,6 @@ extern time_t curdir_mtime;
 #define KEY_BACKSPACE 8
 #define KEY_DELETE    127
 #define KEY_ENTER     13
-#define KEY_KILL      21 /* Ctrl-u */
 
 /* Macros to specify suggestions type */
 #define NO_SUG         0
@@ -699,7 +734,7 @@ extern time_t curdir_mtime;
 #define ELN_SUG        8
 #define FIRST_WORD     9
 #define JCMD_SUG       10
-#define JCMD_SUG_NOACD 11 /* No auto-cd */
+/*#define JCMD_SUG_NOACD 11 // No auto-cd */
 #define VAR_SUG	       12
 #define SEL_SUG	       13
 #define BACKDIR_SUG	   14
@@ -724,20 +759,15 @@ extern time_t curdir_mtime;
 #define WS_PREFIX_SUG  33 /* Workspace name (w:NAME) */
 #define WS_NUM_PREFIX_SUG 34
 
-/* 46 == \x1b[00;38;02;000;000;000;00;48;02;000;000;000m\0 (24bit, RGB
- * true color format including foreground and background colors, the SGR
- * (Select Graphic Rendition) parameter, and, of course, the terminating
- * null byte. */
-#define MAX_COLOR 46
-
 /* Macros to control file descriptors in exec functions. */
 #define E_NOFLAG   0
 #define E_NOSTDIN  (1 << 1)
 #define E_NOSTDOUT (1 << 2)
 #define E_NOSTDERR (1 << 3)
+#define E_SETSID   (1 << 4)
 #define E_MUTE     (E_NOSTDOUT | E_NOSTDERR)
 
-/* Amount of available suggestion strategies (a,b,c,e,f,h,j).
+/* Number of available suggestion strategies (a,b,c,e,f,h,j).
  * 'b' is deprecated (kept only for compatibility with old versions) */
 #define SUG_STRATS 7
 
@@ -754,7 +784,7 @@ extern time_t curdir_mtime;
 #define CS_FREEBUF 1
 #define CS_KEEPBUF 0
 
-/* Macros for the xmagic function */
+/* Macros for the get_file_type function */
 #define MIME_TYPE 1
 #define TEXT_DESC 0
 
@@ -770,9 +800,9 @@ extern time_t curdir_mtime;
 #define SET_COLOR    1
 #define INFORM_COLOR 0
 
-/* Are we trimming a file name with a extension? */
-#define TRIM_NO_EXT 1
-#define TRIM_EXT    2
+/* Are we truncating a filename with an extension? */
+#define TRUNC_NO_EXT 1
+#define TRUNC_EXT    2
 
 /* OpenBSD recommends the use of 10 trailing X's. See mkstemp(3). */
 #if defined(__OpenBSD__)
@@ -848,16 +878,19 @@ extern time_t curdir_mtime;
 #define PROP_SIZE_HUMAN  2
 
 /* Macros for fzf_preview_border_type. */
-#define FZF_BORDER_BOLD    0
-#define FZF_BORDER_BOTTOM  1
-#define FZF_BORDER_DOUBLE  2
-#define FZF_BORDER_HORIZ   3
-#define FZF_BORDER_LEFT    4
-#define FZF_BORDER_NONE    5
-#define FZF_BORDER_ROUNDED 6
-#define FZF_BORDER_SHARP   7
-#define FZF_BORDER_TOP     8
-#define FZF_BORDER_VERT    9
+#define FZF_BORDER_BOLD      0
+#define FZF_BORDER_BOTTOM    1
+#define FZF_BORDER_DOUBLE    2
+#define FZF_BORDER_HORIZ     3
+#define FZF_BORDER_LEFT      4
+#define FZF_BORDER_NONE      5
+#define FZF_BORDER_ROUNDED   6
+#define FZF_BORDER_SHARP     7
+#define FZF_BORDER_TOP       8
+#define FZF_BORDER_VERT      9
+#define FZF_BORDER_BLOCK     10
+#define FZF_BORDER_THINBLOCK 11
+#define FZF_BORDER_RIGHT     12
 
 /* Flags to skip fuzzy matching based on what we're comparing. */
 #define FUZZY_FILES_ASCII 0
@@ -887,15 +920,28 @@ extern time_t curdir_mtime;
 #define FULL_STAT   2
 
 /* Length to be added to a filename length if icons are enabled. */
-#define ICON_LEN 3
+#define ICON_LEN (2 + conf.icons_gap) /* Fixed icon width + gap */
+
+/* Macros for alternative prompts (alt_prompt) */
+#define FILES_PROMPT       1 /* Allow filenames completion */
+#define PERMISSIONS_PROMPT 2 /* pc command */
+#define OWNERSHIP_PROMPT   3 /* oc command */
+#define BOOKMARKS_PROMPT   4 /* bookmarks screen */
+
+#define AUTOCMD_MSG_NONE   0
+#define AUTOCMD_MSG_MINI   1
+#define AUTOCMD_MSG_SHORT  2
+#define AUTOCMD_MSG_LONG   3
+#define AUTOCMD_MSG_FULL   4
+#define AUTOCMD_MSG_PROMPT 5
 
 /* Function macros */
 #define atoi xatoi /* xatoi is just a secure atoi */
 
 #ifndef _NO_GETTEXT
-# define _(String) gettext(String)
+# define _(str) gettext(str)
 #else
-# define _(String) String
+# define _(str) str
 #endif /* !_GETTEXT */
 
 /* Log the message and print it to STDERR, but do not store it into the
@@ -916,23 +962,21 @@ extern time_t curdir_mtime;
  * NOTE: In case of directories themselves (excluding their content), their
  * apparent size is always zero as well (see 'info du'). */
 #define FILE_TYPE_NON_ZERO_SIZE(m) (S_ISDIR((m)) || S_ISREG((m)) \
-		|| S_ISLNK((m)))
+	|| S_ISLNK((m)))
 /* du(1) also checks for
  * S_TYPEISSHM(struct stat *) // shared memory objects
  * S_TYPEISTMO(struct stat *) // typed memory objects */
 
 #define FILE_SIZE_PTR(s) (conf.apparent_size == 1 ? (s)->st_size \
-		: (s)->st_blocks * S_BLKSIZE)
+	: (s)->st_blocks * S_BLKSIZE)
 #define FILE_SIZE(s) (conf.apparent_size == 1 ? (s).st_size \
-		: (s).st_blocks * S_BLKSIZE)
+	: (s).st_blocks * S_BLKSIZE)
 
 /* Do we have the sort method S in light mode? */
 #define ST_IN_LIGHT_MODE(s) ((s) == SNAME || (s) == SVER || (s) == SINO \
-		|| (s) == SEXT || (s) == SNONE || (s) == STYPE)
+	|| (s) == SEXT || (s) == SNONE || (s) == STYPE)
 
 #define UNUSED(x) (void)(x) /* Just silence the compiler's warning */
-#define TOUPPER(c) (((c) >= 'a' && (c) <= 'z') ? ((c) - 'a' + 'A') : (c))
-#define TOLOWER(c) (((c) >= 'A' && (c) <= 'Z') ? ((c) - 'A' + 'a') : (c))
 
 /* A yottabyte takes 26 digits, 28 if we count the negative sign and the NULL
  * terminator, so that 32 bytes is more than enough to represent any given
@@ -947,41 +991,50 @@ extern time_t curdir_mtime;
 
 /* UINT_MAX is 4294967295 == 10 digits */
 #define DIGINUM(n) (((n) < 10) ? 1 \
-		: ((n) < 100)        ? 2   \
-		: ((n) < 1000)       ? 3   \
-		: ((n) < 10000)      ? 4   \
-		: ((n) < 100000)     ? 5   \
-		: ((n) < 1000000)    ? 6   \
-		: ((n) < 10000000)   ? 7   \
-		: ((n) < 100000000)  ? 8   \
-		: ((n) < 1000000000) ? 9   \
+	: ((n) < 100)        ? 2   \
+	: ((n) < 1000)       ? 3   \
+	: ((n) < 10000)      ? 4   \
+	: ((n) < 100000)     ? 5   \
+	: ((n) < 1000000)    ? 6   \
+	: ((n) < 10000000)   ? 7   \
+	: ((n) < 100000000)  ? 8   \
+	: ((n) < 1000000000) ? 9   \
 				             : 10)
 
 /* SIZE_MAX == 18446744073709551615 == 20 digits */
 #define DIGINUM_BIG(n) (((n) < 10) ? 1 \
-		: ((n) < 100)        ? 2   \
-		: ((n) < 1000)       ? 3   \
-		: ((n) < 10000)      ? 4   \
-		: ((n) < 100000)     ? 5   \
-		: ((n) < 1000000)    ? 6   \
-		: ((n) < 10000000)   ? 7   \
-		: ((n) < 100000000)  ? 8   \
-		: ((n) < 1000000000) ? 9   \
-		: ((off_t)(n) < 10000000000) ? 10 \
-		: ((off_t)(n) < 100000000000) ? 11 \
-		: ((off_t)(n) < 1000000000000) ? 12 \
-		: ((off_t)(n) < 10000000000000) ? 13 \
-		: ((off_t)(n) < 100000000000000) ? 14 \
-		: ((off_t)(n) < 1000000000000000) ? 15 \
-		: ((off_t)(n) < 10000000000000000) ? 16 \
-		: ((off_t)(n) < 100000000000000000) ? 17 \
-		: ((off_t)(n) < 1000000000000000000) ? 18 \
+	: ((n) < 100)        ? 2   \
+	: ((n) < 1000)       ? 3   \
+	: ((n) < 10000)      ? 4   \
+	: ((n) < 100000)     ? 5   \
+	: ((n) < 1000000)    ? 6   \
+	: ((n) < 10000000)   ? 7   \
+	: ((n) < 100000000)  ? 8   \
+	: ((n) < 1000000000) ? 9   \
+	: ((off_t)(n) < 10000000000) ? 10 \
+	: ((off_t)(n) < 100000000000) ? 11 \
+	: ((off_t)(n) < 1000000000000) ? 12 \
+	: ((off_t)(n) < 10000000000000) ? 13 \
+	: ((off_t)(n) < 100000000000000) ? 14 \
+	: ((off_t)(n) < 1000000000000000) ? 15 \
+	: ((off_t)(n) < 10000000000000000) ? 16 \
+	: ((off_t)(n) < 100000000000000000) ? 17 \
+	: ((off_t)(n) < 1000000000000000000) ? 18 \
                                              : 19)
 
-#define IS_DIGIT(c)    ((unsigned int)(c) >= '0' && (unsigned int)(c) <= '9')
-#define IS_ALPHA(c)    ((unsigned int)(c) >= 'a' && (unsigned int)(c) <= 'z')
-#define IS_ALPHA_UP(c) ((unsigned int)(c) >= 'A' && (unsigned int)(c) <= 'Z')
-#define IS_ALNUM(c)    (IS_ALPHA((c)) || IS_ALPHA_UP((c)) || IS_DIGIT((c)))
+#define IS_DIGIT(c)     ((unsigned int)(c) >= '0' && (unsigned int)(c) <= '9')
+#define IS_ALPHA_LOW(c) ((unsigned int)(c) >= 'a' && (unsigned int)(c) <= 'z')
+#define IS_ALPHA_UP(c)  ((unsigned int)(c) >= 'A' && (unsigned int)(c) <= 'Z')
+#define IS_ALNUM(c)     (IS_ALPHA_LOW((c)) || IS_ALPHA_UP((c)) || IS_DIGIT((c)))
+#define IS_HEX_DIGIT(c) (IS_DIGIT((c)) || ((c) >= 'a' && (c) <= 'f') \
+	|| ((c) >= 'A' && (c) <= 'F'))
+
+#define TOUPPER(c) (IS_ALPHA_LOW((c)) ? ((c) & ~0x20) : (c))
+#define TOLOWER(c) (IS_ALPHA_UP((c)) ? ((c) | 0x20) : (c))
+
+#define IS_UTF8_LEAD_BYTE(c) (((c) & 0xc0) == 0xc0)
+#define IS_UTF8_CONT_BYTE(c) (((c) & 0xc0) == 0x80)
+#define IS_UTF8_CHAR(c)      (IS_UTF8_LEAD_BYTE((c)) || IS_UTF8_CONT_BYTE((c)))
 
 #define IS_COMMENT(c)  ((unsigned int)(c) == '#' || (unsigned int)(c) == ';')
 #define IS_CTRL_CHR(c) ((unsigned int)(c) < ' ')
@@ -991,53 +1044,12 @@ extern time_t curdir_mtime;
 #define SELFORPARENT(s) (*(s) == '.' && (!(s)[1] || ((s)[1] == '.' && !(s)[2])))
 
 #define FILE_URI_PREFIX_LEN 7
-#define IS_FILE_URI(f) ((f)[4] == ':'                                  \
-				&& (f)[FILE_URI_PREFIX_LEN]                            \
-				&& strncmp((f), "file://", FILE_URI_PREFIX_LEN) == 0)
+#define IS_FILE_URI(file, len) ((len) > FILE_URI_PREFIX_LEN     \
+	&& (file)[4] == ':' && (file)[FILE_URI_PREFIX_LEN]          \
+	&& strncmp((file), "file://", FILE_URI_PREFIX_LEN) == 0)
 
 #define IS_HELP(s) (*(s) == '-' && (((s)[1] == 'h' && !(s)[2]) \
-				|| strcmp((s), "--help") == 0))
-
-/* TERMINAL ESCAPE CODES */
-#define CLEAR \
-	if (term_caps.home == 1 && term_caps.clear == 1) { \
-		if (term_caps.del_scrollback == 1)             \
-			fputs("\x1b[H\x1b[2J\x1b[3J", stdout);     \
-		else                                           \
-			fputs("\x1b[H\x1b[J", stdout);             \
-	}
-
-#define MOVE_CURSOR_DOWN(n)      printf("\x1b[%dB", (n))  /* CUD */
-
-/* ######## Escape sequences used by the suggestions system */
-#define MOVE_CURSOR_UP(n)        printf("\x1b[%dA", (n))  /* CUU */
-#define MOVE_CURSOR_RIGHT(n)     printf("\x1b[%dC", (n))  /* CUF */
-#define MOVE_CURSOR_LEFT(n)      printf("\x1b[%dD", (n))  /* CUB */
-#define ERASE_TO_RIGHT           fputs("\x1b[0K", stdout) /* EL0 */
-#define ERASE_TO_LEFT            fputs("\x1b[1K", stdout) /* EL1 */
-#define ERASE_TO_RIGHT_AND_BELOW fputs("\x1b[J", stdout)  /* ED0 */
-
-#define	SUGGEST_BAEJ(offset, color) printf("\x1b[%dC%s%c\x1b[0m ", \
-	(offset), (color), BAEJ_SUG_POINTER)
-/* ######## */
-
-/* Sequences used by the pad_filename function (listing.c):
- * MOVE_CURSOR_RIGHT() */
-
-/* Sequences used by the pager (listing.c):
- * MOVE_CURSOR_DOWN(n)
- * ERASE_TO_RIGHT */
-
-#define META_SENDS_ESC  fputs("\x1b[?1036h", stdout)
-#define HIDE_CURSOR     fputs(term_caps.hide_cursor == 1 ? "\x1b[?25l" : "", stdout) /* DECTCEM */
-#define UNHIDE_CURSOR   fputs(term_caps.hide_cursor == 1 ? "\x1b[?25h" : "", stdout)
-
-#define RESTORE_COLOR   fputs("\x1b[0;39;49m", stdout)
-#define SET_RVIDEO      fputs("\x1b[?5h", stderr) /* DECSCNM: Enable reverse video */
-#define UNSET_RVIDEO    fputs("\x1b[?5l", stderr)
-#define SET_LINE_WRAP   fputs("\x1b[?7h", stderr) /* DECAWM */
-#define UNSET_LINE_WRAP fputs("\x1b[?7l", stderr)
-#define RING_BELL       fputs("\007", stderr)
+	|| strcmp((s), "--help") == 0))
 
 /* Get the maximum value for a given data type.
  * Taken from coreutils (https://github.com/coreutils/gnulib/blob/master/lib/intprops.h) */
@@ -1091,6 +1103,16 @@ extern time_t curdir_mtime;
 typedef ssize_t filesn_t;
 extern filesn_t files;
 
+struct default_answer_t {
+	char remove;
+	char trash;
+	char bulk_rename;
+	char overwrite;
+	char default_;
+	char default_all;
+	char pad[2];
+};
+
 /* User settings (mostly from the config file) */
 struct config_t {
 	char *opener;
@@ -1099,6 +1121,7 @@ struct config_t {
 	char *encoded_prompt;
 	char *term;
 	char *time_str;
+	char *priority_sort_char;
 	char *ptime_str;
 	char *rprompt_str;
 	char *welcome_message_str;
@@ -1111,15 +1134,17 @@ struct config_t {
 	char *usr_cscheme;
 	char *fzftab_options;
 
+	struct default_answer_t default_answer;
 	int apparent_size;
 	int auto_open;
 	int autocd;
+	int autocmd_msg;
 	int autols;
 	int bell_style;
 	int case_sens_dirjump;
 	int case_sens_path_comp;
 	int case_sens_search;
-	int case_sens_list; /* files list */
+	int case_sens_list; /* File list */
 	int cd_on_quit;
 	int check_cap;
 	int check_ext;
@@ -1143,6 +1168,7 @@ struct config_t {
 	int fzf_preview;
 	int highlight;
 	int icons;
+	int icons_gap;
 	int int_vars;
 	int light_mode;
 	int link_creat_mode;
@@ -1160,7 +1186,7 @@ struct config_t {
 	int max_name_len_bk;
 	int max_printselfiles;
 	int min_jump_rank;
-	int min_name_trim;
+	int min_name_trunc;
 	int mv_cmd;
 	int no_eln;
 	int only_dirs;
@@ -1168,6 +1194,7 @@ struct config_t {
 	int pager_once;
 	int pager_view;
 	int purge_jumpdb;
+	int preview_max_size;
 	int print_dir_cmds;
 	int print_selfiles;
 	int private_ws_settings;
@@ -1198,7 +1225,7 @@ struct config_t {
 	int time_follows_sort;
 	int timestamp_mark;
 	int tips;
-	int trim_names;
+	int trunc_names;
 #ifndef _NO_TRASH
 	int tr_as_rm;
 	int trash_force;
@@ -1208,6 +1235,7 @@ struct config_t {
 #endif /* !_NO_TRASH */
 	int warning_prompt;
 	int welcome_message;
+	int pad3;
 };
 
 extern struct config_t conf;
@@ -1223,7 +1251,7 @@ struct filter_t {
 
 extern struct filter_t filter;
 
-/* Struct to store information about the current user */
+/* Struct to store information about the current user. */
 struct user_t {
 	char *home;
 	char *name;
@@ -1335,8 +1363,11 @@ struct fileinfo {
 	char *name;
 	filesn_t filesn;
 	blkcnt_t blocks;
-	size_t len;    /* File name len (columns needed to display file name) */
-	size_t bytes;  /* Bytes consumed by file name */
+	size_t len;    /* Filename len (columns needed to display filename) */
+	size_t bytes;  /* Bytes consumed by filename */
+#ifdef TIGHT_COLUMNS
+	size_t total_entry_len;
+#endif
 	time_t ltime;  /* For long view mode */
 	time_t time;
 	ino_t inode;
@@ -1347,9 +1378,9 @@ struct fileinfo {
 	mode_t mode;   /* Store st_mode (for long view mode) */
 	mode_t type;   /* Store d_type value */
 	int dir;
-	int eln_n;     /* Amount of digits in ELN */
+	int eln_n;     /* Number of digits in ELN */
 	int exec;
-	int ruser;     /* User read permission for dir */
+	int user_access; /* Read-exec for dirs and read for files */
 	int symlink;
 	int sel;
 	int xattr;
@@ -1391,19 +1422,24 @@ struct devino_t {
 extern struct devino_t *sel_devino;
 
 struct autocmds_t {
+	struct filter_t filter;
 	char *pattern;
 	char *color_scheme;
 	char *cmd;
+	int files_counter;
+	int full_dir_size;
 	int long_view;
 	int light_mode;
-	int files_counter;
+	int match;
 	int max_files;
 	int max_name_len;
+	int only_dirs;
+	int pager;
+	int pattern_rev;
 	int show_hidden;
 	int sort;
 	int sort_reverse;
-	int pager;
-	int only_dirs;
+	int temp;
 };
 
 extern struct autocmds_t *autocmds;
@@ -1412,6 +1448,7 @@ struct opts_t {
 	struct filter_t filter;
 	char *color_scheme;
 	int files_counter;
+	int full_dir_size;
 	int light_mode;
 	int list_dirs_first;
 	int long_view;
@@ -1422,7 +1459,6 @@ struct opts_t {
 	int show_hidden;
 	int sort;
 	int sort_reverse;
-	int pad0;
 };
 
 extern struct opts_t opts;
@@ -1482,6 +1518,7 @@ struct param_t {
 #endif /* !_NO_ICONS */
 	int icons_use_file_color;
 	int int_vars;
+	int kitty_keys; /* kitty keyboard protocol */
 	int list_and_quit;
 	int light_mode;
 	int list_dirs_first;
@@ -1506,6 +1543,7 @@ struct param_t {
 	int readonly;
 	int refresh_on_empty_line;
 	int refresh_on_resize;
+	int report_cwd;
 	int restore_last_path;
 	int rl_vi_mode;
 	int secure_cmds;
@@ -1528,7 +1566,8 @@ struct param_t {
 #ifndef _NO_TRASH
 	int trasrm;
 #endif /* !_NO_TRASH */
-	int trim_names;
+	int trunc_names;
+	int unicode;
 	int virtual_dir_full_paths;
 	int vt100;
 	int welcome_message;
@@ -1579,7 +1618,9 @@ extern struct sel_t *sel_elements;
 /* File statistics for the current directory. Used by the 'stats' command */
 struct stats_t {
 	size_t dir;
+	size_t empty_dir;
 	size_t reg;
+	size_t empty_reg;
 	size_t exec;
 	size_t hidden;
 	size_t suid;
@@ -1597,6 +1638,7 @@ struct stats_t {
 	size_t extended;
 	size_t unknown;
 	size_t unstat; /* Non-statable file */
+	size_t excluded; /* Files not displayed */
 #ifdef __sun
 	size_t door;
 	size_t port;
@@ -1652,12 +1694,14 @@ struct termcaps_t {
 	int clear; /* ED (erase display) */
 	int del_scrollback; /* E3 */
 	int req_cur_pos; /* CPR (cursor position request) */
+	int req_dev_attrs; /* Primary DA (device attributes request) */
+	int unicode;
 };
 extern struct termcaps_t term_caps;
 
 /* Data to be displayed in the properties string in long mode */
 struct props_t {
-	int counter; /* Files counter */
+	int counter; /* File counter */
 	int ids; /* User/group IDs: either NUMBER or NAME */
 	int inode; /* File inode number */
 	int len; /* Approx len of the entire properties string taking into account
@@ -1673,13 +1717,30 @@ struct props_t {
 };
 extern struct props_t prop_fields;
 
+#define ALL_CMDS        (1 << 0) // Check all commands (ignore parameters)
+#define NO_PARAM        (1 << 1) // Command takes no parameter
+#define PARAM_STR       (1 << 2) // Command takes a string (not filename)
+#define PARAM_FNAME     (1 << 3) // Command takes filenames
+#define PARAM_NUM       (1 << 4) // Command takes numbers
+#define NO_FNAME_NUM    (NO_PARAM | PARAM_STR) // Neither filename nor number
+#define PARAM_FNAME_NUM (PARAM_FNAME | PARAM_NUM) // Either filename or number
+
 struct cmdslist_t {
 	char *name;
 	size_t len;
+	int flag;
+	int pad;
+};
+
+extern const struct cmdslist_t internal_cmds[];
+
+struct nameslist_t {
+	char *name;
+	size_t len;
 };
 
-extern const struct cmdslist_t param_str[];
-extern const struct cmdslist_t internal_cmds[];
+extern const struct nameslist_t param_str[];
+extern const struct nameslist_t kb_cmds[];
 
 extern size_t internal_cmds_n;
 
@@ -1743,15 +1804,31 @@ struct dircmds_t {
 };
 extern struct dircmds_t dir_cmds;
 
+struct pmsgs_t {
+	char *text;
+	int read;
+	int pad0;
+};
+extern struct pmsgs_t *messages;
+
 struct dir_info_t {
 	unsigned long long dirs;
 	unsigned long long files;
 	unsigned long long links;
 	off_t size;
+	blkcnt_t blocks;
 	int status;
 	int pad0;
 };
 
+/* Store user defined mimetypes */
+struct mime_t {
+	char *ext;
+	char *mimetype;
+	size_t ext_hash;
+};
+extern struct mime_t *user_mimetypes;
+
 enum tab_mode {
 	STD_TAB =   0,
 	FZF_TAB =   1,
@@ -1822,7 +1899,8 @@ enum comp_type {
 	TCMP_DIRHIST =    35,
 	TCMP_MIME_LIST =  36,
 	TCMP_WS_PREFIX =  37, /* 'w:' keyword expansion */
-	TCMP_MIME_FILES = TCMP_FILE_TYPES_FILES /* Same behavior */
+	TCMP_MIME_FILES = TCMP_FILE_TYPES_FILES, /* Same behavior */
+	TCMP_FILE_TEMPLATES = 38
 };
 
 extern enum comp_type cur_comp_type;
@@ -1839,7 +1917,7 @@ extern int
 
 /* Internal state flags */
 extern int
-	alt_prompt, /* We're running a secondary prompt for the rename function */
+	alt_prompt, /* We're running a secondary prompt */
 	argc_bk, /* A copy of argc taken from main() */
 	autocmd_set,
 	bg_proc,
@@ -1853,8 +1931,9 @@ extern int
 	dirhist_total_index,
 	exit_code,
 	fzftab,
+	fzf_ext_border,
 	fzf_border_type,
-	fzf_height_set,
+	fzf_height_value,
 	fzf_preview_border_type,
 	hist_status,
 	home_ok,
@@ -1946,9 +2025,9 @@ extern char
 	*alt_bm_file,
 	*alt_config_file,
 	*alt_kbinds_file,
+	*alt_mimelist_file,
 	*alt_preview_file,
 	*alt_profile,
-	*bin_name,
 	*bm_file,
 	*cmds_log_file,
 	*colors_dir,
@@ -1975,7 +2054,6 @@ extern char
 	*quote_chars,
 	*rl_callback_handler_input,
 	*remotes_file,
-/*	*right_prompt, */
 	*sel_file,
 	*smenutab_options_env,
 	*stdin_tmp_dir,
@@ -1984,6 +2062,8 @@ extern char
 	*suggestion_buf,
 #endif /* !_NO_SUGGESTIONS */
 	*tags_dir,
+	*templates_dir,
+	*thumbnails_dir,
 	*tmp_rootdir,
 	*tmp_dir,
 #ifndef _NO_TRASH
@@ -1997,7 +2077,7 @@ extern char
 	**bin_commands,
 	**cdpaths,
 	**color_schemes,
-	**messages,
+	**file_templates,
 	**old_pwd,
 	**profile_names,
 	**prompt_cmds,
@@ -2008,14 +2088,20 @@ extern regex_t regex_hist;    /* Commands history */
 extern regex_t regex_dirhist; /* Directory history */
 extern char **environ;
 
-/* A buffer to store file names to be displayed (wide string) */
+/* A buffer to store filenames to be displayed (wide string) */
 #define NAME_BUF_SIZE (NAME_MAX + 1)
 extern char name_buf[NAME_BUF_SIZE * sizeof(wchar_t)];
 
-/* To store all the 39 color variables we use, with 46 bytes each, we need
- * a total of 1,8Kb. It's not much but it could be less if we'd use
- * dynamically allocated arrays for them (which, on the other side,
- * would make the whole thing slower and more tedious) */
+/* Longest supported color:
+ * \x1b[4:4;38;2;000;000;000;48;2;000;000;000;58;2;000;000;000m\0
+ * This is a complete SGR sequence, with 24bit colors: foreground, background,
+ * and underline (kitty). */
+#define MAX_COLOR 64
+
+/* For the almost 100 color variables we use, we need more or less 6Kb.
+ * It's not much, but it could be less if we'd use dynamically allocated
+ * arrays for them (which, on the other side, would make the whole thing
+ * slower and more tedious). */
 
 /* Colors */
 extern char
@@ -2049,14 +2135,15 @@ extern char
 	uf_c[MAX_COLOR], /* Non-'stat'able file */
 
 	/* Interface */
+	ac_c[MAX_COLOR + 2], /* Autocmd indicator */
 	df_c[MAX_COLOR], /* Default color */
 	dl_c[MAX_COLOR], /* Dividing line */
 	el_c[MAX_COLOR], /* ELN color */
-	fc_c[MAX_COLOR], /* Files counter */
+	fc_c[MAX_COLOR], /* File counter */
 	lc_c[MAX_COLOR], /* Symlink character (ColorLinkAsTarget only) */
 	mi_c[MAX_COLOR], /* Misc indicators */
-	ts_c[MAX_COLOR], /* TAB completion suffix */
-	tt_c[MAX_COLOR], /* Tilde for trimmed files */
+	ts_c[MAX_COLOR], /* Tab completion suffix */
+	tt_c[MAX_COLOR], /* Tilde for truncated filenames */
 	wc_c[MAX_COLOR], /* Welcome message */
 	wp_c[MAX_COLOR], /* Warning prompt */
 
@@ -2064,11 +2151,11 @@ extern char
 	sb_c[MAX_COLOR], /* Auto-suggestions: shell builtins */
 	sc_c[MAX_COLOR], /* Auto-suggestions: external commands and aliases */
 	sd_c[MAX_COLOR], /* Auto-suggestions: internal commands description */
-	sf_c[MAX_COLOR], /* Auto-suggestions: file names */
+	sf_c[MAX_COLOR], /* Auto-suggestions: filenames */
 	sh_c[MAX_COLOR], /* Auto-suggestions: history */
 	sp_c[MAX_COLOR], /* Auto-suggestions: BAEJ suggestions pointer */
 	sx_c[MAX_COLOR], /* Auto-suggestions: internal commands and parameters */
-	sz_c[MAX_COLOR], /* Auto-suggestions: file names (fuzzy) */
+	sz_c[MAX_COLOR], /* Auto-suggestions: filenames (fuzzy) */
 
 #ifndef _NO_ICONS
     dir_ico_c[MAX_COLOR], /* Directories icon color */
@@ -2118,7 +2205,7 @@ extern char
 
 	em_c[MAX_COLOR + 2], /* Error msg */
 	li_c[MAX_COLOR + 2], /* Sel indicator */
-	li_cb[MAX_COLOR],    /* Sel indicator (for the files list) */
+	li_cb[MAX_COLOR],    /* Sel indicator (for the file list) */
 	nm_c[MAX_COLOR + 2], /* Notice msg */
 	ti_c[MAX_COLOR + 2], /* Trash indicator */
 	tx_c[MAX_COLOR + 2], /* Text */
@@ -2126,6 +2213,7 @@ extern char
 	si_c[MAX_COLOR + 2], /* Stealth indicator */
 	wm_c[MAX_COLOR + 2], /* Warning msg */
 	xs_c[MAX_COLOR + 2], /* Exit code: success */
+	xs_cb[MAX_COLOR],    /* Exit code: success (Unicode success indicator) */
 	xf_c[MAX_COLOR + 2], /* Exit code: failure */
 	xf_cb[MAX_COLOR],    /* Exit code: failure (dir read) */
 
