@@ -1,17 +1,15 @@
-/* highlight.c -- a simple function to perform syntax highlighting */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,9 +20,10 @@
  * MA 02110-1301, USA.
 */
 
+/* highlight.c -- a simple function to perform syntax highlighting */
+
 #include "helpers.h"
 
-#include <stdio.h>
 #ifdef __OpenBSD__
 typedef char *rl_cpvfunc_t;
 #include <ereadline/readline/readline.h>
@@ -61,7 +60,7 @@ change_word_color(const char *_last_word, const int offset, const char *color)
  * STR and print the color if SET_COLOR is set to 1 (in which case NULL is
  * returned); otherwise, just return a pointer to the corresponding color.
  * This function is used to colorize input, history entries, and accepted
- * suggestions */
+ * suggestions. */
 char *
 rl_highlight(const char *str, const size_t pos, const int flag)
 {
@@ -105,7 +104,7 @@ rl_highlight(const char *str, const size_t pos, const int flag)
 		} else {
 			char cc = c;
 			*(str + pos) = '\0';
-			int ret = is_internal_f(str);
+			int ret = is_internal_cmd(str, PARAM_FNAME_NUM, 0, 0);
 			*(str + pos) = cc;
 			if (ret) {
 				cl = hn_c;
@@ -267,7 +266,7 @@ recolorize_line(void)
 	rl_delete_text(start, rl_end);
 	rl_point = rl_end = start;
 
-	if (start == 0 && end_bk > 1)
+	if (start == 0 && end_bk >= 1)
 		/* First char of a non-empty recolored line (recovering from wrong cmd) */
 		rl_redisplay();
 
