@@ -1,17 +1,15 @@
-/* history.c -- functions for the history system */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,10 +20,11 @@
  * MA 02110-1301, USA.
 */
 
+/* history.c -- functions for the history system */
+
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <time.h>
 #include <readline/history.h>
@@ -58,7 +57,8 @@ get_date(void)
 	return date;
 }
 
-/* Print available logs, for messages, if MSG is 1, or for commands otherwise. */
+/* Print available logs, for messages, if FLAG is MSG_LOGS, or for
+ * commands otherwise. */
 int
 print_logs(const int flag)
 {
@@ -82,7 +82,7 @@ print_logs(const int flag)
 }
 
 static int
-gen_file(char *file)
+gen_file(const char *file)
 {
 	int fd = 0;
 	FILE *fp = open_fwrite(file, &fd);
@@ -193,14 +193,53 @@ write_msg_into_logfile(const char *msg_str)
 	free(date);
 }
 
+static void
+send_kitty_notification(const char *msg)
+{
+	static int kitty_msg_id = 1;
+	const int urgency = pmsg == ERROR ? 2 : (pmsg == WARNING ? 1 : 0);
+
+	printf("\x1b]99;i=%d:d=0:p=title;%s\x1b\\",
+		kitty_msg_id, PROGRAM_NAME);
+	printf("\x1b]99;i=%d:d=1:n=file-manager:f=%s:u=%d:p=body;%s\x1b\\",
+		kitty_msg_id, PROGRAM_NAME, urgency, msg);
+
+	fflush(stdout);
+
+	if (kitty_msg_id < INT_MAX)
+		kitty_msg_id++;
+}
+
 /* Let's send a desktop notification */
 static void
 send_desktop_notification(char *msg)
 {
-/*	if (!msg || !*msg || !(flags & GUI) || desktop_noti != 1) */
-	if (!msg || !*msg)
+	if (!msg || !*msg || *msg == '\n')
 		return;
 
+	size_t mlen = strlen(msg);
+	if (msg[mlen - 1] == '\n') {
+		msg[mlen - 1] = '\0';
+		mlen--;
+	}
+
+	/* Some messages are written in the form "PROGRAM_NAME: MSG". We only
+	 * want the MSG part. */
+	int ret = 0;
+	const size_t s = sizeof(PROGRAM_NAME) - 1;
+	char *p = msg;
+	if (strncmp(msg, PROGRAM_NAME, s) == 0
+	&& msg[s] == ':' && msg[s + 1] == ' ') {
+		p += s + 2;
+		if (!*p)
+			return;
+	}
+
+	if (conf.desktop_notifications == DESKTOP_NOTIF_KITTY) {
+		send_kitty_notification(p);
+		return;
+	}
+
 	char type[12];
 	*type = '\0';
 
@@ -223,27 +262,6 @@ send_desktop_notification(char *msg)
 #endif /* __HAIKU__ */
 	}
 
-	size_t mlen = strlen(msg);
-	if (mlen > 0 && msg[mlen - 1] == '\n') {
-		msg[mlen - 1] = '\0';
-		mlen--;
-		if (mlen == 0)
-			return;
-	}
-
-	/* Some messages are written in the form PROGRAM_NAME: MSG. We only
-	 * want the MSG part */
-	char name[NAME_MAX];
-	snprintf(name, sizeof(name), "%s: ", PROGRAM_NAME);
-	char *p = msg;
-	const size_t nlen = strnlen(name, sizeof(name));
-	int ret = strncmp(p, name, nlen);
-	if (ret == 0) {
-		if (mlen <= nlen)
-			return;
-		p = msg + nlen;
-	}
-
 #if defined(__HAIKU__)
 	char *cmd[] = {"notify", "--type", type, "--title", PROGRAM_NAME, p, NULL};
 	ret = launch_execv(cmd, FOREGROUND, E_MUTE);
@@ -304,13 +322,16 @@ log_msg(char *msg_str, const int print_prompt, const int logme,
 
 	if (add_to_msgs_list == 1) {
 		msgs_n++;
-		messages = xnrealloc(messages, (size_t)(msgs_n + 1), sizeof(char *));
-		messages[msgs_n - 1] = savestring(msg_str, msg_len);
-		messages[msgs_n] = (char *)NULL;
+		messages = xnrealloc(messages, (size_t)(msgs_n + 1),
+			sizeof(struct pmsgs_t));
+		messages[msgs_n - 1].text = savestring(msg_str, msg_len);
+		messages[msgs_n - 1].read = 0;
+		messages[msgs_n].text = (char *)NULL;
+		messages[msgs_n].read = 0;
 	}
 
 	if (print_prompt == 1) {
-		if (conf.desktop_notifications == 1 && logme != 0)
+		if (conf.desktop_notifications > 0 && logme != 0)
 			send_desktop_notification(msg_str);
 		else
 			print_msg = 1;
@@ -423,7 +444,7 @@ edit_history(char **args)
 	 * opened. */
 	if (mtime_bfr != attr.st_mtime) {
 		ret = reload_history();
-		print_reload_msg(_("File modified. History entries reloaded\n"));
+		print_reload_msg(NULL, NULL, _("File modified. History entries reloaded.\n"));
 		return ret;
 	}
 
@@ -433,7 +454,7 @@ edit_history(char **args)
 static int
 clear_history_func(char **args)
 {
-	if (rl_get_y_or_n(_("Clear history? [y/n] ")) == 0)
+	if (rl_get_y_or_n(_("Clear history?"), conf.default_answer.remove) == 0)
 		return FUNC_SUCCESS;
 
 	/* Let's overwrite whatever was there. */
@@ -473,7 +494,7 @@ print_history_list(const int timestamp)
 }
 
 static int
-print_last_items(char *str, const int timestamp)
+print_last_items(const char *str, const int timestamp)
 {
 	int num = atoi(str);
 
@@ -509,7 +530,7 @@ toggle_history(const char *arg)
 
 	switch (*arg) {
 	case 'o':
-		hist_status = (*(arg + 1) == 'n' ? 1 : 0);
+		hist_status = (arg[1] == 'n' ? 1 : 0);
 		return print_hist_status();
 	case 's': return print_hist_status();
 	default: puts(_(HISTORY_USAGE)); return FUNC_FAILURE;
@@ -655,7 +676,7 @@ add_to_cmdhist(char *cmd)
 
 /* Returns 1 if INPUT should be saved on history or 0 if not. */
 int
-record_cmd(char *input)
+record_cmd(const char *input)
 {
 	if (!input || !*input)
 		return 0;
