@@ -1,17 +1,15 @@
-/* history.h */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* history.h */
+
 #ifndef HISTORY_H
 #define HISTORY_H
 
@@ -44,7 +44,7 @@ int  log_cmd(void);
 void log_msg(char *msg_str, const int print_prompt, const int logme,
 	const int add_to_msgs_list);
 int  print_logs(const int flag);
-int  record_cmd(char *input);
+int  record_cmd(const char *input);
 
 __END_DECLS
 
