@@ -1,22 +1,15 @@
-/* icons-emoji.h - Icon definitions for CliFM */
-
-/* Taken from
- * https://github.com/jarun/nnn, licensed under BSD-2-Clause.
- * All changes are licensed under GPL-2.0-or-later.
-*/
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -27,6 +20,13 @@
  * MA 02110-1301, USA.
 */
 
+/* icons-emoji.h - Icon definitions for Clifm */
+
+/* Taken from
+ * https://github.com/jarun/nnn, licensed under BSD-2-Clause.
+ * All changes are licensed under GPL-2.0-or-later.
+*/
+
 /* For a list of emojis consult:
  * https://unicode.org/Public/emoji/5.0/emoji-test.txt */
 
@@ -51,7 +51,7 @@
 #define EMOJI_CLOJURE EMOJI_CODE
 #define EMOJI_CODE "📑"
 #define EMOJI_CHANGELOG "🔺"
-#define EMOJI_CHESS EMOJI_FILE
+#define EMOJI_CHESS "♟️ "
 #define EMOJI_CONF "🔧"
 #define EMOJI_COFFEE EMOJI_JAVA
 #define EMOJI_CPP EMOJI_C
@@ -69,7 +69,7 @@
 #define EMOJI_ELM EMOJI_CODE
 #define EMOJI_ENCRYPTED "🔒"
 #define EMOJI_ERLANG EMOJI_CODE
-#define EMOJI_EXEC "⚙ "
+#define EMOJI_EXEC "⚙️ "
 #define EMOJI_FILE "📄"
 #define EMOJI_FOLDER "📂"
 #define EMOJI_FONT "🔤"
@@ -86,7 +86,7 @@
 #define EMOJI_JSON EMOJI_CONF
 #define EMOJI_KEY "🔑"
 #define EMOJI_KOTLIN "🇰 "
-#define EMOJI_LICENSE "⚖ "
+#define EMOJI_LICENSE "⚖️ "
 #define EMOJI_LINK "🔗"
 #define EMOJI_LINUX "🐧"
 #define EMOJI_LIST "✅"
@@ -95,6 +95,7 @@
 #define EMOJI_MAKE "🛠 "
 #define EMOJI_MANUAL "❔"
 #define EMOJI_MARKDOWN "⬇ "
+#define EMOJI_MATLAB EMOJI_CODE
 #define EMOJI_MOVIE "🎬"
 #define EMOJI_MUSIC "🎧"
 #define EMOJI_NOTE "📝"
@@ -103,6 +104,7 @@
 #define EMOJI_PDF "🔖"
 #define EMOJI_PERL "🐪"
 #define EMOJI_PHOTO "📸"
+#define EMOJI_PHOTOSHOP "🖌️ "
 #define EMOJI_PICTURE "📷"
 #define EMOJI_PLAYLIST EMOJI_MUSIC
 #define EMOJI_POSTSCRIPT EMOJI_PDF
