@@ -1,22 +1,15 @@
-/* icons-nerdfont.h - Icon defintions for CliFM */
-
-/* Taken from
- * https://github.com/jarun/nnn, licensed under BSD-2-Clause.
- * All changes are licensed under GPL-2.0-or-later.
-*/
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -27,6 +20,14 @@
  * MA 02110-1301, USA.
 */
 
+/* icons-nerdfont.h - Icon defintions for Clifm */
+
+/* Taken from
+ * https://github.com/jarun/nnn, licensed under BSD-2-Clause.
+ * All changes are licensed under GPL-2.0-or-later.
+*/
+
+
 /* For more codepoints see https://www.nerdfonts.com/cheat-sheet */
 
 #ifndef ICONS_NERDFONT
@@ -99,6 +100,7 @@
 #define NERD_LINUX         ""
 #define NERD_LUA           ""
 #define NERD_MARKDOWN      ""
+#define NERD_MATLAB        ""
 #define NERD_MYSQL         ""
 #define NERD_NIX           ""
 #define NERD_OCAML         ""
