@@ -1,22 +1,15 @@
-/* icons.h - Icon definitions for CliFM */
-
-/* Taken from
- * https://github.com/jarun/nnn, licensed under BSD-2-Clause.
- * All changes are licensed under GPL-2.0-or-later.
-*/
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -27,6 +20,13 @@
  * MA 02110-1301, USA.
 */
 
+/* icons.h - Icon definitions for CliFM */
+
+/* Taken from
+ * https://github.com/jarun/nnn, licensed under BSD-2-Clause.
+ * All changes are licensed under GPL-2.0-or-later.
+*/
+
 #ifndef ICONS_H
 #define ICONS_H
 
@@ -104,6 +104,7 @@ char
     ICON_MAKEFILE[] = NERD_MAKEFILE,
     ICON_MANPAGE[] = NERD_MANUAL,
     ICON_MARKDOWN[] = NERD_MARKDOWN,
+    ICON_MATLAB[] = NERD_MATLAB,
     ICON_MYSQL[] = NERD_MYSQL,
     ICON_ASM[] = NERD_ASM,
     ICON_OCAML[] = NERD_OCAML,
@@ -220,6 +221,7 @@ char
     ICON_LUA[] = FILE_LUA,
     ICON_MANPAGE[] = FILE_MANPAGE,
     ICON_MAKEFILE[] = FILE_MAPBOX,
+    ICON_MATLAB[] = FILE_MATLAB,
     ICON_MARKDOWN[] = OCT_MARKDOWN,
     ICON_MYSQL[] = MFIZZ_MYSQL,
     ICON_ASM[] = FILE_NASM,
@@ -338,6 +340,7 @@ char
     ICON_MAKEFILE[] = EMOJI_MAKE,
     ICON_MANPAGE[] = EMOJI_MANUAL,
     ICON_MARKDOWN[] = EMOJI_MARKDOWN,
+    ICON_MATLAB[] = EMOJI_MATLAB,
     ICON_MYSQL[] = EMOJI_DATABASE,
     ICON_ASM[] = EMOJI_ASM,
     ICON_OCAML[] = EMOJI_OCAML,
@@ -347,7 +350,7 @@ char
     ICON_PATCH[] = EMOJI_PATCH,
     ICON_PDF[] = EMOJI_PDF,
     ICON_PERL[] = EMOJI_PERL,
-    ICON_PHOTOSHOP[] = EMOJI_PHOTO,
+    ICON_PHOTOSHOP[] = EMOJI_PHOTOSHOP,
     ICON_PHP[] = EMOJI_WEB,
     ICON_POSTSCRIPT[] = EMOJI_POSTSCRIPT,
     ICON_POWERPOINT[] = EMOJI_PRESENTATION,
@@ -405,17 +408,11 @@ char
 # define WHITE "\x1b[0;37m"
 # define B_WHITE "\x1b[1;37m"
 # define YELLOW "\x1b[0;33m"
-# define B_YELLOW "\x1b[1;33m"
 # define GREEN "\x1b[0;32m"
 # define B_GREEN "\x1b[1;32m"
 # define CYAN "\x1b[0;36m"
-# define B_CYAN "\x1b[1;36m"
 # define MAGENTA "\x1b[0;35m"
-# define B_MAGENTA "\x1b[1;35m"
 # define RED "\x1b[0;31m"
-# define B_RED "\x1b[1;31m"
-# define BLACK "\x1b[0;30m"
-# define B_BLACK "\x1b[1;30m"
 #else
 # define EMPTY_STR ""
 # define BLUE EMPTY_STR
@@ -423,23 +420,27 @@ char
 # define WHITE EMPTY_STR
 # define B_WHITE EMPTY_STR
 # define YELLOW EMPTY_STR
-# define B_YELLOW EMPTY_STR
 # define GREEN EMPTY_STR
 # define B_GREEN EMPTY_STR
 # define CYAN EMPTY_STR
-# define B_CYAN EMPTY_STR
 # define MAGENTA EMPTY_STR
-# define B_MAGENTA EMPTY_STR
 # define RED EMPTY_STR
-# define B_RED EMPTY_STR
-# define BLACK EMPTY_STR
-# define B_BLACK EMPTY_STR
 #endif /* _NERD || _ICONS_IN_TERMINAL */
 
-/* Default icons and colors for directories and files */
-#define DEF_DIR_ICON ICON_DIR
+/* Default icons and colors */
+#define DEF_DIR_ICON ICON_DIR /* Directory */
 #define DEF_DIR_ICON_COLOR YELLOW
-#define DEF_FILE_ICON ICON_REG
+
+#define DEF_LINK_ICON ICON_LINK /* Symbolic link */
+#define DEF_LINK_ICON_COLOR CYAN
+
+#define DEF_EXEC_ICON ICON_EXEC /* Executable file */
+#define DEF_EXEC_ICON_COLOR WHITE
+
+#define DEF_NOPERM_ICON ICON_LOCK /* File without access permission */
+#define DEF_NOPERM_ICON_COLOR YELLOW
+
+#define DEF_FILE_ICON ICON_REG /* None of the above */
 #define DEF_FILE_ICON_COLOR WHITE
 
 /* Per file extension icons */
@@ -574,13 +575,16 @@ struct icons_t const icon_ext[] = {
     {"lhs", ICON_HASKELL, BLUE},
     {"log", ICON_TXT, WHITE},
     {"lua", ICON_LUA, WHITE},
+    {"lz", ICON_ARCHIVE, YELLOW},
     {"lz4", ICON_ARCHIVE, YELLOW},
     {"lzh", ICON_ARCHIVE, YELLOW},
     {"lzma", ICON_ARCHIVE, YELLOW},
 
+    {"m", ICON_MATLAB, YELLOW},
     {"m4a", ICON_AUDIO, YELLOW},
     {"m4v", ICON_VID, BLUE},
     {"markdown", ICON_MARKDOWN, WHITE},
+    {"mat", ICON_MATLAB, YELLOW},
     {"md", ICON_MARKDOWN, WHITE},
     {"mk", ICON_MAKEFILE, WHITE},
     {"mkv", ICON_VID, BLUE},
@@ -725,7 +729,7 @@ struct icons_t const icon_dirnames[] = {
     {"Videos", ICON_VIDEOS, DEF_DIR_ICON_COLOR},
 };
 
-/* Icons for some specific file names */
+/* Icons for some specific filenames */
 struct icons_t const icon_filenames[] = {
 /* More specific filenames from here https://github.com/alexanderjeurissen/ranger_devicons/blob/main/devicons.py */
     {".bash_history", ICON_CONF, WHITE},
