@@ -1,17 +1,15 @@
-/* init.h */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,12 +20,14 @@
  * MA 02110-1301, USA.
 */
 
+/* init.h */
+
 #ifndef INIT_H
 #define INIT_H
 
 __BEGIN_DECLS
 
-int  backup_argv(const int argc, char **argv);
+void backup_argv(const int argc, char **argv);
 void check_env_filter(void);
 void check_options(void);
 void get_aliases(void);
@@ -47,11 +47,12 @@ void init_conf_struct(void);
 int  init_gettext(void);
 int  init_history(void);
 void init_shell(void);
-int  init_workspaces(void);
+void init_workspaces(void);
 void init_workspaces_opts(void);
 int  load_actions(void);
 int  load_bookmarks(void);
 int  load_dirhist(void);
+void load_file_templates(void);
 void load_jumpdb(void);
 int  load_pinned_dir(void);
 int  load_prompts(void);
