@@ -1,17 +1,15 @@
-/* jump.c -- functions for Kangaroo, the directory jumper function */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -20,12 +18,13 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  * MA 02110-1301, USA.
-*/
+ */
+
+/* jump.c -- functions for Kangaroo, the directory jumper function */
 
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h> /* unlink(3) */
@@ -120,7 +119,7 @@ calculate_bonus_credit(const char *entry, const char *query, int *keep)
 
 	int bonus = 0;
 
-	char *tmp = query ? strrchr(entry, '/') : (char *)NULL;
+	const char *tmp = query ? strrchr(entry, '/') : (char *)NULL;
 	if (tmp && *(++tmp)) {
 		if (strstr(tmp, query))
 			bonus += BASENAME_BONUS;
@@ -263,7 +262,7 @@ add_to_jumpdb(char *dir)
 	return add_new_jump_entry(dir, dir_len);
 }
 
-/* Store the jump database into a file. */
+/* Save the jump database into a file. */
 void
 save_jumpdb(void)
 {
@@ -289,7 +288,7 @@ save_jumpdb(void)
 			continue;
 
 		int days_since_first = 0, hours_since_last = 0;
-		int rank = rank_entry(i, now, &days_since_first, &hours_since_last);
+		const int rank = rank_entry(i, now, &days_since_first, &hours_since_last);
 		UNUSED(days_since_first); UNUSED(hours_since_last);
 
 		jump_db[i].rank = rank;
@@ -373,7 +372,7 @@ edit_jumpdb(char *app)
 
 /* Save jump entry into the suggestions buffer. */
 static int
-save_jump_suggestion(char *str)
+save_jump_suggestion(const char *str)
 {
 	if (!str || !*str)
 		return FUNC_FAILURE;
@@ -396,9 +395,6 @@ save_jump_suggestion(char *str)
 static char *
 get_directory_color(const char *filename, const struct stat *a)
 {
-	if (check_file_access(a->st_mode, a->st_uid, a->st_gid) == 0)
-		return nd_c;
-
 	if (S_ISLNK(a->st_mode)) {
 		char *linkname = xrealpath(filename, NULL);
 		if (linkname) {
@@ -408,7 +404,7 @@ get_directory_color(const char *filename, const struct stat *a)
 		return or_c;
 	}
 
-	return get_dir_color(filename, a->st_mode, a->st_nlink, -1);
+	return get_dir_color(filename, a, -1);
 }
 
 /* Compare ranks A and B (used to sort jump entries by rank). */
@@ -441,7 +437,7 @@ print_jump_table_header(void)
 		"it is bookmarked, pinned, or currently active in some "
 		"workspace.\n"), item);
 	printf(_("%s A plus sign next rank values means that the "
-		"corresponding directory is marked as permanent (it won't be "
+		"corresponding directory is marked as permanent (it will not be "
 		"removed).\n"), item);
 
 	if (conf.min_jump_rank <= 0) {
@@ -457,11 +453,17 @@ print_jump_table_header(void)
 		conf.colorize == 1 ? BOLD : "", NC);
 }
 
+/* Print the jump database, field by field, incuding the current rank.
+ * If REDUCE is greater than zero, each rank is divided by this value to keep
+ * the total database rank below MaxJumpTotalRank.
+ * NOW is the current time in seconds since epoch, and is used to calculate
+ * each rank.
+ * It always returns FUNC_SUCESS. */
 static int
 print_jump_table(const int reduce, const time_t now)
 {
 	if (jump_n == 0) {
-		printf("jump: Database still empty\n");
+		puts(_("jump: Database still empty"));
 		return FUNC_SUCCESS;
 	}
 
@@ -483,11 +485,9 @@ print_jump_table(const int reduce, const time_t now)
 
 		int days_since_first = 0, hours_since_last = 0;
 		int rank = rank_entry((int)i, now, &days_since_first, &hours_since_last);
-//		int keep = jump_db[i].keep;
-//		jump_db[i].keep = 0;
 
 		if (reduce) {
-			int tmp_rank = rank;
+			const int tmp_rank = rank;
 			rank = tmp_rank / reduce;
 		}
 
@@ -496,7 +496,6 @@ print_jump_table(const int reduce, const time_t now)
 
 		tmp_jump[i].path = jump_db[i].path;
 		tmp_jump[i].keep = jump_db[i].keep;
-//		tmp_jump[i].keep = keep;
 		tmp_jump[i].rank = rank;
 		tmp_jump[i].len = jump_db[i].len;
 		tmp_jump[i].visits = jump_db[i].visits;
@@ -525,7 +524,7 @@ print_jump_table(const int reduce, const time_t now)
 		if (!tmp_jump[i].path)
 			continue;
 
-		char *color = (workspaces[cur_ws].path
+		const char *color = (workspaces[cur_ws].path
 		&& workspaces[cur_ws].path[1] == tmp_jump[i].path[1]
 		&& strcmp(workspaces[cur_ws].path, tmp_jump[i].path) == 0) ? mi_c : df_c;
 
@@ -533,7 +532,8 @@ print_jump_table(const int reduce, const time_t now)
 		if (lstat(tmp_jump[i].path, &a) == -1)
 			color = uf_c;
 
-		char *dir_color = color == uf_c ? uf_c : get_directory_color(tmp_jump[i].path, &a);
+		const char *dir_color = color == uf_c ? uf_c
+			: get_directory_color(tmp_jump[i].path, &a);
 		const int keep = tmp_jump[i].keep;
 		const char keep_char = keep == 1 ? '*'
 			: ((keep == JUMP_ENTRY_PERMANENT)
@@ -577,7 +577,8 @@ purge_invalid_entries(void)
 			continue;
 
 		if (stat(jump_db[i].path, &a) == -1) {
-			printf("%s->%s %s%s%s\n", mi_c, df_c, uf_c, jump_db[i].path, df_c);
+			printf("%s%s%s %s%s%s\n", mi_c, SET_MSG_PTR, df_c, uf_c,
+				jump_db[i].path, df_c);
 			jump_db[i].rank = JUMP_ENTRY_PURGED;
 			c++;
 		}
@@ -597,7 +598,7 @@ purge_low_ranked_entries(const int limit)
 {
 	int i = (int)jump_n;
 	int c = 0;
-	time_t now = time(NULL);
+	const time_t now = time(NULL);
 
 	while (--i >= 0) {
 		if (!IS_VALID_JUMP_ENTRY(i))
@@ -617,7 +618,8 @@ purge_low_ranked_entries(const int limit)
 				continue;
 			}
 			jump_db[i].keep = 0;
-			printf("%s->%s %s (%d)\n", mi_c, df_c, jump_db[i].path, rank);
+			printf("%s%s%s %s (%d)\n", mi_c, SET_MSG_PTR, df_c,
+				jump_db[i].path, rank);
 			jump_db[i].rank = JUMP_ENTRY_PURGED;
 			c++;
 		}
@@ -633,7 +635,7 @@ purge_low_ranked_entries(const int limit)
 }
 
 static int
-purge_jump_database(char *arg)
+purge_jump_database(const char *arg)
 {
 	if (!arg || !*arg)
 		return purge_invalid_entries();
@@ -755,6 +757,29 @@ rank_tmp_entry(const struct jump_entry_t *entry, const time_t now,
 	return rank;
 }
 
+static int
+check_dir(char *param, const int mode, int *ret)
+{
+	char *dir = (mode == NO_SUG_JUMP && strchr(param, '\\'))
+		? unescape_str(param, 0) : param;
+
+	struct stat a;
+	if (lstat(dir, &a) == -1) {
+		if (dir != param)
+			free(dir);
+		return 0;
+	}
+
+	*ret = mode == NO_SUG_JUMP
+		? cd_function(dir, CD_NO_PRINT_ERROR)
+		: save_jump_suggestion(param);
+
+	if (dir != param)
+		free(dir);
+
+	return 1;
+}
+
 /* Found the best ranked directory matching query strings in ARGS.
  *
  * The rank is calculated as frecency. The algorithm is based on Mozilla,
@@ -789,7 +814,7 @@ dirjump(char **args, const int mode)
 		reduce = (jump_total_rank / conf.max_jump_total_rank) + 1;
 
 	if (mode == NO_SUG_JUMP) {
-		int ret = check_jump_params(args, now, reduce);
+		const int ret = check_jump_params(args, now, reduce);
 		if (ret != -1)
 			return ret;
 	}
@@ -807,13 +832,10 @@ dirjump(char **args, const int mode)
 		return FUNC_FAILURE;
 	}
 
-	/* If ARG is an actual directory, just cd into it. */
-	struct stat attr;
-	if (args[1] && !args[2] && lstat(args[1], &attr) != -1) {
-		if (mode == NO_SUG_JUMP)
-			return cd_function(args[1], CD_PRINT_ERROR);
-		return save_jump_suggestion(args[1]);
-	}
+	/* If ARG is an actual directory, just change to it. */
+	int ret = 0;
+	if (args[1] && !args[2] && check_dir(args[1], mode, &ret) == 1)
+		return ret;
 
 	/* Find the best ranked directory using ARGS as filter(s). */
 	size_t i;
