@@ -1,17 +1,15 @@
-/* long_view.c -- Construct entries in long view mode */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,9 +20,10 @@
  * MA 02110-1301, USA.
 */
 
+/* long_view.c -- Construct entries in long view mode */
+
 #include "helpers.h"
 
-#include <stdio.h>  /* printf(3), putchar(3), fputs(3) */
 #include <string.h> /* memcpy(3), strrchr(3) */
 #include <time.h>   /* strftime(3) */
 
@@ -35,27 +34,43 @@
 #include "misc.h"   /* gen_diff_str() */
 #include "properties.h" /* get_color_age, get_color_size, get_file_perms */
 
+/* Remaining space in the properties string buffer. */
+static size_t buf_rem_space = 0;
+
+/* Precomputed colors without the bold attribute for the file type field
+ * in the permissions string. */
+static char bd_nb[MAX_COLOR];
+static char cd_nb[MAX_COLOR];
+static char df_nb[MAX_COLOR];
+static char di_nb[MAX_COLOR];
+static char dn_nb[MAX_COLOR];
+static char fi_nb[MAX_COLOR];
+static char ln_nb[MAX_COLOR];
+#ifdef SOLARIS_DOORS
+static char oo_nb[MAX_COLOR];
+#endif /* SOLARIS_DOORS */
+static char pi_nb[MAX_COLOR];
+static char so_nb[MAX_COLOR];
+
 static char *
 get_ext_info_long(const struct fileinfo *props, const size_t name_len,
-	int *trim, size_t *ext_len)
+	int *trunc, size_t *ext_len)
 {
-	char *ext_name = (char *)NULL;
-	char *e = strrchr(props->name, '.');
-	if (e && e != props->name && e[1]) {
-		ext_name = e;
-		*trim = TRIM_EXT;
-		if (props->utf8 == 0)
-			*ext_len = name_len - (size_t)(ext_name - props->name);
-		else
-			*ext_len = wc_xstrlen(ext_name);
-	}
+	/* At this point, TRUNC is set to TRUNC_NO_EXT and EXT_LEN to zero. */
+	if (!props->ext_name)
+		return (char *)NULL;
 
-	if ((int)*ext_len >= conf.max_name_len || (int)*ext_len <= 0) {
+	if (props->utf8 == 0)
+		*ext_len = name_len - (size_t)(props->ext_name - props->name);
+	else
+		*ext_len = wc_xstrlen(props->ext_name);
+
+	if ((int)*ext_len >= conf.max_name_len || (int)*ext_len <= 0)
 		*ext_len = 0;
-		*trim = TRIM_NO_EXT;
-	}
+	else
+		*trunc = TRUNC_EXT;
 
-	return ext_name;
+	return props->ext_name;
 }
 
 /* Calculate the relative time of AGE, which is the difference between
@@ -98,15 +113,15 @@ static void
 construct_and_print_filename(const struct fileinfo *props,
 	const int max_namelen)
 {
-	/* If file name length is greater than max, truncate it to max (later a
-	 * tilde (~) will be appended to let the user know the file name was
-	 * truncated). */
-	int trim = 0;
+	/* If the filename length is greater than MAX_NAMELEN, truncate it to
+	 * MAX_NAMELEN (later a tilde (~) will be appended to let the user know
+	 * the filename was truncated). */
+	int trunc = 0;
 
-	/* Handle file names with embedded control characters */
+	/* Handle filenames with embedded control characters. */
 	size_t plen = props->len;
 	char *wname = (char *)NULL;
-	if (props->len == 0) {
+	if (plen == 0) {
 		wname = replace_invalid_chars(props->name);
 		plen = wc_xstrlen(wname);
 	}
@@ -115,7 +130,7 @@ construct_and_print_filename(const struct fileinfo *props,
 		&& files > (filesn_t)conf.max_files) ? (filesn_t)conf.max_files : files;
 
 	size_t cur_len = (size_t)DIGINUM(n) + 1 + plen
-		+ (conf.icons == 1 ? ICON_LEN : 0);
+		+ (conf.icons == 1 ? (size_t)ICON_LEN : 0);
 
 	int diff = 0;
 	char *name = wname ? wname : props->name;
@@ -123,14 +138,14 @@ construct_and_print_filename(const struct fileinfo *props,
 
 	if (cur_len > (size_t)max_namelen) {
 		const int rest = (int)cur_len - max_namelen;
-		trim = TRIM_NO_EXT;
+		trunc = TRUNC_NO_EXT;
 		size_t ext_len = 0;
-		ext_name = get_ext_info_long(props, plen, &trim, &ext_len);
+		ext_name = get_ext_info_long(props, plen, &trunc, &ext_len);
 
-		int trim_point = (int)plen - rest - 1 - (int)ext_len;
-		if (trim_point <= 0) {
-			trim_point = (int)plen - rest - 1;
-			trim = TRIM_NO_EXT;
+		int trunc_point = (int)plen - rest - 1 - (int)ext_len;
+		if (trunc_point <= 0) {
+			trunc_point = (int)plen - rest - 1;
+			trunc = TRUNC_NO_EXT;
 		}
 
 		if (props->utf8 == 1) {
@@ -139,12 +154,12 @@ construct_and_print_filename(const struct fileinfo *props,
 			else /* memcpy is faster: use it whenever possible. */
 				memcpy(name_buf, name, props->bytes + 1);
 
-			diff = u8truncstr(name_buf, (size_t)trim_point);
+			diff = u8truncstr(name_buf, (size_t)trunc_point);
 		} else { /* Let's avoid u8truncstr() to get some extra speed. */
-			const char c = name[trim_point];
-			name[trim_point] = '\0';
+			const char c = name[trunc_point];
+			name[trunc_point] = '\0';
 			mbstowcs((wchar_t *)name_buf, name, NAME_BUF_SIZE);
-			name[trim_point] = c;
+			name[trunc_point] = c;
 		}
 
 		cur_len -= (size_t)rest;
@@ -154,51 +169,52 @@ construct_and_print_filename(const struct fileinfo *props,
 
 	free(wname);
 
-	/* Calculate pad for each file name */
+	/* Calculate pad for each filename */
 	int pad = max_namelen - (int)cur_len;
 	if (pad < 0)
 		pad = 0;
 
-	const char *trim_diff = diff > 0 ? gen_diff_str(diff) : "";
+	const char *trunc_diff = diff > 0 ? gen_diff_str(diff) : "";
 
-	static char trim_s[2] = {0};
-	*trim_s = trim > 0 ? TRIMFILE_CHR : 0;
+	static char trunc_s[2] = {0};
+	*trunc_s = trunc > 0 ? TRUNC_FILE_CHR : 0;
 
 	printf("%s%s%s%s%s%ls%s%s%-*s%s\x1b[0m%s%s\x1b[0m%s%s%s  ",
 		(conf.colorize == 1 && conf.icons == 1) ? props->icon_color : "",
 		conf.icons == 1 ? props->icon : "", conf.icons == 1 ? " " : "", df_c,
-
 		conf.colorize == 1 ? props->color : "",
-		(wchar_t *)name_buf, trim_diff,
+		(wchar_t *)name_buf, trunc_diff,
 		conf.light_mode == 1 ? "\x1b[0m" : df_c, pad, "", df_c,
-		trim ? tt_c : "", trim_s,
-		trim == TRIM_EXT ? props->color : "",
-		trim == TRIM_EXT ? ext_name : "",
-		trim == TRIM_EXT ? df_c : "");
+		trunc ? tt_c : "", trunc_s,
+		trunc == TRUNC_EXT ? props->color : "",
+		trunc == TRUNC_EXT ? ext_name : "",
+		trunc == TRUNC_EXT ? df_c : "");
 }
 
-static void
-construct_file_size(const struct fileinfo *props, char *size_str,
+static size_t
+gen_size(const struct fileinfo *props, char *size_str,
 	const int size_max, const int file_perm)
 {
 	if (prop_fields.size == 0) {
 		*size_str = '\0';
-		return;
+		return 0;
 	}
 
+	int bytes = 0;
+
 	if (props->stat_err == 1) {
-		snprintf(size_str, SIZE_STR_LEN, "%*s", size_max
+		bytes = snprintf(size_str, buf_rem_space, "%*s", size_max
 			+ (prop_fields.size == PROP_SIZE_HUMAN), UNKNOWN_STR);
-		return;
+		return bytes > 0 ? (size_t)bytes : 0;
 	}
 
 	const int no_dir_access =
 		(file_perm == 0 && props->dir == 1 && conf.full_dir_size == 1);
 	if (S_ISCHR(props->mode) || S_ISBLK(props->mode) || no_dir_access == 1) {
-		snprintf(size_str, SIZE_STR_LEN, "%s%*c%s", dn_c, size_max
+		bytes = snprintf(size_str, buf_rem_space, "%s%*c%s", dn_c, size_max
 			+ (prop_fields.size == PROP_SIZE_HUMAN),
 			no_dir_access == 1 ? UNKNOWN_CHR : '-', df_c);
-		return;
+		return bytes > 0 ? (size_t)bytes : 0;
 	}
 
 	const off_t size = (FILE_TYPE_NON_ZERO_SIZE(props->mode)
@@ -208,82 +224,85 @@ construct_file_size(const struct fileinfo *props, char *size_str,
 	/* Let's construct the color for the current file size */
 	char *csize = dz_c;
 	static char sf[MAX_SHADE_LEN];
-	if (conf.colorize == 1) {
-		if (!*dz_c) {
-			get_color_size(size, sf, sizeof(sf));
-			csize = sf;
-		}
+	if (!*dz_c && conf.colorize == 1) {
+		get_color_size(size, sf, sizeof(sf));
+		csize = sf;
 	}
 
 	if (prop_fields.size != PROP_SIZE_HUMAN) {
-		snprintf(size_str, SIZE_STR_LEN, "%s%*jd%s%c", csize,
+		char err_char[2] = {0};
+		err_char[0] = props->du_status != 0 ? DU_ERR_CHAR : 0;
+		bytes = snprintf(size_str, buf_rem_space, "%s%*jd%s%s", csize,
 			(props->du_status != 0 && size_max > 0) ? size_max - 1 : size_max,
-			(intmax_t)size, df_c,
-			props->du_status != 0 ? DU_ERR_CHAR : 0);
-		return;
+			(intmax_t)size, df_c, err_char);
+		return bytes > 0 ? (size_t)bytes : 0;
 	}
 
-	const int du_err = (props->dir == 1 && conf.full_dir_size == 1
-		&& props->du_status != 0);
-	const char *unit_color = conf.colorize == 0
-		? (du_err == 1 ? "\x1b[1m" : "")
-		: (du_err == 1 ? xf_cb : dim_c);
+	const int du_err = (props->du_status != 0 && props->dir == 1
+		&& conf.full_dir_size == 1);
+	const char *unit_color = conf.colorize == 1
+		? (du_err == 1 ? xf_cb : dim_c)
+		: ((du_err == 1 && xargs.no_bold != 1) ? "\x1b[1m" : "");
 
-	snprintf(size_str, SIZE_STR_LEN, "%s%*s%s%c\x1b[0m%s",
+	bytes = snprintf(size_str, buf_rem_space, "%s%*s%s%c\x1b[0m%s",
 		csize, size_max,
 		*props->human_size.str ? props->human_size.str : UNKNOWN_STR,
 		unit_color, props->human_size.unit, df_c);
+
+	return bytes > 0 ? (size_t)bytes : 0;
 }
 
-static void
-construct_file_perms(const mode_t mode, char *perm_str, const char file_type,
+static size_t
+gen_perms(const mode_t mode, char *perm_str, const char file_type,
 	const char *ctype)
 {
-	static char tmp_ctype[MAX_COLOR + 1];
-	xstrsncpy(tmp_ctype, (file_type == UNK_PCHR ? df_c : ctype),
-		sizeof(tmp_ctype));
-	if (xargs.no_bold != 1)
-		remove_bold_attr(tmp_ctype);
+	int bytes = 0;
 
 	if (prop_fields.perm == PERM_SYMBOLIC) {
 		const struct perms_t perms = get_file_perms(mode);
-		snprintf(perm_str, PERM_STR_LEN,
+		bytes = snprintf(perm_str, buf_rem_space,
 			"%s%c%s/%s%c%s%c%s%c%s.%s%c%s%c%s%c%s.%s%c%s%c%s%c%s",
-			tmp_ctype, file_type, dn_c,
+			ctype, file_type, dn_c,
 			perms.cur, perms.ur, perms.cuw, perms.uw, perms.cux, perms.ux, dn_c,
 			perms.cgr, perms.gr, perms.cgw, perms.gw, perms.cgx, perms.gx, dn_c,
 			perms.cor, perms.or, perms.cow, perms.ow, perms.cox, perms.ox, df_c);
 
 	} else /* PERM_NUMERIC */ {
-		snprintf(perm_str, PERM_STR_LEN, "%s%04o%s", do_c, mode & 07777, df_c);
-
+		bytes = snprintf(perm_str, PERM_STR_LEN,
+			"%s%04o%s", do_c, mode & 07777, df_c);
 	}
+
+	return bytes > 0 ? (size_t)bytes : 0;
 }
 
-static char *
+static const char *
 get_time_char(void)
 {
 	if (conf.time_follows_sort == 1) {
 		switch (conf.sort) {
 		case SATIME: return conf.relative_time == 1 ? "A" : "a";
-		case SCTIME: return conf.relative_time == 1 ? "C" : "c";
 		case SBTIME: return conf.relative_time == 1 ? "B" : "b";
-		case SMTIME: return conf.relative_time == 1 ? "M" : "m";
-		default: break;
+		case SCTIME: return conf.relative_time == 1 ? "C" : "c";
+		case SMTIME: /* fallthrough */
+		default: return conf.relative_time == 1 ? "M" : "m";
 		}
 	}
 
 	switch (prop_fields.time) {
 	case PROP_TIME_ACCESS: return conf.relative_time == 1 ? "A" : "a";
-	case PROP_TIME_BIRTH: return conf.relative_time == 1 ? "B" : "b";
+	case PROP_TIME_BIRTH:  return conf.relative_time == 1 ? "B" : "b";
 	case PROP_TIME_CHANGE: return conf.relative_time == 1 ? "C" : "c";
 	case PROP_TIME_MOD: /* fallthrough */
 	default: return conf.relative_time == 1 ? "M" : "m";
 	}
 }
 
-static void
-construct_timestamp(char *time_str, const struct fileinfo *props)
+/* GCC (not clang) complains about tfmt being not a string literal. Let's
+ * silence this warning until we find a better approach. */
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
+static size_t
+gen_time(char *time_str, const struct fileinfo *props)
 {
 	const time_t t = props->ltime;
 
@@ -321,17 +340,7 @@ construct_timestamp(char *time_str, const struct fileinfo *props)
 			const char *tfmt = conf.time_str ? conf.time_str :
 				(recent ? DEF_TIME_STYLE_RECENT : DEF_TIME_STYLE_OLDER);
 
-			/* GCC (not clang) complains about tfmt being not a string
-			 * literal. Let's silence this warning until we find a better
-			 * approach. */
-#ifdef GCC_ALLOWS_PRAGMA_IN_FUNC
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif /* GCC_ALLOWS_PRAGMA_IN_FUNC */
 			strftime(file_time, sizeof(file_time), tfmt, &tm);
-#ifdef GCC_ALLOWS_PRAGMA_IN_FUNC
-# pragma GCC diagnostic pop
-#endif /* GCC_ALLOWS_PRAGMA_IN_FUNC */
 		} else {
 			xstrsncpy(file_time, invalid_time_str, sizeof(file_time));
 		}
@@ -341,15 +350,19 @@ construct_timestamp(char *time_str, const struct fileinfo *props)
 		xstrsncpy(file_time, invalid_time_str, sizeof(file_time));
 	}
 
-	snprintf(time_str, TIME_STR_LEN, "%s%s%s%s%s", cdate, *file_time
-		? file_time : UNKNOWN_STR, dt_c,
+	const int bytes = snprintf(time_str, buf_rem_space, "%s%s%s%s%s", cdate,
+		*file_time ? file_time : UNKNOWN_STR, dt_c,
 		conf.timestamp_mark == 1 ? get_time_char() : "", df_c);
-}
 
-static void
-construct_id_field(const struct fileinfo *props, char *id_str,
+	return bytes > 0 ? (size_t)bytes : 0;
+}
+#pragma GCC diagnostic pop
+
+static size_t
+gen_id(const struct fileinfo *props, char *id_str,
 	const struct maxes_t *maxes, const int file_perm)
 {
+	int bytes = 0;
 	const char *uid_color =
 		(file_perm == 1 && conf.colorize == 1) ? du_c : df_c;
 
@@ -361,230 +374,250 @@ construct_id_field(const struct fileinfo *props, char *id_str,
 	if (prop_fields.no_group == 1) {
 		if (prop_fields.ids == PROP_ID_NUM) {
 			if (props->stat_err == 1) {
-				snprintf(id_str, ID_STR_LEN, "%s%*s%s", uid_color,
+				bytes = snprintf(id_str, buf_rem_space, "%s%*s%s", uid_color,
 					maxes->id_user, UNKNOWN_STR, df_c);
 			} else {
-				snprintf(id_str, ID_STR_LEN, "%s%*u%s", uid_color,
+				bytes = snprintf(id_str, buf_rem_space, "%s%*u%s", uid_color,
 					maxes->id_user, props->uid, df_c);
 			}
 		} else { /* PROPS_ID_NAME */
-			snprintf(id_str, ID_STR_LEN, "%s%-*s%s", uid_color,
+			bytes = snprintf(id_str, buf_rem_space, "%s%-*s%s", uid_color,
 				maxes->id_user, USER_NAME, df_c);
 		}
 
-		return;
+		return bytes > 0 ? (size_t)bytes : 0;
 	}
 
-//	const char *dim = conf.colorize == 0 ? "" : dim_c;
 	const char *gid_color = conf.colorize == 0 ? "" :
 		(file_perm == 1 ? dg_c : dim_c);
 
 	if (prop_fields.ids == PROP_ID_NUM) {
 		if (props->stat_err == 1) {
-			snprintf(id_str, ID_STR_LEN, "%s%*c %*c",
+			bytes = snprintf(id_str, buf_rem_space, "%s%*c %*c",
 				df_c, maxes->id_user, UNKNOWN_CHR,
 				maxes->id_group, UNKNOWN_CHR);
 		} else {
-			snprintf(id_str, ID_STR_LEN, "%s%*u %s%*u%s",
+			bytes = snprintf(id_str, buf_rem_space, "%s%*u %s%*u%s",
 				uid_color, maxes->id_user, props->uid,
 				gid_color, maxes->id_group, props->gid, df_c);
 		}
 	} else { /* PROPS_ID_NAME */
-		snprintf(id_str, ID_STR_LEN, "%s%-*s %s%-*s%s",
+		bytes = snprintf(id_str, buf_rem_space, "%s%-*s %s%-*s%s",
 			uid_color, maxes->id_user, USER_NAME,
 			props->stat_err == 1 ? "" : gid_color,
 			maxes->id_group, GROUP_NAME, df_c);
 	}
+#undef USER_NAME
+#undef GROUP_NAME
+	return bytes > 0 ? (size_t)bytes : 0;
 }
 
-static void
-construct_files_counter(const struct fileinfo *props, char *fc_str,
+static size_t
+gen_filecounter(const struct fileinfo *props, char *fc_str,
 	const int max)
 {
+	int bytes = 0;
+
 	if (props->filesn > 0) {
-		snprintf(fc_str, FC_STR_LEN, "%s%*d%s", fc_c, max,
-			(int)props->filesn, df_c);
+		bytes = snprintf(fc_str, buf_rem_space, "%s%*zu%s", fc_c, max,
+			props->filesn, df_c);
 	} else {
-		snprintf(fc_str, FC_STR_LEN, "%s%*c%s", dn_c, max,
+		bytes = snprintf(fc_str, buf_rem_space, "%s%*c%s", dn_c, max,
 			props->filesn < 0 ? UNKNOWN_CHR /* Dir with no read permission */
-			: (props->dir == 1 ? '0' : '-'),
-			df_c);
+			: (props->dir == 1 ? '0' : '-'), df_c);
 	}
+
+	return bytes > 0 ? (size_t)bytes : 0;
+}
+
+static size_t
+gen_inode(const struct fileinfo *props, char *ino_str, const int max)
+{
+	int bytes = 0;
+
+	if (props->stat_err == 1) {
+		bytes = snprintf(ino_str, buf_rem_space,
+			"\x1b[0m%*s%s", max, UNKNOWN_STR, df_c);
+	} else {
+		bytes = snprintf(ino_str, buf_rem_space, "\x1b[0m%s%*ju%s", de_c,
+			max, (uintmax_t)props->inode, df_c);
+	}
+
+	return bytes > 0 ? (size_t)bytes : 0;
+}
+
+static size_t
+gen_links(const struct fileinfo *props, char *links_str, const int max)
+{
+	int bytes = 0;
+
+	if (props->stat_err == 1) {
+		bytes = snprintf(links_str, buf_rem_space, "\x1b[0m%*s%s",
+			max, UNKNOWN_STR, df_c);
+	} else {
+		bytes = snprintf(links_str, buf_rem_space, "\x1b[0m%s%s%*ju%s",
+			dk_c, props->linkn > 1 ? BOLD : "", max,
+			(uintmax_t)props->linkn, df_c);
+	}
+
+	return bytes > 0 ? (size_t)bytes : 0;
+}
+
+static size_t
+gen_blocks(const struct fileinfo *props, char *blk_str, const int max)
+{
+	int bytes = 0;
+
+	if (props->stat_err == 1) {
+		bytes = snprintf(blk_str, buf_rem_space,
+			"\x1b[0m%*s%s", max, UNKNOWN_STR, df_c);
+	} else {
+		bytes = snprintf(blk_str, buf_rem_space, "\x1b[0m%s%*jd%s",
+			db_c, max, (intmax_t)props->blocks, df_c);
+	}
+
+	return bytes > 0 ? (size_t)bytes : 0;
 }
 
 static void
-set_file_type_and_color(const struct fileinfo *props, char *type, char **color)
+set_no_bold_colors(void)
 {
+	xstrsncpy(bd_nb, bd_c, sizeof(bd_nb)); remove_bold_attr(bd_nb);
+	xstrsncpy(cd_nb, cd_c, sizeof(cd_nb)); remove_bold_attr(cd_nb);
+	xstrsncpy(df_nb, df_c, sizeof(df_nb)); remove_bold_attr(df_nb);
+	xstrsncpy(di_nb, di_c, sizeof(di_nb)); remove_bold_attr(di_nb);
+	xstrsncpy(dn_nb, dn_c, sizeof(dn_nb)); remove_bold_attr(dn_nb);
+	xstrsncpy(fi_nb, fi_c, sizeof(fi_nb)); remove_bold_attr(fi_nb);
+	xstrsncpy(ln_nb, ln_c, sizeof(ln_nb)); remove_bold_attr(ln_nb);
+#ifdef SOLARIS_DOORS
+	xstrsncpy(oo_nb, oo_c, sizeof(oo_nb)); remove_bold_attr(oo_nb);
+#endif /* SOLARIS_DOORS */
+	xstrsncpy(pi_nb, pi_c, sizeof(pi_nb)); remove_bold_attr(pi_nb);
+	xstrsncpy(so_nb, so_c, sizeof(so_nb)); remove_bold_attr(so_nb);
+}
+
+static char
+set_file_type_and_color(const struct fileinfo *props, char **color)
+{
+	/* Precompute file type colors without the bold attribute for the
+	 * file type field in the permissions string. Let's do this only once,
+	 * and each time the color scheme is switched. */
+	static char *cscheme_bk = NULL;
+	if (!cscheme_bk || cscheme_bk != cur_cscheme) {
+		set_no_bold_colors();
+		cscheme_bk = cur_cscheme;
+	}
+
 	struct stat a;
 	if (props->stat_err == 1 && conf.follow_symlinks_long == 1
 	&& conf.long_view == 1 && conf.follow_symlinks == 1
 	&& lstat(props->name, &a) == 0 && S_ISLNK(a.st_mode)) {
-		*type = LNK_PCHR;
-		*color = conf.colorize == 1 ? ln_c : df_c;
-		return;
+		*color = conf.colorize == 1 ? ln_nb : df_nb;
+		return LNK_PCHR;
 	}
 
+	char type = 0;
+
 	switch (props->mode & S_IFMT) {
-	case S_IFREG:  *type = REG_PCHR; break;
-	case S_IFDIR:  *type = DIR_PCHR; *color = di_c; break;
-	case S_IFLNK:  *type = LNK_PCHR; *color = ln_c; break;
-	case S_IFIFO:  *type = FIFO_PCHR; *color = pi_c; break;
-	case S_IFSOCK: *type = SOCK_PCHR; *color = so_c; break;
-	case S_IFBLK:  *type = BLKDEV_PCHR; *color = bd_c; break;
-	case S_IFCHR:  *type = CHARDEV_PCHR; *color = cd_c; break;
+	case S_IFREG:  type = REG_PCHR; *color = dn_nb; break;
+	case S_IFDIR:  type = DIR_PCHR; *color = di_nb; break;
+	case S_IFLNK:  type = LNK_PCHR; *color = ln_nb; break;
+	case S_IFIFO:  type = FIFO_PCHR; *color = pi_nb; break;
+	case S_IFSOCK: type = SOCK_PCHR; *color = so_nb; break;
+	case S_IFBLK:  type = BLKDEV_PCHR; *color = bd_nb; break;
+	case S_IFCHR:  type = CHARDEV_PCHR; *color = cd_nb; break;
 #ifndef _BE_POSIX
 # ifdef S_ARCH1
-	case S_ARCH1:  *type = ARCH1_PCHR; *color = fi_c; break;
-	case S_ARCH2:  *type = ARCH2_PCHR; *color = fi_c; break;
+	case S_ARCH1:  type = ARCH1_PCHR; *color = fi_nb; break;
+	case S_ARCH2:  type = ARCH2_PCHR; *color = fi_nb; break;
 # endif /* S_ARCH1 */
 # ifdef SOLARIS_DOORS
-	case S_IFDOOR: *type = DOOR_PCHR; *color = oo_c; break;
-	case S_IFPORT: *type = PORT_PCHR; *color = oo_c; break;
+	case S_IFDOOR: type = DOOR_PCHR; *color = oo_nb; break;
+	case S_IFPORT: type = PORT_PCHR; *color = oo_nb; break;
 # endif /* SOLARIS_DOORS */
 # ifdef S_IFWHT
-	case S_IFWHT:  *type = WHT_PCHR; *color = fi_c; break;
+	case S_IFWHT:  type = WHT_PCHR; *color = fi_nb; break;
 # endif /* S_IFWHT */
 #endif /* !_BE_POSIX */
-	default:       *type = UNK_PCHR; break;
+	default:       type = UNK_PCHR; *color = dn_nb; break;
 	}
 
 	if (conf.colorize == 0)
-		*color = df_c;
+		*color = df_nb;
+
+	return type;
 }
 
-static void
-construct_inode_num(const struct fileinfo *props, char *ino_str, const int max)
-{
-	if (props->stat_err == 1) {
-		snprintf(ino_str, INO_STR_LEN, "\x1b[0m%*s%s", max, UNKNOWN_STR, df_c);
-		return;
-	}
-
-	snprintf(ino_str, INO_STR_LEN, "\x1b[0m%s%*ju%s", de_c,
-		max, (uintmax_t)props->inode, df_c);
-}
-
-static void
-construct_links_str(const struct fileinfo *props, char *links_str, const int max)
-{
-	if (props->stat_err == 1) {
-		snprintf(links_str, LINKS_STR_LEN, "\x1b[0m%*s%s",
-			max, UNKNOWN_STR, df_c);
-		return;
-	}
-
-	snprintf(links_str, LINKS_STR_LEN, "\x1b[0m%s%s%*ju%s",
-		dk_c, props->linkn > 1 ? BOLD : "", max, (uintmax_t)props->linkn, df_c);
-}
-
-static void
-construct_blocks_str(const struct fileinfo *props, char *blk_str, const int max)
-{
-	if (props->stat_err == 1) {
-		snprintf(blk_str, BLK_STR_LEN, "\x1b[0m%*s%s", max, UNKNOWN_STR, df_c);
-		return;
-	}
-
-	snprintf(blk_str, BLK_STR_LEN, "\x1b[0m%s%*jd%s",
-		db_c, max, (intmax_t)props->blocks, df_c);
-}
-
-/* Compose the properties line for the current file name.
- * This function is called by list_dir(), in listing.c, for each file name
+/* Compose the properties line for the current filename.
+ * This function is called by list_dir(), in listing.c, for each filename
  * in the current directory when running in long view mode (after
  * printing the corresponding ELN). */
 int
 print_entry_props(const struct fileinfo *props, const struct maxes_t *maxes,
 	const int have_xattr)
 {
-	char file_type = 0; /* File type indicator */
-	char *ctype = dn_c; /* Color for file type indicator */
+	static char buf[MAX_PROP_STR + 1]; /* Store generated fields */
+	size_t len = 0; /* Bytes written into buf so far. */
 
-	set_file_type_and_color(props, &file_type, &ctype);
-	const int file_perm =
-		check_file_access(props->mode, props->uid, props->gid);
-
-	/* Let's compose each properties field individually to be able to
-	 * print only the desired ones. This is specified via the PropFields
-	 * option in the config file. */
+	char *ctype = NULL; /* Color for the file type indicator */
+	const char file_type = set_file_type_and_color(props, &ctype);
+	const int file_perm = conf.light_mode == 1
+		? check_file_access(props->mode, props->uid, props->gid)
+		: (props->stat_err != 1 && props->user_access != 0);
+	const char xattr_char =
+		have_xattr == 1 ? (props->xattr == 1 ? XATTR_CHAR : ' ') : 0;
+	const size_t prop_fields_gap = (size_t)conf.prop_fields_gap;
+	const int file_counter =
+		(conf.files_counter != 0 && maxes->files_counter != 0);
 
 	construct_and_print_filename(props, maxes->name);
 
-	static char perm_str[PERM_STR_LEN]; *perm_str = '\0';
-	if (prop_fields.perm != 0)
-		construct_file_perms(props->mode, perm_str, file_type, ctype);
-
-	static char time_str[TIME_STR_LEN]; *time_str = '\0';
-	if (prop_fields.time != 0)
-		construct_timestamp(time_str, props);
-
-	static char size_str[SIZE_STR_LEN]; *size_str = '\0';
-	if (prop_fields.size != 0)
-		construct_file_size(props, size_str, maxes->size, file_perm);
-
-	static char id_str[ID_STR_LEN]; *id_str = '\0';
-	if (prop_fields.ids != 0)
-		construct_id_field(props, id_str, maxes, file_perm);
-
-	static char links_str[LINKS_STR_LEN]; *links_str = '\0';
-	if (prop_fields.links != 0)
-		construct_links_str(props, links_str, maxes->links);
-
-	static char ino_str[INO_STR_LEN]; *ino_str = '\0';
-	if (prop_fields.inode != 0)
-		construct_inode_num(props, ino_str, maxes->inode);
-
-	static char blocks_str[BLK_STR_LEN]; *blocks_str = '\0';
-	if (prop_fields.blocks != 0)
-		construct_blocks_str(props, blocks_str, maxes->blocks);
-
-	static char fc_str[FC_STR_LEN]; *fc_str = '\0';
-	/* FC_MAX is zero if there are no subdirs in the current dir */
-	if (prop_fields.counter != 0 && conf.files_counter != 0
-	&& maxes->files_counter != 0)
-		construct_files_counter(props, fc_str, maxes->files_counter);
-
-	/* We only need a single character to print the xattributes indicator,
-	 * which would be normally printed like this:
-	 * printf("%c", x ? 'x' : 0);
-	 * However, some terminals, like 'cons25', print the 0 above graphically,
-	 * as a space, which is not what we want here. To fix this, let's
-	 * construct this char as a string. */
-	static char xattr_str[2] = {0};
-	*xattr_str = have_xattr == 1 ? (props->xattr == 1 ? XATTR_CHAR : ' ') : 0;
-
-	/* Print stuff */
+	/* Let's print fields according to the value of PropFields in the
+	 * config file (prop_fields_str). */
 	for (size_t i = 0; i < PROP_FIELDS_SIZE && prop_fields_str[i]; i++) {
-		int print_space = prop_fields_str[i + 1] ? 1 : 0;
+		if (len >= sizeof(buf) - 1)
+			break;
+
+		buf_rem_space = sizeof(buf) - len;
 
 		switch (prop_fields_str[i]) {
-		case 'B': if (*blocks_str) fputs(blocks_str, stdout); break;
-		case 'f': fputs(fc_str, stdout); break;
-		case 'd': if (*ino_str) fputs(ino_str, stdout); break;
+		case 'B': len += gen_blocks(props, buf + len, maxes->blocks); break;
+		case 'f':
+			if (file_counter == 1)
+				len += gen_filecounter(props, buf + len, maxes->files_counter);
+			break;
+		case 'd': len += gen_inode(props, buf + len, maxes->inode); break;
 		case 'p': /* fallthrough */
-		case 'n': fputs(perm_str, stdout);
-			if (*xattr_str) fputs(xattr_str, stdout);
+		case 'n':
+			len += gen_perms(props->mode, buf + len, file_type, ctype);
+			if (xattr_char != 0) buf[len++] = xattr_char;
 			break;
 		case 'i': /* fallthrough */
-		case 'I': fputs(id_str, stdout); break;
-		case 'l': if (*links_str) fputs(links_str, stdout); break;
+		case 'I': len += gen_id(props, buf + len, maxes, file_perm); break;
+		case 'l': len += gen_links(props, buf + len, maxes->links); break;
 		case 'a': /* fallthrough */
 		case 'b': /* fallthrough */
 		case 'm': /* fallthrough */
-		case 'c': fputs(time_str, stdout); break;
+		case 'c': len += gen_time(buf + len, props); break;
 		case 's': /* fallthrough */
-		case 'S': fputs(size_str, stdout); break;
-		default: print_space = 0; break;
+		case 'S':
+			len += gen_size(props, buf + len, maxes->size, file_perm); break;
+		default: continue; /* Unknown option character. Skip it. */
 		}
 
-		if (print_space == 0)
-			continue;
+		/* If not the last field, add some space to separate the current
+		 * field from the next one. */
+		const int last_field = prop_fields_str[i + 1] == '\0';
+		if (last_field == 1 || (sizeof(buf) - len) <= prop_fields_gap)
+			break;
 
-		if (conf.prop_fields_gap <= 1)
-			putchar(' ');
-		else
-			MOVE_CURSOR_RIGHT(conf.prop_fields_gap);
+		buf[len++] = ' ';
+		if (conf.prop_fields_gap > 1) /* PropFieldsGap is at most 2. */
+			buf[len++] = ' ';
 	}
+
+	buf[len] = '\0';
+	fputs(buf, stdout);
 	putchar('\n');
 
 	return FUNC_SUCCESS;
