@@ -1,16 +1,9 @@
 /*
- * SPDX-FileCopyrightText: 2016-2022 L. Abramovich <leo.clifm@outlook.com>
+ * This file is part of Clifm
+ *
  * SPDX-License-Identifier: GPL-2.0-or-later
- */
-
-			/*  ########################################
-			 *  #               CliFM                  #
-			 *  # 	  The command line file manager    #
-			 *  ######################################## */
-
-/* Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -27,12 +20,13 @@
  * MA 02110-1301, USA.
  */
 
+/* main.c -- Main file for Clifm */
+
 #include "helpers.h"
 
 #include <errno.h>
 #include <langinfo.h> /* nl_langinfo() */
 #include <locale.h>
-#include <stdio.h>
 #include <string.h>
 #include <termios.h>
 #include <unistd.h>
@@ -47,6 +41,7 @@
 #include "jump.h"
 #include "keybinds.h"
 #include "listing.h"
+#include "mimetypes.h" /* load_user_mimetypes() */
 #include "misc.h"
 #ifndef _NO_PROFILES
 # include "profiles.h"
@@ -58,6 +53,7 @@
 #ifdef SECURITY_PARANOID
 # include "sanitize.h"
 #endif /* SECURITY_PARANOID */
+#include "term.h" /* set_term_title() */
 
 /* Globals */
 
@@ -98,6 +94,8 @@ struct ext_mnt_t *ext_mnt = (struct ext_mnt_t *)NULL;
 struct groups_t *sys_users = (struct groups_t *)NULL;
 struct groups_t *sys_groups = (struct groups_t *)NULL;
 struct dircmds_t dir_cmds = {UNSET, 0};
+struct pmsgs_t *messages = (struct pmsgs_t *)NULL;
+struct mime_t *user_mimetypes = (struct mime_t *)NULL;
 
 const struct sort_t sort_methods[] = {
     {"none", 0, 0},
@@ -165,8 +163,9 @@ int
 	exit_code = 0,
 
 	fzftab = UNSET,
+	fzf_ext_border = UNSET,
 	fzf_border_type = UNSET,
-	fzf_height_set = 0,
+	fzf_height_value = 0,
 	fzf_preview_border_type = 0,
 
 	hist_status = UNSET,
@@ -255,9 +254,9 @@ char
 	*alt_trash_dir = (char *)NULL,
 	*alt_config_file = (char *)NULL,
 	*alt_kbinds_file = (char *)NULL,
+	*alt_mimelist_file = (char *)NULL,
 	*alt_preview_file = (char *)NULL,
 	*alt_profile = (char *)NULL,
-	*bin_name = (char *)NULL,
 	*bm_file = (char *)NULL,
 	*cmds_log_file = (char *)NULL,
 	*colors_dir = (char *)NULL,
@@ -284,7 +283,6 @@ char
 	*quote_chars = (char *)NULL,
 	*rl_callback_handler_input = (char *)NULL,
 	*remotes_file = (char *)NULL,
-/*	*right_prompt = (char *)NULL, */
 	*sel_file = (char *)NULL,
 	*smenutab_options_env = (char *)NULL,
 	*stdin_tmp_dir = (char *)NULL,
@@ -294,6 +292,8 @@ char
 #endif /* !_NO_SUGGESTIONS */
 	*sys_shell = (char *)NULL,
 	*tags_dir = (char *)NULL,
+	*templates_dir = (char *)NULL,
+	*thumbnails_dir = (char *)NULL,
 	*tmp_rootdir = (char *)NULL,
 	*tmp_dir = (char *)NULL,
 #ifndef _NO_TRASH
@@ -306,7 +306,7 @@ char
 	**bin_commands = (char **)NULL,
 	**cdpaths = (char **)NULL,
 	**color_schemes = (char **)NULL,
-	**messages = (char **)NULL,
+	**file_templates = (char **)NULL,
 	**old_pwd = (char **)NULL,
 	**profile_names = (char **)NULL,
 	**prompt_cmds = (char **)NULL,
@@ -344,26 +344,27 @@ char
 	uf_c[MAX_COLOR], /* Non-'stat'able file */
 
 	/* Interface */
+	ac_c[MAX_COLOR + 2], /* Autocmd indicator */
 	df_c[MAX_COLOR], /* Default color */
 	dl_c[MAX_COLOR], /* Dividing line index */
 	el_c[MAX_COLOR], /* ELN */
-	fc_c[MAX_COLOR], /* Files counter */
+	fc_c[MAX_COLOR], /* File counter */
 	lc_c[MAX_COLOR], /* Symlink character (ColorLinkAsTarget only) */
 	mi_c[MAX_COLOR], /* Misc indicators */
-	ts_c[MAX_COLOR], /* TAB completion suffix */
+	ts_c[MAX_COLOR], /* Tab completion suffix */
 	wc_c[MAX_COLOR], /* Welcome message color */
 	wp_c[MAX_COLOR], /* Warning prompt */
-	tt_c[MAX_COLOR], /* Tilde for trimmed files */
+	tt_c[MAX_COLOR], /* Character to mark truncated filenames */
 
 	/* Suggestions */
 	sb_c[MAX_COLOR], /* Auto-suggestions: shell builtins */
 	sc_c[MAX_COLOR], /* Auto-suggestions: external commands */
 	sd_c[MAX_COLOR], /* Auto-suggestions: internal commands description */
 	sh_c[MAX_COLOR], /* Auto-suggestions: history */
-	sf_c[MAX_COLOR], /* Auto-suggestions: file names */
+	sf_c[MAX_COLOR], /* Auto-suggestions: filenames */
 	sx_c[MAX_COLOR], /* Auto-suggestions: internal commands and params */
 	sp_c[MAX_COLOR], /* Auto-suggestions: suggestions pointer */
-	sz_c[MAX_COLOR], /* Auto-suggestions: file names (fuzzy) */
+	sz_c[MAX_COLOR], /* Auto-suggestions: filenames (fuzzy) */
 
 #ifndef _NO_ICONS
 	dir_ico_c[MAX_COLOR], /* Directories icon color */
@@ -412,7 +413,7 @@ char
 
 	em_c[MAX_COLOR + 2], /* Error msg color */
 	li_c[MAX_COLOR + 2], /* Sel indicator color */
-	li_cb[MAX_COLOR], /* Sel indicator color (for the files list) */
+	li_cb[MAX_COLOR], /* Sel indicator color (for the file list) */
 	nm_c[MAX_COLOR + 2], /* Notice msg color */
 	wm_c[MAX_COLOR + 2], /* Warning msg color */
 	ro_c[MAX_COLOR + 2], /* read-only indicator color */
@@ -420,177 +421,177 @@ char
 	ti_c[MAX_COLOR + 2], /* Trash indicator color */
 	tx_c[MAX_COLOR + 2], /* Text color */
 	xs_c[MAX_COLOR + 2], /* Exit code: success */
+	xs_cb[MAX_COLOR],    /* Exit code: success (Unicode success indicator) */
 	xf_c[MAX_COLOR + 2], /* Exit code: failure */
 	xf_cb[MAX_COLOR],    /* Exit code: failure (dir read) */
 
 	tmp_color[MAX_COLOR + 2], /* A temp buffer to store color codes */
 	dim_c[] = "\x1b[2m";
 
-/* A buffer to store file names to be displayed (wide string) */
+/* A buffer to store filenames to be displayed (wide string) */
 char name_buf[NAME_BUF_SIZE * sizeof(wchar_t)];
 
-/* A list of all internal commands, with short and long formats.
- * We use two more lists of commands: one of commands dealing with file names
- * (is_internal()), and another one listing commands taking ELN's/numbers
- * as parameters (is_internal_f()), both in checks.c */
+/* A list of all internal commands, including command name, name length,
+ * and parameter type (as a bit flag). Used by is_internal_cmd(), in checks.c. */
 const struct cmdslist_t internal_cmds[] = {
-	{",", 1},
-	{"?", 1},
-	{"help", 4},
-	{"ac", 2},
-	{"ad", 2},
-	{"acd", 3},
-	{"autocd", 6},
-	{"actions", 7},
-	{"alias", 5},
-	{"ao", 2},
-	{"auto-open", 9},
-	{"b", 1},
-	{"back", 4},
-	{"bb", 2},
-	{"bleach", 6},
-	{"bd", 2},
-	{"dh", 2},
-	{"bl", 2},
-	{"bm", 2},
-	{"bookmarks", 9},
-	{"br", 2},
-	{"bulk", 4},
-	{"c", 1}, //"cp",
-	{"colors", 6},
-	{"cd", 2},
-	{"cl", 2},
-	{"columns", 7},
-	{"cmd", 3},
-	{"commands", 8},
-	{"cs", 2},
-	{"colorschemes", 12},
-	{"cwd", 3}, // deprecate this one, just as 'path'
-	{"d", 1},
-	{"dup", 3},
-	{"ds", 2},
-	{"desel", 5},
-	{"config", 6},
-	{"exp", 3},
-	{"export", 6},
-	{"ext", 3},
-	{"f", 1},
-	{"forth", 5},
-	{"fc", 2},
-	{"ff", 2},
-	{"dirs-first", 10},
-	{"ft", 2},
-	{"filter", 6},
-	{"fz", 2},
-	{"hh", 2},
-	{"hf", 2},
-	{"hidden", 6},
-	{"history", 7},
-	{"icons", 5},
-	{"j", 1},
-	{"jump", 4},
-	{"je", 2},
-	{"jc", 2},
-	{"jl", 2},
-	{"jp", 2},
-	{"k", 1},
-	{"kk", 2},
-//	{"jo", 2},
-	{"kb", 2},
-	{"keybinds", 8},
-	{"l", 1},
-	{"le", 2},
-	{"lm", 2},
-	{"log", 3},
-	{"ll", 2},
-	{"lv", 2},
-	{"m", 1},
-	{"md", 2},
-	{"media", 5},
-	{"mf", 2},
-	{"mm", 2},
-	{"mime", 4},
-	{"mp", 2},
-	{"mountpoints", 11},
-	{"msg", 3},
-	{"messages", 8},
-	{"n", 1},
-	{"new", 3},
-	{"net", 3},
-	{"o", 1},
-	{"oc", 2},
-	{"open", 4},
-	{"ow", 2},
-	{"opener", 6},
-	{"p", 1},
-	{"pc", 2},
-	{"pp", 2},
-	{"pr", 2},
-	{"prop", 4},
-	{"path", 4}, // deprecate this one, just as 'cwd'
-	{"paste", 5},
-	{"pf", 2},
-	{"prof", 4},
-	{"profile", 7},
-	{"pg", 2},
-	{"pager", 5},
-	{"pin", 3},
-	{"unpin", 5},
-	{"prompt", 6},
-	{"pwd", 3},
-	{"q", 1},
-	{"Q", 1},
-	{"quit", 4},
-	{"exit", 4},
-	{"r", 1}, //"rm",
-	{"rf", 2},
-	{"refresh", 7},
-	{"rl", 2},
-	{"reload", 6},
-	{"rr", 2},
-	{"s", 1},
-	{"sel", 3},
-	{"sb", 2},
-	{"selbox", 6},
-	{"splash", 6},
-	{"st", 2},
-	{"sort", 4},
-	{"stats", 5},
-	{"t", 1},
-	{"tr", 2},
-	{"trash", 5},
-	{"tag", 3},
-	{"ta", 2},
-	{"td", 2},
-	{"tl", 2},
-	{"tm", 2},
-	{"tn", 2},
-	{"tu", 2},
-	{"ty", 2},
-	{"te", 2},
-	{"tips", 4},
-	{"u", 1},
-	{"undel", 5},
-	{"unset", 5},
-	{"untrash", 7},
-	{"umask", 5},
-	{"v", 1},
-	{"vv", 2},
-	{"ver", 3},
-	{"version", 7},
-	{"view", 4},
-	{"ws", 2},
-	{"x", 1},
-	{"X", 1},
-	{NULL, 0}
+	{",", 1, NO_PARAM, 0},
+	{"?", 1, NO_PARAM, 0},
+	{"help", 4, NO_PARAM, 0},
+	{"ac", 2, PARAM_FNAME, 0},
+	{"ad", 2, PARAM_FNAME, 0},
+	{"acd", 3, PARAM_STR, 0},
+	{"auto", 4, PARAM_STR, 0},
+	{"autocd", 6, PARAM_STR, 0},
+	{"actions", 7, PARAM_STR, 0},
+	{"alias", 5, PARAM_FNAME, 0}, // diff (old 0110, new 1110) // 'alias import' takes filenames
+	{"ao", 2, PARAM_STR, 0},
+	{"auto-open", 9, PARAM_STR, 0},
+	{"b", 1, PARAM_STR, 0},
+	{"back", 4, PARAM_STR, 0},
+	{"bb", 2, PARAM_FNAME, 0},
+	{"bleach", 6, PARAM_FNAME, 0},
+	{"bd", 2, PARAM_STR, 0},
+	{"dh", 2, PARAM_FNAME, 0}, // diff (old 0110, new 1110)
+	{"bl", 2, PARAM_FNAME, 0},
+	{"bm", 2, PARAM_FNAME, 0},
+	{"bookmarks", 9, PARAM_FNAME, 0},
+	{"br", 2, PARAM_FNAME, 0},
+	{"bulk", 4, PARAM_FNAME, 0},
+	{"c", 1, PARAM_FNAME, 0}, //"cp",
+	{"colors", 6, NO_PARAM, 0},
+	{"cd", 2, PARAM_FNAME, 0},
+	{"cl", 2, PARAM_STR, 0},
+	{"columns", 7, PARAM_STR, 0},
+	{"cmd", 3, NO_PARAM, 0},
+	{"commands", 8, NO_PARAM, 0},
+	{"cs", 2, PARAM_STR, 0},
+	{"colorschemes", 12, PARAM_STR, 0},
+	{"cwd", 3, NO_PARAM, 0}, // deprecate this one, just as 'path'
+	{"d", 1, PARAM_FNAME, 0},
+	{"dup", 3, PARAM_FNAME, 0},
+	{"ds", 2, PARAM_FNAME, 0}, // diff (old 0110, new 1110)
+	{"desel", 5, PARAM_FNAME, 0}, // diff (old 0110, new 1110)
+	{"config", 6, PARAM_STR, 0},
+	{"exp", 3, PARAM_FNAME, 0},
+	{"export", 6, PARAM_FNAME | PARAM_STR, 0}, // diff (old 1101, new 1111)
+	{"ext", 3, PARAM_STR, 0},
+	{"f", 1, PARAM_STR, 0},
+	{"forth", 5, PARAM_STR, 0},
+	{"fc", 2, PARAM_STR, 0},
+	{"ff", 2, PARAM_STR, 0},
+	{"dirs-first", 10, PARAM_STR, 0},
+	{"ft", 2, PARAM_STR, 0},
+	{"filter", 6, PARAM_STR, 0},
+	{"fz", 2, PARAM_STR, 0},
+	{"hh", 2, PARAM_STR, 0},
+	{"hf", 2, PARAM_STR, 0},
+	{"hidden", 6, PARAM_STR, 0},
+	{"history", 7, PARAM_STR, 0},
+	{"icons", 5, PARAM_STR, 0},
+	{"j", 1, PARAM_STR, 0},
+	{"jump", 4, PARAM_STR, 0},
+	{"je", 2, PARAM_STR, 0},
+	{"jc", 2, PARAM_STR, 0}, // diff (old 1101, new 0101)
+	{"jl", 2, PARAM_STR, 0},
+	{"jp", 2, PARAM_STR, 0}, // diff (old 1101, new 0101)
+	{"k", 1, NO_PARAM, 0},
+	{"kk", 2, NO_PARAM, 0},
+	{"kb", 2, PARAM_STR, 0},
+	{"keybinds", 8, PARAM_STR, 0},
+	{"l", 1, PARAM_FNAME, 0},
+	{"le", 2, PARAM_FNAME, 0},
+	{"lm", 2, PARAM_STR, 0},
+	{"log", 3, PARAM_STR, 0},
+	{"ll", 2, PARAM_STR, 0},
+	{"lv", 2, PARAM_STR, 0},
+	{"m", 1, PARAM_FNAME, 0},
+	{"md", 2, PARAM_FNAME, 0}, // diff (old 0110, 1110)
+	{"media", 5, NO_PARAM, 0},
+	{"mf", 2, PARAM_NUM, 0},
+	{"mm", 2, PARAM_FNAME, 0},
+	{"mime", 4, PARAM_FNAME, 0},
+	{"mp", 2, NO_PARAM, 0},
+	{"mountpoints", 11, NO_PARAM, 0},
+	{"msg", 3, PARAM_STR, 0},
+	{"messages", 8, PARAM_STR, 0},
+	{"n", 1, PARAM_FNAME, 0},
+	{"new", 3, PARAM_FNAME, 0},
+	{"net", 3, PARAM_STR, 0},
+	{"o", 1, PARAM_FNAME, 0},
+	{"oc", 2, PARAM_FNAME, 0},
+	{"open", 4, PARAM_FNAME, 0},
+	{"ow", 2, PARAM_FNAME, 0}, // diff (old 0110, new 1110)
+	{"opener", 6, PARAM_STR, 0},
+	{"p", 1, PARAM_FNAME, 0},
+	{"pc", 2, PARAM_FNAME, 0},
+	{"pp", 2, PARAM_FNAME, 0},
+	{"pr", 2, PARAM_FNAME, 0},
+	{"prop", 4, PARAM_FNAME, 0},
+	{"path", 4, NO_PARAM, 0}, // deprecate this one, just as 'cwd'
+	{"paste", 5, PARAM_FNAME, 0},
+	{"pf", 2, PARAM_STR, 0},
+	{"prof", 4, PARAM_STR, 0},
+	{"profile", 7, PARAM_STR, 0},
+	{"pg", 2, PARAM_STR, 0},
+	{"pager", 5, PARAM_STR, 0},
+	{"pin", 3, PARAM_FNAME, 0},
+	{"unpin", 5, NO_PARAM, 0},
+	{"prompt", 6, PARAM_STR, 0},
+	{"pwd", 3, PARAM_STR, 0},
+	{"q", 1, NO_PARAM, 0},
+	{"quit", 4, NO_PARAM, 0},
+	{"exit", 4, NO_PARAM, 0},
+	{"r", 1, PARAM_FNAME, 0}, //"rm",
+	{"rf", 2, NO_PARAM, 0},
+	{"refresh", 7, NO_PARAM, 0},
+	{"rl", 2, NO_PARAM, 0},
+	{"reload", 6, NO_PARAM, 0},
+	{"rr", 2, PARAM_FNAME, 0},
+	{"s", 1, PARAM_FNAME, 0},
+	{"sel", 3, PARAM_FNAME, 0},
+	{"sb", 2, NO_PARAM, 0},
+	{"selbox", 6, NO_PARAM, 0},
+	{"st", 2, PARAM_NUM, 0},
+	{"sort", 4, PARAM_NUM, 0},
+	{"stats", 5, NO_PARAM, 0},
+	{"t", 1, PARAM_FNAME, 0},
+	{"tr", 2, PARAM_FNAME, 0},
+	{"trash", 5, PARAM_FNAME, 0},
+	{"tag", 3, PARAM_FNAME, 0},
+	{"ta", 2, PARAM_FNAME, 0},
+	{"td", 2, PARAM_STR, 0},
+	{"tl", 2, PARAM_STR, 0}, // diff (old 0110, new 0101)
+	{"tm", 2, PARAM_STR, 0},
+	{"tn", 2, PARAM_STR, 0},
+	{"tu", 2, PARAM_STR, 0},
+	{"ty", 2, PARAM_STR, 0},
+	{"te", 2, PARAM_FNAME, 0},
+	{"tips", 4, NO_PARAM, 0},
+	{"u", 1, PARAM_STR, 0},
+	{"undel", 5, PARAM_STR, 0},
+	{"unset", 5, PARAM_STR, 0},
+	{"untrash", 7, PARAM_STR, 0},
+	{"umask", 5, PARAM_STR, 0},
+	{"vv", 2, PARAM_FNAME, 0},
+	{"ver", 3, NO_PARAM, 0},
+	{"version", 7, NO_PARAM, 0},
+	{"view", 4, PARAM_STR, 0},
+	{"ws", 2, PARAM_NUM, 0},
+	{"x", 1, PARAM_FNAME, 0}, // diff (old 0110, new 1110)
+	{"X", 1, PARAM_FNAME, 0}, // diff (old 0110, new 1110)
+	{NULL, 0, 0, 0}
 };
+
 size_t internal_cmds_n = 0;
 
 /* A list of internal commands and fixed parameters for the auto-suggestions
  * system. */
-const struct cmdslist_t param_str[] = {
+const struct nameslist_t param_str[] = {
 	{"actions edit", 12},
 	{"actions list", 12},
+	{"auto unset", 10},
+	{"auto list", 9},
+	{"auto none", 9},
 	{"autocd on", 9},
 	{"acd on", 6},
 	{"autocd off", 10},
@@ -617,6 +618,8 @@ const struct cmdslist_t param_str[] = {
 	{"bookmarks edit", 14},
 	{"c -f", 4},
 	{"c --force", 9},
+	{"cs check-ext", 12},
+	{"colorschemes check-ext", 22},
 	{"colorschemes edit", 17},
 	{"cs edit", 7},
 	{"colorschemes preview", 20},
@@ -626,6 +629,7 @@ const struct cmdslist_t param_str[] = {
 	{"config", 6},
 	{"config edit", 11},
 	{"config dump", 11},
+	{"config reload", 13},
 	{"config reset", 12},
 	{"ext on", 6},
 	{"ext off", 7},
@@ -657,6 +661,7 @@ const struct cmdslist_t param_str[] = {
 	{"help file-details", 17},
 	{"help file-filters", 17},
 	{"help file-previews", 18},
+	{"help image-previews", 19},
 	{"help file-tags", 14},
 	{"help navigation", 15},
 	{"help plugins", 12},
@@ -670,13 +675,19 @@ const struct cmdslist_t param_str[] = {
 	{"help trash", 10},
 	{"hf on", 5},
 	{"hf off", 6},
+	{"hf first", 8},
+	{"hf last", 7},
 	{"hf status", 9},
 	{"hh on", 5},
 	{"hh off", 6},
 	{"hh status", 9},
+	{"hh last", 7},
+	{"hh status", 9},
 	{"hidden on", 9},
 	{"hidden off", 10},
 	{"hidden status", 13},
+	{"hidden last", 11},
+	{"hidden first", 12},
 	{"history clear", 13},
 	{"history edit", 12},
 	{"history on", 10},
@@ -687,6 +698,8 @@ const struct cmdslist_t param_str[] = {
 	{"icons off", 9},
 	{"j --purge", 9},
 	{"j --edit", 8},
+	{"kb bind", 7},
+	{"keybinds bind", 13},
 	{"kb conflict", 11},
 	{"keybinds conflict", 17},
 	{"kb edit", 7},
@@ -807,6 +820,86 @@ const struct cmdslist_t param_str[] = {
 	{"undel all", 9},
 	{"untrash all", 11},
 	{"view edit", 9},
+	{"view purge", 10},
+	{NULL, 0}
+};
+
+const struct nameslist_t kb_cmds[] = {
+	{"archive-sel", 11},
+	{"bookmarks", 9},
+	{"cmd-hist", 8},
+	{"clear-line", 10},
+	{"clear-msgs", 10},
+	{"create-file", 11},
+	{"deselect-all", 12},
+	{"dirs-first", 10},
+	{"edit-color-scheme", 17},
+	{"export-sel", 10},
+	{"home-dir", 8},
+	{"launch-view", 11},
+	{"lock", 4},
+	{"mountpoints", 11},
+	{"move-sel", 8},
+	{"new-instance", 12},
+	{"next-dir", 8},
+	{"next-profile", 12},
+	{"only-dirs", 9},
+	{"open-bookmarks", 14},
+	{"open-config", 11},
+	{"open-jump-db", 12},
+	{"open-keybinds", 13},
+	{"open-mime", 9},
+	{"open-preview", 12},
+	{"open-sel", 8},
+	{"parent-dir", 10},
+	{"pinned-dir", 10},
+	{"plugin1", 7},
+	{"plugin2", 7},
+	{"plugin3", 7},
+	{"plugin4", 7},
+	{"plugin5", 7},
+	{"plugin6", 7},
+	{"plugin7", 7},
+	{"plugin8", 7},
+	{"plugin9", 7},
+	{"plugin10", 8},
+	{"plugin11", 8},
+	{"plugin12", 8},
+	{"plugin13", 8},
+	{"plugin14", 8},
+	{"plugin15", 8},
+	{"plugin16", 8},
+	{"paste-sel", 10},
+	{"prepend-sudo", 12},
+	{"previous-dir", 12},
+	{"previous-profile", 16},
+	{"quit", 4},
+	{"refresh-screen", 14},
+	{"remove-sel", 10},
+	{"rename-sel", 10},
+	{"root-dir", 8},
+	{"run-pager", 9},
+	{"selbox", 6},
+	{"select-all", 10},
+	{"show-dirhist", 12},
+	{"sort-previous", 13},
+	{"sort-next", 9},
+	{"show-manpage", 12},
+	{"show-cmds", 9},
+	{"show-kbinds", 11},
+	{"toggle-disk-usage", 17},
+	{"toggle-follow-links-long", 24},
+	{"toggle-hidden", 13},
+	{"toggle-light", 12},
+	{"toggle-long", 11},
+	{"toggle-max-name-len", 19},
+	{"toggle-virtualdir-full-paths", 28},
+	{"trash-sel", 9},
+	{"untrash-sel", 11},
+	{"workspace1", 10},
+	{"workspace2", 10},
+	{"workspace3", 10},
+	{"workspace4", 10},
 	{NULL, 0}
 };
 
@@ -815,9 +908,6 @@ int inotify_fd = UNSET, inotify_wd = UNSET;
 int watch = UNSET;
 unsigned int INOTIFY_MASK =
 	IN_CREATE | IN_DELETE | IN_DELETE_SELF | IN_MOVE | IN_MOVE_SELF
-/*#if LINUX_VERSION_CODE >= KERNEL_VERSION(4, 18, 0)
-	| IN_DONT_FOLLOW | IN_EXCL_UNLINK | IN_ONLYDIR | IN_MASK_CREATE;
-#else */
 # if LINUX_VERSION_CODE >= KERNEL_VERSION(2, 6, 15)
 	| IN_DONT_FOLLOW | IN_ONLYDIR
 # endif /* LINUX >= 2.6.15 */
@@ -837,22 +927,37 @@ time_t curdir_mtime = 0;
 #endif /* LINUX_INOTIFY */
 
 #ifdef RUN_CMD
+# ifndef _NO_TRASH
+static inline void
+init_trash(void)
+{
+	if (trash_ok == 1) {
+		const filesn_t ret = count_dir(trash_files_dir, NO_CPOP);
+		trash_n = ret <= 2 ? 0 : (size_t)ret - 2;
+	}
+}
+# endif /* _NO_TRASH */
+
 /* Run the command passed via --cmd and exit */
 static void
 run_and_exit(void)
 {
-	/* 1) Parse input string */
+# ifndef _NO_TRASH
+	init_trash();
+# endif /* !_NO_TRASH */
+
+	/* 1) Parse input string. */
 	int i = 0;
 	char **cmd = parse_input_str(cmd_line_cmd);
 	if (!cmd)
 		exit(EXIT_FAILURE);
 
-	/* 2) Execute input string */
+	/* 2) Execute input string. */
 	char **alias_cmd = check_for_alias(cmd);
 	if (alias_cmd) {
 		/* If an alias is found, check_for_alias() frees CMD and returns
-		 * ALIAS_CMD in its place to be executed by exec_cmd() */
-		exec_cmd(alias_cmd);
+		 * ALIAS_CMD in its place to be executed by exec_cmd_tm(). */
+		exec_cmd_tm(alias_cmd);
 
 		for (i = 0; alias_cmd[i]; i++)
 			free(alias_cmd[i]);
@@ -861,7 +966,7 @@ run_and_exit(void)
 	}
 
 	if (!(flags & FAILED_ALIAS))
-		exec_cmd(cmd);
+		exec_cmd_tm(cmd);
 	flags &= ~FAILED_ALIAS;
 
 	for (i = 0; cmd[i]; i++)
@@ -873,7 +978,7 @@ run_and_exit(void)
 }
 #endif /* RUN_CMD */
 
-/* This is the main structure of any basic shell (a REPL)
+/* This is the main structure of a basic shell (a REPL)
 	 1 - Grab user input
 	 2 - Parse user input
 	 3 - Execute command
@@ -885,31 +990,30 @@ static void
 run_main_loop(void)
 {
 #ifdef RUN_CMD
-	if (cmd_line_cmd)
-		run_and_exit();
+	if (cmd_line_cmd != NULL)
+		run_and_exit(); /* No return */
 #endif /* RUN_CMD */
 
 	int i;
-	/* 1) Infinite loop to keep the program running */
+	/* 1) Infinite loop to keep the program running. */
 	while (1) {
-		/* 2) Grab input string from the prompt */
-		char *input = prompt(PROMPT_SHOW);
+		/* 2) Grab the input string from the command prompt. */
+		char *input = prompt(PROMPT_SHOW, PROMPT_SCREEN_REFRESH);
 		if (!input)
 			continue;
 
-		/* 3) Parse input string */
+		/* 3) Parse the input string. */
 		char **cmd = parse_input_str(input);
 		free(input);
-		input = (char *)NULL;
 
 		if (!cmd)
 			continue;
 
-		/* 4) Execute input string */
+		/* 4) Execute the command. */
 		char **alias_cmd = check_for_alias(cmd);
 		if (alias_cmd) {
 			/* If an alias is found, check_for_alias() frees CMD and returns
-			 * ALIAS_CMD in its place to be executed by exec_cmd() */
+			 * ALIAS_CMD in its place to be executed by exec_cmd_tm(). */
 			exec_cmd_tm(alias_cmd);
 
 			for (i = 0; alias_cmd[i]; i++)
@@ -934,8 +1038,8 @@ set_root_indicator(void)
 {
 	if (user.uid == 0) {
 		const char *bold_red = conf.colorize == 1 ? "\x1b[1;31m" : "";
-		err(ERR_NO_LOG, PRINT_PROMPT, _("%s->%s Running as root%s\n"),
-			conf.colorize == 1 ? mi_c : "", bold_red,
+		err(ERR_NO_LOG, PRINT_PROMPT, _("%s%s%s Running as root%s\n"),
+			conf.colorize == 1 ? mi_c : "", SET_MSG_PTR, bold_red,
 			conf.colorize == 1 ? df_c : "");
 	}
 }
@@ -944,7 +1048,7 @@ static inline void
 list_files(void)
 {
 #ifdef RUN_CMD
-	if (cmd_line_cmd)
+	if (cmd_line_cmd != NULL)
 		return;
 #endif /* RUN_CMD */
 
@@ -978,7 +1082,7 @@ LIST:
 static inline void
 print_splash_screen(void)
 {
-	if (conf.splash_screen) {
+	if (conf.splash_screen == 1) {
 		splash();
 		conf.splash_screen = 0;
 		CLEAR;
@@ -989,15 +1093,9 @@ print_splash_screen(void)
 static inline void
 set_term_win_title(void)
 {
-	if (!(flags & GUI) || xargs.list_and_quit == 1 || xargs.vt100 == 1)
-		return;
-
-	if (xargs.cwd_in_title == 0) {
-		printf("\033]2;%s\007", PROGRAM_NAME);
-		fflush(stdout);
-	} else {
-		set_term_title(workspaces[cur_ws].path);
-	}
+	if ((flags & GUI) && xargs.list_and_quit != 1 && xargs.vt100 != 1)
+		set_term_title(xargs.cwd_in_title == 1
+			? workspaces[cur_ws].path : (char *)NULL);
 }
 
 static inline void
@@ -1006,50 +1104,24 @@ check_working_directory(void)
 	if (workspaces == (struct ws_t *)NULL || !workspaces[cur_ws].path
 	|| !*workspaces[cur_ws].path) {
 		err(0, NOPRINT_PROMPT, _("%s: Fatal error! Failure "
-			"retrieving current working directory\n"), PROGRAM_NAME);
+			"retrieving the current working directory\n"), PROGRAM_NAME);
 		exit(EXIT_FAILURE);
 	}
 }
 
-/* Check whether we have a working shell */
-/*
-static inline void
-check_working_shell(void)
-{
-	if (access(user.shell, X_OK) == -1) {
-		err('w', PRINT_PROMPT, _("%s: %s: System shell not found. "
-			"Please edit the configuration file to specify a working "
-			"shell.\n"), PROGRAM_NAME, user.shell);
-	}
-} */
-
-#ifndef _NO_TRASH
-static inline void
-init_trash(void)
-{
-	if (trash_ok) {
-		filesn_t ret = count_dir(trash_files_dir, NO_CPOP);
-		if (ret <= 2)
-			trash_n = 0;
-		else
-			trash_n = (size_t)ret;
-	}
-}
-#endif /* _NO_TRASH */
-
 static inline void
 get_hostname(void)
 {
 	if (gethostname(hostname, sizeof(hostname)) == -1) {
-		hostname[0] = '?';
+		hostname[0] = UNKNOWN_CHR;
 		hostname[1] = '\0';
 		err('e', PRINT_PROMPT, _("%s: Error getting hostname\n"),
 			PROGRAM_NAME);
 	}
 }
 
-/* Initialize the files filter struct */
-static void
+/* Initialize the file filter struct */
+static inline void
 init_filter(void)
 {
 	filter.str = (char *)NULL;
@@ -1059,13 +1131,13 @@ init_filter(void)
 }
 
 /* Initialize the msgs struct */
-static void
+static inline void
 init_msgs(void)
 {
 	msgs.error = msgs.notice = msgs.warning = 0;
 }
 
-static void
+static inline void
 set_locale(void)
 {
 	/* Use the locale specified by the environment */
@@ -1073,11 +1145,11 @@ set_locale(void)
 	if (strcmp(nl_langinfo(CODESET), "UTF-8") != 0) {
 		err('w', PRINT_PROMPT, _("%s: Locale is not UTF-8. To avoid "
 			"encoding issues you might want to set an UTF-8 locale. For "
-			"example: export LANG=es_AR.UTF-8\n"), PROGRAM_NAME);
+			"example: 'export LANG=es_AR.UTF-8'.\n"), PROGRAM_NAME);
 	}
 }
 
-static void
+static inline void
 check_gui(void)
 {
 	/* Running on a graphical environment? */
@@ -1114,7 +1186,7 @@ set_security_paranoid_mode(void)
 #endif /* SECURITY_PARANOID */
 
 #ifdef HAVE_PLEDGE
-static void
+static inline void
 set_pledge(void)
 {
 	if (pledge("stdio rpath wpath cpath dpath tmppath fattr "
@@ -1151,24 +1223,14 @@ list_files_and_quit(void)
 	exit(EXIT_SUCCESS); /* Never reached. */
 }
 
-/*
-static void
-init_file_flags(void)
-{
-	flags |= CONFIG_OK;
-	flags |= HOME_OK;
-	flags |= SELFILE_OK;
-	flags |= TRASH_OK;
-} */
-
 				/**
 				 * #############################
 				 * #           MAIN            #
 				 * #############################
 				 * */
 
-/* 1. Initialize stuff
- * 2. Run the main program loop */
+/* 1. Initialize stuff.
+ * 2. Run the main program loop. */
 int
 main(int argc, char *argv[])
 {
@@ -1188,8 +1250,6 @@ main(int argc, char *argv[])
 	/* Make sure all initialization is made with restrictive permissions. */
 	const mode_t old_mask = umask(0077); /* flawfinder: ignore */
 
-	check_term(); /* Let's check terminal capabilities. */
-
 	/* # 1. INITIALIZE EVERYTHING WE NEED # */
 
 	init_conf_struct();
@@ -1198,7 +1258,6 @@ main(int argc, char *argv[])
 #ifndef _NO_ICONS
 	init_icons_hashes();
 #endif /* !_NO_ICONS */
-/*	init_file_flags(); */
 
 	set_locale();
 
@@ -1232,11 +1291,12 @@ main(int argc, char *argv[])
 	set_security_paranoid_mode();
 #endif /* SECURITY_PARANOID */
 
+	check_term(); /* Let's check terminal capabilities. */
+
 	/* Get paths from PATH environment variable. These paths will be
 	 * used later by get_path_programs (for the autocomplete function)
 	 * and get_cmd_path(). */
-	if (!(flags & PATH_PROGRAMS_ALREADY_LOADED))
-		path_n = get_path_env(1);
+	path_n = get_path_env(1);
 	cdpath_n = get_cdpath();
 
 	check_env_filter();
@@ -1253,7 +1313,7 @@ main(int argc, char *argv[])
 		do_stat_and_exit(xargs.stat == FULL_STAT ? 1 : 0);
 
 	if (xargs.list_and_quit == 1)
-		list_files_and_quit();
+		list_files_and_quit(); /* No return. */
 
 	set_sel_file();
 	create_tmp_files();
@@ -1262,8 +1322,7 @@ main(int argc, char *argv[])
 
 	/* Get the list of available programs in PATH to be used by the
 	 * custom TAB-completion function (tab_complete(), in tabcomp.c). */
-	if (!(flags & PATH_PROGRAMS_ALREADY_LOADED))
-		get_path_programs();
+	get_path_programs();
 
 	/* Check third-party programs availability: finders (fzf, fnf, smenu),
 	 * udevil, and udisks2. */
@@ -1301,6 +1360,7 @@ main(int argc, char *argv[])
 
 	shell = get_sys_shell();
 	create_kbinds_file();
+	load_file_templates();
 	load_bookmarks();
 	load_keybinds();
 	load_tags();
@@ -1311,18 +1371,13 @@ main(int argc, char *argv[])
 	init_shell();
 	initialize_readline();
 	get_prompt_cmds();
-
-#ifndef _NO_TRASH
-	init_trash();
-#endif /* !_NO_TRASH */
-
 	get_hostname();
 	set_env(0);
 
 	if (config_ok == 1)
 		init_history();
 
-	/* Store history into an array to be able to manipulate it. */
+	/* Store history in an array to be able to manipulate it. */
 	get_history();
 
 #ifndef _NO_PROFILES
@@ -1331,9 +1386,10 @@ main(int argc, char *argv[])
 
 	load_pinned_dir();
 	init_workspaces_opts();
+	load_user_mimetypes();
 
 	/* Restore user umask */
-	umask(old_mask);
+	umask(old_mask); /* flawfinder: ignore */
 
 	/* # 2. MAIN PROGRAM LOOP # */
 	run_main_loop();
