@@ -1,17 +1,15 @@
-/* media.c -- functions to manage local filesystems */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,11 +20,12 @@
  * MA 02110-1301, USA.
 */
 
+/* media.c -- functions to manage local filesystems */
+
 #ifndef NO_MEDIA_FUNC
 
 #include "helpers.h"
 
-#include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <errno.h>
@@ -54,9 +53,7 @@
 #endif /* __linux__ || __CYGWIN__ */
 
 #include "aux.h"
-#ifdef __sun
-# include "checks.h" /* check_file_access() */
-#endif /* __sun */
+#include "colors.h" /* get_dir_color() */
 #include "history.h"
 #include "jump.h"
 #include "listing.h"
@@ -137,7 +134,7 @@ unmount_dev(const size_t i, const int n)
 {
 	if (xargs.mount_cmd == UNSET) {
 		xerror(_("%s: No mount application found. Install either "
-			"udevil or udisks2\n"), PROGRAM_NAME);
+			"udevil or udisks2.\n"), PROGRAM_NAME);
 		return FUNC_FAILURE;
 	}
 
@@ -279,15 +276,18 @@ list_mounted_devs(const int mode)
 		if (strncmp(ent->mnt_fsname, "/dev/", 5) != 0)
 			continue;
 
+		struct stat a;
+		if (stat(ent->mnt_dir, &a) == -1 || !S_ISDIR(a.st_mode))
+			continue;
+
+		const char *dir_color = get_dir_color(ent->mnt_dir, &a, -1);
+
 		if (mode == MEDIA_LIST) {
 			printf("%s%zu%s %s%s%s [%s]\n", el_c, mp_n + 1,
-				df_c, (access(ent->mnt_dir, R_OK | X_OK) == 0) ? di_c : nd_c,
-				ent->mnt_dir, df_c, ent->mnt_fsname);
+				df_c, dir_color, ent->mnt_dir, df_c, ent->mnt_fsname);
 		} else {
 			printf("%s%zu%s %s [%s%s%s]\n", el_c, mp_n + 1,
-				df_c, ent->mnt_fsname,
-				(access(ent->mnt_dir, R_OK | X_OK) == 0) ? di_c
-				: nd_c, ent->mnt_dir, df_c);
+				df_c, ent->mnt_fsname, dir_color, ent->mnt_dir, df_c);
 		}
 
 		media = xnrealloc(media, mp_n + 2, sizeof(struct mnt_t));
@@ -313,7 +313,7 @@ mount_dev(const int n)
 {
 	if (xargs.mount_cmd == UNSET) {
 		xerror(_("%s: No mount application found. Install either "
-			"udevil or udisks2\n"), PROGRAM_NAME);
+			"udevil or udisks2.\n"), PROGRAM_NAME);
 		return FUNC_FAILURE;
 	}
 
@@ -326,6 +326,11 @@ mount_dev(const int n)
 		return FUNC_FAILURE;
 
 	int stdout_bk = dup(STDOUT_FILENO); /* Save original stdout */
+	if (stdout_bk == -1) {
+		unlinkat(fd, file, 0);
+		return FUNC_FAILURE;
+	}
+
 	dup2(fd, STDOUT_FILENO); /* Redirect stdout to the desired file */
 	close(fd);
 
@@ -429,12 +434,15 @@ list_mountpoints_bsd(struct statfs *fslist)
 		if (strncmp(fslist[i].f_mntfromname, "/dev/", 5) != 0)
 			continue;
 
-		printf("%s%zu%s %s%s%s (%s)\n", el_c, j + 1, df_c,
-		    (access(fslist[i].f_mntonname, R_OK | X_OK) == 0)
-		    ? di_c : nd_c, fslist[i].f_mntonname,
-		    df_c, fslist[i].f_mntfromname);
+		struct stat a;
+		if (stat(fslist[i].f_mntonname, &a) == -1 || !S_ISDIR(a.st_mode))
+			continue;
 
-		/* Store the mountpoint into the mounpoints struct */
+		const char *dir_color = get_dir_color(fslist[i].f_mntonname, &a, -1);
+		printf("%s%zu%s %s%s%s (%s)\n", el_c, j + 1, df_c, dir_color,
+			fslist[i].f_mntonname, df_c, fslist[i].f_mntfromname);
+
+		/* Store the mountpoint in the mounpoints struct */
 		media = xnrealloc(media, j + 2, sizeof(struct mnt_t));
 		media[j].mnt = savestring(fslist[i].f_mntonname,
 			strlen(fslist[i].f_mntonname));
@@ -463,12 +471,12 @@ get_mnt_input(const int mode, int *info)
 	while (!input) {
 #ifdef HAVE_PROC_MOUNTS
 		if (mode == MEDIA_LIST)
-			input = rl_no_hist(_("Choose a mountpoint: "));
+			input = rl_no_hist(_("Select a mountpoint (by ELN): "), 0);
 		else
-			input = rl_no_hist(_("Choose a mountpoint/device: "));
+			input = rl_no_hist(_("Select a mountpoint/device (by ELN): "), 0);
 #else
 		UNUSED(mode);
-		input = rl_no_hist(_("Choose a mountpoint: "));
+		input = rl_no_hist(_("Select a mountpoint (by ELN): "), 0);
 #endif /* HAVE_PROC_MOUNTS */
 		if (!input || !*input) {
 			free(input);
@@ -476,7 +484,7 @@ get_mnt_input(const int mode, int *info)
 			continue;
 		}
 
-		if (*input == 'q' && *(input + 1) == '\0') {
+		if (*input == 'q' && !input[1]) {
 			if (conf.autols == 1)
 				reload_dirlist();
 			free(input);
@@ -491,7 +499,7 @@ get_mnt_input(const int mode, int *info)
 
 		const int atoi_num = atoi(p);
 		if (atoi_num <= 0 || atoi_num > (int)mp_n) {
-			xerror("%s: %s: Invalid ELN\n", PROGRAM_NAME, input);
+			xerror(_("%s: %s: Invalid ELN\n"), PROGRAM_NAME, input);
 			free(input);
 			input = (char *)NULL;
 			continue;
@@ -538,9 +546,9 @@ xgetmntinfo_sun(void)
 		if (stat(mp, &a) == -1 || !S_ISDIR(a.st_mode))
 			continue;
 
-		const int perm = check_file_access(a.st_mode, a.st_uid, a.st_gid);
+		const char *dir_color = get_dir_color(mp, &a, -1);
 		printf("%s%zu%s %s%s%s [%s]\n", el_c, n + 1, df_c,
-			perm == 1 ? di_c : nd_c, mp, df_c, ent.mnt_special);
+			dir_color, mp, df_c, ent.mnt_special);
 
 		media = xnrealloc(media, n + 2, sizeof(struct mnt_t));
 		media[n].mnt = savestring(mp, strlen(mp));
@@ -581,7 +589,7 @@ media_menu(const int mode)
 
 	if (mode == MEDIA_MOUNT && xargs.mount_cmd == UNSET) {
 		xerror("%s\n", _("media: No mount application found. Install either "
-			"udevil or udisks2"));
+			"udevil or udisks2."));
 		return FUNC_FAILURE;
 	}
 
