@@ -1,17 +1,15 @@
-/* mime.c -- functions controlling Lira, the resource opener */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,46 +20,220 @@
  * MA 02110-1301, USA.
 */
 
-#ifndef _NO_LIRA
+/* mime.c -- functions controlling Lira, the file opener */
 
 #include "helpers.h"
 
-#include <errno.h>
-#include <stdio.h>
-#include <string.h>
-#include <unistd.h>
-#include <readline/tilde.h>
-
 #ifndef _NO_MAGIC
 # include <magic.h>
-#endif /* !_NO_MAGIC */
-
-#if defined(_NO_MAGIC)
+#else
 # if !defined(_BE_POSIX)
 #  include <paths.h>
 # endif /* !_BE_POSIX */
 # ifndef _PATH_DEVNULL
 #  define _PATH_DEVNULL "/dev/null"
 # endif /* _PATH_DEVNULL */
-#endif /* _NO_MAGIC */
+#endif /* !_NO_MAGIC */
 
-#ifndef _NO_ARCHIVING
-#include "archives.h"
-#endif /* !_NO_ARCHIVING */
-#include "aux.h"
-#include "checks.h"
-#include "config.h"
-#include "listing.h"
-#include "messages.h"
-#include "mime.h"
-#include "misc.h"
-#include "readline.h"
-#include "sanitize.h"
-#include "spawn.h"
+#ifndef _NO_LIRA
+# include <errno.h>
+# include <string.h>
+# include <unistd.h>
+# include <readline/tilde.h>
 
+# ifndef _NO_ARCHIVING
+#  include "archives.h"
+# endif /* !_NO_ARCHIVING */
+# include "aux.h"
+# include "checks.h"
+# include "config.h"
+# include "listing.h"
+# include "messages.h"
+# include "mime.h"
+# include "misc.h"
+# include "readline.h"
+# include "sanitize.h"
+# include "spawn.h"
+#else
+# include <string.h>
+# include <unistd.h>
+# include "aux.h" /* open_f* functions */
+# include "spawn.h" /* launch_execv() */
+#endif /* !_NO_LIRA */
+
+#ifndef _NO_LIRA
 static char *err_name = (char *)NULL;
-static int mime_match = 0;
+static int g_mime_match = 0;
+static char *g_mime_type = (char *)NULL;
+#endif /* !_NO_LIRA */
 
+/* Return the MIME type associated to the current file based on its extension.
+ * Associations are taken from ~/.mime.types (or $CLIFM_MIMETYPES_FILE) and
+ * stored in the user_mimetypes struct by load_user_mimetypes() (mimetypes.c). */
+static char *
+check_user_mimetypes(const char *file)
+{
+	char *ext = strrchr(file, '.');
+	if (!ext || ext == file || !*(++ext))
+		return (char *)NULL;
+
+	const size_t hash = hashme(ext, conf.case_sens_list);
+
+	static filesn_t n = 0;
+	if (n == 0)
+		for (n = 0; user_mimetypes[n].mimetype; n++);
+
+	filesn_t i = n;
+	while (--i >= 0)
+		/* An extension name starting with NUL byte is duplicated. Skip it. */
+		if (hash == user_mimetypes[i].ext_hash && *user_mimetypes[i].ext)
+			return user_mimetypes[i].mimetype;
+
+	return (char *)NULL;
+}
+
+#ifndef _NO_MAGIC
+/* Get FILE's type using the libmagic library.
+ * Return the MIME type if QUERY_MIME is set to 1, or a text description
+ * otherwise.
+ * NULL is returned in case of error. */
+char *
+xmagic(const char *file, const int query_mime)
+{
+	if (!file || !*file)
+		return (char *)NULL;
+
+	if (query_mime == 1 && user_mimetypes) {
+		const char *mime = check_user_mimetypes(file);
+		if (mime)
+			return strdup(mime);
+	}
+
+	magic_t cookie = magic_open(query_mime ? (MAGIC_MIME_TYPE | MAGIC_ERROR)
+		: MAGIC_ERROR);
+	if (!cookie)
+		return (char *)NULL;
+
+	magic_load(cookie, NULL);
+
+	const char *mime = magic_file(cookie, file);
+
+	char *str = mime ? savestring(mime, strlen(mime)) : (char *)NULL;
+	magic_close(cookie);
+
+	return str;
+}
+
+#else /* _NO_MAGIC */
+/* Get FILE's type using file(1).
+ * Return the MIME type if QUERY_MIME is set to 1, or a text description
+ * otherwise.
+ * NULL is returned in case of error. */
+char *
+xmagic(const char *file, const int query_mime)
+{
+	if (!file || !*file)
+		return (char *)NULL;
+
+	if (query_mime == 1 && user_mimetypes) {
+		const char *mime = check_user_mimetypes(file);
+		if (mime)
+			return strdup(mime);
+	}
+
+	char *mime_type = (char *)NULL;
+	char *rand_ext = gen_rand_str(RAND_SUFFIX_LEN);
+
+	char tmp_file[PATH_MAX + 1];
+	snprintf(tmp_file, sizeof(tmp_file), "%s/mime.%s", tmp_dir,
+		rand_ext ? rand_ext : "Hu?+6545jk");
+	free(rand_ext);
+
+	int fd = 0;
+	FILE *fp_out = open_fwrite(tmp_file, &fd);
+	if (!fp_out)
+		return (char *)NULL;
+
+	FILE *fp_err = fopen(_PATH_DEVNULL, "w");
+	if (!fp_err)
+		goto END;
+
+	int stdout_bk = dup(STDOUT_FILENO); /* Store original stdout */
+	int stderr_bk = dup(STDERR_FILENO); /* Store original stderr */
+
+	if (stdout_bk == -1 || stderr_bk == -1)
+		goto ERROR;
+
+	/* Redirect stdout to the desired file */
+	if (dup2(fileno(fp_out), STDOUT_FILENO) == -1)
+		goto ERROR;
+
+	/* Redirect stderr to /dev/null */
+	if (dup2(fileno(fp_err), STDERR_FILENO) == -1)
+		goto ERROR;
+
+	fclose(fp_out);
+	fclose(fp_err);
+
+/* --mime-type is only available since file 4.24 (Mar, 2008), while the -i
+ * flag (-I in MacOS) is supported since 3.30 (Apr, 2000).
+ * NOTE: the -i flag in the POSIX file(1) specification is a completely
+ * different thing. */
+#ifdef __APPLE__
+	char *cmd[] = {"file", query_mime ? "-bI" : "-b", (char *)file, NULL};
+#else
+	char *cmd[] = {"file", query_mime ? "-bi" : "-b", (char *)file, NULL};
+#endif /* __APPLE__ */
+	int ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
+
+	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
+	dup2(stderr_bk, STDERR_FILENO); /* Restore original stderr */
+	close(stdout_bk);
+	close(stderr_bk);
+
+	if (ret != FUNC_SUCCESS
+	|| (fp_out = fopen(tmp_file, "r")) == NULL) {
+		unlink(tmp_file);
+		return (char *)NULL;
+	}
+
+	/* According to the RFC-4288, both type and subtype of a MIME type cannot
+	 * be longer than 127 characters each, So adding the separating slash, we
+	 * get a max of 255 characters.
+	 * See https://datatracker.ietf.org/doc/html/rfc4288#section-4.2 */
+	char line[NAME_MAX + 1]; *line = '\0';
+	if (fgets(line, (int)sizeof(line), fp_out) == NULL)
+		goto END;
+
+	char *s = query_mime ? strrchr(line, ';') : (char *)NULL;
+	if (s)
+		*s = '\0';
+
+	size_t len = strlen(line);
+	if (len > 0 && line[len - 1] == '\n') {
+		line[len - 1] = '\0';
+		len--;
+	}
+
+	mime_type = len > 0 ? savestring(line, len) : (char *)NULL;
+
+END:
+	fclose(fp_out);
+	unlink(tmp_file);
+
+	return mime_type;
+
+ERROR:
+	fclose(fp_out);
+	fclose(fp_err);
+	unlink(tmp_file);
+	close(stdout_bk);
+	close(stderr_bk);
+	return (char *)NULL;
+}
+#endif /* !_NO_MAGIC */
+
+#ifndef _NO_LIRA
 /* Expand all environment variables in the string S.
  * Returns the expanded string or NULL on error. */
 static char *
@@ -123,14 +295,14 @@ skip_line_prefix(char *line)
 	char *p = line;
 
 	if (!(flags & GUI)) {
-		if (*p == 'X' && *(p + 1) == ':')
+		if (*p == 'X' && p[1] == ':')
 			return (char *)NULL;
-		if (*p == '!' && *(p + 1) == 'X' && *(p + 2) == ':')
+		if (*p == '!' && p[1] == 'X' && p[2] == ':')
 			p += 3;
 	} else {
-		if (*p == '!' && *(p + 1) == 'X')
+		if (*p == '!' && p[1] == 'X')
 			return (char *)NULL;
-		if (*p == 'X' && *(p + 1) == ':')
+		if (*p == 'X' && p[1] == ':')
 			p += 2;
 	}
 
@@ -173,7 +345,7 @@ test_pattern(const char *pattern, const char *filename, const char *mime)
 	regex_t regex;
 
 	if (filename && (*pattern == 'N' || *pattern == 'E')
-	&& *(pattern + 1) == ':') {
+	&& pattern[1] == ':') {
 		if (regcomp(&regex, pattern + 2,
 		REG_NOSUB | REG_EXTENDED | REG_ICASE) == 0
 		&& regexec(&regex, filename, 0, NULL, 0) == 0)
@@ -181,7 +353,7 @@ test_pattern(const char *pattern, const char *filename, const char *mime)
 	} else {
 		if (regcomp(&regex, pattern, REG_NOSUB | REG_EXTENDED) == 0
 		&& regexec(&regex, mime, 0, NULL, 0) == 0) {
-			mime_match = 1;
+			g_mime_match = 1;
 			ret = FUNC_SUCCESS;
 		}
 	}
@@ -191,9 +363,9 @@ test_pattern(const char *pattern, const char *filename, const char *mime)
 }
 
 /* Return 1 if APP is a valid and existent application (2 if it's located in
- * the home directory), or zero otherwise. */
+ * the home directory), or 0 otherwise. */
 static int
-check_app_existence(char **app, char **arg)
+check_app_existence(char **app, const char *params, const size_t params_len)
 {
 	if (*(*app) == 'a' && *(*app + 1) == 'd' && !*(*app + 2))
 		/* No need to check: 'ad' is an internal command. */
@@ -203,7 +375,7 @@ check_app_existence(char **app, char **arg)
 	if (*(*app) == '~' && *(*app + 1) == '/' && *(*app + 2)) {
 		size_t len = user.home_len + strlen(*app);
 		const size_t cmd_len = len > 0 ? len - 1 : 0;
-		len += *arg ? strlen(*arg) + 1 : 0;
+		len += params ? params_len + 1 : 0;
 
 		char *tmp_cmd = xnmalloc(len, sizeof(char));
 		snprintf(tmp_cmd, len, "%s/%s", user.home, *app + 2);
@@ -213,23 +385,18 @@ check_app_existence(char **app, char **arg)
 			return 0;
 		}
 
-		if (*arg) /* Append arguments */
-			snprintf(tmp_cmd + cmd_len, len, " %s", *arg);
+		if (params) { /* Append command paramters */
+			tmp_cmd[cmd_len] = ' ';
+			memcpy(tmp_cmd + cmd_len + 1, params, params_len + 1);
+		}
 
-		*app = xnrealloc(*app, len + 2, sizeof(char));
-		xstrsncpy(*app, tmp_cmd, len + 2);
-		free(tmp_cmd);
+		free(*app);
+		*app = tmp_cmd;
 		return 2;
 	}
 
 	/* Either a command name or an absolute path */
-	char *cmd_path = get_cmd_path(*app);
-	if (cmd_path) {
-		free(cmd_path);
-		return 1;
-	}
-
-	return 0;
+	return is_cmd_in_path(*app);
 }
 
 /* Return a copy the first cmd found in LINE (NULL terminated) or NULL.
@@ -245,7 +412,7 @@ get_cmd_from_line(char **line)
 
 	/* Get the first field in LINE */
 	while (*l != '\0' && *l != ';' && *l != '\n'
-	&& *l != '\'' && *l != '"' && len < PATH_MAX - 1) {
+	&& *l != '\'' && *l != '"' && len < sizeof(tmp) - 1) {
 		tmp[len] = *l;
 		len++;
 		l++;
@@ -254,10 +421,7 @@ get_cmd_from_line(char **line)
 	tmp[len] = '\0';
 	*line = l;
 
-	if (len == 0)
-		return (char *)NULL;
-
-	return savestring(tmp, len);
+	return len > 0 ? savestring(tmp, len) : NULL;
 }
 
 /* Return the first valid and existent opening application in LINE or NULL */
@@ -291,13 +455,13 @@ retrieve_app(char *line)
 		if (ret)
 			*ret = '\0';
 
-		const size_t param_len = (ret && *(ret + 1)) ? strlen(ret + 1) : 0;
+		const size_t param_len = (ret && ret[1]) ? strlen(ret + 1) : 0;
 
 		char *params = (char *)NULL;
 		if (*app == '~' && param_len > 0)
 			params = savestring(ret + 1, param_len);
 
-		int exists = check_app_existence(&app, &params);
+		const int exists = check_app_existence(&app, params, param_len);
 		free(params);
 
 		if (exists != 2 && ret) /* App not in HOME */
@@ -314,7 +478,7 @@ retrieve_app(char *line)
 	return (char *)NULL; /* No app was found */
 }
 
-/* Get application associated to a given MIME type or file name.
+/* Get application associated to a given MIME type or filename.
  * Returns the first matching line in the MIME file or NULL if none is
  * found. */
 static char *
@@ -345,7 +509,7 @@ get_app(const char *mime, const char *filename)
 		/* PATTERN points now to the beginning of the null terminated pattern,
 		 * while CMDS points to the beginning of the list of opening cmds. */
 
-		mime_match = 0;
+		g_mime_match = 0;
 		/* Global. Are we matching a MIME type? It will be set by test_pattern. */
 		if (test_pattern(pattern, filename, mime) == FUNC_FAILURE)
 			continue;
@@ -359,140 +523,8 @@ get_app(const char *mime, const char *filename)
 	return app;
 }
 
-#ifndef _NO_MAGIC
-/* Get FILE's MIME type using the libmagic library */
-char *
-xmagic(const char *file, const int query_mime)
-{
-	if (!file || !*file)
-		return (char *)NULL;
-
-	magic_t cookie = magic_open(query_mime ? (MAGIC_MIME_TYPE | MAGIC_ERROR)
-		: MAGIC_ERROR);
-	if (!cookie)
-		return (char *)NULL;
-
-	magic_load(cookie, NULL);
-
-	const char *mime = magic_file(cookie, file);
-	if (!mime) {
-		magic_close(cookie);
-		return (char *)NULL;
-	}
-
-	char *str = savestring(mime, strlen(mime));
-	magic_close(cookie);
-	return str;
-}
-
-#else /* _NO_MAGIC */
-static char *
-get_mime(char *file)
-{
-	if (!file || !*file) {
-		xerror("%s\n", _("Error opening temporary file"));
-		return (char *)NULL;
-	}
-
-	char *mime_type = (char *)NULL;
-	char *rand_ext = gen_rand_str(RAND_SUFFIX_LEN);
-
-	char tmp_file[PATH_MAX + 1];
-	snprintf(tmp_file, sizeof(tmp_file), "%s/mime.%s", tmp_dir,
-		rand_ext ? rand_ext : "Hu?+6545jk");
-	free(rand_ext);
-
-	int fd = 0;
-	FILE *fp_out = open_fwrite(tmp_file, &fd);
-	if (!fp_out) {
-		xerror("%s: fopen: '%s': %s\n", err_name, tmp_file, strerror(errno));
-		return (char *)NULL;
-	}
-
-	FILE *fp_err = fopen(_PATH_DEVNULL, "w");
-	if (!fp_err) {
-		xerror("%s: '%s': %s\n", err_name, _PATH_DEVNULL, strerror(errno));
-		goto END;
-	}
-
-	int stdout_bk = dup(STDOUT_FILENO); /* Store original stdout */
-	int stderr_bk = dup(STDERR_FILENO); /* Store original stderr */
-
-	if (stdout_bk == -1 || stderr_bk == -1)
-		goto ERROR;
-
-	/* Redirect stdout to the desired file */
-	if (dup2(fileno(fp_out), STDOUT_FILENO) == -1)
-		goto ERROR;
-
-	/* Redirect stderr to /dev/null */
-	if (dup2(fileno(fp_err), STDERR_FILENO) == -1)
-		goto ERROR;
-
-	fclose(fp_out);
-	fclose(fp_err);
-
-/* --mime-type is only available since file 4.24 (Mar, 2008), while the -i
- * flag (-I in MacOS) is supported since 3.30 (Apr, 2000).
- * NOTE: the -i flag in the POSIX file(1) specification is a completely
- * different thing. */
-#ifdef __APPLE__
-	char *cmd[] = {"file", "-bI", file, NULL};
-#else
-	char *cmd[] = {"file", "-bi", file, NULL};
-#endif /* __APPLE__ */
-	int ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
-
-	dup2(stdout_bk, STDOUT_FILENO); /* Restore original stdout */
-	dup2(stderr_bk, STDERR_FILENO); /* Restore original stderr */
-	close(stdout_bk);
-	close(stderr_bk);
-
-	if (ret != FUNC_SUCCESS
-	|| (fp_out = fopen(tmp_file, "r")) == NULL) {
-		unlink(tmp_file);
-		return (char *)NULL;
-	}
-
-	/* According to the RFC-4288, both type and subtype of a MIME type cannot
-	 * be longer than 127 characters each, So adding the separating slash, we
-	 * get a max of 255 characters.
-	 * See https://datatracker.ietf.org/doc/html/rfc4288#section-4.2 */
-	char line[NAME_MAX + 1]; *line = '\0';
-	if (fgets(line, (int)sizeof(line), fp_out) == NULL)
-		goto END;
-
-	char *s = strrchr(line, ';');
-	if (s)
-		*s = '\0';
-
-	size_t len = strlen(line);
-	if (len > 0 && line[len - 1] == '\n') {
-		line[len - 1] = '\0';
-		len--;
-	}
-
-	mime_type = len > 0 ? savestring(line, len) : (char *)NULL;
-
-END:
-	fclose(fp_out);
-	unlink(tmp_file);
-
-	return mime_type;
-
-ERROR:
-	xerror("%s: %s\n", err_name, strerror(errno));
-	fclose(fp_out);
-	fclose(fp_err);
-	unlink(tmp_file);
-	close(stdout_bk);
-	close(stderr_bk);
-	return (char *)NULL;
-}
-#endif /* !_NO_MAGIC */
-
-/* Import MIME associations from the system and store them into FILE.
- * Returns the amount of associations found, if any, or -1 in case of error
+/* Import MIME associations from the system and save them into FILE.
+ * Returns the number of associations found, if any, or -1 in case of error
  * or no association found */
 static int
 mime_import(char *file)
@@ -509,12 +541,12 @@ mime_import(char *file)
 
 	if (!(flags & GUI)) { /* Not in X, exit */
 		xerror(_("%s: Nothing was imported. No graphical "
-			"environment found\n"), err_name);
+			"environment found.\n"), err_name);
 		return (-1);
 	}
 
 	if (!user.home) {
-		xerror(_("%s: Error getting home directory\n"), err_name);
+		xerror(_("%s: Error getting the home directory\n"), err_name);
 		return (-1);
 	}
 
@@ -553,7 +585,7 @@ mime_import(char *file)
 	    "/etc/xdg/mimeapps.list",
 	    NULL};
 
-	/* Check each mimeapps.list file and store its associations into FILE */
+	/* Check each mimeapps.list file and save its associations into FILE */
 	size_t i;
 	int mime_defs = 0;
 
@@ -605,7 +637,7 @@ mime_import(char *file)
 
 	if (mime_defs == 0)
 		xerror(_("%s: Nothing was imported. No MIME association "
-			"found\n"), err_name);
+			"found.\n"), err_name);
 
 	fclose(mime_fp);
 	return mime_defs;
@@ -620,7 +652,7 @@ mime_edit(char **args)
 	}
 
 	if (!mime_file || !*mime_file) {
-		xerror("%s: The mimelist file name is undefined\n", err_name);
+		xerror("%s: The mimelist filename is undefined\n", err_name);
 		return FUNC_FAILURE;
 	}
 
@@ -658,7 +690,7 @@ mime_edit(char **args)
 
 	if (stat(mime_file, &a) != -1 && a.st_mtime != prev) {
 		reload_dirlist();
-		print_reload_msg(_(CONFIG_FILE_UPDATED));
+		print_reload_msg(NULL, NULL, _(CONFIG_FILE_UPDATED));
 	}
 
 	return exit_status;
@@ -676,14 +708,14 @@ get_basename(char *file_path)
 
 /* Get user input for the 'open with' function.
  * NUM is a pointer to an integer storing the item selected by the user.
- * MAX is the amount of available items. */
+ * MAX is the number of available items. */
 static int
 get_user_input(const size_t max)
 {
 	char *input = (char *)NULL;
 
 	while (!input) {
-		input = rl_no_hist(_("Choose an application ('q' to quit): "));
+		input = rl_no_hist(_("Select an application ('q' to quit): "), 0);
 		if (!input || !*input) {
 			free(input);
 			input = (char *)NULL;
@@ -720,11 +752,11 @@ set_exec_flags(const char *str, int *exec_flags)
 {
 	if (*str == 'E') {
 		*exec_flags |= E_NOSTDERR;
-		if (*(str + 1) == 'O')
+		if (str[1] == 'O')
 			*exec_flags |= E_NOSTDOUT;
 	} else if (*str == 'O') {
 		*exec_flags |= E_NOSTDOUT;
-		if (*(str + 1) == 'E')
+		if (str[1] == 'E')
 			*exec_flags |= E_NOSTDERR;
 	}
 }
@@ -737,8 +769,8 @@ copy_field(char **dst, const char *src)
 	xstrsncpy(*dst, src, src_len + 1);
 }
 
-/* Expand %[f|x] placeholder, stderr/stdout flags, and environment variables
- * in the opening application line. */
+/* Expand %[f|m|u|x] placeholders, stderr/stdout flags, and environment
+ * variables in the opening application line. */
 static size_t
 expand_app_fields(char ***cmd, size_t *n, char *fpath, int *exec_flags)
 {
@@ -753,18 +785,36 @@ expand_app_fields(char ***cmd, size_t *n, char *fpath, int *exec_flags)
 			copy_field(&a[i], fpath);
 			f = 1;
 			set_exec_flags("EO", exec_flags);
+			*exec_flags |= E_SETSID;
 			bg_proc = 1;
 			i++;
 			break;
 		}
 
-		/* Expand %f placeholder */
+		/* Expand %f placeholder to the file's absolute path */
 		if (*a[i] == '%' && a[i][1] == 'f') {
 			copy_field(&a[i], fpath);
 			f = 1;
 			continue;
 		}
 
+		/* Expand %m placeholder to the file's MIME type */
+		if (*a[i] == '%' && a[i][1] == 'm' && g_mime_type) {
+			copy_field(&a[i], g_mime_type);
+			continue;
+		}
+
+		/* Expand %u to the file URI for the original filename */
+		if (*a[i] == '%' && a[i][1] == 'u') {
+			char *p = url_encode(fpath, 1);
+			if (p) {
+				copy_field(&a[i], p);
+				free(p);
+				f = 1;
+			}
+			continue;
+		}
+
 		/* Set execution flags */
 		if (*a[i] == '!' && (a[i][1] == 'E' || a[i][1] == 'O')) {
 			set_exec_flags(a[i] + 1, exec_flags);
@@ -776,11 +826,10 @@ expand_app_fields(char ***cmd, size_t *n, char *fpath, int *exec_flags)
 		/* Expand environment variable */
 		if (*a[i] == '$' && a[i][1] >= 'A' && a[i][1] <= 'Z') {
 			char *p = expand_env(a[i]);
-			if (!p)
-				continue;
-
-			copy_field(&a[i], p);
-			free(p);
+			if (p) {
+				copy_field(&a[i], p);
+				free(p);
+			}
 			continue;
 		}
 
@@ -796,7 +845,7 @@ expand_app_fields(char ***cmd, size_t *n, char *fpath, int *exec_flags)
 	return f;
 }
 
-/* Open the file named FILE via the application APP, splitting APP and
+/* Open the file named FILE using the application APP, splitting APP and
  * expanding fields to the appropriate values. */
 static int
 run_mime_app(char *app, char *file)
@@ -809,7 +858,7 @@ run_mime_app(char *app, char *file)
 	size_t i = 0;
 	const size_t f = expand_app_fields(&cmd, &i, file, &exec_flags);
 
-	/* If no %f placeholder was found, append file name */
+	/* If no %f placeholder was found, append filename. */
 	if (f == 0) {
 		cmd = xnrealloc(cmd, i + 2, sizeof(char *));
 		cmd[i] = savestring(file, strlen(file));
@@ -826,7 +875,7 @@ run_mime_app(char *app, char *file)
 	return ret;
 }
 
-/* Open the file named FILE via the application APP.
+/* Open the file named FILE using the application APP.
  * No field expansion is made on APP, since it must be just an application
  * name. If expansion is required, use run_mime_app() instead. */
 static int
@@ -865,8 +914,8 @@ mime_list_open(char **apps, char *file)
 	for (i = 0; apps[i]; i++)
 		printf("%s%*zu%s %s\n", el_c, pad, i + 1, df_c, apps[i]);
 
-	int n = get_user_input(i);
-	if (n == -1) /* 'q' or Ctrl-d */
+	const int n = get_user_input(i);
+	if (n == -1) /* 'q' or Ctrl+d */
 		return FUNC_SUCCESS;
 
 	char *app = apps[n - 1];
@@ -877,7 +926,7 @@ mime_list_open(char **apps, char *file)
 
 	if (strchr(app, ' '))
 		ret = run_mime_app(app, file);
-	else /* We have just a command name: no parameter nor placeholder */
+	else /* We have just a command name: no parameter nor placeholder. */
 		ret = run_cmd(app, file);
 
 	return ret;
@@ -886,9 +935,7 @@ mime_list_open(char **apps, char *file)
 static int
 is_dup_entry(const char *prefix, char **apps, const char *app)
 {
-	size_t i;
-
-	for (i = prefix ? 1 : 0; apps && apps[i]; i++) {
+	for (size_t i = prefix ? 1 : 0; apps && apps[i]; i++) {
 		if (*app == *apps[i] && strcmp(app, apps[i]) == 0)
 			return 1;
 	}
@@ -898,8 +945,8 @@ is_dup_entry(const char *prefix, char **apps, const char *app)
 
 /* Return the list of opening apps for FILE_NAME, whose MIME type is MIME,
  * reading the file whose file pointer is FP.
- * If PREFIX is not NULL, we're TAB completing.
- * If ONLY_NAMES is 1, we're TAB completing for 'edit' subcommands (in which
+ * If PREFIX is not NULL, we're tab completing.
+ * If ONLY_NAMES is 1, we're tab completing for 'edit' subcommands (in which
  * case we want only command names, not parameters). */
 static char **
 get_apps_from_file(FILE *fp, char *file_name, const char *mime,
@@ -912,18 +959,18 @@ get_apps_from_file(FILE *fp, char *file_name, const char *mime,
 	size_t appsn = prefix != NULL ? 1 : 0;
 	const size_t prefix_len = prefix ? strlen(prefix) : 0;
 
-	char *base_name = get_basename(file_name);
+	const char *base_name = get_basename(file_name);
 
 	while (getline(&line, &line_size, fp) > 0) {
 		if (*line == '#' || *line == '[' || *line == '\n')
 			continue;
 
-		char *p = skip_line_prefix(line);
+		const char *p = skip_line_prefix(line);
 		if (!p)
 			continue;
 
 		char *tmp = strchr(p, '=');
-		if (!tmp || !*(tmp + 1))
+		if (!tmp || !tmp[1])
 			continue;
 
 		/* Truncate line in '=' to get only the ext/mimetype pattern/string */
@@ -1019,7 +1066,7 @@ get_apps_from_file(FILE *fp, char *file_name, const char *mime,
 				continue;
 			}
 
-			/* If the app exists, store it into the APPS array */
+			/* If the app exists, store it in the APPS array */
 			if (*app != '/') {
 				free(file_path);
 				file_path = (char *)NULL;
@@ -1101,12 +1148,7 @@ mime_open_with_tab(char *filename, const char *prefix, const int only_names)
 	if (!name)
 		return (char **)NULL;
 
-#ifndef _NO_MAGIC
 	char *mime = xmagic(name, MIME_TYPE);
-#else
-	err_name = "open";
-	char *mime = get_mime(name);
-#endif /* !_NO_MAGIC */
 	if (!mime) {
 		free(name);
 		return (char **)NULL;
@@ -1120,7 +1162,7 @@ mime_open_with_tab(char *filename, const char *prefix, const int only_names)
 	}
 
 	/* Do not let PREFIX be NULL, so that get_apps_from_file() knows
-	 * we're TAB completing. */
+	 * we're tab completing. */
 	char **apps = get_apps_from_file(fp, name, mime,
 		prefix ? prefix : "", only_names);
 
@@ -1195,12 +1237,13 @@ append_params(char **args, char *name, char ***cmd, int *exec_flags)
 			(*cmd)[n] = savestring(name, strlen(name));
 			f = 1;
 			set_exec_flags("EO", exec_flags);
+			*exec_flags |= E_SETSID;
 			bg_proc = 1;
 			n++;
 			break;
 		}
 
-		if (*args[i] == '%' && *(args[i] + 1) == 'f' && !*(args[i] + 2)) {
+		if (*args[i] == '%' && args[i][1] == 'f' && !args[i][2]) {
 			f = 1;
 			(*cmd)[n] = savestring(name, strlen(name));
 			n++;
@@ -1212,8 +1255,28 @@ append_params(char **args, char *name, char ***cmd, int *exec_flags)
 			continue;
 		}
 
-		if (*args[i] == '$' && *(args[i] + 1) >= 'A'
-		&& *(args[i] + 1) <= 'Z') {
+		/* Expand %m placeholder to the file's MIME type */
+		if (*args[i] == '%' && args[i][1] == 'm') {
+			char *mime = xmagic(name, MIME_TYPE);
+			if (mime) {
+				(*cmd)[n] = mime;
+				n++;
+			}
+			continue;
+		}
+
+		/* Expand %u to the file URI for the original filename */
+		if (*args[i] == '%' && args[i][1] == 'u') {
+			char *p = url_encode(name, 1);
+			if (p) {
+				(*cmd)[n] = p;
+				n++;
+				f = 1;
+			}
+			continue;
+		}
+
+		if (*args[i] == '$' && IS_ALPHA_UP(args[i][1])) {
 			char *env = expand_env(args[i]);
 			if (env) {
 				(*cmd)[n] = savestring(env, strlen(env));
@@ -1317,7 +1380,7 @@ mime_open_with(char *filename, char **args)
 
 	/* ow FILE APP [ARGS]
 	 * We already have the opening app. Just join the app, option
-	 * parameters, and file name, and execute the command. */
+	 * parameters, and filename, and execute the command. */
 	if (args && args[0]) {
 		const int ret = join_and_run(args, name);
 		free(name);
@@ -1325,12 +1388,8 @@ mime_open_with(char *filename, char **args)
 	}
 
 	/* Find out the appropriate opening application via either mime type
-	 * or file name. */
-#ifndef _NO_MAGIC
+	 * or filename. */
 	char *mime = xmagic(name, MIME_TYPE);
-#else
-	char *mime = get_mime(name);
-#endif /* !_NO_MAGIC */
 	if (!mime) {
 		xerror(_("%s: Error getting MIME type\n"), err_name);
 		goto FAIL;
@@ -1345,17 +1404,20 @@ mime_open_with(char *filename, char **args)
 	char **apps = get_apps_from_file(fp, name, mime, NULL, 0);
 
 	fclose(fp);
-	free(mime);
 
 	if (!apps) {
 		xerror(_("%s: No opening application found\n"
 			"Tip: Run 'APP FILE', or 'mm edit' to add an opening "
 			"application\n"), err_name);
 		free(name);
+		free(mime);
 		return FUNC_FAILURE;
 	}
 
+	g_mime_type = mime;
 	const int ret = mime_list_open(apps, name);
+	g_mime_type = (char *)NULL;
+	free(mime);
 
 	size_t i;
 	for (i = 0; apps[i]; i++)
@@ -1383,7 +1445,7 @@ mime_open_url(char *url)
 	err_name = (xargs.open == 1 || xargs.preview == 1) ? PROGRAM_NAME : "lira";
 
 	char *app = get_app("text/html", 0);
-	if (!app) /* The error message might not be printed by get_app(). Fix. */
+	if (!app) /* The error message may not be printed by get_app(). Fix. */
 		return FUNC_FAILURE;
 
 	char *p = strchr(app, ' ');
@@ -1409,8 +1471,8 @@ import_mime(void)
 	const int mime_defs = mime_import(new);
 	if (mime_defs > 0) {
 		printf(_("%d MIME association(s) imported from the system.\n"
-			"File stored as '%s'\nAdd these new associations to your mimelist "
-			"file running 'mm edit'.\n"), mime_defs, new);
+			"File saved as '%s'\nAdd these new associations to your mimelist "
+			"file by running 'mm edit'.\n"), mime_defs, new);
 		return FUNC_SUCCESS;
 	}
 
@@ -1418,7 +1480,7 @@ import_mime(void)
 }
 
 static int
-mime_info(char *arg, char **fpath, char **deq)
+check_mime_info_file(char *arg, char **fpath)
 {
 	if (!arg) {
 		fprintf(stderr, "%s\n", _(MIME_USAGE));
@@ -1426,10 +1488,9 @@ mime_info(char *arg, char **fpath, char **deq)
 	}
 
 	if (strchr(arg, '\\')) {
-		*deq = unescape_str(arg, 0);
-		*fpath = xrealpath(*deq, NULL);
-		free(*deq);
-		*deq = (char *)NULL;
+		char *deq = unescape_str(arg, 0);
+		*fpath = xrealpath(deq, NULL);
+		free(deq);
 	} else {
 		*fpath = xrealpath(arg, NULL);
 	}
@@ -1441,20 +1502,13 @@ mime_info(char *arg, char **fpath, char **deq)
 		return isnum == 1 ? FUNC_FAILURE : errno;
 	}
 
-	if (access(*fpath, R_OK) == -1) {
-		xerror("%s: '%s': %s\n", err_name, *fpath, strerror(errno));
-		free(*fpath);
-		*fpath = (char *)NULL;
-		return errno;
-	}
-
 	return FUNC_SUCCESS;
 }
 
 /* Get the full path of the file to be opened by mime
  * Returns 0 on success and 1 on error */
 static int
-get_open_file_path(char **args, char **fpath, char **deq)
+get_open_file_path(char **args, char **fpath)
 {
 	char *f = (char *)NULL;
 	if (*args[1] == 'o' && strcmp(args[1], "open") == 0 && args[2])
@@ -1462,12 +1516,11 @@ get_open_file_path(char **args, char **fpath, char **deq)
 	else
 		f = args[1];
 
-	/* Only dequote the file name if coming from the mime command */
+	/* Only dequote the filename if coming from the mime command. */
 	if (*args[0] == 'm' && strchr(f, '\\')) {
-		*deq = unescape_str(f, 0);
-		*fpath = xrealpath(*deq, NULL);
-		free(*deq);
-		*deq = (char *)NULL;
+		char *deq = unescape_str(f, 0);
+		*fpath = xrealpath(deq, NULL);
+		free(deq);
 	}
 
 	if (!*fpath) {
@@ -1488,8 +1541,12 @@ handle_no_app(const int info, char **fpath, char **mime, const char *arg)
 	if (xargs.preview == 1) {
 		/* When running the previewer, MIME_FILE points to the path to
 		 * preview.clifm file. */
-		xerror(_("shotgun: '%s': No associated application found\n"
-			"Fix this in the configuration file:\n%s\n"), arg, mime_file);
+		free(*fpath);
+		free(*mime);
+		xerror(_("%s: '%s': No associated application found\n"
+			"Fix this in the configuration file:\n%s\n"
+			"(run 'view edit' if running %s)\n"), PROGRAM_NAME,
+			arg, mime_file, PROGRAM_NAME);
 		return FUNC_FAILURE;
 	}
 
@@ -1531,23 +1588,46 @@ print_error_no_mime(char **fpath)
 }
 
 static void
-print_info_name_mime(char *filename, char *mime)
+print_info_name_mime(const char *filename, const char *mime)
 {
 	printf(_("Name: %s\n"), filename ? filename : _("None"));
-	printf(_("MIME type: %s\n"), mime);
+	printf(_("MIME type: %s\n"), mime ? mime : _("unknown"));
 }
 
 static int
 print_mime_info(char **app, char **fpath, char **mime)
 {
 	if (*(*app) == 'a' && (*app)[1] == 'd' && !(*app)[2]) {
-		printf(_("Associated application: ad [built-in] [%s]\n"),
-			mime_match ? "MIME" : "FILENAME");
+		printf(_("Opening application:    ad [builtin] [%s]\n"),
+			g_mime_match ? "MIME" : "FILENAME");
 	} else {
-		printf(_("Associated application: %s [%s]\n"), *app,
-			mime_match ? "MIME" : "FILENAME");
+		printf(_("Opening application:    '%s' [%s]\n"), *app,
+			g_mime_match ? "MIME" : "FILENAME");
 	}
 
+	if (!config_dir || !*config_dir)
+		goto END;
+
+	char *mime_file_ptr = mime_file;
+
+	char buf[PATH_MAX + 1];
+	if (alt_preview_file && *alt_preview_file) {
+		mime_file = alt_preview_file;
+	} else {
+		snprintf(buf, sizeof(buf), "%s/preview.clifm", config_dir);
+		mime_file = buf;
+	}
+
+	char *preview_app = get_app(*mime, *fpath);
+	printf(_("Previewing application: '%s' %s\n"),
+		(preview_app && *preview_app) ? preview_app : "None",
+		(preview_app && *preview_app)
+		? (g_mime_match ? "[MIME]" : "[FILENAME]") : "");
+
+	mime_file = mime_file_ptr;
+	free(preview_app);
+
+END:
 	free(*fpath);
 	free(*mime);
 	free(*app);
@@ -1557,35 +1637,19 @@ print_mime_info(char **app, char **fpath, char **mime)
 
 #ifndef _NO_ARCHIVING
 static int
-run_archiver(char **fpath, char **app)
+run_archiver(char **fpath, char **app, char **mime_type)
 {
 	char *cmd[] = {"ad", *fpath, NULL};
 	const int exit_status = archiver(cmd, 'd');
 
 	free(*fpath);
 	free(*app);
+	free(*mime_type);
 
 	return exit_status;
 }
 #endif /* _NO_ARCHIVING */
 
-#ifdef _NO_MAGIC
-/* Check the existence of the 'file' command */
-static int
-check_file_cmd(void)
-{
-	char *p = get_cmd_path("file");
-	if (!p) {
-		xerror("%s: Cannot retrieve MIME type: 'file' command "
-			"not found\n", err_name);
-		return FUNC_FAILURE;
-	}
-
-	free(p);
-	return FUNC_SUCCESS;
-}
-#endif /* _NO_MAGIC */
-
 static int
 print_mime_help(void)
 {
@@ -1612,20 +1676,17 @@ mime_open(char **args)
 		return mime_edit(args);
 
 	char *file_path = (char *)NULL;
-	char *deq_file = (char *)NULL;
-	int info = 0, file_index = 0;
+	const int info = (*args[1] == 'i' && strcmp(args[1], "info") == 0);
+	const int file_index = info == 1 ? 2 : 1;
 
-	if (*args[1] == 'i' && strcmp(args[1], "info") == 0) {
-		const int ret = mime_info(args[2], &file_path, &deq_file);
+	if (info == 1) {
+		const int ret = check_mime_info_file(args[2], &file_path);
 		if (ret != FUNC_SUCCESS)
 			return ret;
-		info = 1;
-		file_index = 2;
 	} else {
-		const int ret = get_open_file_path(args, &file_path, &deq_file);
+		const int ret = get_open_file_path(args, &file_path);
 		if (ret != FUNC_SUCCESS)
 			return ret;
-		file_index = 1;
 	}
 
 	if (!file_path) {
@@ -1634,20 +1695,11 @@ mime_open(char **args)
 	}
 
 	/* Get file's MIME type */
-#ifndef _NO_MAGIC
 	char *mime = xmagic(file_path, MIME_TYPE);
-#else
-	if (check_file_cmd() == FUNC_FAILURE) {
-		free(file_path);
-		return FUNC_FAILURE;
-	}
-	char *mime = get_mime(file_path);
-#endif /* !_NO_MAGIC */
-
 	if (!mime)
 		return print_error_no_mime(&file_path);
 
-	char *filename = get_basename(file_path);
+	const char *filename = get_basename(file_path);
 
 	if (info == 1)
 		print_info_name_mime(filename, mime);
@@ -1660,32 +1712,30 @@ mime_open(char **args)
 	if (info == 1)
 		return print_mime_info(&app, &file_path, &mime);
 
-	free(mime);
-
 	/* Construct and execute the command */
 #ifndef _NO_ARCHIVING
 	if (*app == 'a' && app[1] == 'd' && !app[2])
-		return run_archiver(&file_path, &app);
+		return run_archiver(&file_path, &app, &mime);
 #endif /* !_NO_ARCHIVING */
 
-	int ret = 0;
+	g_mime_type = mime;
 #ifdef __CYGWIN__
 	/* Some Windows programs, like Word and Powerpoint (but not Excel!!), do
-	 * not like absolute paths when the file name contains spaces. So, let's
-	 * pass the file name as it was passed to this function, without
+	 * not like absolute paths when the filename contains spaces. So, let's
+	 * pass the filename as it was passed to this function, without
 	 * expanding it to an absolute path.
 	 * This hack must be removed as soon as the real cause is discovered:
-	 * why Word/Powerpoint fails to open absolute paths when the file name
+	 * why Word/Powerpoint fails to open absolute paths when the filename
 	 * contains spaces? */
-	ret = run_mime_app(app, args[file_index]);
+	const int ret = run_mime_app(app, args[file_index]);
 #else
-	ret = run_mime_app(app, file_path);
+	const int ret = run_mime_app(app, file_path);
 #endif /* __CYGWIN__ */
 
+	free(mime);
+	g_mime_type = (char *)NULL;
 	free(app);
 	free(file_path);
 	return ret;
 }
-#else
-void *_skip_me_lira;
 #endif /* !_NO_LIRA */
