@@ -1,17 +1,15 @@
-/* misc.c -- functions that do not fit in any other file */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* misc.c -- functions that do not fit in any other file */
+
 /* The err function is based on littlstar's asprintf implementation
  * (https://github.com/littlstar/asprintf.c/blob/master/asprintf.c),
  * licensed under MIT.
@@ -32,7 +32,6 @@
 #include <errno.h>
 #include <signal.h>
 #include <stdarg.h>
-#include <stdio.h>
 #include <string.h>
 #include <sys/ioctl.h>
 #include <termios.h> /* tcsetattr */
@@ -60,6 +59,7 @@ typedef char *rl_cpvfunc_t;
 #endif /* LINUX_INOTIFY */
 
 #include "aux.h"
+#include "autocmds.h" /* update_autocmd_opts() */
 #include "bookmarks.h"
 #include "checks.h"
 #include "file_operations.h"
@@ -116,7 +116,8 @@ is_blank_name(const char *s)
 char *
 get_newname(const char *msg, char *old_name, int *quoted)
 {
-	alt_prompt = rl_nohist = 1;
+	rl_nohist = 1;
+	alt_prompt = FILES_PROMPT;
 	int poffset_bk = prompt_offset;
 	prompt_offset = 3;
 
@@ -161,7 +162,7 @@ END:
 	return new_name;
 }
 
-/* Set ELN color according to the current workspace */
+/* Set ELN color according to the current workspace. */
 void
 set_eln_color(void)
 {
@@ -185,7 +186,7 @@ set_eln_color(void)
 		return;
 	}
 
-	/* Remove leading and trainling control characters (\001 and \002)*/
+	/* Remove leading and trailing control characters (\001 and \002). */
 	size_t cl_len = 0;
 	if (*cl == 001) {
 		cl++;
@@ -210,7 +211,7 @@ set_eln_color(void)
  * to the previous one, without printing any message mark.
  * MSG_TYPE also accepts ERR_NO_LOG (-1) and ERR_NO_STORE (-2) as values:
  * ERR_NO_LOG: Print the message but do not log it.
- * ERR_NO_STORE: Log but do not store the message into the messages array.
+ * ERR_NO_STORE: Log but do not store the message in the messages array.
  *
  * PROMPT_FLAG tells whether to print the message immediately before the next
  * prompt or rather in place.
@@ -244,8 +245,8 @@ err(const int msg_type, const int prompt_flag, const char *format, ...)
 		{free(buf); goto ERROR;}
 
 	/* If the new message is the same as the last message, skip it. */
-	if (msgs_n > 0 && msg_type != 'f' && *messages[msgs_n - 1] == *buf
-	&& strcmp(messages[msgs_n - 1], buf) == 0)
+	if (msgs_n > 0 && msg_type != 'f' && *messages[msgs_n - 1].text == *buf
+	&& strcmp(messages[msgs_n - 1].text, buf) == 0)
 		{free(buf); goto ERROR;}
 
 	if (msg_type >= 'e') {
@@ -263,6 +264,11 @@ err(const int msg_type, const int prompt_flag, const char *format, ...)
 	if (msg_type == ERR_NO_STORE) {
 		add_to_msgs_list = 0;
 		logme = 1;
+/*		if (prompt_flag == NOPRINT_PROMPT && term_caps.unicode == 1) {
+			// Invoked as xerror()
+			printf("%s%s%s ", xf_c, ERROR_PTR_STR_U, df_c);
+			fflush(stdout);
+		} */
 	}
 
 	log_msg(buf, prompt_flag, logme, add_to_msgs_list);
@@ -278,11 +284,13 @@ ERROR:
 
 /* Print format string MSG, as "> MSG" (colored), if autols is on, or just
  * as "MSG" if off.
+ * Use PTR as pointer, or ">" if PTR is NULL.
+ * Use COLOR as pointer color, or mi_c if COLOR is NULL.
  * This function is used to inform the user about changes that require a
- * a files list reload (either upon files or interface modifications). */
-__attribute__((__format__(__printf__, 1, 0)))
+ * a file list reload (either upon files or interface modifications). */
+__attribute__((__format__(__printf__, 3, 4)))
 int
-print_reload_msg(const char *msg, ...)
+print_reload_msg(const char *ptr, const char *color, const char *msg, ...)
 {
 	va_list arglist;
 
@@ -294,7 +302,8 @@ print_reload_msg(const char *msg, ...)
 		return FUNC_FAILURE;
 
 	if (conf.autols == 1)
-		printf("%s%s%s ", mi_c, MSG_PTR_STR, df_c);
+		printf("%s%s%s ", color ? color : mi_c,
+			ptr ? ptr : SET_MSG_PTR, df_c);
 
 	char *buf = xnmalloc((size_t)size + 1, sizeof(char));
 
@@ -352,7 +361,7 @@ read_inotify(void)
 	char inotify_buf[EVENT_BUF_LEN];
 
 	memset((void *)inotify_buf, '\0', sizeof(inotify_buf));
-	i = (int)read(inotify_fd, inotify_buf, sizeof(inotify_buf));
+	i = (int)read(inotify_fd, inotify_buf, sizeof(inotify_buf)); /* flawfinder: ignore */
 
 	if (i <= 0) {
 # ifdef INOTIFY_DEBUG
@@ -404,7 +413,7 @@ read_inotify(void)
 					break;
 			}
 
-			/* If destiny file name is already in the files list (j >= 0),
+			/* If destiny filename is already in the file list (j >= 0),
 			 * ignore this event. */
 			ignore_event = (j < 0) ? 0 : 1;
 		}
@@ -483,19 +492,6 @@ read_kqueue(void)
 }
 #endif /* LINUX_INOTIFY */
 
-void
-set_term_title(char *str)
-{
-	int free_tmp = 0;
-	char *tmp = home_tilde(str, &free_tmp);
-
-	printf("\033]2;%s - %s\007", PROGRAM_NAME, tmp ? tmp : str);
-	fflush(stdout);
-
-	if (free_tmp == 1)
-		free(tmp);
-}
-
 void
 set_filter_type(const char c)
 {
@@ -524,7 +520,7 @@ unset_filter(void)
 	if (conf.autols == 1)
 		reload_dirlist();
 
-	print_reload_msg(_("Filter unset\n"));
+	print_reload_msg(NULL, NULL, _("Filter unset\n"));
 	return FUNC_SUCCESS;
 }
 
@@ -532,19 +528,23 @@ static int
 validate_file_type_filter(void)
 {
 	if (!filter.str || !*filter.str || *filter.str != '='
-	|| !*(filter.str + 1) || *(filter.str + 2))
+	|| !filter.str[1] || filter.str[2])
 		return FUNC_FAILURE;
 
-	const char c = *(filter.str + 1);
+	const char c = filter.str[1];
 	if (c == 'b' || c == 'c' || c == 'd' || c == 'f'
+#ifdef SOLARIS_DOORS
+	|| c == 'l' || c == 'p' || c == 's' || c == 'O' || c == 'P')
+#else
 	|| c == 'l' || c == 'p' || c == 's')
+#endif /* SOLARIS_DOORS */
 		return FUNC_SUCCESS;
 
 	if (conf.light_mode == 1)
 		return FUNC_FAILURE;
 
 	if (c == 'g' || c == 'h' || c == 'o' || c == 't'
-	|| c == 'u' || c == 'x')
+	|| c == 'u' || c == 'x' || c == 'D' || c == 'F' || c == 'L')
 		return FUNC_SUCCESS;
 
 	return FUNC_FAILURE;
@@ -574,9 +574,6 @@ compile_filter(void)
 	if (conf.autols == 1)
 		reload_dirlist();
 
-	print_reload_msg(_("%s%s: New filter set\n"),
-		filter.rev == 1 ? "!" : "", filter.str);
-
 	return FUNC_SUCCESS;
 
 ERR:
@@ -591,7 +588,7 @@ filter_function(char *arg)
 {
 	if (!arg) {
 		printf(_("Current filter: %c%s\n"), filter.rev == 1 ? '!' : 0,
-			filter.str ? filter.str : "none");
+			filter.str ? filter.str : _("none"));
 		return FUNC_SUCCESS;
 	}
 
@@ -600,8 +597,11 @@ filter_function(char *arg)
 		return FUNC_SUCCESS;
 	}
 
-	if (*arg == 'u' && strcmp(arg, "unset") == 0)
-		return unset_filter();
+	if (*arg == 'u' && strcmp(arg, "unset") == 0) {
+		const int ret = unset_filter();
+		update_autocmd_opts(AC_FILTER);
+		return ret;
+	}
 
 	free(filter.str);
 	regfree(&regex_exp);
@@ -624,32 +624,48 @@ filter_function(char *arg)
 
 	set_filter_type(*p);
 	filter.str = savestring(p, strlen(p));
+	update_autocmd_opts(AC_FILTER);
 
 	return compile_filter();
 }
 
 /* Check whether the conditions to run the new_instance function are
  * fulfilled */
-static inline int
+static int
 check_new_instance_init_conditions(void)
 {
-	if (!conf.term) {
-		xerror(_("%s: Default terminal not set. Use the "
-			"configuration file (F10) to set it\n"), PROGRAM_NAME);
-		return FUNC_FAILURE;
-	}
-
 	if (!(flags & GUI)) {
 		xerror(_("%s: Function only available for graphical "
 			"environments\n"), PROGRAM_NAME);
 		return FUNC_FAILURE;
 	}
 
+	if (!conf.term || !*conf.term) {
+		xerror(_("%s: Default terminal not set. Use the "
+			"configuration file (F10) to set it.\n"), PROGRAM_NAME);
+		return FUNC_FAILURE;
+	}
+
+	/* Check command existence. */
+	char *s = strchr(conf.term, ' ');
+	if (s)
+		*s = '\0';
+
+	if (is_cmd_in_path(conf.term) == 0) {
+		xerror("%s: %s: %s\n", PROGRAM_NAME, conf.term, NOTFOUND_MSG);
+		if (s)
+			*s = ' ';
+		return E_NOTFOUND;
+	}
+
+	if (s)
+		*s = ' ';
+
 	return FUNC_SUCCESS;
 }
 
-/* Just check that DIR exists and is a directory */
-static inline int
+/* Just check if DIR exists and it is a directory. */
+static int
 check_dir(char **dir)
 {
 	int ret = FUNC_SUCCESS;
@@ -668,7 +684,7 @@ check_dir(char **dir)
 }
 
 /* Construct absolute path for DIR */
-static inline char *
+static char *
 get_path_dir(char **dir)
 {
 	char *path_dir = (char *)NULL;
@@ -751,13 +767,13 @@ confirm_sudo_cmd(char **cmd)
 	if (i > 0)
 		putchar('\n');
 
-	return rl_get_y_or_n(_("Run command? [y/n] "));
+	return rl_get_y_or_n(_("Run command?"), 0);
 }
 
 /* Launch a new instance using CMD. If CMD is NULL, try "CONF.TERM clifm".
  * Returns the exit status of the executed command. */
 static int
-launch_new_instance_cmd(char ***cmd, char **self, char **_sudo,
+launch_new_instance_cmd(char ***cmd, char **self, char **sudo_prog,
 	char **dir, int sudo)
 {
 	int ret = 0;
@@ -765,34 +781,29 @@ launch_new_instance_cmd(char ***cmd, char **self, char **_sudo,
 	sudo = 0;
 #endif /* __HAIKU__ */
 
-//	setenv("CLIFM_OWN_CHILD", "1", 1);
-
 	if (*cmd) {
 		ret = (sudo == 0 || confirm_sudo_cmd(*cmd) == 1)
-			? launch_execv(*cmd, BACKGROUND, E_NOFLAG)
+			? launch_execv(*cmd, BACKGROUND, E_SETSID)
 			: FUNC_SUCCESS;
-		size_t i;
-		for (i = 0; (*cmd)[i]; i++)
+		for (size_t i = 0; (*cmd)[i]; i++)
 			free((*cmd)[i]);
 		free(*cmd);
 	} else {
 		if (sudo == 1) {
-			char *tcmd[] = {conf.term, *_sudo, *self, *dir, NULL};
+			char *tcmd[] = {conf.term, *sudo_prog, *self, *dir, NULL};
 			ret = (confirm_sudo_cmd(tcmd) == 1)
-				? launch_execv(tcmd, BACKGROUND, E_NOFLAG)
+				? launch_execv(tcmd, BACKGROUND, E_SETSID)
 				: FUNC_SUCCESS;
 		} else {
 			char *tcmd[] = {conf.term, *self, *dir, NULL};
-			ret = launch_execv(tcmd, BACKGROUND, E_NOFLAG);
+			ret = launch_execv(tcmd, BACKGROUND, E_SETSID);
 		}
 	}
 
-	free(*_sudo);
+	free(*sudo_prog);
 	free(*self);
 	free(*dir);
 
-//	unsetenv("CLIFM_OWN_CHILD");
-
 	return ret;
 }
 
@@ -802,43 +813,48 @@ launch_new_instance_cmd(char ***cmd, char **self, char **_sudo,
 /* Open DIR in a new instance of the program (using TERM, set in the config
  * file, as terminal emulator). */
 int
-new_instance(char *dir, const int sudo)
+new_instance(char *dir, int sudo)
 {
-	if (check_new_instance_init_conditions() == FUNC_FAILURE)
-		return FUNC_FAILURE;
+	int ret = check_new_instance_init_conditions();
+	if (ret != FUNC_SUCCESS)
+		return ret;
 
 	if (!dir)
 		return EINVAL;
 
-	char *_sudo = (char *)NULL;
+	/* Do not run with sudo if already root */
+	if (user.uid == 0)
+		sudo = 0;
+
+	char *sudo_prog = (char *)NULL;
 #ifndef __HAIKU__
-	if (sudo == 1 && !(_sudo = get_sudo_path()))
+	if (sudo == 1 && !(sudo_prog = get_sudo_path()))
 		return errno;
 #endif /* !__HAIKU__ */
 
 	char *deq_dir = unescape_str(dir, 0);
 	if (!deq_dir) {
-		free(_sudo);
-		xerror(_("%s: '%s': Cannot escape file name\n"), PROGRAM_NAME, dir);
+		free(sudo_prog);
+		xerror(_("%s: '%s': Cannot escape filename\n"), PROGRAM_NAME, dir);
 		return FUNC_FAILURE;
 	}
 
 	char *self = get_cmd_path(PROGRAM_NAME);
 	if (!self) {
-		free(_sudo); free(deq_dir);
+		free(sudo_prog); free(deq_dir);
 		xerror("%s: %s: %s\n", PROGRAM_NAME, PROGRAM_NAME, strerror(errno));
 		return errno;
 	}
 
-	const int ret = check_dir(&deq_dir);
+	ret = check_dir(&deq_dir);
 	if (ret != FUNC_SUCCESS) {
-		free(deq_dir); free(self); free(_sudo);
+		free(deq_dir); free(self); free(sudo_prog);
 		return ret;
 	}
 
 	char *path_dir = get_path_dir(&deq_dir);
-	char **cmd = get_cmd(path_dir, _sudo, self, sudo);
-	return launch_new_instance_cmd(&cmd, &self, &_sudo, &path_dir, sudo);
+	char **cmd = get_cmd(path_dir, sudo_prog, self, sudo);
+	return launch_new_instance_cmd(&cmd, &self, &sudo_prog, &path_dir, sudo);
 }
 
 /* Import (copy to main config file) aliases from the file named FILE.
@@ -856,7 +872,7 @@ alias_import(char *file)
 
 	char *npath = normalize_path(file, strlen(file));
 	if (!npath) {
-		xerror(_("alias: '%s': Error normalizing file name\n"), file);
+		xerror(_("alias: '%s': Error normalizing filename\n"), file);
 		return FUNC_FAILURE;
 	}
 
@@ -872,7 +888,7 @@ alias_import(char *file)
 		return errno;
 	}
 
-	/* Open CliFM's config file as well. */
+	/* Open clifm's config file as well. */
 	FILE *config_fp = open_fappend(config_file);
 	if (!config_fp) {
 		xerror("alias: '%s': %s\n", config_file, strerror(errno));
@@ -896,7 +912,7 @@ alias_import(char *file)
 		if (!alias_name)
 			continue;
 
-		if (is_internal_c(alias_name)) {
+		if (is_internal_cmd(alias_name, ALL_CMDS, 1, 1)) {
 			xerror(_("'%s': Alias conflicts with internal "
 				"command\n"), alias_name);
 			free(alias_name);
@@ -967,7 +983,7 @@ alias_import(char *file)
 	/* Add new aliases to the internal list of aliases. */
 	get_aliases();
 
-	/* Add new aliases to the commands list for TAB completion. */
+	/* Add new aliases to the commands list for tab completion. */
 	if (bin_commands) {
 		for (i = 0; bin_commands[i]; i++)
 			free(bin_commands[i]);
@@ -996,9 +1012,8 @@ parse_usrvar_value(const char *str, const char c)
 
 	/* Remove trailing spaces, tabs, new line chars, and quotes */
 	const size_t tmp_len = strlen(tmp);
-	size_t i;
 
-	for (i = tmp_len - 1; tmp[i] && i > 0; i--) {
+	for (size_t i = tmp_len - 1; tmp[i] && i > 0; i--) {
 		if (tmp[i] != ' ' && tmp[i] != '\t' && tmp[i] != '"' && tmp[i] != '\''
 		&& tmp[i] != '\n')
 			break;
@@ -1050,15 +1065,58 @@ create_usr_var(const char *str)
 	return FUNC_SUCCESS;
 }
 
-void
-free_autocmds(void)
+/*
+// Resize the autocmds array leaving only temporary autocommands.
+static void
+keep_temp_autocmds(void)
 {
+	size_t i;
+	size_t c = 0;
+	struct autocmds_t *a = (struct autocmds_t *)NULL;
+
+	for (i = 0; i < autocmds_n; i++) {
+		if (autocmds[i].temp == 1) {
+			a = xnrealloc(a, c + 2, sizeof(struct autocmds_t));
+			memmove(a + c, autocmds + i, sizeof(struct autocmds_t));
+			c++;
+			a[c] = (struct autocmds_t){0};
+		}
+
+		free(autocmds[i].pattern);
+		free(autocmds[i].cmd);
+		free(autocmds[i].filter.str);
+		autocmds[i] = (struct autocmds_t){0};
+	}
+
+	autocmds_n = c;
+	free(autocmds);
+	autocmds = a;
+//	autocmds = xnrealloc(autocmds, autocmds_n, sizeof(struct autocmds_t));
+} */
+
+void
+free_autocmds(const int keep_temp)
+{
+	UNUSED(keep_temp);
+/*	int i = (int)autocmds_n;
+
+	if (keep_temp == 1) {
+		while (--i >= 0) {
+			if (autocmds[i].temp == 1) {
+				keep_temp_autocmds();
+				return;
+			}
+		}
+	} */
+
 	int i = (int)autocmds_n;
 	while (--i >= 0) {
 		free(autocmds[i].pattern);
 		free(autocmds[i].cmd);
+		free(autocmds[i].filter.str);
 		autocmds[i].color_scheme = (char *)NULL;
 	}
+
 	free(autocmds);
 	autocmds = (struct autocmds_t *)NULL;
 	autocmds_n = 0;
@@ -1082,8 +1140,7 @@ free_remotes(const int exit)
 	if (exit)
 		autounmount_remotes();
 
-	size_t i;
-	for (i = 0; i < remotes_n; i++) {
+	for (size_t i = 0; i < remotes_n; i++) {
 		free(remotes[i].name);
 		free(remotes[i].desc);
 		free(remotes[i].mountpoint);
@@ -1211,8 +1268,7 @@ xrl_discard_keymap(Keymap map)
 void
 free_workspaces_filters(void)
 {
-	size_t i;
-	for (i = 0; i < MAX_WS; i++) {
+	for (size_t i = 0; i < MAX_WS; i++) {
 		free(workspace_opts[i].filter.str);
 		workspace_opts[i].filter.str = (char *)NULL;
 		workspace_opts[i].filter.rev = 0;
@@ -1277,8 +1333,8 @@ handle_last_path(void)
 	 * directory specifcied in ~/.config/clifm/.last if this file is found (it
 	 * is actually a symlink to ~/.config/clifm/profiles/PROFILE/.last).
 	 * Remove the file to prevent the function from changing the directory
-	 * if cd-on-quit is disabled (e.g., not exiting via 'Q'). If necessary,
-	 * it will be recreated by save_last_path() below. */
+	 * if cd-on-quit is disabled. If necessary, it will be recreated by}
+	 * save_last_path() below. */
 	const size_t len = strlen(config_dir_gral) + 7;
 	char *last_path_tmp = xnmalloc(len, sizeof(char));
 	snprintf(last_path_tmp, len, "%s/.last", config_dir_gral);
@@ -1294,6 +1350,20 @@ handle_last_path(void)
 	free(last_path_tmp);
 }
 
+static void
+free_file_templates(void)
+{
+	if (!file_templates)
+		return;
+
+	filesn_t i;
+	for (i = 0; file_templates[i]; i++)
+		free(file_templates[i]);
+
+	free(file_templates);
+	file_templates = (char **)NULL;
+}
+
 /* This function is called by atexit() to clear whatever is there at exit
  * time and avoid thus memory leaks */
 void
@@ -1306,9 +1376,18 @@ free_stuff(void)
 	free(alt_config_file);
 	free(alt_bm_file);
 	free(alt_kbinds_file);
+	free(alt_mimelist_file);
 	free(alt_preview_file);
 	free(alt_profile);
 
+	if (user_mimetypes) {
+		for (i = 0; user_mimetypes[i].mimetype; i++) {
+			free(user_mimetypes[i].ext);
+			free(user_mimetypes[i].mimetype);
+		}
+		free(user_mimetypes);
+	}
+
 	if (sys_users) {
 		for (i = 0; sys_users[i].name; i++)
 			free(sys_users[i].name);
@@ -1340,6 +1419,7 @@ free_stuff(void)
 
 	free(conf.time_str);
 	free(conf.ptime_str);
+	free(conf.priority_sort_char);
 
 #ifdef LINUX_INOTIFY
 	/* Shutdown inotify */
@@ -1356,19 +1436,18 @@ free_stuff(void)
 
 	free_prompts();
 	free(prompts_file);
-	free_autocmds();
+	free_autocmds(0);
 	free_tags();
 	free_remotes(1);
+	free_file_templates();
 
 	if (xargs.stealth_mode != 1)
 		save_jumpdb();
 
 	handle_last_path();
 
-	free(bin_name);
 	free_bookmarks();
 	free(conf.encoded_prompt);
-/*	free(right_prompt); */
 	free_dirlist();
 	free(conf.opener);
 	free(conf.rprompt_str);
@@ -1391,8 +1470,7 @@ free_stuff(void)
 		free(jump_db);
 	}
 
-	if (pinned_dir)
-		free(pinned_dir);
+	free(pinned_dir);
 
 //	ADD FILTER TYPE CHECK!
 	if (filter.str) {
@@ -1494,10 +1572,10 @@ free_stuff(void)
 		free(prompt_cmds[i]);
 	free(prompt_cmds);
 
-	if (msgs_n) {
+	if (msgs_n > 0) {
 		i = (int)msgs_n;
 		while (--i >= 0)
-			free(messages[i]);
+			free(messages[i].text);
 		free(messages);
 	}
 
@@ -1513,10 +1591,8 @@ free_stuff(void)
 	if (workspaces && workspaces[0].path) {
 		i = MAX_WS;
 		while (--i >= 0) {
-			if (workspaces[i].path)
-				free(workspaces[i].path);
-			if (workspaces[i].name)
-				free(workspaces[i].name);
+			free(workspaces[i].path);
+			free(workspaces[i].name);
 		}
 		free(workspaces);
 	}
@@ -1545,6 +1621,8 @@ free_stuff(void)
 #endif /* !_NO_SUGGESTIONS */
 
 	free(sel_file);
+	free(templates_dir);
+	free(thumbnails_dir);
 	free(tmp_rootdir);
 	free(tmp_dir);
 	free(user.name);
@@ -1582,11 +1660,8 @@ free_stuff(void)
 	if (conf.colorize == 1 && xargs.list_and_quit != 1)
 		RESTORE_COLOR;
 
-	const int ret = restore_shell();
-	if (ret < 0) {
-		fprintf(stderr, "%s: tcsetattr: %s\n", PROGRAM_NAME, strerror(ret));
-		exit(ret);
-	}
+	if (xargs.kitty_keys == 1)
+		UNSET_KITTY_KEYS;
 }
 
 /* Get current terminal dimensions and store them in TERM_COLS and
@@ -1618,41 +1693,16 @@ get_term_size(void)
 		term_lines = (unsigned short)value;
 }
 
-#ifndef _BE_POSIX
-/* Get new window size and update/refresh the screen accordingly */
-static void
-sigwinch_handler(int sig)
-{
-	UNUSED(sig);
-	if (xargs.refresh_on_resize == 0 || conf.pager == 1 || kbind_busy == 1)
-		return;
-
-	get_term_size();
-	flags |= DELAYED_REFRESH;
-}
-#endif /* !_BE_POSIX */
-
-void
-set_signals_to_ignore(void)
-{
-	signal(SIGINT, SIG_IGN);  /* C-c */
-	signal(SIGQUIT, SIG_IGN); /* C-\ */
-	signal(SIGTSTP, SIG_IGN); /* C-z */
-#ifndef _BE_POSIX
-	signal(SIGWINCH, sigwinch_handler);
-#endif /* !_BE_POSIX */
-}
-
 static int
 create_virtual_dir(const int user_provided)
 {
 	if (!stdin_tmp_dir || !*stdin_tmp_dir) {
 		if (user_provided == 1) {
-			err('e', PRINT_PROMPT, "%s: Empty buffer for virtual "
-				"directory name. Trying with default value\n", PROGRAM_NAME);
+			err('e', PRINT_PROMPT, _("%s: Empty buffer for virtual "
+				"directory name. Trying with default value\n"), PROGRAM_NAME);
 		} else {
-			err('e', PRINT_PROMPT, "%s: Empty buffer for virtual "
-				"directory name\n", PROGRAM_NAME);
+			err('e', PRINT_PROMPT, _("%s: Empty buffer for virtual "
+				"directory name\n"), PROGRAM_NAME);
 		}
 		return FUNC_FAILURE;
 	}
@@ -1664,8 +1714,8 @@ create_virtual_dir(const int user_provided)
 			: (ret == E_NOEXEC ? NOEXEC_MSG : (char *)NULL));
 
 		if (user_provided == 1) {
-			err('e', PRINT_PROMPT, "%s: mkdir: '%s': %s. Trying with "
-				"default value\n", PROGRAM_NAME, stdin_tmp_dir,
+			err('e', PRINT_PROMPT, _("%s: mkdir: '%s': %s. Trying with "
+				"default value\n"), PROGRAM_NAME, stdin_tmp_dir,
 				errmsg ? errmsg : strerror(ret));
 		} else {
 			err('e', PRINT_PROMPT, "%s: mkdir: '%s': %s\n",
@@ -1686,7 +1736,7 @@ construct_name(char *file, const size_t flen)
 	/* Should we construct destiny file as full path or using only the
 	 * last path component (the file's basename)? */
 	if (xargs.virtual_dir_full_paths != 1) {
-		char *p = strrchr(file, '/');
+		const char *p = strrchr(file, '/');
 		if (!p || !*(++p))
 			name = savestring(file, flen);
 		else
@@ -1697,8 +1747,8 @@ construct_name(char *file, const size_t flen)
 
 	if (!name || !*name) {
 		free(name);
-		err('w', PRINT_PROMPT, "%s: '%s': Error constructing "
-			"file name\n", PROGRAM_NAME, file);
+		err('w', PRINT_PROMPT, _("%s: '%s': Error constructing "
+			"filename\n"), PROGRAM_NAME, file);
 		return (char *)NULL;
 	}
 
@@ -1731,15 +1781,15 @@ gen_symlink(char *file, const char *cwd)
 
 	struct stat attr;
 	if (lstat(file, &attr) == -1) {
-		/* "~" fails here. No need to check in construct_name() */
+		/* "~" fails here. No need to check in construct_name(). */
 		err('w', PRINT_PROMPT, "%s: '%s': %s\n",
 			PROGRAM_NAME, file, strerror(errno));
 		return 0;
 	}
 
-	/* Construct source and destiny files */
+	/* Construct source and destiny files. */
 
-	/* symlink(3) doesn't like file names ending with slash */
+	/* symlink(3) doesn't like filenames ending with slash. */
 	size_t file_len = strlen(file);
 	if (file_len > 1 && file[file_len - 1] == '/') {
 		file[file_len - 1] = '\0';
@@ -1759,27 +1809,31 @@ gen_symlink(char *file, const char *cwd)
 	char dest[PATH_MAX + 32];
 	snprintf(dest, sizeof(dest), "%s/%s", stdin_tmp_dir, name);
 
-	errno = 0;
-	int suffix = 1;
-	while (symlink(source, dest) == -1 && errno == EEXIST) {
-		snprintf(dest, sizeof(dest), "%s/%s-%d", stdin_tmp_dir,
-			name, suffix);
-		suffix++;
+	int suffix = 0;
+
+	while (1) {
 		errno = 0;
+		if (symlink(source, dest) == 0)
+			break;
+
+		if (errno != EEXIST) {
+			err('w', PRINT_PROMPT, _("%s: Cannot create symbolic "
+				"link '%s': %s\n"), PROGRAM_NAME, dest, strerror(errno));
+			free(name);
+			return 0;
+		}
+
+		suffix++;
 		if (suffix == INT_MAX) {
 			free(name);
 			return 0;
 		}
+
+		snprintf(dest, sizeof(dest), "%s/%s-%d", stdin_tmp_dir,
+			name, suffix);
 	}
 
 	free(name);
-
-	if (errno != 0) {
-		err('w', PRINT_PROMPT, "%s: Cannot create symbolic "
-			"link '%s': %s\n", PROGRAM_NAME, dest, strerror(errno));
-		return 0;
-	}
-
 	return 1;
 }
 
@@ -1803,18 +1857,17 @@ handle_stdin(void)
 	 * == (65535 * PATH_MAX)
 	 * == 262MiB of data ((65535 * PATH_MAX) / 1024). */
 
-	size_t chunk = (size_t)512 * 1024;
+	const size_t chunk = (size_t)512 * 1024;
+	const size_t max_chunks = 512;
 	size_t chunks_n = 1;
 	size_t total_len = 0;
-	size_t max_chunks = 512;
-
 	ssize_t input_len = 0;
 
 	/* Initial buffer allocation == 1 chunk */
 	char *buf = xnmalloc(chunk, sizeof(char));
 
 	while (chunks_n < max_chunks) {
-		input_len = read(STDIN_FILENO, buf + total_len, chunk);
+		input_len = read(STDIN_FILENO, buf + total_len, chunk); /* flawfinder: ignore */
 
 		/* Error */
 		if (input_len < 0) {
@@ -1833,8 +1886,11 @@ handle_stdin(void)
 		buf = xnrealloc(buf, chunks_n + 1, chunk);
 	}
 
-	if (total_len == 0)
-		goto FREE_N_EXIT;
+	if (total_len == 0) {
+		free(buf);
+		fprintf(stderr, _("%s: No entries\n"), PROGRAM_NAME);
+		exit(EXIT_FAILURE);
+	}
 
 	/* Null terminate the input buffer */
 	buf[total_len] = '\0';
@@ -1887,7 +1943,7 @@ handle_stdin(void)
 
 	if (links_counter == 0) { /* No symlink was created. Exit */
 		dup2(STDOUT_FILENO, STDIN_FILENO);
-		xerror(_("%s: Empty file names buffer. Nothing to do\n"), PROGRAM_NAME);
+		xerror(_("%s: Empty filenames buffer. Nothing to do\n"), PROGRAM_NAME);
 		if (getenv("CLIFM_VT_RUNNING"))
 			press_any_key_to_continue(0);
 
@@ -2128,7 +2184,7 @@ bonus_function(void)
 		"At the beginning, software was hardware; but today hardware is "
 		"being absorbed by software",
 		"\"Juremos con gloria morir\"",
-		"\"Given enough eyeballs, all bugs are shallow.\" (Linus' law)",
+		"\"Given enough eyeballs, all bugs are shallow.\" (E. Raymond)",
 		"\"We're gonna need a bigger boat.\" (Caleb)",
 		"\"Ein Verletzter, Alarm, Alarm!\"",
 		"\"There is not knowledge that is not power\"",
