@@ -1,17 +1,15 @@
-/* misc.h */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* misc.h */
+
 #ifndef MISC_H
 #define MISC_H
 
@@ -36,7 +36,7 @@ int  confirm_sudo_cmd(char **cmd);
 int  create_usr_var(const char *str);
 int  expand_prompt_name(char *name);
 int  filter_function(char *arg);
-void free_autocmds(void);
+void free_autocmds(const int keep_temp);
 void free_prompts(void);
 void free_stuff(void);
 int  free_remotes(const int exit);
@@ -49,13 +49,11 @@ int  handle_stdin(void);
 int  is_blank_name(const char *s);
 int  list_mountpoints(void);
 int  new_instance(char *, int);
-int  print_reload_msg(const char *msg, ...);
+int  print_reload_msg(const char *ptr, const char *color, const char *msg, ...);
 int  pin_directory(char *dir);
 void save_last_path(char *last_path_tmp);
 void set_eln_color(void);
-void set_signals_to_ignore(void);
 void set_filter_type(const char c);
-void set_term_title(char *str);
 void splash(void);
 int  unpin_dir(void);
 void version_function(const int full);
