@@ -1,16 +1,15 @@
-/* name_cleaner.c -- functions to clean up file names
- * 
- * This file is part of CliFM
+/* 
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -21,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* name_cleaner.c -- functions to sanitize filenames */
+
 /* check_width, unpack_start, and unpack_cont macros, just
  * as get_utf_8_width and get_uft8_dec_value functions are taken from
  * https://github.com/dharple/detox/blob/main/src/clean_utf_8.c, licensed
@@ -31,7 +32,6 @@
 
 #include "helpers.h"
 
-#include <stdio.h>
 #include <unistd.h>
 #include <string.h>
 #include <time.h>
@@ -74,8 +74,8 @@
 #define UTF_8_ENCODED_2_BYTES_MASK 0xE0
 #define UTF_8_ENCODED_2_BYTES      0xC0
 
-#define BLEACH_TMP_HEADER "# CliFM - Bleach\n\
-# Edit replacement file names as you wish, save, and close the editor.\n\
+#define BLEACH_TMP_HEADER "# Clifm - Bleach\n\
+# Edit replacement filenames as you wish, save, and close the editor.\n\
 # You will be asked for confirmation at exit.\n\n"
 
 #define check_width(chr, size) if (((chr) & UTF_8_ENCODED_ ## size ## _BYTES_MASK) == UTF_8_ENCODED_ ## size ## _BYTES) { return size; }
@@ -98,9 +98,8 @@ get_utf_8_width(const char c)
 		check_width(c, 6);
     }
 
-	if ((c & UTF_8_ENCODED_MASK) == UTF_8_ENCODED_CONT) { /* UTF-8 continuation byte */
+	if ((c & UTF_8_ENCODED_MASK) == UTF_8_ENCODED_CONT) /* UTF-8 continuation byte */
 		return -1;
-	}
 
 	return 1;
 }
@@ -113,8 +112,7 @@ static int
 translate_unsafe_char(const unsigned char c)
 {
 	unsigned char t = 0;
-	if ((c >= '0' && c <= '9') || (c >= 'a' && c <= 'z')
-	|| (c >= 'A' && c <= 'Z') || c == '.' || c == '_' || c == '-')
+	if (IS_ALNUM(c) || c == '.' || c == '_' || c == '-')
 		t = c;
 	else if (c == '(' || c == ')' || c == '[' || c == ']'
 	|| c == '{' || c == '}')
@@ -139,7 +137,7 @@ get_uft8_dec_value(size_t *i, const char *str)
 		case 4: new_value = unpack_start(c, 4); break;
 		case 5: new_value = unpack_start(c, 5); break;
 		case 6: new_value = unpack_start(c, 6); break;
-		default: return (-1); /* -1 */
+		default: return (-1);
 	}
 
 	int expected_chars = utf8_width - 1;
@@ -170,16 +168,16 @@ get_uft8_dec_value(size_t *i, const char *str)
  * without an ASCII alternative/similar character, or by translating (based
  * on the unitable table (in cleaner_table.h)) extended-ASCII/Unicode characters
  * into an alternative ASCII character based on familiarity/similarity. Disallowed
- * characters (NUL and slash) will be simply removed. The file name length will
- * be trimmed to NAME_MAX (usually 255). If the replacement file name is
+ * characters (NUL and slash) will be simply removed. The filename length will
+ * be trimmed to NAME_MAX (usually 255). If the replacement filename is
  * only one character long, "bleach" will be appended to avoid too short
- * file names.
+ * filenames.
  *
- * @parameter: the file name to be cleaned up. If it contains a slash, only
- * the string after the slash will be taken a the actual file name
+ * @parameter: the filename to be sanitized. If it contains a slash, only
+ * the string after the slash will be taken a the actual filename
  *
- * @return: the cleaned file name or NULL in case of error. If the translated
- * file name is empty, it will be replaced by "bleach.YYYYMMDDHHMMS"
+ * @return: the sanitized filename or NULL in case of error. If the translated
+ * filename is empty, it will be replaced by "bleach.YYYYMMDDHHMMS"
  * */
 static char *
 clean_file_name(char *restrict name)
@@ -217,7 +215,7 @@ clean_file_name(char *restrict name)
 		}
 
 		if (n <= 127) {
-			int ret = translate_unsafe_char(n);
+			const int ret = translate_unsafe_char(n);
 			if (ret == -1)
 				continue;
 
@@ -255,7 +253,7 @@ clean_file_name(char *restrict name)
 		}
 
 		/* Extended ASCII and Unicode chars */
-		int dec_value = get_uft8_dec_value(&i, name);
+		const int dec_value = get_uft8_dec_value(&i, name);
 		if (!name[i])
 			break;
 
@@ -297,13 +295,13 @@ clean_file_name(char *restrict name)
 	if (too_long)
 		p[NAME_MAX] = '\0';
 
-	/* Handle some file names that should be avoided */
+	/* Handle some filenames that should be avoided */
 
-	if (!*p) { /* Empty file name */
+	if (!*p) { /* Empty filename */
 		time_t rawtime = time(NULL);
 		struct tm t;
 		char *suffix = localtime_r(&rawtime, &t)
-			? gen_date_suffix(t) : (char *)NULL;
+			? gen_date_suffix(t, 0) : (char *)NULL;
 		if (!suffix) {
 			free(p);
 			return (char *)NULL;
@@ -311,8 +309,8 @@ clean_file_name(char *restrict name)
 		snprintf(p, NAME_MAX, "%s.%s", FUNC_NAME, suffix);
 		free(suffix);
 	} else {
-		if (!*(p + 1)) {
-			/* Avoid one character long file names. Specially because files
+		if (!p[1]) {
+			/* Avoid one character long filenames. Specially because files
 			 * named with a single dot should be avoided */
 			char c = *p;
 			snprintf(p, NAME_MAX, "%c.%s", c, FUNC_NAME);
@@ -323,19 +321,19 @@ clean_file_name(char *restrict name)
 	if (*name != '.' && *p == '.')
 		*p = DEFAULT_TRANSLATION;
 
-	/* File names shouldn't start with a dash/hyphen (reserved for command options)*/
+	/* Filenames shouldn't start with a dash/hyphen (reserved for command options)*/
 	if (*p == '-')
 		*p = DEFAULT_TRANSLATION;
 
-	/* No file name should be named dot-dot (..) */
+	/* No filename should be named dot-dot (..) */
 	if (cur_len == 3 && *p == '.' && *(p + 1) == '.' && !*(p + 2))
 		*(p + 1) = DEFAULT_TRANSLATION;
 
 	return p;
 }
 
-/* Let the user edit replacement file names via a text editor and return
- * a new array with the updated file names. If the temp file is not modified
+/* Let the user edit replacement filenames via a text editor and return
+ * a new array with the updated filenames. If the temp file is not modified
  * the original list of files is returned instead. In case of error
  * returns NULL */
 static struct bleach_t *
@@ -443,12 +441,12 @@ edit_replacements(struct bleach_t *bfiles, size_t *n, int *edited_names)
 				bfiles[i].original = (char *)NULL;
 			}
 			bfiles[i].original = savestring(p, strlen(p));
-		/* Do not store the replacement file name is there is no original */
+		/* Do not store the replacement filename is there is no original */
 		} else if (strncmp(line, "replacement: ", 13) == 0
 		&& bfiles[i].original) {
 			bfiles[i].replacement = savestring(p, strlen(p));
-			printf("%s %s->%s %s\n", bfiles[i].original, mi_c, df_c,
-				bfiles[i].replacement);
+			printf("%s %s%s%s %s\n", bfiles[i].original, mi_c, SET_MSG_PTR,
+				df_c, bfiles[i].replacement);
 			i++;
 		} else {
 			continue;
@@ -489,9 +487,9 @@ ERROR:
 	return bfiles;
 }
 
-/* Clean up the list of file names (NAMES), print the list of the cleaned
- * file names (allowing the user to edit this list), and finally
- * rename the original file names into the clean ones */
+/* Clean up the list of filenames (NAMES), print the list of the sanitized
+ * filenames (allowing the user to edit this list), and finally
+ * rename the original filenames into the clean ones */
 int
 bleach_files(char **names)
 {
@@ -507,7 +505,7 @@ bleach_files(char **names)
 	for (; names[i]; i++) {
 		char *dstr = unescape_str(names[i], 0);
 		if (!dstr) {
-			xerror(_("bleach: '%s': Error unescaping file name\n"), names[i]);
+			xerror(_("bleach: '%s': Error unescaping filename\n"), names[i]);
 			continue;
 		}
 
@@ -542,8 +540,8 @@ bleach_files(char **names)
 		} else {
 			bfiles[f].replacement = savestring(p, strlen(p));
 		}
-		printf("%s %s->%s %s\n",
-			bfiles[f].original, mi_c, df_c, bfiles[f].replacement);
+		printf("%s %s%s%s %s\n",
+			bfiles[f].original, mi_c, SET_MSG_PTR, df_c, bfiles[f].replacement);
 		f++;
 		free(p);
 	}
@@ -558,8 +556,7 @@ bleach_files(char **names)
 
 CONFIRM:
 	while (!input) {
-		input = rl_no_hist(_("Is this OK? [y/n/(e)dit] "));
-
+		input = rl_no_hist(_("Is this OK? [y/n/(e)dit] "), 0);
 		if (!input)
 			continue;
 
@@ -600,19 +597,23 @@ CONFIRM:
 	free(input);
 
 	if (f == 0) {
-		/* Just in case either the original or the replacement file name
+		/* Just in case either the original or the replacement filename
 		 * was removed from the list by the user, leaving only one of the two. */
-		free(bfiles[0].original);
-		free(bfiles[0].replacement);
+		if (bfiles) {
+			free(bfiles[0].original);
+			free(bfiles[0].replacement);
+		}
 		free(bfiles);
 		printf(_("%s: Nothing to do\n"), FUNC_NAME);
 		return FUNC_SUCCESS;
 	}
 
 	if (edited_names == -1 || quit_func == 1) { /* ERROR or quit */
-		for (i = 0; i < f; i++) {
-			free(bfiles[i].original);
-			free(bfiles[i].replacement);
+		if (bfiles) {
+			for (i = 0; i < f; i++) {
+				free(bfiles[i].original);
+				free(bfiles[i].replacement);
+			}
 		}
 		free(bfiles);
 		return (quit_func == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
@@ -622,11 +623,13 @@ CONFIRM:
 	 * Ask for confirmation in case the user just wanted to see what would
 	 * be done. */
 	if (do_edit == 1) {
-		printf(_("%zu file name(s) will be bleached\n"), f);
-		if (rl_get_y_or_n(_("Continue? [y/n] ")) != 1) {
-			for (i = 0; i < f; i++) {
-				free(bfiles[i].original);
-				free(bfiles[i].replacement);
+		printf(_("%zu filename(s) will be bleached\n"), f);
+		if (rl_get_y_or_n(_("Continue?"), 0) != 1) {
+			if (bfiles) {
+				for (i = 0; i < f; i++) {
+					free(bfiles[i].original);
+					free(bfiles[i].replacement);
+				}
 			}
 			free(bfiles);
 			return FUNC_SUCCESS;
@@ -642,10 +645,13 @@ CONFIRM:
 	int exit_status = FUNC_SUCCESS;
 
 	for (i = 0; i < f; i++) {
-		char *o = bfiles[i].original ? bfiles[i].original : (char *)NULL;
-		char *r = bfiles[i].replacement ? bfiles[i].replacement : (char *)NULL;
+		char *o = (bfiles && bfiles[i].original)
+			? bfiles[i].original : (char *)NULL;
+		char *r = (bfiles && bfiles[i].replacement)
+			? bfiles[i].replacement : (char *)NULL;
+
 		if (o && *o && r && *r && rename) {
-			/* Make sure the replacement file name does not exist. If
+			/* Make sure the replacement filename does not exist. If
 			 * it does, append REP_SUFFIX and try again. */
 			struct stat a;
 			while (lstat(r, &a) == 0) {
@@ -658,7 +664,7 @@ CONFIRM:
 			}
 
 			if (renameat(XAT_FDCWD, o, XAT_FDCWD, r) == -1) {
-				xerror("bleach: Cannot rename '%s' to '%s': %s\n",
+				xerror(_("bleach: Cannot rename '%s' to '%s': %s\n"),
 					o, r, strerror(errno));
 				total_rename--;
 				exit_status = FUNC_FAILURE;
@@ -672,11 +678,12 @@ CONFIRM:
 	free(bfiles);
 
 	if (exit_status == FUNC_FAILURE || total_rename == 0) {
-		printf(_("%s: %d file names(s) bleached\n"), FUNC_NAME, total_rename);
+		printf(_("%s: %d filenames(s) bleached\n"), FUNC_NAME, total_rename);
 	} else {
 		if (conf.autols == 1)
 			reload_dirlist();
-		print_reload_msg(_("%d file name(s) bleached\n"), total_rename);
+		print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+			_("%d filename(s) bleached\n"), total_rename);
 	}
 
 	return exit_status;
