@@ -1,16 +1,15 @@
-/* name_cleaner.h
+/*
+ * This file is part of Clifm
  *
- * This file is part of CliFM
- * 
- * Copyright (C) 2016-2023, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -21,12 +20,14 @@
  * MA 02110-1301, USA.
 */
 
+/* name_cleaner.h */
+
 #ifndef NAME_CLEANER_H
 #define NAME_CLEANER_H
 
 __BEGIN_DECLS
 
-int bleach_files(char **);
+int bleach_files(char **names);
 
 __END_DECLS
 
