@@ -1,17 +1,15 @@
-/* navigation.c -- functions to control the navigation system */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,10 +20,11 @@
  * MA 02110-1301, USA.
 */
 
+/* navigation.c -- functions to control the navigation system */
+
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <readline/tilde.h>
@@ -40,6 +39,7 @@
 #include "misc.h"        /* xerror */
 #include "navigation.h"
 #include "readline.h"    /* rl_no_hist */
+#include "term.h" 		 /* report_cwd(), set_term_title() */
 
 #define BD_CONTINUE 2
 
@@ -83,7 +83,7 @@ pwd_function(const char *arg)
 	}
 
 	char p[PATH_MAX + 1]; *p = '\0';
-	char *real_path = xrealpath(pwd, p);
+	const char *real_path = xrealpath(pwd, p);
 	if (!real_path) {
 		xerror("pwd: '%s': %s\n", pwd, strerror(errno));
 		return errno;
@@ -93,7 +93,7 @@ pwd_function(const char *arg)
 	return FUNC_SUCCESS;
 }
 
-/* Return the list of paths in CWD matching STR */
+/* Return the list of paths in CWD matching STR. */
 char **
 get_bd_matches(const char *str, int *n, const int mode)
 {
@@ -104,16 +104,15 @@ get_bd_matches(const char *str, int *n, const int mode)
 	char **matches = (char **)NULL;
 
 	if (mode == BD_TAB) {
-		/* matches will be passed to readline for TAB completion, so
-		 * that we need to reserve the first slot to hold the query
-		 * string */
+		/* matches will be passed to readline for tab completion, so that
+		 * we need to reserve the first slot to hold the query string. */
 		*n = 1;
 		matches = xnmalloc(2, sizeof(char *));
 	}
 
 	while (1) {
 		char *p = (char *)NULL;
-		if (str && *str) { /* Non-empty query string */
+		if (str && *str) { /* Non-empty query string. */
 			p = conf.case_sens_path_comp
 				? strstr(cwd, str) : xstrcasestr(cwd, (char *)str);
 			if (!p)
@@ -130,11 +129,11 @@ get_bd_matches(const char *str, int *n, const int mode)
 
 		matches = xnrealloc(matches, (size_t)*n + 2, sizeof(char *));
 		if (mode == BD_TAB) {
-			/* Print only the path base name */
+			/* Print only the path base name. */
 			char *ss = strrchr(workspaces[cur_ws].path, '/');
 			if (ss && *(++ss))
 				matches[*n] = savestring(ss, strlen(ss));
-			else /* Last slash is the first and only char: We have root dir */
+			else /* Last slash is the first and only char: we have the root dir. */
 				matches[*n] = savestring("/", 1);
 			(*n)++;
 		} else {
@@ -155,20 +154,20 @@ get_bd_matches(const char *str, int *n, const int mode)
 	}
 
 	if (mode == BD_TAB) {
-		if (*n == 1) { /* No matches */
+		if (*n == 1) { /* No matches. */
 			free(matches);
 			return (char **)NULL;
-		} else if (*n == 2) { /* One match */
+		} else if (*n == 2) { /* One match. */
 			char *p = escape_str(matches[1]);
 			if (!p) {
+				free(matches[1]);
 				free(matches);
 				return (char **)NULL;
 			}
-			matches[0] = savestring(p, strlen(p));
+			matches[0] = p;
 			free(matches[1]);
 			matches[1] = (char *)NULL;
-			free(p);
-		} else { /* Multiple matches */
+		} else { /* Multiple matches. */
 			matches[0] = savestring(str, strlen(str));
 			matches[*n] = (char *)NULL;
 		}
@@ -187,7 +186,7 @@ grab_bd_input(const int n)
 	putchar('\n');
 
 	while (!input) {
-		input = rl_no_hist(_("Choose a directory ('q' to quit): "));
+		input = rl_no_hist(_("Select a directory ('q' to quit): "), 0);
 		if (!input || !*input) {
 			free(input);
 			input = (char *)NULL;
@@ -205,7 +204,7 @@ grab_bd_input(const int n)
 				continue;
 			}
 
-		} else if (*input == 'q' && !*(input + 1)) {
+		} else if (*input == 'q' && !input[1]) {
 			free(input);
 			return (-1);
 
@@ -216,7 +215,7 @@ grab_bd_input(const int n)
 		}
 	}
 
-	return (-1); /* Never reached */
+	return (-1); /* Never reached. */
 }
 
 static int
@@ -236,7 +235,7 @@ backdir_directory(char *dir, const char *str)
 		dir = exp_path;
 	}
 
-	/* If STR is a directory, just change to it */
+	/* If STR is a directory, just change to it. */
 	struct stat a;
 	if (stat(dir, &a) == 0 && S_ISDIR(a.st_mode))
 		return cd_function(dir, CD_PRINT_ERROR);
@@ -244,7 +243,7 @@ backdir_directory(char *dir, const char *str)
 	return BD_CONTINUE;
 }
 
-/* If multiple matches, print a menu to choose from */
+/* If multiple matches, print a menu to select from. */
 static int
 backdir_menu(char **matches)
 {
@@ -252,13 +251,13 @@ backdir_menu(char **matches)
 	for (i = 0; matches[i]; i++) {
 		char *sl = strrchr(matches[i], '/');
 		int flag = 0;
-		if (sl && *(sl + 1)) {
+		if (sl && sl[1]) {
 			*sl = '\0';
 			sl++;
 			flag = 1;
 		}
 		printf("%s%d%s %s%s%s\n", el_c, i + 1, df_c, di_c, sl ? sl : "/", df_c);
-		if (flag) {
+		if (flag == 1) {
 			sl--;
 			*sl = '/';
 		}
@@ -287,7 +286,7 @@ help_or_root(const char *str)
 	return FUNC_FAILURE;
 }
 
-/* Change to parent directory matching STR */
+/* Change to parent directory matching STR. */
 int
 backdir(char *str)
 {
@@ -317,12 +316,13 @@ backdir(char *str)
 		return FUNC_FAILURE;
 	}
 
-	int exit_status = FUNC_SUCCESS, i = n;
-	if (n == 1) /* Just one match: change to it */
+	int exit_status = FUNC_SUCCESS;
+	if (n == 1) /* Just one match: change to it. */
 		exit_status = cd_function(matches[0], CD_PRINT_ERROR);
-	else if (n > 1) /* Multiple matches: print a menu to choose from */
+	else /* Multiple matches: print a menu to select from. */
 		exit_status = backdir_menu(matches);
 
+	int i = n;
 	while (--i >= 0)
 		free(matches[i]);
 	free(matches);
@@ -361,7 +361,7 @@ xchdir(char *dir, const int cd_flag)
 
 	if (ret == 0 && cd_flag == SET_TITLE) {
 		char tmp[PATH_MAX + 1]; *tmp = '\0';
-		char *p = get_cwd(tmp, sizeof(tmp), 0);
+		const char *p = get_cwd(tmp, sizeof(tmp), 0);
 
 		/* Do not set OLDPWD if changing to the same directory ("cd ."
 		 * and similar commands). */
@@ -370,8 +370,13 @@ xchdir(char *dir, const int cd_flag)
 
 		setenv("PWD", dir, 1);
 
-		if (xargs.cwd_in_title == 1)
-			set_term_title(dir);
+		if (xargs.vt100 != 1) { /* --vt100 */
+			if (xargs.report_cwd != 0) /* --no-report-cwd */
+				report_cwd(dir); /* OSC-7 escape sequence */
+
+			if (xargs.cwd_in_title == 1)
+				set_term_title(dir); /* --cwd-in-title */
+		}
 	}
 
 	return ret;
@@ -380,19 +385,18 @@ xchdir(char *dir, const int cd_flag)
 static char *
 check_cdpath(const char *name)
 {
-	if (cdpath_n == 0 || !name || !*name)
+	if (!name || !*name)
 		return (char *)NULL;
 
 	if (*name == '/' || (*name == '.' && name[1] == '/')
 	|| (*name == '.' && name[1] == '.' && name[2] == '/'))
 		return (char *)NULL;
 
-	size_t i;
 	const size_t namelen = strlen(name);
 	char *p = (char *)NULL;
 	struct stat a;
 
-	for (i = 0; cdpaths[i]; i++) {
+	for (size_t i = 0; cdpaths[i]; i++) {
 		const size_t len = strlen(cdpaths[i]);
 		const size_t tmp_len = len + namelen + 2;
 		char *tmp = xnmalloc(tmp_len, sizeof(char));
@@ -406,7 +410,7 @@ check_cdpath(const char *name)
 		if (*tmp == '~')
 			exp_path = tilde_expand(tmp);
 
-		char *dir = exp_path ? exp_path : tmp;
+		const char *dir = exp_path ? exp_path : tmp;
 		if (stat(dir, &a) != -1 && S_ISDIR(a.st_mode)) {
 			p = savestring(dir, strlen(dir));
 			free(tmp);
@@ -426,9 +430,9 @@ check_cdpath(const char *name)
 	return p;
 }
 
-/* Change the current directory to the home directory */
+/* Change the current directory to the home directory. */
 static int
-go_home(const int cd_flag)
+change_to_home_dir(const int cd_flag)
 {
 	if (!user.home) {
 		if (cd_flag == CD_PRINT_ERROR)
@@ -444,13 +448,13 @@ go_home(const int cd_flag)
 
 	if (workspaces) {
 		free(workspaces[cur_ws].path);
-		workspaces[cur_ws].path = savestring(user.home, strlen(user.home));
+		workspaces[cur_ws].path = savestring(user.home, user.home_len);
 	}
 
 	return FUNC_SUCCESS;
 }
 
-/* Change current directory to NEW_PATH */
+/* Change current directory to NEW_PATH. */
 static int
 change_to_path(char *new_path, const int cd_flag)
 {
@@ -462,52 +466,59 @@ change_to_path(char *new_path, const int cd_flag)
 	if (strchr(new_path, '\\')) {
 		char *deq_path = unescape_str(new_path, 0);
 		if (deq_path) {
+			/* deq_path is guaranteed to be shorter than new_path. */
 			xstrsncpy(new_path, deq_path, strlen(deq_path) + 1);
 			free(deq_path);
 		}
 	}
 
-	char *p = check_cdpath(new_path);
+	char *cdpath_path = cdpath_n > 0 ? check_cdpath(new_path) : NULL;
 	errno = 0;
-	char *r = p ? p : new_path;
-	char *q = normalize_path(r, strlen(r));
+	char *tmp = cdpath_path ? cdpath_path : new_path;
+	char *dest_dir = normalize_path(tmp, strlen(tmp));
+	free(cdpath_path);
 
-	if (!q) {
+	if (!dest_dir) {
 		if (cd_flag == CD_PRINT_ERROR)
 			xerror(_("cd: '%s': Error normalizing path\n"), new_path);
-		free(p);
 		return FUNC_FAILURE;
 	}
 
-	free(p);
-
-	if (xchdir(q, SET_TITLE) != FUNC_SUCCESS) {
+	if (xchdir(dest_dir, SET_TITLE) != FUNC_SUCCESS) {
 		if (cd_flag == CD_PRINT_ERROR)
 			xerror("cd: '%s': %s\n", new_path, strerror(errno));
 
-		free(q);
+		free(dest_dir);
 
 		/* Most shells return 1 in case of EACCESS/ENOENT error. However, 1, as
 		 * a general error code, is not quite informative. Why not to return the
 		 * actual error code returned by chdir(3)? Note that POSIX only requires
 		 * for cd to return >0 in case of error (see cd(1p)). */
-		if (errno == EACCES || errno == ENOENT)
-			return FUNC_FAILURE;
-
-		return errno;
+		return (errno == EACCES || errno == ENOENT) ? 1 : errno;
 	}
 
 	if (workspaces) {
 		free(workspaces[cur_ws].path);
-		workspaces[cur_ws].path = savestring(q, strlen(q));
+		workspaces[cur_ws].path = savestring(dest_dir, strlen(dest_dir));
 	}
 
-	free(q);
+	free(dest_dir);
 
 	return FUNC_SUCCESS;
 }
 
+/* Implementation of the shell 'cd -' command. */
 static int
+change_to_previous_dir(void)
+{
+	static int state = 0;
+	char *cmd[] = { state == 0 ? "b" : "f", NULL };
+
+	state = !state;
+	return *cmd[0] == 'b' ? back_function(cmd) : forth_function(cmd);
+}
+
+static inline int
 skip_directory(const char *dir)
 {
 	return (conf.dirhistignore_regex && *conf.dirhistignore_regex
@@ -519,24 +530,15 @@ skip_directory(const char *dir)
 int
 cd_function(char *new_path, const int cd_flag)
 {
-	/* If no argument, change to home */
+	/* If no argument, change to home. */
 	int ret = FUNC_SUCCESS;
 
 	if (!new_path || !*new_path) {
-		if ((ret = go_home(cd_flag)) != FUNC_SUCCESS)
+		if ((ret = change_to_home_dir(cd_flag)) != FUNC_SUCCESS)
 			return ret;
 
 	} else if (*new_path == '-' && !new_path[1]) {
-		/* Implementation of the shell 'cd -' command */
-		static int state = 0;
-		char *c[] = { state == 0 ? "b" : "f", NULL };
-		if (state == 0) {
-			state = 1;
-			return back_function(c);
-		} else {
-			state = 0;
-			return forth_function(c);
-		}
+		return change_to_previous_dir();
 
 	} else {
 		if ((ret = change_to_path(new_path, cd_flag)) != FUNC_SUCCESS)
@@ -560,8 +562,8 @@ cd_function(char *new_path, const int cd_flag)
 
 /* Return a pointer to the first occurrence in the string STR of a byte that
  * is not C. Otherwise, if only C is found, NULL is returned. */
-static char *
-xstrcpbrk(char *str, const char c)
+static const char *
+xstrcpbrk(const char *restrict str, const char c)
 {
 	if (!str || !*str)
 		return (char *)NULL;
@@ -578,66 +580,44 @@ xstrcpbrk(char *str, const char c)
 /* Convert "... n" into "../.. n"
  * and "../.. n" into the corresponding path. */
 char *
-fastback(char *str)
+fastback(const char *str)
 {
 	if (!str || !*str || xstrcpbrk(str, '.'))
 		return (char *)NULL;
 
-	char *p = str;
-	size_t dots = 0;
+	/* At this point we know STR contains only dots. */
+	const size_t dots = strlen(str);
 
-	char *rem = (char *)NULL;
-	while (*p) {
-		if (*p != '.') {
-			rem = p;
-			break;
-		}
-		dots++;
-		p++;
-	}
+	if (dots <= 2)
+		return dots == 2 ? normalize_path("..", 2) : NULL;
 
-	if (dots <= 2) {
-		if (dots == 2)
-			return normalize_path("..", 2);
-		return (char *)NULL;
-	}
-
-	char q[PATH_MAX];
-	q[0] = '.';
-	q[1] = '.';
+	char dir[PATH_MAX + 1];
+	dir[0] = '.';
+	dir[1] = '.';
 
+	/* Replace each dot after ".." by "/.." */
 	size_t i, c = 2;
 	for (i = 2; c < dots;) {
-		q[i] = '/'; i++;
-		q[i] = '.'; i++;
-		q[i] = '.'; i++;
+		dir[i] = '/'; i++;
+		dir[i] = '.'; i++;
+		dir[i] = '.'; i++;
 		c++;
 	}
 
-	q[i] = '\0';
+	dir[i] = '\0';
 
-	if (rem) {
-		if (*rem != '/') {
-			q[i] = '/';
-			q[i + 1] = '\0';
-			i++;
-		}
-
-		xstrncat(q, strnlen(q, sizeof(q)), rem, sizeof(q));
-	}
-
-	return normalize_path(q, i);
+	return normalize_path(dir, i);
 }
 
 void
 print_dirhist(char *query)
 {
-	int n = DIGINUM(dirhist_total_index), i;
+	const int n = DIGINUM(dirhist_total_index);
 	const size_t len = (query && conf.fuzzy_match == 1) ? strlen(query) : 0;
-	int fuzzy_str_type = (len > 0 && contains_utf8(query) == 1)
+	const int fuzzy_str_type = (len > 0 && contains_utf8(query) == 1)
 		? FUZZY_FILES_UTF8 : FUZZY_FILES_ASCII;
 
-	for (i = 0; i < dirhist_total_index; i++) {
+	for (int i = 0; i < dirhist_total_index; i++) {
 		if (!old_pwd[i] || *old_pwd[i] == KEY_ESC)
 			continue;
 
@@ -671,7 +651,7 @@ clear_dirhist(void)
 }
 
 /* Change to the specified directory number (N) in the directory
- * history list */
+ * history list. */
 static int
 change_to_dirhist_num(int n)
 {
@@ -724,7 +704,7 @@ surf_hist(char **args)
 	return change_to_dirhist_num(n);
 }
 
-/* Set the path of the current workspace to NEW_PATH */
+/* Set the path of the current workspace to NEW_PATH. */
 static int
 set_path(const char *new_path)
 {
@@ -742,7 +722,7 @@ set_path(const char *new_path)
 	return FUNC_SUCCESS;
 }
 
-/* Go back one entry in dirhist */
+/* Go back one entry in dirhist. */
 int
 back_function(char **args)
 {
@@ -756,7 +736,7 @@ back_function(char **args)
 		return FUNC_SUCCESS;
 	}
 
-	/* Find the previous non-consecutive equal and valid entry */
+	/* Find the previous non-consecutive equal and valid entry. */
 	int i = dirhist_cur_index;
 	while (--i >= 0) {
 		if (old_pwd[i] && *old_pwd[i] != KEY_ESC && (!workspaces[cur_ws].path
@@ -774,7 +754,7 @@ back_function(char **args)
 
 	xerror("cd: '%s': %s\n", old_pwd[dirhist_cur_index], strerror(errno));
 
-	/* Invalidate this entry */
+	/* Invalidate this entry. */
 	*old_pwd[dirhist_cur_index] = KEY_ESC;
 	if (dirhist_cur_index > 0)
 		dirhist_cur_index--;
@@ -782,7 +762,7 @@ back_function(char **args)
 	return FUNC_FAILURE;
 }
 
-/* Go forth one entry in dirhist */
+/* Go forth one entry in dirhist. */
 int
 forth_function(char **args)
 {
@@ -796,7 +776,7 @@ forth_function(char **args)
 		return FUNC_SUCCESS;
 	}
 
-	/* Find the next valid entry */
+	/* Find the next valid entry. */
 	int i = dirhist_cur_index;
 	while (++i <= dirhist_total_index) {
 		if (old_pwd[i] && (*old_pwd[i] != KEY_ESC && (!workspaces[cur_ws].path
@@ -815,7 +795,7 @@ forth_function(char **args)
 
 	xerror("cd: '%s': %s\n", old_pwd[dirhist_cur_index], strerror(errno));
 
-	/* Invalidate this entry */
+	/* Invalidate this entry. */
 	*old_pwd[dirhist_cur_index] = KEY_ESC;
 	if (dirhist_cur_index < dirhist_total_index
 	&& old_pwd[dirhist_cur_index + 1])
