@@ -1,17 +1,15 @@
-/* profiles.c -- functions for manipulating user profiles */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,12 +20,13 @@
  * MA 02110-1301, USA.
 */
 
+/* profiles.c -- functions for manipulating user profiles */
+
 #ifndef _NO_PROFILES
 
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <readline/history.h>
@@ -64,7 +63,7 @@ get_profile_names(void)
 	snprintf(pf_dir, len, "%s/profiles", config_dir_gral);
 
 	struct dirent **profs = (struct dirent **)NULL;
-	int files_n = scandir(pf_dir, &profs, NULL, xalphasort);
+	const int files_n = scandir(pf_dir, &profs, NULL, xalphasort);
 
 	if (files_n == -1) {
 		free(pf_dir);
@@ -112,8 +111,7 @@ get_profile_names(void)
 static int
 check_profile(const char *name)
 {
-	int i;
-	for (i = 0; profile_names[i]; i++) {
+	for (int i = 0; profile_names[i]; i++) {
 		if (*name == *profile_names[i] && strcmp(name, profile_names[i]) == 0)
 			return i;
 	}
@@ -265,7 +263,7 @@ profile_set(const char *prof)
 	if (conf.autols == 1)
 		reload_dirlist();
 
-	print_reload_msg(_("Switched to profile %s%s%s\n"), BOLD, prof, NC);
+	print_reload_msg(NULL, NULL, _("Switched to profile %s%s%s\n"), BOLD, prof, NC);
 	return FUNC_SUCCESS;
 }
 
@@ -303,7 +301,7 @@ profile_add(const char *prof)
 		return FUNC_FAILURE;
 	}
 
-	/* If the config dir is fine, generate config file names */
+	/* If the config dir is fine, generate config filenames */
 	int exit_status = FUNC_SUCCESS;
 	const size_t config_len = strlen(nconfig_dir);
 
@@ -406,12 +404,13 @@ static int
 print_profiles(void)
 {
 	size_t i;
-	char *cur_prof = alt_profile ? alt_profile : "default";
+	const char *cur_prof = alt_profile ? alt_profile : "default";
 
+	const char *ptr = SET_MISC_PTR;
 	for (i = 0; profile_names[i]; i++) {
 		if (*cur_prof == *profile_names[i]
 		&& strcmp(cur_prof, profile_names[i]) == 0)
-			printf("%s>%s %s\n", mi_c, df_c, profile_names[i]);
+			printf("%s%s%s %s\n", mi_c, ptr, df_c, profile_names[i]);
 		else
 			printf("  %s\n", profile_names[i]);
 	}
@@ -525,7 +524,12 @@ rename_profile(char **args)
 		return errno;
 	}
 
-	int fd = dirfd(dir);
+	const int fd = dirfd(dir);
+	if (fd == -1) {
+		xerror("pf: '%s': %s\n", src_pf_name, strerror(errno));
+		closedir(dir);
+		return errno;
+	}
 
 	p = unescape_str(args[1], 0);
 	if (p) {
