@@ -1,17 +1,15 @@
-/* prompt.c -- functions controlling the appearance and behavior of the prompt */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,12 +20,13 @@
  * MA 02110-1301, USA.
 */
 
+/* prompt.c -- functions controlling the appearance and behavior of the prompt */
+
 /* The decode_prompt function is taken from Bash (1.14.7), licensed under
  * GPL-2.0-or-later, and modified to fit our needs. */
 
 #include "helpers.h"
 
-#include <stdio.h>
 #include <string.h>
 #if !defined(__HAIKU__) && !defined(__OpenBSD__) && !defined(__ANDROID__)
 # include <wordexp.h>
@@ -48,7 +47,9 @@
 #include "misc.h"
 #include "navigation.h"
 #include "prompt.h"
+#include "properties.h" /* get_file_perms() */
 #include "sanitize.h"
+#include "sort.h" /* num_to_sort_name() */
 #include "spawn.h"
 #ifndef _NO_SUGGESTIONS
 # include "suggestions.h"
@@ -68,7 +69,9 @@ struct p_mod_paths_t {
 static struct p_mod_paths_t p_mod_paths[MAX_PMOD_PATHS];
 #endif /* __HAIKU__ || __OpenBSD__ || __ANDROID__ */
 
-static inline char *
+int g_prompt_ignore_empty_line = 0;
+
+static char *
 gen_time(const int c)
 {
 	char *temp = (char *)NULL;
@@ -102,7 +105,32 @@ gen_time(const int c)
 	return temp;
 }
 
-static inline char *
+static char *
+gen_rl_vi_mode(const int alloc)
+{
+	if (rl_editing_mode == RL_EMACS_MODE
+	/*"Ignore if running in vi mode and notifications are enabled: the
+	 * notification will be printed by the prompt itself. */
+	|| (alloc == 1 && prompt_notif == 1)) {
+		if (alloc == 0)
+			return "";
+		char *tmp = xnmalloc(1, sizeof(char));
+		*tmp = '\0';
+		return tmp;
+	}
+
+	Keymap keymap = rl_get_keymap();
+	if (keymap == vi_insertion_keymap) {
+		return alloc == 1
+			? savestring(RL_VI_INS_MODESTR, RL_VI_INS_MODESTR_LEN)
+			: RL_VI_INS_MODESTR;
+	}
+
+	return alloc == 1 ? savestring(RL_VI_CMD_MODESTR, RL_VI_CMD_MODESTR_LEN)
+		: RL_VI_CMD_MODESTR;
+}
+
+static char *
 get_dir_basename(const char *str)
 {
 	/* If not root dir (/), get last path component */
@@ -115,7 +143,7 @@ get_dir_basename(const char *str)
 	return savestring(ret + 1, strlen(ret + 1));
 }
 
-static inline char *
+static char *
 reduce_path(const char *str)
 {
 	char *temp = (char *)NULL;
@@ -137,8 +165,7 @@ reduce_path(const char *str)
 static size_t
 copy_char(char *buf, char *s)
 {
-	const int bytes = ((*s & 0xc0) == 0xc0 || (*s & 0xc0) == 0x80)
-		? utf8_bytes((unsigned char)*s) : 1;
+	const int bytes = IS_UTF8_CHAR(*s) ? utf8_bytes((unsigned char)*s) : 1;
 
 	if (bytes == 1) {
 		*buf = *s;
@@ -155,7 +182,7 @@ copy_char(char *buf, char *s)
 	return (size_t)c;
 }
 
-static inline char *
+static char *
 reduce_path_fish(char *str)
 {
 	if ((*str == '~' || *str == '/') && !str[1])
@@ -228,7 +255,7 @@ reduce_path_fish(char *str)
 	return buf;
 }
 
-static inline char *
+static char *
 gen_pwd(const int c)
 {
 	char *temp = (char *)NULL;
@@ -257,12 +284,12 @@ gen_pwd(const int c)
 	return temp;
 }
 
-static inline char *
+static char *
 gen_workspace(void)
 {
 	/* An int (or workspace name) + workspaces color + NUL byte */
 	char s[NAME_MAX + sizeof(ws1_c) + 1];
-	char *cl = (char *)NULL;
+	char *cl = df_c;
 
 	if (conf.colorize == 1) {
 		switch (cur_ws + 1) {
@@ -276,8 +303,6 @@ gen_workspace(void)
 		case 8: cl = *ws8_c ? ws8_c : DEF_WS8_C; break;
 		default: break;
 		}
-	} else {
-		cl = df_c;
 	}
 
 	if (workspaces[cur_ws].name)
@@ -289,7 +314,7 @@ gen_workspace(void)
 	return savestring(s, strnlen(s, sizeof(s)));
 }
 
-static inline char *
+static char *
 gen_exit_status(void)
 {
 	const size_t code_len = (size_t)DIGINUM(exit_code);
@@ -303,7 +328,7 @@ gen_exit_status(void)
 	return temp;
 }
 
-static inline char *
+static char *
 gen_escape_char(char **line, int *c)
 {
 	(*line)++;
@@ -316,15 +341,22 @@ gen_escape_char(char **line, int *c)
 	return temp;
 }
 
-static inline char *
+static char *
 gen_octal(char **line, int *c)
 {
 	char octal_string[4];
-
 	xstrsncpy(octal_string, *line, sizeof(octal_string));
-	octal_string[3] = '\0';
 
-	const int n = read_octal(octal_string);
+	int n = octal2int(octal_string);
+#ifdef CHAR_MAX
+	/* Max octal number: 0177 (char is signed) or 0377 (char is unsigned). */
+	if (n > CHAR_MAX)
+		n = CHAR_MAX;
+#else
+	if (n > 127)
+		n = 127;
+#endif /* CHAR_MAX */
+
 	char *temp = xnmalloc(3, sizeof(char));
 
 	if (n == CTLESC || n == CTLNUL) {
@@ -346,7 +378,7 @@ gen_octal(char **line, int *c)
 	return temp;
 }
 
-static inline char *
+static char *
 gen_profile(void)
 {
 	if (!alt_profile)
@@ -355,7 +387,7 @@ gen_profile(void)
 	return savestring(alt_profile, strlen(alt_profile));
 }
 
-static inline char *
+static char *
 gen_user_name(void)
 {
 	if (!user.name)
@@ -364,7 +396,13 @@ gen_user_name(void)
 	return savestring(user.name, strlen(user.name));
 }
 
-static inline char *
+static char *
+gen_sort_name(void)
+{
+	return strdup(num_to_sort_name(conf.sort, 1));
+}
+
+static char *
 gen_hostname(const int c)
 {
 	/* Using strnlen() here avoids a Redhat hardened compilation warning. */
@@ -379,7 +417,7 @@ gen_hostname(const int c)
 	return temp;
 }
 
-static inline char *
+static char *
 gen_user_flag(void)
 {
 	char *temp = xnmalloc(2, sizeof(char));
@@ -389,22 +427,17 @@ gen_user_flag(void)
 	return temp;
 }
 
-static inline char *
+static char *
 gen_mode(void)
 {
 	char *temp = xnmalloc(2, sizeof(char));
-
-	if (conf.light_mode == 1) {
-		*temp = LIGHT_MODE_CHAR;
-		temp[1] = '\0';
-	} else {
-		*temp = '\0';
-	}
+	*temp = conf.light_mode == 1 ? LIGHT_MODE_CHAR : '\0';
+	temp[1] = '\0';
 
 	return temp;
 }
 
-static inline char*
+static char *
 gen_misc(const int c)
 {
 	char *temp = xnmalloc(2, sizeof(char));
@@ -421,17 +454,17 @@ gen_misc(const int c)
 	return temp;
 }
 
-static inline char *
+static char *
 gen_non_print_sequence(const int c)
 {
 	char *temp = xnmalloc(2, sizeof(char));
-	*temp = (c == '[') ? RL_PROMPT_START_IGNORE : RL_PROMPT_END_IGNORE;
+	*temp = c == '[' ? RL_PROMPT_START_IGNORE : RL_PROMPT_END_IGNORE;
 	temp[1] = '\0';
 
 	return temp;
 }
 
-static inline char *
+static char *
 gen_shell_name(void)
 {
 	if (user.shell && user.shell_basename)
@@ -440,7 +473,7 @@ gen_shell_name(void)
 	return savestring("unknown", 7);
 }
 
-static inline void
+static void
 add_string(char **tmp, const int c, char **line, char **res, size_t *len)
 {
 	if (!*tmp)
@@ -464,7 +497,7 @@ add_string(char **tmp, const int c, char **line, char **res, size_t *len)
 }
 
 #ifndef NO_WORDEXP
-static inline void
+static void
 reset_ifs(const char *value)
 {
 	if (value)
@@ -473,7 +506,7 @@ reset_ifs(const char *value)
 		unsetenv("IFS");
 }
 
-static inline void
+static void
 substitute_cmd(char **line, char **res, size_t *len)
 {
 	char *p = strchr(*line, ')');
@@ -523,7 +556,7 @@ END:
 }
 #endif /* !NO_WORDEXP */
 
-static inline char *
+static char *
 gen_emergency_prompt(void)
 {
 	static int f = 0;
@@ -535,7 +568,7 @@ gen_emergency_prompt(void)
 	return savestring(EMERGENCY_PROMPT, sizeof(EMERGENCY_PROMPT) - 1);
 }
 
-static inline char *
+static char *
 gen_stats_str(const int flag)
 {
 	size_t val = 0;
@@ -564,6 +597,10 @@ gen_stats_str(const int flag)
 	case STATS_STICKY: val = stats.sticky; break;
 	case STATS_UNKNOWN: val = stats.unknown; break;
 	case STATS_UNSTAT: val = stats.unstat; break;
+	case STATS_NON_DIR:
+		val = stats.reg + stats.block_dev + stats.char_dev
+			+ stats.socket + stats.fifo;
+		break;
 	default: break;
 	}
 
@@ -580,15 +617,33 @@ gen_stats_str(const int flag)
 	return p;
 }
 
-static inline char *
+static size_t
+count_autocmd_matches(void)
+{
+	if (autocmds_n == 0)
+		return 0;
+
+	size_t c = 0;
+	ssize_t i = (ssize_t)autocmds_n;
+	while (--i >= 0)
+		c += (autocmds[i].match == 1);
+
+	return c;
+}
+
+static char *
 gen_notification(const int flag)
 {
-	const size_t len = MAX_INT_STR + 1;
+	const size_t len = MAX_INT_STR + 2;
 
 	char *p = xnmalloc(len, sizeof(char));
 	*p = '\0';
 
 	switch (flag) {
+	case NOTIF_AUTOCMD:
+		if (count_autocmd_matches() > 0)
+			{ *p = 'A'; p[1] = '\0'; }
+		break;
 	case NOTIF_ERROR:
 		if (msgs.error > 0)
 			snprintf(p, len, "E%zu", msgs.error);
@@ -619,7 +674,7 @@ gen_notification(const int flag)
 	return p;
 }
 
-static inline char *
+static char *
 gen_nesting_level(const int mode)
 {
 	char *p = (char *)NULL;
@@ -637,14 +692,14 @@ gen_nesting_level(const int mode)
 		return p;
 	}
 
-	const size_t len = (MAX_COLOR * 2) + MAX_INT_STR;
+	const size_t len = MAX_INT_STR + 3;
 	p = xnmalloc(len, sizeof(char));
 	snprintf(p, len, "(%d)", nesting_level);
 
 	return p;
 }
 
-static char *
+static const char *
 get_color_attribute(const char *line)
 {
 	if (!line || !line[0] || line[1] != ':')
@@ -679,9 +734,7 @@ is_valid_hex(const char *s)
 
 	size_t i;
 	for (i = 0; s[i]; i++) {
-		if ( !( (s[i] >= '0' && s[i] <= '9')
-		|| (s[i] >= 'a' && s[i] <= 'f')
-		|| (s[i] >= 'A' && s[i] <= 'F') ) )
+		if (!IS_HEX_DIGIT(s[i]))
 			return 0;
 	}
 
@@ -701,7 +754,7 @@ gen_color(char **line)
 	char *l = (*line) + 1; /* L is now "color}" */
 
 	const int bg = (l[0] == 'k' && l[1] == ':' && l[2]);
-	char *attr = bg == 0 ? get_color_attribute(l) : (char *)NULL;
+	const char *attr = bg == 0 ? get_color_attribute(l) : (char *)NULL;
 	if (bg == 1 || attr)
 		l += 2; /* Remove background/attribute prefix ("x:") */
 
@@ -793,6 +846,13 @@ gen_color(char **line)
 	*p = '}'; /* Restore the trailing '}' */
 	*line = p; /* Set LINE to the end of the color notation */
 	return temp;
+
+#undef GEN_COLOR
+#undef GEN_ATTR
+#undef C_START
+#undef C_END
+#undef C_ESC
+#undef C_LEN
 }
 
 #ifndef NO_WORDEXP
@@ -908,6 +968,20 @@ END:
 	return (char *)NULL;
 }
 
+static char *
+gen_cwd_perms(void)
+{
+	struct stat a;
+	if (!workspaces || !workspaces[cur_ws].path
+	|| stat(workspaces[cur_ws].path, &a) == -1)
+		return savestring(UNKNOWN_STR, sizeof(UNKNOWN_STR) - 1);
+
+	char *buf = xnmalloc(5, sizeof(char));
+	snprintf(buf, 5, "%04o", a.st_mode & 07777);
+
+	return buf;
+}
+
 /* Decode the prompt string (encoded_prompt global variable) taken from
  * the configuration file. */
 char *
@@ -934,7 +1008,7 @@ decode_prompt(char *line)
 			/* Now move on to the next char */
 			c = (int)*line;
 			switch (c) {
-			/* Files statistics */
+			/* File statistics */
 			case 'B': temp = gen_stats_str(STATS_BLK); goto ADD_STRING;
 			case 'C': temp = gen_stats_str(STATS_CHR); goto ADD_STRING;
 			case 'D': temp = gen_stats_str(STATS_DIR); goto ADD_STRING;
@@ -946,6 +1020,7 @@ decode_prompt(char *line)
 			case 'M': temp = gen_stats_str(STATS_MULTI_L); goto ADD_STRING;
 			case 'o': temp = gen_stats_str(STATS_BROKEN_L); goto ADD_STRING;
 			case 'O': temp = gen_stats_str(STATS_OTHER_W); goto ADD_STRING;
+			case 'Q': temp = gen_stats_str(STATS_NON_DIR); goto ADD_STRING;
 			case 'R': temp = gen_stats_str(STATS_REG); goto ADD_STRING;
 			case 'U': temp = gen_stats_str(STATS_SUID); goto ADD_STRING;
 			case 'x': temp = gen_stats_str(STATS_CAP); goto ADD_STRING;
@@ -966,12 +1041,17 @@ decode_prompt(char *line)
 			case '(': temp = gen_notification(NOTIF_ERROR); goto ADD_STRING;
 			case '=': temp = gen_notification(NOTIF_NOTICE); goto ADD_STRING;
 
+			case 'v': temp = gen_rl_vi_mode(1); goto ADD_STRING;
+			case 'y': temp = gen_notification(NOTIF_AUTOCMD); goto ADD_STRING;
+
 			case 'z': /* Exit status of last executed command */
 				temp = gen_exit_status(); goto ADD_STRING;
 
 			case 'e': /* Escape char */
 				temp = gen_escape_char(&line, &c); goto ADD_STRING;
 
+			case 'j': temp = gen_cwd_perms(); goto ADD_STRING;
+
 			case '0': /* fallthrough */ /* Octal char */
 			case '1': /* fallthrough */
 			case '2': /* fallthrough */
@@ -1002,6 +1082,9 @@ decode_prompt(char *line)
 			case 'u': /* User name */
 				temp = gen_user_name(); goto ADD_STRING;
 
+			case 'g':
+				temp = gen_sort_name(); goto ADD_STRING;
+
 			case 'h': /* fallthrough */ /* Hostname up to first '.' */
 			case 'H': /* Full hostname */
 				temp = gen_hostname(c); goto ADD_STRING;
@@ -1093,7 +1176,7 @@ ADD_STRING:
 }
 
 /* Make sure CWD exists; if not, go up to the parent, and so on */
-static inline void
+static void
 check_cwd(void)
 {
 	int cwd_change = 0;
@@ -1113,13 +1196,12 @@ check_cwd(void)
 }
 
 /* Remove all final slash(es) from path, if any */
-static inline void
+static void
 trim_final_slashes(void)
 {
 	const size_t path_len = strlen(workspaces[cur_ws].path);
-	size_t i;
 
-	for (i = path_len - 1; workspaces[cur_ws].path[i] && i > 0; i--) {
+	for (size_t i = path_len - 1; workspaces[cur_ws].path[i] && i > 0; i--) {
 		if (workspaces[cur_ws].path[i] != '/')
 			break;
 		else
@@ -1168,7 +1250,7 @@ print_user_message(void)
 	fputs(df_c, stdout);
 }
 
-static inline void
+static void
 print_welcome_msg(void)
 {
 	static int message_shown = 0;
@@ -1186,7 +1268,7 @@ print_welcome_msg(void)
 	message_shown = 1;
 }
 
-static inline void
+static void
 print_tips_func(void)
 {
 	if (conf.tips == 0)
@@ -1199,7 +1281,7 @@ print_tips_func(void)
 	}
 }
 
-static inline void
+static void
 run_prompt_cmds(void)
 {
 	if (conf.ext_cmd_ok == 0 || prompt_cmds_n == 0)
@@ -1207,9 +1289,8 @@ run_prompt_cmds(void)
 
 	const int tflags = flags;
 	flags &= ~DELAYED_REFRESH;
-	size_t i;
 
-	for (i = 0; i < prompt_cmds_n; i++) {
+	for (size_t i = 0; i < prompt_cmds_n; i++) {
 		if (xargs.secure_cmds == 0
 		|| sanitize_cmd(prompt_cmds[i], SNT_PROMPT) == FUNC_SUCCESS)
 			launch_execl(prompt_cmds[i]);
@@ -1237,14 +1318,11 @@ update_trash_indicator(void)
 	trash_files_dir_mtime = a.st_mtime;
 
 	const filesn_t n = count_dir(trash_files_dir, NO_CPOP);
-	if (n <= 2)
-		trash_n = 0;
-	else
-		trash_n = (size_t)n - 2;
+	trash_n = n <= 2 ? 0 : (size_t)n - 2;
 }
 #endif /* !_NO_TRASH */
 
-static inline void
+static void
 setenv_prompt(void)
 {
 	if (prompt_notif == 1)
@@ -1266,21 +1344,22 @@ setenv_prompt(void)
 	setenv("CLIFM_STAT_STEALTH", (xargs.stealth_mode == 1) ? "1" : "0", 1);
 }
 
-static inline size_t
-set_prompt_length(const size_t decoded_prompt_len)
+static size_t
+set_prompt_length(const size_t decoded_prompt_len, const size_t ac_matches)
 {
 	size_t len = 0;
 
 	if (prompt_notif == 1) {
 		len = (size_t)(decoded_prompt_len
+		+ (ac_matches > 0 ? N_IND : 0)
 		+ (xargs.stealth_mode == 1 ? STEALTH_IND_SIZE : 0)
 		+ (user.uid == 0 ? ROOT_IND_SIZE : 0)
 		+ (conf.readonly == 1 ? RDONLY_IND_SIZE : 0)
-		+ ((sel_n > 0) ? N_IND : 0)
-		+ ((trash_n > 0) ? N_IND : 0)
-		+ ((msgs.error > 0) ? N_IND : 0)
-		+ ((msgs.warning > 0) ? N_IND : 0)
-		+ ((msgs.notice > 0) ? N_IND : 0)
+		+ (sel_n > 0 ? N_IND : 0)
+		+ (trash_n > 0 ? N_IND : 0)
+		+ (msgs.error > 0 ? N_IND : 0)
+		+ (msgs.warning > 0 ? N_IND : 0)
+		+ (msgs.notice > 0 ? N_IND : 0)
 		+ 6 + sizeof(tx_c) + 1 + 2);
 	} else {
 		len = (size_t)(decoded_prompt_len + 6 + sizeof(tx_c) + 1);
@@ -1289,15 +1368,25 @@ set_prompt_length(const size_t decoded_prompt_len)
 	return len;
 }
 
-static inline char *
+static char *
 construct_prompt(const char *decoded_prompt)
 {
+	char *rl_vi_mode = (char *)NULL;
 	/* Construct indicators: MSGS (ERR, WARN, and NOTICE), SEL, and TRASH */
 	char err_ind[N_IND], warn_ind[N_IND],
-		notice_ind[N_IND], trash_ind[N_IND], sel_ind[N_IND];
+		notice_ind[N_IND], trash_ind[N_IND], sel_ind[N_IND],
+		acmd_ind[N_IND];
 	*err_ind = *warn_ind = *notice_ind = *trash_ind = *sel_ind = '\0';
+	*acmd_ind = '\0';
+
+	size_t ac_matches = 0;
 
 	if (prompt_notif == 1) {
+		if (rl_editing_mode == RL_VI_MODE)
+			rl_vi_mode = gen_rl_vi_mode(0);
+		if (conf.autocmd_msg == AUTOCMD_MSG_PROMPT
+		&& (ac_matches = count_autocmd_matches()) > 0)
+			snprintf(acmd_ind, N_IND, "%sA%s", ac_c, RL_NC);
 		if (msgs.error > 0)
 			snprintf(err_ind, N_IND, "%sE%zu%s", em_c, msgs.error, RL_NC);
 		if (msgs.warning > 0)
@@ -1312,12 +1401,15 @@ construct_prompt(const char *decoded_prompt)
 				sel_n, RL_NC);
 	}
 
-	const size_t prompt_len = set_prompt_length(strlen(decoded_prompt));
+	const size_t prompt_len = (rl_vi_mode ? strlen(rl_vi_mode) : 0) +
+		set_prompt_length(strlen(decoded_prompt), ac_matches);
 	char *the_prompt = xnmalloc(prompt_len, sizeof(char));
 
 	if (prompt_notif == 1) {
 		snprintf(the_prompt, prompt_len,
-			"%s%s%s%s%s%s%s%s%s%s%s%s\001%s\002",
+			"%s%s%s%s%s%s%s%s%s%s%s%s%s%s\001%s\002",
+			rl_vi_mode ? rl_vi_mode : "",
+			ac_matches > 0 ? acmd_ind : "",
 			(user.uid == 0) ? (conf.colorize == 1
 				? ROOT_IND : ROOT_IND_NO_COLOR) : "",
 			(conf.readonly == 1) ? ro_c : "",
@@ -1338,7 +1430,21 @@ construct_prompt(const char *decoded_prompt)
 	return the_prompt;
 }
 
-static inline void
+static void
+print_prompt_messages(void)
+{
+	for (size_t i = 0; i < msgs_n; i++) {
+		if (messages[i].read == 1)
+			continue;
+
+		fputs(messages[i].text, stderr);
+		messages[i].read = 1;
+	}
+
+	print_msg = 0;
+}
+
+static void
 initialize_prompt_data(const int prompt_flag)
 {
 	check_cwd();
@@ -1348,16 +1454,16 @@ initialize_prompt_data(const int prompt_flag)
 
 	/* If autols is disabled, and since terminal dimensions are gathered
 	 * in list_dir() via get_term_size(), let's get terminal dimensions
-	 * here. We need them to print suggestions */
+	 * here. We need them to print suggestions. */
 	if (conf.autols == 0 && conf.suggestions == 1)
 		get_term_size();
 
-	/* Set foreground color to default */
+	/* Set foreground color to default. */
 	fputs(df_c, stdout);
 	fflush(stdout);
 
+	/* If just updating the prompt, there's no need to run prompt commands. */
 	if (prompt_flag != PROMPT_UPDATE)
-	/* We're just updating the prompt string: no need to run prompt commands. */
 		run_prompt_cmds();
 
 #ifndef _NO_TRASH
@@ -1376,14 +1482,11 @@ initialize_prompt_data(const int prompt_flag)
 	}
 #endif /* !_NO_SUGGESTIONS */
 
-	/* Print error messages */
-	if (print_msg == 1 && msgs_n > 0) {
-		fputs(messages[msgs_n - 1], stderr);
-		print_msg = 0; /* Print messages only once */
-	}
+	if (print_msg == 1 && msgs_n > 0)
+		print_prompt_messages();
 }
 
-static inline void
+static void
 log_and_record(char *input)
 {
 	if (conf.log_cmds == 1) {
@@ -1439,7 +1542,7 @@ get_rprompt_len(char *rprompt)
 {
 	char *p = rprompt;
 	while (*p) {
-		if ((*p & 0xc0) == 0xc0 || (*p & 0xc0) == 0x80)
+		if (IS_UTF8_CHAR(*p))
 			return get_rprompt_len_utf8(rprompt);
 		p++;
 	}
@@ -1543,27 +1646,25 @@ expand_history(char **input)
 }
 
 static char *
-handle_empty_line(void)
+handle_empty_line(const int screen_refresh)
 {
 	if (conf.autols == 1 && ((flags & DELAYED_REFRESH)
-	|| xargs.refresh_on_empty_line == 1) && rl_pending_input == 0) {
-		flags &= ~DELAYED_REFRESH;
+	|| xargs.refresh_on_empty_line == 1)
+	&& screen_refresh == PROMPT_SCREEN_REFRESH
+	&& g_prompt_ignore_empty_line == 0)
 		refresh_screen();
-	} else {
-		flags &= ~DELAYED_REFRESH;
-	}
 
-	rl_pending_input = 0;
+	g_prompt_ignore_empty_line = 0;
+	flags &= ~DELAYED_REFRESH;
 	return (char *)NULL;
 }
 
 /* Print the prompt and return the string entered by the user, to be
  * parsed later by parse_input_str() */
 char *
-prompt(const int prompt_flag)
+prompt(const int prompt_flag, const int screen_refresh)
 {
 	initialize_prompt_data(prompt_flag);
-
 	/* Generate the prompt string using the prompt line in the config
 	 * file (stored in encoded_prompt at startup). */
 	char *decoded_prompt = decode_prompt(conf.encoded_prompt);
@@ -1595,16 +1696,16 @@ prompt(const int prompt_flag)
 
 	if (!input || !*input || rl_end == 0) {
 		free(input);
-		return handle_empty_line();
+		return handle_empty_line(screen_refresh);
 	}
 
+	g_prompt_ignore_empty_line = 0;
 	flags &= ~DELAYED_REFRESH;
 
 	if (expand_history(&input) != FUNC_SUCCESS)
 		return (char *)NULL;
 
 	log_and_record(input);
-
 	return input;
 }
 
@@ -1612,18 +1713,18 @@ static int
 list_prompts(void)
 {
 	if (prompts_n == 0) {
-		puts(_("prompt: No extra prompts found. Using the default prompt"));
+		puts(_("prompt: No extra prompts found. Using the default prompt."));
 		return FUNC_SUCCESS;
 	}
 
-	size_t i;
-	for (i = 0; i < prompts_n; i++) {
+	const char *ptr = SET_MISC_PTR;
+	for (size_t i = 0; i < prompts_n; i++) {
 		if (!prompts[i].name)
 			continue;
 
 		if (*cur_prompt_name == *prompts[i].name
 		&& strcmp(cur_prompt_name, prompts[i].name) == 0)
-			printf("%s>%s %s\n", mi_c, df_c, prompts[i].name);
+			printf("%s%s%s %s\n", mi_c, ptr, df_c, prompts[i].name);
 		else
 			printf("  %s\n", prompts[i].name);
 	}
@@ -1707,8 +1808,8 @@ set_default_prompt(void)
 	return FUNC_SUCCESS;
 }
 
-/* Read environment variables controling options for the '\b', '\f', and
- * '\p' prompt codes and set the appropriate values. */
+/* Read environment variables controlling options for the '\b', '\f', and
+ * '\p' prompt escape codes, and set the appropriate values. */
 void
 set_prompt_options(void)
 {
@@ -1727,6 +1828,8 @@ set_prompt_options(void)
 	const int f_is_set = CHECK_PROMPT_OPT("\\f");
 	const int p_is_set = CHECK_PROMPT_OPT("\\p");
 
+#undef CHECK_PROMPT_OPT
+
 	conf.prompt_b_is_set = b_is_set;
 
 	if (f_is_set == 1) {
@@ -1793,7 +1896,7 @@ edit_prompts_file(char *app)
 		reload_dirlist();
 
 	ret = load_prompts();
-	print_reload_msg(_("File modified. Prompts reloaded\n"));
+	print_reload_msg(NULL, NULL, _("File modified. Prompts reloaded.\n"));
 
 	if (*cur_prompt_name)
 		set_prompt(cur_prompt_name);
