@@ -1,17 +1,15 @@
-/* properties.c -- Home of the p/pp, pc, oc, and stats commands */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* properties.c -- Home of the p/pp, pc, oc, and stats commands */
+
 /* These four functions: get_color_size, get_color_size256, get_color_age,
  * and get_color_age256 are based on https://github.com/leahneukirchen/lr
  * (licenced MIT) and modified to fit our needs.
@@ -33,7 +33,6 @@
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <time.h>
 #include <unistd.h>
@@ -94,9 +93,7 @@
 #include "checks.h" /* check_file_access(), is_number() */
 #include "colors.h" /* get_dir_color(), get_regfile_color() */
 #include "messages.h"
-#ifndef _NO_MAGIC
 # include "mime.h"  /* xmagic() */
-#endif /* !_NO_MAGIC */
 #include "misc.h"
 #include "properties.h"
 #include "readline.h"   /* Required by the 'pc' command */
@@ -204,10 +201,7 @@ get_link_color(const char *name)
 		return color;
 
 	if (S_ISDIR(a.st_mode)) {
-		if (check_file_access(a.st_mode, a.st_uid, a.st_gid) == 1)
-			color = get_dir_color(name, a.st_mode, a.st_nlink, -1);
-		else
-			color = nd_c;
+		color = get_dir_color(name, &a, -1);
 	} else {
 		switch (a.st_mode & S_IFMT) {
 		case S_IFLNK:  color = stat(name, &a) == -1 ? or_c : ln_c; break;
@@ -384,7 +378,7 @@ static int
 validate_new_perms(const char *s)
 {
 	const size_t l = strlen(s);
-	if (*s >= '0' && *s <= '9')
+	if (IS_DIGIT(*s))
 		return validate_octal_perms(s, l);
 
 	if (l != 9) {
@@ -435,14 +429,14 @@ get_new_perms(const char *str, const int diff)
 {
 	const int poffset_bk = prompt_offset;
 	prompt_offset = 3;
-	alt_prompt = 2; /* Disable TAB completion for this prompt */
+	alt_prompt = PERMISSIONS_PROMPT;
 	rl_nohist = 1;
 
 	if (diff == 1) {
 		printf(_("%sFiles with different sets of permissions\n"
 			"Only shared permission bits are set in the template\n"), tx_c);
 	}
-	printf(_("%sEdit file permissions (Ctrl-d to quit)\n"
+	printf(_("%sEdit file permissions (Ctrl+d to quit)\n"
 		"Both symbolic and numeric notation are supported\n"), tx_c);
 	char m[(MAX_COLOR * 2) + 7];
 	snprintf(m, sizeof(m), "\001%s\002>\001%s\002 ", mi_c, tx_c);
@@ -613,14 +607,14 @@ get_new_ownership(const char *str, const int diff)
 {
 	const int poffset_bk = prompt_offset;
 	prompt_offset = 3;
-	alt_prompt = 3; /* Allow completion only for user and group names */
+	alt_prompt = OWNERSHIP_PROMPT;
 	rl_nohist = 1;
 
 	if (diff == 1) {
 		printf(_("%sFiles with different owners\n"
 			"Only common owners are set in the template\n"), tx_c);
 	}
-	printf(_("%sEdit file ownership (Ctrl-d to quit)\n"
+	printf(_("%sEdit file ownership (Ctrl+d to quit)\n"
 		"Both ID numbers and names are supported\n"), tx_c);
 	char m[(MAX_COLOR * 2) + 7];
 	snprintf(m, sizeof(m), "\001%s\002>\001%s\002 ", mi_c, tx_c);
@@ -711,7 +705,7 @@ set_file_owner(char **args)
 		return exit_status;
 
 	/* Neither owners nor primary groups are common */
-	if (*own == ':' && !*(own + 1))
+	if (*own == ':' && !own[1])
 		*own = '\0';
 
 	char *new_own = get_new_ownership(own, diff);
@@ -725,7 +719,7 @@ set_file_owner(char **args)
 	char *new_group = (strchr(new_own, ':'));
 	if (new_group) {
 		*new_group = '\0';
-		if (!*(new_group + 1))
+		if (!new_group[1])
 			new_group = (char *)NULL;
 	}
 
@@ -780,14 +774,16 @@ set_file_owner(char **args)
 		}
 
 		if (*new_own && owner->pw_uid != a.st_uid) {
-			printf(_("%s->%s %s: User set to %d (%s%s%s)\n"),
-				mi_c, NC, args[i], owner->pw_uid, BOLD, owner->pw_name, NC);
+			printf(_("%s%s%s %s: User set to %d (%s%s%s)\n"),
+				mi_c, SET_MSG_PTR, NC, args[i], owner->pw_uid,
+				BOLD, owner->pw_name, NC);
 			new_o++;
 		}
 
 		if (new_group && group->gr_gid != a.st_gid) {
-			printf(_("%s->%s %s: Primary group set to %d (%s%s%s)\n"),
-				mi_c, NC, args[i], group->gr_gid, BOLD, group->gr_name, NC);
+			printf(_("%s%s%s %s: Primary group set to %d (%s%s%s)\n"),
+				mi_c, SET_MSG_PTR, NC, args[i], group->gr_gid, BOLD,
+				group->gr_name, NC);
 			new_g++;
 		}
 	}
@@ -1013,10 +1009,7 @@ get_file_type_and_color(const char *filename, const struct stat *attr,
 		if (conf.colorize == 0)
 			break;
 
-		if (check_file_access(attr->st_mode, attr->st_uid, attr->st_gid) == 0)
-			color = nd_c;
-		else
-			color = get_dir_color(filename, attr->st_mode, attr->st_nlink, -1);
+		color = get_dir_color(filename, attr, -1);
 		break;
 
 	case S_IFLNK: {
@@ -1093,8 +1086,10 @@ print_filename(char *filename, const char *color, const int follow_link,
 			char t[PATH_MAX * sizeof(wchar_t) + 3]; *t = '\0';
 			if (detect_space(link_target) == 1)
 				snprintf(t, sizeof(t), "'%s'", link_target);
-			printf(_("\tName: %s%s%s %s<-%s %s%s%s\n"), color,
-				*t ? t : link_target, df_c, dn_c, df_c, ln_c, n, df_c);
+			printf(_("\tName: %s%s%s %s%s%s %s%s%s\n"), color,
+				*t ? t : link_target, df_c, dn_c,
+				term_caps.unicode == 1 ? MSG_PTR_STR_LEFT_U : MSG_PTR_STR_LEFT,
+				df_c, ln_c, n, df_c);
 		} else {
 			printf(_("\tName: %s%s%s\n"), color, n, df_c);
 		}
@@ -1121,16 +1116,16 @@ print_filename(char *filename, const char *color, const int follow_link,
 	struct stat a;
 	if (tlen != -1 && *target && lstat(target, &a) != -1) {
 		char *link_color = get_link_color(target);
-		printf(_("\tName: %s%s%s %s->%s %s%s%s\n"), ln_c, n, df_c,
-			dn_c, df_c, link_color, *t ? t : target, df_c);
+		printf(_("\tName: %s%s%s %s%s%s %s%s%s\n"), ln_c, n, df_c,
+			dn_c, SET_MSG_PTR, df_c, link_color, *t ? t : target, df_c);
 
 	} else { /* Broken link */
 		if (tlen != -1 && *target) {
-			printf(_("\tName: %s%s%s %s->%s %s%s%s (broken link)\n"), or_c,
-				n, df_c, dn_c, df_c, uf_c, *t ? t : target, df_c);
+			printf(_("\tName: %s%s%s %s%s%s %s%s%s (broken link)\n"), or_c,
+				n, df_c, dn_c, SET_MSG_PTR, df_c, uf_c, *t ? t : target, df_c);
 		} else {
-			printf(_("\tName: %s%s%s %s-> ???%s\n"), or_c, n, df_c,
-				dn_c, df_c);
+			printf(_("\tName: %s%s%s %s%s ???%s\n"), or_c, n, df_c,
+				dn_c, SET_MSG_PTR, df_c);
 		}
 	}
 
@@ -1362,7 +1357,7 @@ has_nsec_modifier(char *fmt)
 		return (char *)NULL;
 
 	while (*fmt) {
-		if (*fmt == '%' && *(fmt + 1) == 'N')
+		if (*fmt == '%' && fmt[1] == 'N')
 			return fmt;
 		++fmt;
 	}
@@ -1370,6 +1365,8 @@ has_nsec_modifier(char *fmt)
 	return (char *)NULL;
 }
 
+#pragma GCC diagnostic push
+#pragma GCC diagnostic ignored "-Wformat-nonliteral"
 static void
 gen_user_time_str(char *buf, size_t buf_size, struct tm *t, const size_t nsec)
 {
@@ -1377,14 +1374,7 @@ gen_user_time_str(char *buf, size_t buf_size, struct tm *t, const size_t nsec)
 	if (!ptr) {
 		/* GCC (not clang) complains about format being not a string literal.
 		 * Let's silence this warning until we find a better approach. */
-#ifdef GCC_ALLOWS_PRAGMA_IN_FUNC
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif /* GCC_ALLOWS_PRAGMA_IN_FUNC */
 		strftime(buf, buf_size, conf.ptime_str, t);
-#ifdef GCC_ALLOWS_PRAGMA_IN_FUNC
-# pragma GCC diagnostic pop
-#endif /* GCC_ALLOWS_PRAGMA_IN_FUNC */
 		return;
 	}
 
@@ -1397,14 +1387,7 @@ gen_user_time_str(char *buf, size_t buf_size, struct tm *t, const size_t nsec)
 	size_t len = 0;
 	*ptr = '\0';
 	if (ptr != conf.ptime_str)
-#ifdef GCC_ALLOWS_PRAGMA_IN_FUNC
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif /* GCC_ALLOWS_PRAGMA_IN_FUNC */
 		len = strftime(buf, buf_size, conf.ptime_str, t);
-#ifdef GCC_ALLOWS_PRAGMA_IN_FUNC
-# pragma GCC diagnostic pop
-#endif /* GCC_ALLOWS_PRAGMA_IN_FUNC */
 
 	*ptr = '%';
 	if (len == 0 && ptr != conf.ptime_str) /* Error or exhausted space in BUF. */
@@ -1414,19 +1397,13 @@ gen_user_time_str(char *buf, size_t buf_size, struct tm *t, const size_t nsec)
 	if (len >= buf_size) /* Error or exhausted space in BUF. */
 		return;
 
-	ptr += 2; /* Move past the %N modifier in format string */
+	ptr += 2; /* Move past the %N modifier in format string. */
 	if (!*ptr)
 		return;
 
-#ifdef GCC_ALLOWS_PRAGMA_IN_FUNC
-# pragma GCC diagnostic push
-# pragma GCC diagnostic ignored "-Wformat-nonliteral"
-#endif /* GCC_ALLOWS_PRAGMA_IN_FUNC */
 	strftime(buf + len, buf_size - len, ptr, t);
-#ifdef GCC_ALLOWS_PRAGMA_IN_FUNC
-# pragma GCC diagnostic pop
-#endif /* GCC_ALLOWS_PRAGMA_IN_FUNC */
 }
+#pragma GCC diagnostic pop
 
 /* Write into BUF, whose size is SIZE, the timestamp TIM, with nanoseconds NSEC,
  * in human readable format.
@@ -1453,12 +1430,12 @@ xgen_time_str(char *buf, const size_t buf_size, const time_t tim,
 
 	*buf = '\0';
 
-	if (conf.ptime_str) { /* User-defined time format */
+	if (conf.ptime_str) { /* User-defined time format. */
 		gen_user_time_str(buf, buf_size, &t, nsec);
 		return;
 	}
 
-	/* Default value */
+	/* Default value. */
 	size_t len = strftime(buf, buf_size, "%F %T", &t);
 	if (len == 0) /* Error or exhausted space in BUF. */
 		return;
@@ -1541,14 +1518,14 @@ print_timestamps(char *filename, const struct stat *attr)
 			attrx.ST_BTIME.tv_sec, (size_t)attrx.ST_BTIME.tv_nsec);
 		bt = attrx.ST_BTIME.tv_sec;
 	} else {
-		/* Birthtime is not available */
+		/* Birthtime is not available. */
 		*creation_time = '-';
 		creation_time[1] = '\0';
 	}
 
 # elif defined(__sun)
 	struct timespec birthtim = get_birthtime(filename);
-	if (birthtim.tv_sec == (time_t)-1) { /* Error */
+	if (birthtim.tv_sec == (time_t)-1) { /* Error. */
 		*creation_time = '-';
 		creation_time[1] = '\0';
 	} else {
@@ -1569,7 +1546,7 @@ print_timestamps(char *filename, const struct stat *attr)
 	}
 
 	printf(_("Birth: \t\t%s%s%s\n"), cbdate, creation_time, cend);
-#elif !defined(_BE_POSIX) /* Birthtime not supported */
+#elif !defined(_BE_POSIX) /* Birthtime is not supported. */
 	printf(_("Birth: \t\t%s-%s\n"), dn_c, cend);
 #endif /* ST_BTIME */
 }
@@ -1580,8 +1557,8 @@ err_no_file(const char *filename, const char *target, const int errnum)
 	char *errname = xargs.stat > 0 ? PROGRAM_NAME : "prop";
 
 	if (*target) {
-		xerror(_("%s: %s %s->%s %s: Broken symbolic link\n"),
-			errname, filename, mi_c, df_c, target);
+		xerror(_("%s: %s %s%s%s %s: Broken symbolic link\n"),
+			errname, filename, mi_c, SET_MSG_PTR, df_c, target);
 	} else {
 		xerror("%s: '%s': %s\n", errname, filename, strerror(errnum));
 	}
@@ -1592,10 +1569,6 @@ err_no_file(const char *filename, const char *target, const int errnum)
 static void
 print_file_mime(const char *name)
 {
-#ifdef _NO_MAGIC
-	UNUSED(name);
-	return;
-#else
 	fputs("MIME type:\t", stdout);
 	char *n = xmagic(name, MIME_TYPE);
 	if (n) {
@@ -1604,7 +1577,6 @@ print_file_mime(const char *name)
 	} else {
 		printf("%s%c%s\n", dn_c, UNKNOWN_CHR, df_c);
 	}
-#endif /* _NO_MAGIC */
 }
 
 #ifdef USE_DU1
@@ -1650,7 +1622,7 @@ print_file_size(char *filename, const struct stat *attr, const int file_perm,
 
 	char *size_unit = construct_human_size(size);
 	char *csize = dz_c;
-	char *cend = conf.colorize == 1 ? df_c : "";
+	const char *cend = conf.colorize == 1 ? df_c : "";
 
 	char sf[MAX_SHADE_LEN];
 	*sf = '\0';
@@ -1667,16 +1639,18 @@ print_file_size(char *filename, const struct stat *attr, const int file_perm,
 		if (bigger_than_bytes == 1)
 			printf(" / %s%jd B%s", csize, (intmax_t)size, cend);
 
+		const int is_sparse = (S_ISREG(attr->st_mode) &&
+			(intmax_t)(attr->st_blocks * S_BLKSIZE) < (intmax_t)attr->st_size
+			 && attr->st_blocks > 0);
+
 		printf(" (%s%s%s)\n", conf.apparent_size == 1 ? _("apparent")
 			: _("disk usage"), (xargs.si == 1 && bigger_than_bytes == 1)
-			? ",si" : "", (S_ISREG(attr->st_mode)
-			&& (intmax_t)(attr->st_blocks * S_BLKSIZE) < (intmax_t)attr->st_size)
-			? ",sparse" : "");
+			? ",si" : "", is_sparse == 1 ? ",sparse" : "");
 
 		return;
 	}
 
-	if (full_dirsize == 0) // We're running 'p', not 'pp'.
+	if (full_dirsize == 0) /* We're running 'p', not 'pp'. */
 		return;
 
 	int du_status = 0;
@@ -1684,9 +1658,9 @@ print_file_size(char *filename, const struct stat *attr, const int file_perm,
 		? get_total_size(filename, &du_status) : (-2);
 
 	if (total_size < 0) {
-		if (total_size == -2) // No access
+		if (total_size == -2) /* No access. */
 			printf(_("Total size: \t%s%c%s\n"), dn_c, UNKNOWN_CHR, cend);
-		else // get_total_size returned error (-1)
+		else /* get_total_size returned error (-1). */
 			puts(UNKNOWN_STR);
 		return;
 	}
@@ -1698,7 +1672,7 @@ print_file_size(char *filename, const struct stat *attr, const int file_perm,
 		csize = sf;
 	}
 
-	char *human_size = construct_human_size(total_size);
+	const char *human_size = construct_human_size(total_size);
 	if (!human_size) {
 		puts(UNKNOWN_STR);
 		return;
@@ -1764,11 +1738,13 @@ print_dir_items(const char *dir, const int file_perm)
 
 #else /* !USE_DU1 */
 static void
-print_file_size(const struct stat *attr)
+print_size(const struct stat *attr, const int apparent)
 {
 	const off_t size =
 		(FILE_TYPE_NON_ZERO_SIZE(attr->st_mode) || S_TYPEISSHM(attr)
-		|| S_TYPEISTMO(attr)) ? FILE_SIZE_PTR(attr) : 0;
+		|| S_TYPEISTMO(attr))
+			? (apparent == 1 ? attr->st_size : attr->st_blocks * S_BLKSIZE)
+			: 0;
 
 	char *size_unit = construct_human_size(size);
 	char *csize = dz_c;
@@ -1781,18 +1757,54 @@ print_file_size(const struct stat *attr)
 		csize = sf;
 	}
 
-	printf(_("Size: \t\t%s%s%s"), csize, size_unit ? size_unit : "?", cend);
+	printf("%s%s%s", csize, size_unit ? size_unit : "?", cend);
 
-	const int bigger_than_bytes = size > (xargs.si == 1 ? 1000 : 1024);
+	const int bigger_than_bytes = size >= (xargs.si == 1 ? 1000 : 1024);
+
+	const int is_sparse = (S_ISREG(attr->st_mode) && attr->st_blocks > 0
+		&& (intmax_t)(attr->st_blocks * S_BLKSIZE) < (intmax_t)attr->st_size);
 
 	if (bigger_than_bytes == 1)
 		printf(" / %s%jd B%s", csize, (intmax_t)size, cend);
 
-	printf(" (%s%s%s)\n", conf.apparent_size == 1 ? _("apparent")
-		: _("disk usage"), (xargs.si == 1 && bigger_than_bytes == 1)
-		? ",si" : "", (S_ISREG(attr->st_mode)
-		&& (intmax_t)(attr->st_blocks * S_BLKSIZE) < (intmax_t)attr->st_size)
-		? ",sparse" : "");
+	printf(" (%s%s%s)\n", apparent == 1 ? _("apparent")
+		: _("on disk"), (xargs.si == 1 && bigger_than_bytes == 1)
+		? ",si" : "", is_sparse == 1 ? ",sparse" : "");
+}
+
+static void
+print_file_size(const struct stat *attr)
+{
+	fputs(_("Size:\t\t"), stdout);
+	print_size(attr, 1); /* Apparent size */
+	fputs("\t\t", stdout);
+	print_size(attr, 0); /* Physical size */
+}
+
+static void
+print_dir_size(const off_t dir_size, const int apparent, const char *read_err)
+{
+	char *cend = conf.colorize == 1 ? df_c : "";
+	char *size_color = dz_c;
+	char sf[MAX_SHADE_LEN]; *sf = '\0';
+	if (conf.colorize == 1 && !*dz_c) {
+		get_color_size(dir_size, sf, sizeof(sf));
+		size_color = sf;
+	}
+
+	char *size = construct_human_size(dir_size);
+	if (size) {
+		printf("%s%s%s%s ", read_err, size_color, size, cend);
+
+		const int size_mult_factor = xargs.si == 1 ? 1000 : 1024;
+		if (dir_size > size_mult_factor)
+			printf("/ %s%jd B%s ", size_color, (intmax_t)dir_size, cend);
+
+		printf("(%s%s)\n", apparent == 1 ? _("apparent")
+			: _("on disk"), xargs.si == 1 ? ",si" : "");
+	} else {
+		puts(UNKNOWN_STR);
+	}
 }
 
 static void
@@ -1806,7 +1818,7 @@ print_dir_info(const char *dir, const int file_perm)
 
 	struct dir_info_t info = {0};
 
-	fputs(_("Total size: \t"), stdout);
+	fputs(_("Total size:\t"), stdout);
 
 #define SCANNING_MSG "Scanning..."
 	if (term_caps.suggestions == 1) {
@@ -1825,32 +1837,15 @@ print_dir_info(const char *dir, const int file_perm)
 		fputs(df_c, stdout);
 		fflush(stdout);
 	}
+#undef SCANNING_MSG
 
 	char read_err[5 + (MAX_COLOR * 2)]; *read_err = '\0';
 	if (info.status != 0)
 		snprintf(read_err, sizeof(read_err), "%s%c%s", xf_cb, DU_ERR_CHAR, df_c);
 
-	char *cend = conf.colorize == 1 ? df_c : "";
-	char *size_color = dz_c;
-	char sf[MAX_SHADE_LEN]; *sf = '\0';
-	if (conf.colorize == 1 && !*dz_c) {
-		get_color_size(info.size, sf, sizeof(sf));
-		size_color = sf;
-	}
-
-	char *size = construct_human_size(info.size);
-	if (size) {
-		printf("%s%s%s%s ", read_err, size_color, size, cend);
-
-		const int size_mult_factor = xargs.si == 1 ? 1000 : 1024;
-		if (info.size > size_mult_factor)
-			printf("/ %s%jd B%s ", size_color, (intmax_t)info.size, cend);
-
-		printf("(%s%s)\n", conf.apparent_size == 1 ? _("apparent")
-			: _("disk usage"), xargs.si == 1 ? ",si" : "");
-	} else {
-		puts(UNKNOWN_STR);
-	}
+	print_dir_size(info.size, 1, read_err); /* Apparent */
+	fputs("\t\t", stdout);
+	print_dir_size(info.blocks * S_BLKSIZE, 0, read_err); /* Physical */
 
 	printf(_("Items:\t\t%s%s%llu%s (%s%llu%s %s, %s%llu%s %s, %s%llu%s %s)\n"),
 		read_err, BOLD, info.dirs + info.files + info.links, df_c,
@@ -1871,7 +1866,7 @@ do_stat(char *filename, const int follow_link)
 		return FUNC_FAILURE;
 
 	/* Remove leading "./" */
-	if (*filename == '.' && *(filename + 1) == '/' && *(filename + 2))
+	if (*filename == '.' && filename[1] == '/' && filename[2])
 		filename += 2;
 
 	/* Check file existence. */
@@ -1902,7 +1897,7 @@ do_stat(char *filename, const int follow_link)
 		check_file_access(attr.st_mode, attr.st_uid, attr.st_gid);
 #else
 		/* check_file_access() checks Unix-style permissions, which does
-		 * not make much sense on Windows file systems (particularly, on
+		 * not make much sense on Windows filesystems (particularly, on
 		 * system directories). */
 		1;
 #endif /* !__CYGWIN__ */
@@ -1953,7 +1948,7 @@ properties_function(char **args, const int follow_link)
 		if (strchr(args[i], '\\')) {
 			char *deq_file = unescape_str(args[i], 0);
 			if (!deq_file) {
-				xerror(_("p: '%s': Cannot unescape file name\n"), args[i]);
+				xerror(_("p: '%s': Cannot unescape filename\n"), args[i]);
 				exit_status = FUNC_FAILURE;
 				continue;
 			}
@@ -1986,9 +1981,8 @@ do_stat_and_exit(const int full_stat)
 # if defined(HAVE_GNU_DU)
 	bin_flags |= GNU_DU_BIN_DU;
 # elif !defined(_BE_POSIX)
-	char *p = (char *)NULL;
-	if ((p = get_cmd_path("gdu")) != NULL)
-		{ free(p); bin_flags |= GNU_DU_BIN_GDU; }
+	if (is_cmd_in_path("gdu") == 1)
+		bin_flags |= GNU_DU_BIN_GDU;
 # endif /* HAVE_GNU_DU */
 
 	if (!tmp_dir)
