@@ -1,17 +1,15 @@
-/* properties.h -- Home of the p/pp, pc, oc, and stats commands */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* properties.h */
+
 #ifndef PROPERTIES_H
 #define PROPERTIES_H
 
@@ -56,15 +56,6 @@
 /* Max size for a file size color (used by get_color_size()) */
 #define MAX_SHADE_LEN 26 /* "\x1b[0;x;38;2;xxx;xxx;xxxm\0" */
 
-/* Macros to calculate relative timestamps (used by get_color_age()) */
-#define RT_SECOND 1
-#define RT_MINUTE (60  * RT_SECOND)
-#define RT_HOUR   (60  * RT_MINUTE)
-#define RT_DAY    (24  * RT_HOUR)
-#define RT_WEEK   (7   * RT_DAY)
-#define RT_MONTH  (30  * RT_DAY)
-#define RT_YEAR   (365 * RT_DAY)
-
 /* Struct used by get_file_perms() */
 struct perms_t {
 	/* Field colors */
