@@ -1,17 +1,15 @@
-/* remote.c -- functions to manage remotes */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,10 +20,11 @@
  * MA 02110-1301, USA.
 */
 
+/* remote.c -- functions to manage remotes */
+
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 
 #include "aux.h"
@@ -126,7 +125,7 @@ get_remote(char *name)
 	return i;
 }
 
-static inline int
+static int
 create_mountpoint(const int i)
 {
 	char *cmd[] = {"mkdir", "-p", remotes[i].mountpoint, NULL};
@@ -139,7 +138,7 @@ create_mountpoint(const int i)
 	return FUNC_SUCCESS;
 }
 
-static inline int
+static int
 cd_to_mountpoint(const int i)
 {
 	free(workspaces[cur_ws].path);
@@ -148,21 +147,20 @@ cd_to_mountpoint(const int i)
 	add_to_jumpdb(workspaces[cur_ws].path);
 	add_to_dirhist(workspaces[cur_ws].path);
 
-	free_dirlist();
-	if (list_dir() != FUNC_SUCCESS)
-		return FUNC_FAILURE;
+	dir_changed = 1;
+	reload_dirlist();
 
 	return FUNC_SUCCESS;
 }
 
-static inline int
+static int
 print_cd_error(const int i)
 {
 	xerror("net: '%s': %s\n", remotes[i].mountpoint, strerror(errno));
 	return FUNC_FAILURE;
 }
 
-static inline int
+static int
 print_no_mount_cmd_error(const int i)
 {
 	xerror(_("net: No mount command specified for '%s'\n"), remotes[i].name);
@@ -201,7 +199,7 @@ remotes_mount(char *name)
 	if (conf.autols == 1) {
 		exit_status = cd_to_mountpoint(i);
 	} else {
-		printf(_("%s: '%s': Remote mounted on %s\n"), PROGRAM_NAME,
+		printf(_("net: '%s': Changed to mountpoint (%s)\n"),
 			remotes[i].name, remotes[i].mountpoint);
 	}
 
@@ -306,7 +304,7 @@ remotes_edit(char *app)
 	if (mtime_bfr != attr.st_mtime) {
 		free_remotes(0);
 		load_remotes();
-		print_reload_msg(_("File modified. Remotes reloaded\n"));
+		print_reload_msg(NULL, NULL, _("File modified. Remotes reloaded.\n"));
 	}
 
 	return FUNC_SUCCESS;
