@@ -1,17 +1,15 @@
-/* search.c -- functions for the search system */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,10 +20,11 @@
  * MA 02110-1301, USA.
 */
 
+/* search.c -- functions for the search system */
+
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <sys/ioctl.h>
 #include <unistd.h>
@@ -83,10 +82,8 @@ define_find_name(void)
 	/* Let's run this only once. */
 	if (check == 1) {
 		check = 0;
-		char *p = get_cmd_path("gfind");
-		if (p)
+		if (is_cmd_in_path("gfind") == 1)
 			have_gfind = 1;
-		free(p);
 	}
 
 	return (have_gfind == 1 ? "gfind" : "find");
@@ -179,7 +176,7 @@ set_file_type_and_search_path(char **args, mode_t *file_type,
 	case 'c': *file_type = invert == 1 ? DT_CHR : S_IFCHR; break;
 	case 'd': *file_type = invert == 1 ? DT_DIR : S_IFDIR; break;
 #ifdef SOLARIS_DOORS
-	case 'D': *file_type = invert == 1 ? DT_DOOR : S_IFDOOR; break;
+	case 'O': *file_type = invert == 1 ? DT_DOOR : S_IFDOOR; break;
 	case 'P': *file_type = invert == 1 ? DT_PORT : S_IFPORT; break;
 #endif /* SOLARIS_DOORS */
 	case 'f': *file_type = invert == 1 ? DT_REG : S_IFREG; break;
@@ -202,7 +199,7 @@ chdir_search_path(char **search_path, const char *arg)
 	if (strchr(*search_path, '\\')) {
 		char *deq_dir = unescape_str(*search_path, 0);
 		if (!deq_dir) {
-			xerror(_("search: %s: Error unescaping file name\n"), arg);
+			xerror(_("search: %s: Error unescaping filename\n"), arg);
 			return FUNC_FAILURE;
 		}
 
@@ -289,7 +286,7 @@ get_glob_matches(char **gfiles, const char *search_path,
 
 		matches[n].name = savestring(gfiles[i], strlen(gfiles[i]));
 
-		/* Get the longest file name in the list. */
+		/* Get the longest filename in the list. */
 		/* If not in CWD, we only need to know the file's length (no ELN) */
 		if (search_path) {
 			/* This will be passed to colors_list(): -1 means no ELN */
@@ -436,7 +433,7 @@ get_glob_longest(struct search_t *matches, int *longest_eln,
 	}
 
 	if (conf.icons == 1)
-		*longest_match += ICON_LEN;
+		*longest_match += (size_t)ICON_LEN;
 
 	int longest_name = 0;
 	for (i = 0; matches[i].name; i++) {
@@ -539,7 +536,7 @@ print_glob_matches(struct search_t *matches, const char *search_path)
 				df_c, conf.icons == 1 ? ' ' : 0);
 		}
 
-		/* Print file name. */
+		/* Print filename. */
 		int name_pad = (last_column == 1 || i == (found - 1)) ? NO_PAD :
 		    (int)(flongest - matches[i].len - ( search_path ? 0
 		    : (size_t)(eln_pad - DIGINUM(matches[i].eln)) ) + 1);
@@ -553,13 +550,13 @@ print_glob_matches(struct search_t *matches, const char *search_path)
 
 	tab_offset = tab_offset_bk;
 
-	print_reload_msg(_("Matches found: %d%s\n"), found,
+	print_reload_msg(SET_SUCCESS_PTR, xs_cb, _("Matches found: %d%s\n"), found,
 		conf.search_strategy != GLOB_ONLY ? " (glob)" : "");
 
 	return found;
 }
 
-/* List matching file names in the specified directory. */
+/* List matching filenames in the specified directory. */
 static int
 search_glob(char **args)
 {
@@ -611,8 +608,8 @@ search_glob(char **args)
 	if (conf.list_dirs_first == 1)
 		gfiles = glob_sort_dirs(&globbed_files, &g);
 
-	/* We need to store pointers to matching file names in array of pointers,
-	 * just as the file name length (to construct the columned output), and,
+	/* We need to store pointers to matching filenames in array of pointers,
+	 * just as the filename length (to construct the columned output), and,
 	 * if searching in CWD, its index (ELN) in the dirlist array as well. */
 	struct search_t *list = (struct search_t *)NULL;
 
@@ -754,13 +751,13 @@ load_entry_info(struct dirent **reg_dirlist, const int index)
 	list.name = name;
 	list.eln = reg_dirlist ? -1 : index + 1;
 	list.len = wc_xstrlen(name);
-	list.len += (!reg_dirlist && conf.icons == 1) ? ICON_LEN : 0;
+	list.len += (!reg_dirlist && conf.icons == 1) ? (size_t)ICON_LEN : 0;
 	list.len += reg_dirlist ? 0 : (size_t)(DIGINUM(list.eln) + 1);
 
 	return list;
 }
 
-/* Return the length of the longest entry in the files list LIST. */
+/* Return the length of the longest entry in the file list LIST. */
 static size_t
 get_regex_longest(struct search_t *list, const int total, int *elnpad)
 {
@@ -886,7 +883,7 @@ print_regex_matches(const mode_t file_type, struct dirent **reg_dirlist,
 		print_regex_entry(list[i], name_pad, eln_pad, newline);
 	}
 
-	print_reload_msg(_("Matches found: %zu\n"), count);
+	print_reload_msg(SET_SUCCESS_PTR, xs_cb, _("Matches found: %zu\n"), count);
 
 END:
 	free(list);
@@ -894,7 +891,7 @@ END:
 	return count;
 }
 
-/* List matching (or non-marching if INVERT is set to 1) file names
+/* List matching (or non-matching if INVERT is set to 1) filenames
  * in the specified directory. */
 static int
 search_regex(char **args)
@@ -1014,7 +1011,7 @@ err_glob_no_match(const char *arg)
 	return FUNC_FAILURE;
 }
 
-/* We have three search strategies:
+/* We provide three search strategies:
  * 1 - Only glob
  * 2 - Only regex
  * 3 - Glob-regex */
