@@ -1,17 +1,15 @@
-/* selection.c -- files selection functions */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,10 +20,11 @@
  * MA 02110-1301, USA.
 */
 
+/* selection.c -- files selection functions */
+
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <readline/tilde.h>
 #include <string.h>
 #include <unistd.h>
@@ -65,7 +64,7 @@
 int
 save_sel(void)
 {
-	if (selfile_ok == 0 || config_ok == 0 || !sel_file)
+	if (selfile_ok == 0 || !sel_file)
 		return (xargs.stealth_mode == 1 ? FUNC_SUCCESS : FUNC_FAILURE);
 
 	if (sel_n == 0) {
@@ -665,8 +664,9 @@ print_sel_results(const int new_sel, const char *sel_path,
 	if (conf.autols == 1 && error == 0)
 		reload_dirlist();
 
-	print_reload_msg(_("%d file(s) selected\n"), new_sel);
-	print_reload_msg(_("%zu total selected file(s)\n"), sel_n);
+	print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+		_("%d file(s) selected\n"), new_sel);
+	print_reload_msg(NULL, NULL, _("%zu total selected file(s)\n"), sel_n);
 
 	return FUNC_SUCCESS;
 }
@@ -767,7 +767,7 @@ select_pattern(char *arg, const char *dir, const mode_t filetype, int *err)
 	int ret = sel_glob(arg, dir, filetype);
 
 	/* Glob failed. Try REGEX (if at least one regex metacharacter not being
-	 * asterisk if found). */
+	 * asterisk is found). */
 	if (ret == -1)
 		ret = not_just_star(arg) == 1 ? sel_regex(arg, dir, filetype) : 0;
 
@@ -887,7 +887,7 @@ list_selected_files(void)
 				conf.pager = 0; reset_pager = 1; /* 'q' */
 				break;
 			/* If another key is pressed, go back one position.
-			 * Otherwise, some file names won't be listed.*/
+			 * Otherwise, some filenames won't be listed.*/
 			default:
 				i--;
 				continue;
@@ -956,7 +956,7 @@ edit_selfile(void)
 	if (conf.autols == 1)
 		reload_dirlist();
 
-	print_reload_msg(_("%zu file(s) selected\n"), sel_n);
+	print_reload_msg(SET_SUCCESS_PTR, xs_cb, _("%zu file(s) selected\n"), sel_n);
 	return ret;
 
 ERROR:
@@ -1071,8 +1071,8 @@ desel_entries(char **desel_elements, const size_t desel_n, const int desel_scree
 	if (desel_screen == 1) {
 		free(desel_path);
 	} else if (err == 1) {
-		print_reload_msg(_("%d file(s) deselected\n"), dn);
-		print_reload_msg(_("%zu total selected file(s)\n"), sel_n);
+		print_reload_msg(SET_SUCCESS_PTR, xs_cb, _("%d file(s) deselected\n"), dn);
+		print_reload_msg(NULL, NULL, _("%zu total selected file(s)\n"), sel_n);
 	}
 	free(desel_elements);
 
@@ -1106,22 +1106,23 @@ static int
 deselect_from_args(char **args)
 {
 	char **ds = xnmalloc(args_n + 1, sizeof(char *));
-	size_t j, k = 0;
+	size_t i, j = 0;
 
-	for (j = 1; j <= args_n; j++) {
-		char *pp = normalize_path(args[j], strlen(args[j]));
-		if (!pp)
+	for (i = 1; i <= args_n; i++) {
+		char *ptr = normalize_path(args[i], strlen(args[i]));
+		if (!ptr)
 			continue;
-		ds[k] = savestring(pp, strlen(pp));
-		k++;
-		free(pp);
+
+		ds[j] = savestring(ptr, strlen(ptr));
+		j++;
+
+		free(ptr);
 	}
-	ds[k] = (char *)NULL;
 
-	if (desel_entries(ds, args_n, 0) == FUNC_FAILURE)
-		return FUNC_FAILURE;
+	ds[j] = (char *)NULL;
 
-	return FUNC_SUCCESS;
+	return (desel_entries(ds, j, 0) == FUNC_FAILURE
+		? FUNC_FAILURE : FUNC_SUCCESS);
 }
 
 /* Desel screen: take user input and return an array of input substrings,
@@ -1136,7 +1137,7 @@ get_desel_input(size_t *n)
 	snprintf(dp, sizeof(dp), "\001%s\002>\001%s\002 ", mi_c, tx_c);
 	char *line = NULL;
 	while (!line)
-		line = rl_no_hist(dp);
+		line = rl_no_hist(dp, 0);
 
 	/* get_substr() will try to expand ranges, in which case a range with
 	 * no second field is expanded from the value of the first field to the
@@ -1243,10 +1244,11 @@ end_deselect(const int err, char ***args)
 		reload_dirlist();
 
 	if (argsbk > 0) {
-		print_reload_msg(_("%zu file(s) deselected\n"), desel_files);
-		print_reload_msg(_("%zu total selected file(s)\n"), sel_n);
+		print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+			_("%zu file(s) deselected\n"), desel_files);
+		print_reload_msg(NULL, NULL, _("%zu total selected file(s)\n"), sel_n);
 	} else {
-		print_reload_msg(_("%zu selected file(s)\n"), sel_n);
+		print_reload_msg(NULL, NULL, _("%zu selected file(s)\n"), sel_n);
 	}
 
 	return exit_status;
@@ -1264,8 +1266,9 @@ handle_desel_args(char **args)
 			reload_dirlist();
 
 		if (ret == FUNC_SUCCESS) {
-			print_reload_msg(_("%zu file(s) deselected\n"), n);
-			print_reload_msg(_("0 total selected file(s)\n"));
+			print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+				_("%zu file(s) deselected\n"), n);
+			print_reload_msg(NULL, NULL, _("0 total selected file(s)\n"));
 		}
 		return ret;
 
