@@ -1,17 +1,15 @@
-/* settings.h - User settings default values */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* settings.h - User settings default values */
+
 #ifndef SETTINGS_H
 #define SETTINGS_H
 
@@ -39,11 +39,11 @@
  * (usually $HOME/.config/clifm/colors/) nor in the data dir, (usually
  * /usr/local/share/clifm/colors/). */
 #define DEF_FILE_COLORS "bd=1;33:ca=30;41:cd=1:di=1;34:ed=2;34:\
-ee=32:ef=2:ex=1;32:fi=0:ln=1;36:mh=30;46:nd=4;1;31:nf=2;4;31:\
+ee=32:ef=2:ex=1;32:fi=0:ln=1;36:mh=30;46:nd=4;1;31:nf=:\
 no=31;47:or=2;4;36:ow=34;40:pi=4;45;37:so=45;37:su=37;41:sg=30;43:st=37;44:\
 tw=37;42:uf=2;4;37:"
 
-#define DEF_IFACE_COLORS "db=:dd=:de=:df=0:dg=2;35:dk=:dl=2;37:\
+#define DEF_IFACE_COLORS "ac=:db=:dd=:de=:df=0:dg=2;35:dk=:dl=2;37:\
 dn=2;37:do=:dp=:dr=:dt=:du=35:dw=:dxd=:dxr=:dz=:\
 el=36:em=1;31:fc=2;37:hb=36:hc=2;31:lc=:\
 hd=36:he=0;36:hn=0:hp=36:hq=33:hr=31:hs=32:hv=32:\
@@ -105,7 +105,7 @@ xf=1;31:xs=32:"
 #define DEF_LN_C  "\x1b[1;36m"    /* Symbolic link */
 #define DEF_MH_C  "\x1b[30;46m"   /* Multi-link */
 #define DEF_ND_C  "\x1b[1;31m"    /* Unaccessible dir */
-#define DEF_NF_C  "\x1b[2;31m"    /* Unaccessible file */
+#define DEF_NF_C  ""              /* Unaccessible file */
 #define DEF_NO_C  "\x1b[0;31;47m" /* Unknown file type */
 #ifdef __sun
 #define DEF_OO_C  "\x1b[0;35m"    /* Solaris door/port (ls default is 1;35) */
@@ -121,11 +121,12 @@ xf=1;31:xs=32:"
 #define DEF_UF_C  "\x1b[2;4;37m"  /* Un'stat'able file */
 
 /* Interface */
+#define DEF_AC_C  "\001\x1b[2;37m\002"   /* Autocmd indicator */
 #define DEF_DF_C  "\x1b[0m"      /* Reset attributes: default terminal color */
 #define DEF_DL_C  "\x1b[2;37m"   /* Dividing line */
 #define DEF_EL_C  "\x1b[0;36m"   /* ELN's */
 #define DEF_EM_C  "\001\x1b[1;31m\002" /* Error msg indicator */
-#define DEF_FC_C  "\x1b[0;2;37m" /* Files counter */
+#define DEF_FC_C  "\x1b[0;2;37m" /* File counter */
 #define DEF_LI_C  "\001\x1b[1;32m\002" /* Sel files indicator (prompt) */
 #define DEF_LI_CB "\x1b[1;32m"   /* Sel files indicator (files list) */
 #define DEF_MI_C  "\x1b[1;36m"   /* Misc */
@@ -140,12 +141,13 @@ xf=1;31:xs=32:"
 /* Symlink char indicator (for ColorLinksAsTarget only) */
 #define DEF_LC_C  "\x1b[1;36m"
 
-#define DEF_TS_C  "\x1b[4;35m" /* Matching prefix for TAB completed possible entries */
-#define DEF_TT_C  "\x1b[1;2;36m" /* Tilde for trimmed file names */
+#define DEF_TS_C  "\x1b[4;35m" /* Matching prefix for tab completed possible entries */
+#define DEF_TT_C  "\x1b[1;2;36m" /* Tilde for truncated filenames */
 #define DEF_WP_C  "\x1b[0;2;31m" /* Warning prompt input text */
 #define DEF_XS_C  "\001\x1b[32m\002" /* Exit code: success */
+#define DEF_XS_CB "\x1b[32m"     /* Exit code: Unicode success indicator */
 #define DEF_XF_C  "\001\x1b[1;31m\002" /* Exit code: failure */
-#define DEF_XF_CB "\x1b[1;31m" /* Exit code: failure (dir read) */
+#define DEF_XF_CB "\x1b[1;31m"   /* Exit code: failure (dir read) */
 
 /* Workspaces */
 #define DEF_WS1_C "\001\x1b[34m\002"
@@ -158,14 +160,14 @@ xf=1;31:xs=32:"
 #define DEF_WS8_C "\001\x1b[36m\002"
 
 /* Suggestions */
-#define DEF_SB_C  "\x1b[2;33m"   /* Shell built-ins */
+#define DEF_SB_C  "\x1b[2;33m"   /* Shell builtins */
 #define DEF_SC_C  "\x1b[2;36m"   /* Aliases and binaries in PATH */
 #define DEF_SD_C  "\x1b[2;37m"   /* Internal commands description */
 #define DEF_SH_C  "\x1b[2;37m"   /* Commands history */
 #define DEF_SF_C  "\x1b[2;4;36m" /* ELN's, bookmark, file, and directory names */
 #define DEF_SP_C  "\x1b[2;31m"   /* Suggestions pointer (12 > filename) */
 #define DEF_SX_C  "\x1b[2;32m"   /* Internal commands and parameters */
-#define DEF_SZ_C  "\x1b[2;4;36m" /* File names (fuzzy) */
+#define DEF_SZ_C  "\x1b[2;4;36m" /* Filenames (fuzzy) */
 
 /* Highlight */
 #define DEF_HB_C  "\x1b[0;36m" /* Parenthesis, Brackets ( {[()]} ) */
@@ -208,14 +210,14 @@ xf=1;31:xs=32:"
 		 * ############################################# */
 
 #define DEF_FILE_COLORS_256 "bd=1;38;5;229:ca=30;41:cd=1;38;5;214:di=1;34:\
-ed=2;34:ee=32:ef=2:ex=1;32:fi=0:ln=1;36:mh=7;36:nd=1;31:nf=2;31:no=4;31;47:\
+ed=2;34:ee=32:ef=2:ex=1;32:fi=0:ln=1;36:mh=7;36:nd=1;31:nf=:no=4;31;47:\
 or=4;2;36:ow=48;5;235;38;5;33:pi=38;5;5;48;5;238:sg=30;43:so=1;38;5;5;48;5;238:\
 st=37;44:su=37;41:tw=37;42:uf=4;2;37:"
 
-#define DEF_IFACE_COLORS_256 "db=:dd=:de=:df=0:dg=2;35:dk=:dl=38;5;243:dn=:\
+#define DEF_IFACE_COLORS_256 "ac=:db=:dd=:de=:df=0:dg=2;35:dk=:dl=38;5;243:dn=:\
 do=:dp=:dr=:dt=:du=35:dw=:dxd=:dxr=:dz=:el=36:em=1;31:fc=38;5;246:hb=36:hc=2;37:\
 hd=36:he=36:hn=:hp=36:hq=LY:hr=31:hs=32:hv=32:lc=38;5;43:li=1;32:mi=1;36:\
-nm=1;32:ro=:si=1;34:sb=2;33:sc=2;36:sd=38;5;243:sf=4;2;36:sh=38;5;243:sp=2;31:\
+nm=1;32:ro=:si=1;34:sb=2;33:sc=2;36:sd=38;5;240:sf=4;2;36:sh=38;5;240:sp=38;5;239:\
 sx=2;32:sz=4;2;36:ti=1;36:ts=4;35:tt=1;2;36:tx=0:wc=1;36:wm=38;5;228:ws1=34:\
 ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 
@@ -268,7 +270,7 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_LN_C256  "\x1b[1;36m"       /* Symbolic link */
 #define DEF_MH_C256  "\x1b[7;36m"       /* Multi-link */
 #define DEF_ND_C256  "\x1b[1;31m"       /* Unaccessible dir */
-#define DEF_NF_C256  "\x1b[2;31m"       /* Unaccessible file */
+#define DEF_NF_C256  ""                 /* Unaccessible file */
 #define DEF_NO_C256  "\x1b[4;31;47m"    /* Unknown file type */
 #ifdef __sun
 #define DEF_OO_C256  "\x1b[0;35m"    /* Solaris door/port (ls default is 1;35) */
@@ -284,13 +286,14 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_UF_C256  "\x1b[2;4;37m" /* Un'stat'able file */
 
 /* Interface */
+#define DEF_AC_C256  "\001\x1b[38;5;247m\002" /* Autocmd indicator */
 #define DEF_DF_C256  "\x1b[0m"      /* Reset attributes: default terminal color */
 #define DEF_DL_C256  "\x1b[38;5;243m"   /* Dividing line */
 #define DEF_EL_C256  "\x1b[0;36m"   /* ELN's */
 #define DEF_EM_C256  "\001\x1b[1;31m\002" /* Error msg indicator */
-#define DEF_FC_C256  "\x1b[0;38;5;247m" /* Files counter */
+#define DEF_FC_C256  "\x1b[0;38;5;247m" /* File counter */
 #define DEF_LI_C256  "\001\x1b[1;32m\002" /* Sel files indicator (prompt) */
-#define DEF_LI_CB256 "\x1b[1;32m"   /* Sel files indicator (files list) */
+#define DEF_LI_CB256 "\x1b[1;32m"   /* Sel files indicator (file list) */
 #define DEF_MI_C256  "\x1b[1;36m"   /* Misc */
 #define DEF_NM_C256  "\001\x1b[1;32m\002" /* Notice msg indicator */
 #define DEF_RO_C256  "\001\x1b[1;34m\002" /* Read-only mode indicator */
@@ -303,12 +306,13 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 /* Symlink char indicator (for ColorLinksAsTarget only) */
 #define DEF_LC_C256 "\x1b[0;38;5;43m"
 
-#define DEF_TS_C256   "\x1b[4;35m" /* Matching prefix for TAB completed possible entries */
-#define DEF_TT_C256   "\x1b[1;2;36m" /* Tilde for trimmed file names */
+#define DEF_TS_C256   "\x1b[4;35m" /* Matching prefix for tab completed possible entries */
+#define DEF_TT_C256   "\x1b[1;2;36m" /* Tilde for truncated filenames */
 #define DEF_WP_C256   "\x1b[0;2;31m" /* Warning prompt input text */
 #define DEF_XS_C256   "\001\x1b[32m\002" /* Exit code: success */
+#define DEF_XS_CB256  "\x1b[32m"     /* Exit code: Unicode success indicator */
 #define DEF_XF_C256   "\001\x1b[1;31m\002" /* Exit code: failure */
-#define DEF_XF_CB256  "\x1b[1;31m" /* Exit code: failure (dir read) */
+#define DEF_XF_CB256  "\x1b[1;31m"   /* Exit code: failure (dir read) */
 
 /* Workspaces */
 #define DEF_WS1_C256 "\001\x1b[34m\002"
@@ -321,14 +325,14 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_WS8_C256 "\001\x1b[2;37m\002"
 
 /* Suggestions */
-#define DEF_SB_C256  "\x1b[2;33m"   /* Shell built-ins */
+#define DEF_SB_C256  "\x1b[2;33m"   /* Shell builtins */
 #define DEF_SC_C256  "\x1b[2;36m"   /* Aliases and binaries in PATH */
 #define DEF_SD_C256  "\x1b[38;5;240m"   /* Internal commands description */
 #define DEF_SH_C256  "\x1b[38;5;240m"   /* Commands history */
 #define DEF_SF_C256  "\x1b[2;4;36m" /* ELN's, bookmark, file, and directory names */
-#define DEF_SP_C256  "\x1b[2;31m"   /* Suggestions pointer (12 > filename) */
+#define DEF_SP_C256  "\x1b[38;5;239m"   /* Suggestions pointer (12 > filename) */
 #define DEF_SX_C256  "\x1b[2;32m"   /* Internal commands and parameters */
-#define DEF_SZ_C256  "\x1b[2;4;36m" /* File names (fuzzy) */
+#define DEF_SZ_C256  "\x1b[2;4;36m" /* Filenames (fuzzy) */
 
 /* Highlight */
 #define DEF_HB_C256  "\x1b[0;36m" /* Parenthesis, Brackets ( {[()]} ) */
@@ -401,8 +405,20 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define PORT_PCHR    'P' /* Event port (Solaris) */
 #define UNK_PCHR     UNKNOWN_CHR
 
-#define SELFILE_CHR  '*'
-#define TRIMFILE_CHR '~'
+/* When any of 'nf' or 'nd' color codes is unset, prepend this character
+ * (a single one) to unaccessible files or directories in the file list
+ * (only if not running in light mode and icons are disabled). */
+#define NO_PERM_STR "!"
+
+#define SELFILE_CHR   'S' /* Prompt indicator */
+#define SELFILE_STR   "*" /* Selected file mark for file list (ASCII) */
+#define SELFILE_STR_U "┃" /* Unicode alternative */
+
+/* Symbolic link mark for listed files (if ColorLinksAsTarget is enabled). */
+#define LINK_STR   "@" /* ASCII */
+#define LINK_STR_U "@" /* Unicode alternative */
+
+#define TRUNC_FILE_CHR '~'
 
 /* Character used to mark files with extended attributes (long view) */
 #define XATTR_CHAR '@'
@@ -412,7 +428,7 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DU_ERR_CHAR '!'
 
 /* Character used to replace invalid characters (either a control char or an
- * invalid UTF-8 char) in file names. */
+ * invalid UTF-8 char) in filenames. */
 #define INVALID_CHR '^'
 
 /* A few identifiers used in the prompt */
@@ -422,17 +438,49 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 
 /* String used for the messages pointer (e.g.: "-> 2 file(s) removed")
  * Note: The color used for this string is 'mi' (in the color scheme file). */
-#define MSG_PTR_STR "->"
+#define MSG_PTR_STR   "->"
+#define MSG_PTR_STR_U "→"  /* Unicode alternative */
+
+/* Used only by the 'pp' command on a symbolic link */
+#define MSG_PTR_STR_LEFT   "<-"
+#define MSG_PTR_STR_LEFT_U "←"
+
+/* Pointer used for successful operations (color used is 'xs')*/
+#define SUCCESS_PTR_STR_U "✔"
+#define SUCCESS_PTR_STR   "->"
+
+//#define ERROR_PTR_STR_U   "✗"
+//#define ERROR_PTR_STR     "->"
+
+/* Used in multiple places, e.g., when listing prompts, color schemes, profiles,
+ * and workspaces ('config dump' as well). */
+#define MISC_PTR   ">"
+#define MISC_PTR_U "┃"
+
+/* If PrintDirCmds is enabled */
+#define DIR_CMD_PTR   ">"
+#define DIR_CMD_PTR_U DIR_CMD_PTR
 
 /* Character used to print non-matching suggestions, e.g. "12 > filename". */
-#define BAEJ_SUG_POINTER '>'
+#define SUG_POINTER '>'
 
 #define MNT_UDEVIL 	0
 #define MNT_UDISKS2 1
 
 /* Default options */
+/* Default answers for confirmation prompts: 0 or 'u' (unset), 'y', or 'n' */
+#define DEF_ANSWER_BULK_RENAME 'n' /* 'br' command */
+#define DEF_ANSWER_OVERWRITE   'n' /* 'c' and 'm' commands mostly */
+#define DEF_ANSWER_REMOVE      'n' /* 'r' command */
+#define DEF_ANSWER_TRASH       'y' /* 't' command */
+#define DEF_ANSWER_DEFAULT     'y' /* Remaining prompts */
+#define DEF_ANSWER_DEFAULT_ALL 0   /* All prompts (overrides the above ones) */
+
 #define DEF_APPARENT_SIZE 1
 #define DEF_AUTOCD 1
+/* InformAutocmd values:
+ * AUTOCMD_MSG_NONE, AUTOCMD_MSG_MINI, AUTOCMD_MSG_SHORT, AUTOCMD_MSG_LONG */
+#define DEF_AUTOCMD_MSG AUTOCMD_MSG_PROMPT
 #define DEF_AUTO_OPEN 1
 #define DEF_AUTOLS 1
 #define DEF_CASE_SENS_LIST 0
@@ -441,7 +489,7 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_CASE_SENS_SEARCH 0
 #define DEF_CD_ON_QUIT 0
 #define DEF_CHECK_CAP 1 /* Check files capabilities */
-#define DEF_CHECK_EXT 1 /* Check file names extension (for color) */
+#define DEF_CHECK_EXT 1 /* Check filenames extension (for color) */
 #define DEF_CLASSIFY 1
 #define DEF_CLEAR_SCREEN 1
 #define DEF_CMD_DESC_SUG 1
@@ -451,12 +499,13 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_COLOR_LNK_AS_TARGET 0
 #define DEF_COLUMNS 1
 #define DEF_CP_CMD CP_CP
-#define DEF_CUR_WS 0
+#define DEF_CUR_WS 0 /* First workspace */
 #define DEF_CWD_IN_TITLE 0
 #define DEF_DESKTOP_NOTIFICATIONS 0
 #define DEF_DIRHIST_MAP 0
 #define DEF_DISK_USAGE 0
 #define DEF_DIV_LINE "-"
+#define DEF_DIV_LINE_U "─" /* Unicode alternative */
 #define DEF_ELN_USE_WORKSPACE_COLOR 0
 /* Available styles: QUOTING_STYLE_BACKSLASH, QUOTING_STYLE_SINGLE_QUOTES,
  * and QUOTING_STYLE_DOUBLE_QUOTES */
@@ -474,7 +523,9 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_HIST_STATUS 1
 #define DEF_HISTIGNORE "^[q,Q]$|^quit$|^exit$|^ .*|^#.*|^[0-9]+$|^\\.+$"
 #define DEF_ICONS 0
+#define DEF_ICONS_GAP 1 /* Number of spaces between icons and filenames */
 #define DEF_INT_VARS 0
+#define DEF_KITTY_KEYS 0 /* Kitty keyboard protocol */
 #define DEF_LIGHT_MODE 0
 /* Possible values: LNK_CREAT_REG, LNK_CREAT_ABS, and LNK_CREAT_REL */
 #define DEF_LINK_CREATION_MODE LNK_CREAT_REG
@@ -491,7 +542,7 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_MAX_LOG 1000
 #define DEF_MAX_PRINTSEL 0
 #define DEF_MIN_JUMP_RANK 10
-#define DEF_MIN_NAME_TRIM 20
+#define DEF_MIN_NAME_TRUNC 20
 #define DEF_MOUNT_CMD MNT_UDEVIL
 #define DEF_MV_CMD MV_MV
 #define DEF_NOELN 0
@@ -499,9 +550,11 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_PAGER 0
 /* Possible values: PAGER_AUTO, PAGER_LONG, and PAGER_SHORT */
 #define DEF_PAGER_VIEW PAGER_AUTO
+#define DEF_PREVIEW_MAX_SIZE -1 /* Max size in KiB. -1 == unlimited */
 #define DEF_PRINT_DIR_CMDS 0
 #define DEF_PRINTSEL 0
-#define DEF_PRINT_REMOVED_FILES 1
+#define DEF_PRINT_REMOVED_FILES 0
+#define DEF_PRIORITY_SORT_CHAR ""
 #define DEF_PRIVATE_WS_SETTINGS 0
 #define DEF_PROMPT_B_MIN 0
 #define DEF_PROMPT_B_PRECISION 2
@@ -509,7 +562,7 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_PROMPT_F_FULL_LEN_DIRS 1
 #define DEF_PROMPT_P_MAX_PATH 40
 /* "xfpIsm" =
- * xattrs/caps/ACLs, files counter, perms, owner/grp names, size (human), mtime */
+ * xattrs/caps/ACLs, file counter, perms, owner/grp names, size (human), mtime */
 #define DEF_PROP_FIELDS "xfpIsm"
 #define DEF_PROP_FIELDS_GAP 1 /* Spaces between columns in long view */
 #define DEF_PURGE_JUMPDB 0
@@ -519,19 +572,21 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_REFRESH_ON_EMPTY_LINE 1
 #define DEF_REFRESH_ON_RESIZE 1
 #define DEF_RELATIVE_TIME 0
+#define DEF_REPORT_CWD 0 /* Report CWD to terminal via OSC-7 escape sequence */
 #define DEF_RESTORE_LAST_PATH 1
 #define DEF_RL_EDIT_MODE 1
 #define DEF_SECURE_CMDS 0
 #define DEF_SECURE_ENV 0
 #define DEF_SECURE_ENV_FULL 0
 #define DEF_SHARE_SELBOX 0
-#define DEF_SHOW_HIDDEN 0
+/* Supported values: HIDDEN_FALSE, HIDDEN_TRUE, HIDDEN_FIRST, and HIDDEN_LAST */
+#define DEF_SHOW_HIDDEN HIDDEN_FALSE
 #define DEF_SHOW_BACKUP_FILES 1
 #define DEF_SKIP_NON_ALNUM_PREFIX 1
 #define DEF_SI 0 /* If 1, compute sizes in powers of 1000 instead of 1024 */
-/* Available sort options:
+/* Supported sort options:
  * SNONE, SNAME, STSIZE, SATIME, SBTIME, SCTIME, SMTIME
- * SVER, SEXT, SINO, SOWN, SGRP */
+ * SVER, SEXT, SINO, SOWN, and SGRP */
 #define DEF_SORT SVER
 #define DEF_SORT_REVERSE 0
 #define DEF_SPLASH_SCREEN 0
@@ -550,8 +605,8 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_TIMESTAMP_MARK 0
 #define DEF_TIPS 1
 #define DEF_TRASRM 0
-#define DEF_TRASH_FORCE 1
-#define DEF_TRIM_NAMES 1
+#define DEF_TRASH_FORCE 0
+#define DEF_TRUNC_NAMES 1
 #define DEF_WELCOME_MESSAGE 1
 
 /* This expands to "CliFM > The command line file manager" */
@@ -566,16 +621,16 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 
 /* cp and mv commands
  * All options used for cp(1) and mv(1) are POSIX. No need to check. */
-#define DEFAULT_CP_CMD "cp -iRp"
+#define DEFAULT_CP_CMD "cp -Rp"
 #define DEFAULT_CP_CMD_FORCE "cp -Rp"
-#define DEFAULT_ADVCP_CMD "advcp -giRp"
+#define DEFAULT_ADVCP_CMD "advcp -gRp"
 #define DEFAULT_ADVCP_CMD_FORCE "advcp -gRp"
 #define DEFAULT_WCP_CMD "wcp"
 #define DEFAULT_RSYNC_CMD "rsync -avP"
 
-#define DEFAULT_MV_CMD "mv -i"
+#define DEFAULT_MV_CMD "mv"
 #define DEFAULT_MV_CMD_FORCE "mv"
-#define DEFAULT_ADVMV_CMD "advmv -gi"
+#define DEFAULT_ADVMV_CMD "advmv -g"
 #define DEFAULT_ADVMV_CMD_FORCE "advmv -g"
 
 #define DEF_RM_FORCE 0
@@ -606,17 +661,24 @@ ws2=31:ws3=38;5;228:ws4=32:ws5=36:ws6=38;5;214:ws7=35:ws8=2;37:xf=1;31:xs=32:"
 #define DEF_FZFTAB_OPTIONS "--color=16,prompt:6,fg+:-1,pointer:4,\
 hl:2,hl+:2,gutter:-1,marker:2,border:7:dim --bind tab:accept,right:accept,\
 left:abort,alt-p:toggle-preview,change:top,alt-up:preview-page-up,\
-alt-down:preview-page-down --inline-info --layout=reverse-list \
+alt-down:preview-page-down --tiebreak=begin --inline-info --layout=reverse-list \
 --preview-window=wrap,border-left"
 // --preview-window=noborder (0.19, Nov 15, 2019)
 
 #define DEF_FZFTAB_OPTIONS_NO_COLOR "--color=bw --bind tab:accept,\
 right:accept,left:abort,alt-p:toggle-preview,change:top,alt-up:preview-page-up,\
-alt-down:preview-page-down --inline-info --layout=reverse-list \
+alt-down:preview-page-down --tiebreak=begin --inline-info --layout=reverse-list \
 --preview-window=wrap,border-left"
 
 #define DEF_SMENU_OPTIONS "-a t:2,b b:4 c:r ct:2,r sf:6,r st:5,r mt:5,b"
 
+/* Name of the file to store thumbnails original paths.
+ * Each line in this file has this format: THUMB_FILE@FILE_URI,
+ * where THUMB_FILE is the name of the thumbnail file (MD5 hash of FILE_URI),
+ * and FILE_URI is the file URI for the absolute path to the original filename.
+ * This file is located in $XDG_CACHE_HOME/clifm/thumbnails */
+#define THUMBNAILS_INFO_FILE ".thumbs.info"
+
 /* Should we add __APPLE__ here too? */
 #if defined(__HAIKU__)
 # define DEF_TERM_CMD "Terminal"
