@@ -1,17 +1,15 @@
-/* sort.c -- functions used to sort files */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,15 +20,17 @@
  * MA 02110-1301, USA.
 */
 
+/* sort.c -- functions used to sort files */
+
 #include "helpers.h"
 
-#include <stdio.h>
 #include <string.h>
 #include <unistd.h>
 #include <strings.h> /* str(n)casecmp() */
 
+#include "autocmds.h" /* update_autocmd_opts() */
+#include "aux.h"      /* xatoi */
 #include "checks.h"
-#include "aux.h" /* xatoi */
 #include "listing.h"
 #include "messages.h" /* SORT_USAGE */
 
@@ -42,11 +42,11 @@ skip_files(const struct dirent *ent)
 {
 	/* In case a directory isn't reacheable, like a failed
 	 * mountpoint... */
-	/*  struct stat file_attrib;
+	/*  struct stat a;
 
-	if (lstat(entry->d_name, &file_attrib) == -1) {
-		fprintf(stderr, _("stat: cannot access '%s': %s\n"),
-				entry->d_name, strerror(errno));
+	if (lstat(ent->d_name, &a) == -1) {
+		fprintf(stderr, _("lstat: cannot access '%s': %s\n"),
+			ent->d_name, strerror(errno));
 		return 0;
 	} */
 
@@ -67,14 +67,16 @@ skip_files(const struct dirent *ent)
 }
 
 /* Return a pointer to the first alphanumeric character in NAME, or to the
- * first character if no alphanumeric character is found */
+ * first character if no alphanumeric character is found.
+ * This function is not UTF8 aware, meaning that UTF8 non-alphanemeric
+ * characters are not supported. */
 static inline void
 skip_name_prefixes(char **name)
 {
 	char *s = *name;
 
 	while (*s) {
-		if (IS_ALNUM(*s))
+		if (IS_ALNUM(*s) || IS_UTF8_LEAD_BYTE(*s))
 			break;
 		s++;
 	}
@@ -90,16 +92,44 @@ int
 compare_strings(char **s1, char **s2)
 {
 #if defined(HAVE_STRCOLL)
-	return strcoll(*s2, *s2);
+	return strcoll(*s1, *s2);
 #else
-	int ret = **s1 - **s2;
-	if (ret == 0)
-		ret = strcmp(*s1, *s2);
-
-	return ret;
+	const int ret = **s1 - **s2;
+	return ret == 0 ? strcmp(*s1, *s2) : ret;
 #endif /* HAVE_STRCOLL */
 }
 
+static inline int
+check_hidden_file(const char c1, const char c2)
+{
+	const int ret = ((c1 == '.' && c2 != '.') ? -1
+		: ((c1 != '.' && c2 == '.') ? 1 : 0));
+
+	return (ret == 0 ? 0
+		: (conf.show_hidden == HIDDEN_FIRST ? ret : -ret));
+}
+
+static int
+check_priority_sort_char(const char c1, const char c2)
+{
+	const char *psch = conf.priority_sort_char;
+
+	if (!psch[1]) {  /* Single char */
+		return ((c1 == *psch && c2 != *psch) ? -1
+			: ((c1 != *psch && c2 == *psch) ? 1 : 0));
+	}
+
+	while (*psch) {
+		if (c1 == *psch && c2 != *psch)
+			return -1;
+		if (c1 != *psch && c2 == *psch)
+			return 1;
+		psch++;
+	}
+
+	return 0;
+}
+
 static int
 namecmp(char *s1, char *s2)
 {
@@ -108,23 +138,12 @@ namecmp(char *s1, char *s2)
 		skip_name_prefixes(&s2);
 	}
 
-	/* If both strings start with number, sort them as numbers, not as strings */
-	if (IS_DIGIT(*s1) && IS_DIGIT(*s2)) {
-		char *p1, *p2;
-		const long long n1 = strtoll(s1, &p1, 10);
-		const long long n2 = strtoll(s2, &p2, 10);
-		if (n2 > n1)
-			return -1;
-		if (n2 < n1)
-			return 1;
-	}
+	if (!IS_UTF8_LEAD_BYTE(*s1) && !IS_UTF8_LEAD_BYTE(*s2)) {
+	/* None of the strings begins with a unicode char: compare the first
+	 * byte of both strings. */
+		char ac = *s1, bc = *s2;
 
-	char ac = *s1, bc = *s2;
-
-	if ((*s1 & 0xc0) != 0xc0 && (*s2 & 0xc0) != 0xc0) {
-	/* None of the strings starts with a unicode char: compare the first
-	 * byte of both strings */
-		if (!conf.case_sens_list) {
+		if (conf.case_sens_list == 0) {
 			ac = (char)TOLOWER(*s1);
 			bc = (char)TOLOWER(*s2);
 		}
@@ -136,7 +155,7 @@ namecmp(char *s1, char *s2)
 			return 1;
 	}
 
-	if (!conf.case_sens_list || (*s1 & 0xc0) == 0xc0 || (*s2 & 0xc0) == 0xc0)
+	if (conf.case_sens_list == 0)
 		return strcoll(s1, s2);
 
 	return strcmp(s1, s2);
@@ -145,28 +164,11 @@ namecmp(char *s1, char *s2)
 static inline int
 sort_by_extension(struct fileinfo *pa, struct fileinfo *pb)
 {
-	char *e1 = (char *)NULL;
-	char *e2 = (char *)NULL;
+	const char *e1 = (pa->dir == 0 && pa->ext_name)
+		? pa->ext_name + (pa->ext_name[1] != '\0') : NULL;
 
-	if (pa->dir == 0) {
-		if (pa->ext_name) {
-			e1 = pa->ext_name + (pa->ext_name[1] != '\0');
-		} else {
-			char *p = strrchr(pa->name, '.');
-			if (p && p[1])
-				e1 = p + 1;
-		}
-	}
-
-	if (pb->dir == 0) {
-		if (pb->ext_name) {
-			e2 = pb->ext_name + (pb->ext_name[1] != '\0');
-		} else {
-			char *p = strrchr(pb->name, '.');
-			if (p && p[1])
-				e2 = p + 1;
-		}
-	}
+	const char *e2 = (pb->dir == 0 && pb->ext_name)
+		? pb->ext_name + (pb->ext_name[1] != '\0') : NULL;
 
 	if (e1 || e2) {
 		if (!e1)
@@ -174,7 +176,8 @@ sort_by_extension(struct fileinfo *pa, struct fileinfo *pb)
 		if (!e2)
 			return 1;
 
-		return strcasecmp(e1, e2);
+		return conf.case_sens_list == 1
+			? strcmp(e1, e2) : strcasecmp(e1, e2);
 	}
 
 	return 0;
@@ -220,30 +223,54 @@ sort_by_type(struct fileinfo *pa, struct fileinfo *pb)
 	return sort_by_extension(pa, pb);
 }
 
+static int
+sort_by_version(char *s1, char *s2, const int have_utf8)
+{
+	if (have_utf8 == 1)
+		return namecmp(s1, s2);
+
+	/* xstrverscmp is not UTF-8 aware. */
+	if (conf.skip_non_alnum_prefix == 1) {
+		skip_name_prefixes(&s1);
+		skip_name_prefixes(&s2);
+	}
+
+	return xstrverscmp(s1, s2);
+}
+
 int
 entrycmp(const void *a, const void *b)
 {
 	struct fileinfo *pa = (struct fileinfo *)a;
 	struct fileinfo *pb = (struct fileinfo *)b;
-	int ret = 0;
 	int st = conf.sort;
 
-	if (conf.list_dirs_first == 1) {
-		ret = F_SORT_DIRS(pa->dir, pb->dir);
-		if (ret != 0)
-			return ret;
-	}
+	int ret = conf.list_dirs_first == 1 ? F_SORT_DIRS(pa->dir, pb->dir) : 0;
+	if (ret != 0)
+		return ret;
+
+	ret = (conf.priority_sort_char && *conf.priority_sort_char)
+		? check_priority_sort_char(*pa->name, *pb->name) : 0;
+	if (ret != 0)
+		return ret;
+
+	/* > 1 = Either HIDDEN_FIRST or HIDDEN_LAST */
+	ret = conf.show_hidden > 1 ? check_hidden_file(*pa->name, *pb->name) : 0;
+	if (ret != 0)
+		return ret;
 
 	if (conf.light_mode == 1 && !ST_IN_LIGHT_MODE(st))
 		st = SNAME;
 
+	const int have_utf8 = (pa->utf8 == 1 || pb->utf8 == 1);
+
 	switch (st) {
 	case STSIZE: ret = F_SORT(pa->size, pb->size); break;
 	case SATIME: /* fallthrough */
 	case SBTIME: /* fallthrough */
 	case SCTIME: /* fallthrough */
 	case SMTIME: ret = F_SORT(pa->time, pb->time); break;
-	case SVER: ret = xstrverscmp(pa->name, pb->name); break;
+	case SVER: ret = sort_by_version(pa->name, pb->name, have_utf8); break;
 	case SEXT: ret = sort_by_extension(pa, pb); break;
 	case SINO: ret = F_SORT(pa->inode, pb->inode); break;
 	case SOWN: ret = sort_by_owner(pa, pb); break;
@@ -254,13 +281,10 @@ entrycmp(const void *a, const void *b)
 	default: break;
 	}
 
-	if (!ret)
+	if (ret == 0)
 		ret = namecmp(pa->name, pb->name);
 
-	if (!conf.sort_reverse)
-		return ret;
-
-	return (-ret);
+	return conf.sort_reverse == 0 ? ret : -ret;
 }
 
 /* Same as alphasort, but is uses strcmp instead of sctroll, which is
@@ -282,11 +306,7 @@ xalphasort(const struct dirent **a, const struct dirent **b)
 	else
 		ret = strcmp((*a)->d_name, (*b)->d_name);
 
-	if (!conf.sort_reverse)
-		return ret;
-
-	/* If sort_reverse, return the opposite value */
-	return (-ret);
+	return conf.sort_reverse == 0 ? ret : -ret;
 }
 
 /* This is a modification of the alphasort function that makes it case
@@ -299,14 +319,11 @@ alphasort_insensitive(const struct dirent **a, const struct dirent **b)
 	const int ret = strcasecmp(((*a)->d_name[0] == '.') ? (*a)->d_name + 1
 	: (*a)->d_name, ((*b)->d_name[0] == '.') ? (*b)->d_name + 1 : (*b)->d_name);
 
-	if (!conf.sort_reverse)
-		return ret;
-
-	return (-ret);
+	return conf.sort_reverse == 0 ? ret : -ret;
 }
 
 char *
-num_to_sort_name(const int n)
+num_to_sort_name(const int n, const int abbrev)
 {
 	switch (n) {
 	case SNONE:	 return "none";
@@ -316,22 +333,22 @@ num_to_sort_name(const int n)
 	case SBTIME: return "btime";
 	case SCTIME: return "ctime";
 	case SMTIME: return "mtime";
-	case SVER:   return "version";
-	case SEXT:   return "extension";
-	case SINO:   return "inode";
-	case SOWN:   return "owner";
-	case SGRP:   return "group";
-	case SBLK:   return "blocks";
-	case SLNK:   return "links";
+	case SVER:   return abbrev ? "ver" : "version";
+	case SEXT:   return abbrev ? "ext" : "extension";
+	case SINO:   return abbrev ? "ino" : "inode";
+	case SOWN:   return abbrev ? "own" : "owner";
+	case SGRP:   return abbrev ? "grp" : "group";
+	case SBLK:   return abbrev ? "blk" : "blocks";
+	case SLNK:   return abbrev ? "lnk" : "links";
 	case STYPE:  return "type";
-	default:     return "unknown";
+	default:     return abbrev ? "unk" : "unknown";
 	}
 }
 
 void
 print_sort_method(void)
 {
-	char *name = num_to_sort_name(conf.sort);
+	char *name = num_to_sort_name(conf.sort, 0);
 
 	printf("%s%s%s%s", BOLD, name, NC,
 		(conf.sort_reverse == 1) ? " [rev]" : "");
@@ -342,15 +359,6 @@ print_sort_method(void)
 		putchar('\n');
 }
 
-static inline void
-toggle_sort_reverse(void)
-{
-	if (conf.sort_reverse)
-		conf.sort_reverse = 0;
-	else
-		conf.sort_reverse = 1;
-}
-
 static inline int
 re_sort_files_list(void)
 {
@@ -358,7 +366,7 @@ re_sort_files_list(void)
 		return FUNC_SUCCESS;
 
 	/* sort_switch just tells list_dir() to print a line with the current
-	 * sorting order at the end of the files list. */
+	 * sort order at the end of the file list. */
 	sort_switch = 1;
 	free_dirlist();
 	const int ret = list_dir();
@@ -368,7 +376,7 @@ re_sort_files_list(void)
 }
 
 /* If ARG is a string, write the corresponding integer to ARG itself.
- * Return zero if ARG corresponds to a valid sorting method or one
+ * Return zero if ARG corresponds to a valid sort method or one
  * otherwise. */
 static inline int
 set_sort_by_name(char **arg)
@@ -390,24 +398,24 @@ set_sort_by_name(char **arg)
 		}
 	}
 
-	fprintf(stdout, _("st: %s: No such sorting order\n"), *arg);
+	fprintf(stdout, _("st: %s: No such sort order\n"), *arg);
 	return FUNC_FAILURE;
 }
 
 int
 sort_function(char **arg)
 {
-	/* No argument: Just print current sorting order */
+	/* No argument: Just print the current sort order. */
 	if (!arg[1]) {
 		fputs(_("Sorted by "), stdout);
 		print_sort_method();
 		return FUNC_SUCCESS;
 	}
 
-	/* Argument is alphanumerical string */
+	/* Argument is an alphanumerical string */
 	if (!is_number(arg[1])) {
 		if (*arg[1] == 'r' && strcmp(arg[1], "rev") == 0) {
-			toggle_sort_reverse();
+			conf.sort_reverse = !conf.sort_reverse;
 			return re_sort_files_list();
 		}
 
@@ -420,7 +428,7 @@ sort_function(char **arg)
 
 	if (conf.light_mode == 1 && !ST_IN_LIGHT_MODE(n)) {
 		fprintf(stderr, _("st: %d (%s): Not available in light mode\n"),
-			n, num_to_sort_name(n));
+			n, num_to_sort_name(n, 0));
 		return FUNC_FAILURE;
 	}
 
@@ -435,12 +443,14 @@ sort_function(char **arg)
 		conf.sort = n;
 
 		if (arg[2] && *arg[2] == 'r' && strcmp(arg[2], "rev") == 0)
-			toggle_sort_reverse();
+			conf.sort_reverse = !conf.sort_reverse;
+
+		update_autocmd_opts(AC_SORT);
 
 		return re_sort_files_list();
 	}
 
-	/* If arg1 is a number but is not in the range 0-SORT_TYPES, error */
+	/* If arg1 is a number but is not in the range 0-SORT_TYPES, err. */
 	fprintf(stderr, "%s\n", _(SORT_USAGE));
 	return FUNC_FAILURE;
 }
