@@ -1,17 +1,15 @@
-/* spawn.c -- Execute system commands */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* spawn.c -- Execute system commands */
+
 #include "helpers.h"
 
 #include <errno.h>    /* errno */
@@ -33,9 +33,7 @@
 #else
 # define _PATH_DEVNULL "/dev/null"
 #endif /* !_BE_POSIX */
-#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__sun)
-# include <signal.h>  /* signal */
-#endif /* BSD || __sun */
+#include <signal.h>   /* sigaction */
 #include <string.h>   /* strerror */
 #include <unistd.h>   /* fork, execl, execvp, dup2, close, _exit */
 #include <sys/wait.h> /* waitpid */
@@ -91,17 +89,26 @@ run_in_background(const pid_t pid)
 static void
 set_cmd_signals(void)
 {
-	signal(SIGHUP,  SIG_DFL);
-	signal(SIGINT,  SIG_DFL);
-	signal(SIGQUIT, SIG_DFL);
-	signal(SIGTERM, SIG_DFL);
-	signal(SIGTSTP, SIG_IGN);
+	struct sigaction sa;
+
+	memset(&sa, 0, sizeof(sa));
+	sigemptyset(&sa.sa_mask);
+	sa.sa_flags = 0;
+	sa.sa_handler = SIG_DFL;
+
+	sigaction(SIGHUP, &sa, NULL);
+	sigaction(SIGINT, &sa, NULL);
+	sigaction(SIGQUIT, &sa, NULL);
+	sigaction(SIGTERM, &sa, NULL);
+
+	sa.sa_handler = SIG_IGN;
+	sigaction(SIGTSTP, &sa, NULL);
 }
 
 /* Implementation of system(3).
  * Unlike system(3), which runs a command using '/bin/sh' as the executing
  * shell, xsystem() uses a custom shell (user.shell) specified via CLIFM_SHELL
- * or SHELL, falling back to '/bin/sh' only if none of these variables are set. */
+ * or SHELL, falling back to '/bin/sh' only if none of these variables is set. */
 static int
 xsystem(const char *cmd)
 {
@@ -198,6 +205,11 @@ launch_execv(char **cmd, const int bg, const int xflags)
 				dup2(fd, STDOUT_FILENO);
 			if (xflags & E_NOSTDERR)
 				dup2(fd, STDERR_FILENO);
+			if ((xflags & E_SETSID) && setsid() == (pid_t)-1) {
+				xerror("%s: setsid: %s\n", PROGRAM_NAME, strerror(errno));
+				close(fd);
+				_exit(errno);
+			}
 
 			close(fd);
 		}
