@@ -1,17 +1,15 @@
-﻿/* strings.c -- misc string manipulation functions */
-
-/*
- * This file is part of CliFM
+﻿/*
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* strings.c -- misc string manipulation functions */
+
 /* The xstrsncpy function is taken from https://github.com/jarun/nnn/blob/master/src/nnn.c,
  * licensed under BSD-2-clause.
  * All changes are licensed under GPL-2.0-or-later. */
@@ -35,7 +35,6 @@
 #if defined(__HAIKU__)
 # include <stdint.h>
 #endif /* __HAIKU__ */
-#include <stdio.h>
 #include <string.h>
 #ifndef HAVE_ARC4RANDOM
 # include <time.h> /* time(2) */
@@ -82,14 +81,15 @@ typedef char *rl_cpvfunc_t;
 
 #ifdef SOLARIS_DOORS
 # define IS_FILE_TYPE_FILTER(x) ((x) == 'b' || (x) == 'c' || (x) == 'C' \
-|| (x) == 'd' || (x) == 'f' || (x) == 'g' || (x) == 'h' || (x) == 'l' \
-|| (x) == 'o' || (x) == 'p' || (x) == 's' || (x) == 't' || (x) == 'u' \
-|| (x) == 'x' || (x) == 'D' || (x) == 'P')
+|| (x) == 'd' || (x) == 'D' || (x) == 'f' || (x) == 'F' || (x) == 'g' \
+|| (x) == 'h' || (x) == 'l' || (x) == 'L' || (x) == 'o' || (x) == 'p' \
+|| (x) == 's' || (x) == 't' || (x) == 'u' || (x) == 'x' || (x) == 'O' \
+|| (x) == 'P')
 #else
 # define IS_FILE_TYPE_FILTER(x) ((x) == 'b' || (x) == 'c' || (x) == 'C' \
-|| (x) == 'd' || (x) == 'f' || (x) == 'g' || (x) == 'h' || (x) == 'l' \
-|| (x) == 'o' || (x) == 'p' || (x) == 's' || (x) == 't' || (x) == 'u' \
-|| (x) == 'x')
+|| (x) == 'd' || (x) == 'D' || (x) == 'f' || (x) == 'F' || (x) == 'g' \
+|| (x) == 'h' || (x) == 'l' || (x) == 'L' || (x) == 'o' || (x) == 'p' \
+|| (x) == 's' || (x) == 't' || (x) == 'u' || (x) == 'x')
 #endif /* SOLARIS_DOORS */
 
 #define IS_GLOB(x, y) (((x) == '*' || (x) == '?' || (x) == '{' ) && (y) != ' ')
@@ -113,7 +113,7 @@ quote_str(const char *str)
 
 	const size_t len = strlen(str) + 3;
 	char *p = xnmalloc(len, sizeof(char));
-	char quote_char =
+	const char quote_char =
 		conf.quoting_style == QUOTING_STYLE_DOUBLE_QUOTES ? '"' : '\'';
 
 	snprintf(p, len, "%c%s%c", quote_char, str, quote_char);
@@ -142,34 +142,33 @@ count_chars(const char *restrict s, const char c)
 size_t
 count_words(size_t *start_word, size_t *full_word)
 {
-	size_t words = 0, w = 0, first_non_space = 0;
-	char q = 0;
+	size_t words = 0, i = 0, first_non_space = 0;
+	char quote = 0;
 	char *b = rl_line_buffer;
 
-	for (; b[w]; w++) {
+	for (; b[i]; i++) {
 		/* Keep track of open quotes. */
-		if (b[w] == '\'' || b[w] == '"')
-			q = q == b[w] ? 0 : b[w];
+		if (b[i] == '\'' || b[i] == '"')
+			quote = quote == b[i] ? 0 : b[i];
 
-		if (!first_non_space && b[w] != ' ') {
-			words = 1;
-			*start_word = w;
-			first_non_space = 1;
+		if (first_non_space == 0 && b[i] != ' ') {
+			words = first_non_space = 1;
+			*start_word = i;
 			continue;
 		}
 
-		if (w > 0 && b[w] == ' ' && b[w - 1] != '\\') {
-			if (!*full_word && b[w - 1] != '|'
-			&& b[w - 1] != ';' && b[w - 1] != '&')
-				*full_word = w; /* Index of the end of the first full word (cmd). */
-			if (b[w + 1] && b[w + 1] != ' ')
+		if (i > 0 && b[i] == ' ' && b[i - 1] != '\\') {
+			if (!*full_word && b[i - 1] != '|'
+			&& b[i - 1] != ';' && b[i - 1] != '&')
+				*full_word = i; /* Index of the end of the first full word (cmd). */
+			if (b[i + 1] && b[i + 1] != ' ')
 				words++;
 		}
 
 		/* If a process separator char is found, reset variables so that we
 		 * can start counting again for the new command. */
-		if (!q && cur_color != hq_c && w > 0 && b[w - 1] != '\\'
-		&& ((b[w] == '&' && b[w - 1] == '&') || b[w] == '|' || b[w] == ';'))
+		if (quote == 0 && cur_color != hq_c && i > 0 && b[i - 1] != '\\'
+		&& ((b[i] == '&' && b[i - 1] == '&') || b[i] == '|' || b[i] == ';'))
 			words = first_non_space = *full_word = 0;
 	}
 
@@ -225,13 +224,11 @@ xstrcasechr(char *s, char c)
 	if (!s || !*s)
 		return (char *)NULL;
 
-	const char uc = TOUPPER(c);
+	const char uc = (char)TOUPPER(c);
 	while (*s) {
-		if (TOUPPER(*s) != uc) {
-			s++;
-			continue;
-		}
-		return s;
+		if (TOUPPER(*s) == uc)
+			return s;
+		s++;
 	}
 
 	return (char *)NULL;
@@ -259,7 +256,7 @@ xstrrpbrk(char *s, const char *accept)
 }
 
 #ifdef _BE_POSIX
-/* strcasestr(3) is a not POSIX: let's use this as replacement.
+/* strcasestr(3) is a not POSIX function: let's use this as replacement.
  * Find the first occurrence of the string B in the string A, ignoring case. */
 char *
 x_strcasestr(char *a, char *b)
@@ -324,7 +321,7 @@ xstrncat(char *restrict dst, const size_t dst_len, const char *restrict src,
 	return dst;
 }
 
-/* strverscmp() is a GNU extension, and as such not available on some systems
+/* strverscmp() is a GNU extension, and as such not available on some systems.
  * This function is a modified version of the GLIBC and uClibc strverscmp()
  * taken from here:
  * https://elixir.bootlin.com/uclibc-ng/latest/source/libc/string/strverscmp.c
@@ -332,37 +329,14 @@ xstrncat(char *restrict dst, const size_t dst_len, const char *restrict src,
 
 /* Compare S1 and S2 as strings holding indices/version numbers,
  * returning less than, equal to or greater than zero if S1 is less than,
- * equal to or greater than S2 (for more info, see the texinfo doc). */
+ * equal to or greater than S2 (for more info, see the texinfo doc).
+ * This function is not UTF8 aware. */
 int
 xstrverscmp(const char *s1, const char *s2)
 {
-	if ((*s1 & 0xc0) == 0xc0 || (*s2 & 0xc0) == 0xc0)
-		return strcoll(s1, s2);
-
 	const unsigned char *p1 = (const unsigned char *)s1;
 	const unsigned char *p2 = (const unsigned char *)s2;
 
-	if (conf.skip_non_alnum_prefix == 1) {
-		/* Jump to first alphanumeric character in both strings */
-		while (*p1) {
-			if (IS_ALNUM(*p1))
-				break;
-			p1++;
-		}
-
-		if (!*p1)
-			p1 = (const unsigned char *)s1;
-
-		while (*p2) {
-			if (IS_ALNUM(*p2))
-				break;
-			p2++;
-		}
-
-		if (!*p2)
-			p2 = (const unsigned char *)s2;
-	}
-
 	/* Symbol(s)    0       [1-9]   others
 	 Transition   (10) 0  (01) d  (00) x   */
 	static const uint8_t next_state[] = {
@@ -388,9 +362,9 @@ xstrverscmp(const char *s1, const char *s2)
 		return 0;
 
 	if (!conf.case_sens_list) {
-		c1 = TOLOWER(*p1);
+		c1 = (unsigned char)TOLOWER(*p1);
 		++p1;
-		c2 = TOLOWER(*p2);
+		c2 = (unsigned char)TOLOWER(*p2);
 		++p2;
 	} else {
 		c1 = *p1;
@@ -408,9 +382,9 @@ xstrverscmp(const char *s1, const char *s2)
 
 		state = next_state[state];
 		if (!conf.case_sens_list) {
-			c1 = TOLOWER(*p1);
+			c1 = (unsigned char)TOLOWER(*p1);
 			++p1;
-			c2 = TOLOWER(*p2);
+			c2 = (unsigned char)TOLOWER(*p2);
 			++p2;
 		} else {
 			c1 = *p1;
@@ -469,10 +443,10 @@ wc_xstrlen(const char *restrict str)
 
 /* Truncate an UTF-8 string at width MAX.
  * Returns the difference beetween MAX and the point at which STR was actually
- * trimmed (this difference should be added to STR as spaces to equate MAX
+ * truncated (this difference should be added to STR as spaces to equate MAX
  * and get a correct length).
  * Since a wide char could take two o more columns to be drawn, and since
- * you might want to trim the name in the middle of a wide char, this
+ * you might want to truncate the name in the middle of a wide char, this
  * function won't store the last wide char to avoid taking more columns
  * than MAX. In this case, the programmer should take care of filling the
  * empty spaces (usually no more than one) theyself. */
@@ -585,15 +559,13 @@ check_control_char(const unsigned char *s)
 	if (*s < ' ' || *s == 127)
 		return 1; /* C0 control characters and DEL char */
 
-	if (*s == 0xc2 && *(s + 1) >= 0x80 && *(s + 1) <= 0x9f)
+	if (*s == 0xc2 && s[1] >= 0x80 && s[1] <= 0x9f)
 		return 2; /* C1 controls characters */
 
-	if (*s == 0xe2 && *(s + 1) == 0x80
-	&& (*(s + 2) == 0xa8 || *(s + 2) == 0xa9))
+	if (*s == 0xe2 && s[1] == 0x80 && (s[2] == 0xa8 || s[2] == 0xa9))
 		return 3; /* Line separator (xa8) / Paragraph separator (xa9) */
 
-	if (*s == 0xf3 && *(s + 1) == 0xa9 && *(s + 2) == 0x80
-	&& *(s + 3) == 0x81)
+	if (*s == 0xf3 && s[1] == 0xa9 && s[2] == 0x80 && s[3] == 0x81)
 		return 4; /* Language tag */
 
 	return 0;
@@ -869,7 +841,7 @@ init_quoted_words(void)
  * expansion of the 'sel' keyword, the index 2 points now to a selected file.
  *
  * This function updates quoted word indices taking into account the starting
- * point, i.e. the word after the expandable expression (START), and the amount
+ * point, i.e. the word after the expandable expression (START), and the number
  * of actually expanded fields (N). */
 static void
 update_quoted_words_index(const size_t start, const size_t added_items)
@@ -912,7 +884,7 @@ cmd_keeps_quotes(char *str)
 	char *p = strchr(str, ' ');
 	if (p) {
 		*p = '\0';
-		const int ret = is_internal_c(str);
+		const int ret = is_internal_cmd(str, ALL_CMDS, 1, 1);
 		*p = ' ';
 		/* Let's keep quotes for external commands */
 		if (ret == 0)
@@ -1016,6 +988,13 @@ split_str(char *str, const int update_args)
 
 		case '\'': /* fallthrough */
 		case '"': {
+			if (str_len > 0 && *(str - 1) == '\\') {
+				buf = xnrealloc(buf, buf_len + 1, sizeof(char *));
+				buf[buf_len] = *str;
+				buf_len++;
+				break;
+			}
+
 			const int is_quoted = (keep_quotes == 1
 				|| (str_len > 0 && *(str - 1) == '\\'));
 			/* If the quote is escaped, keep it. */
@@ -1205,14 +1184,14 @@ split_fused_param(char *str)
 			if (c && *(p - 1) != ' ' && *(p - 1) != '|'
 			&& *(p - 1) != '&' && *(p - 1) != ';')
 				words++;
-			if (*(p + 1))
+			if (p[1])
 				pp = p + 1;
 			break;
 		case '&': /* fallthrough */
 		case '|': /* fallthrough */
 		case ';':
 			words = 1;
-			if (*(p + 1))
+			if (p[1])
 				pp = p + 1;
 			break;
 		default: break;
@@ -1223,7 +1202,9 @@ split_fused_param(char *str)
 		&& check_fused_param(p)) {
 			char t = *p;
 			*p = '\0';
-			if (is_internal_f(pp)) {
+			const size_t plen = strlen(pp);
+			if (plen > 0 && pp[plen - 1] != '-'
+			&& is_internal_cmd(pp, PARAM_FNAME_NUM, 0, 0)) {
 				*b = ' ';
 				b++;
 			}
@@ -1278,7 +1259,6 @@ check_shell_functions(const char *str)
 		"if ", "if(",
 		"[ ", "[[ ", "test ",
 		"case ", "case(",
-		"echo ", "printf ",
 		"declare ",
 		"(( ",
 		"set ",
@@ -1301,18 +1281,18 @@ check_shell_functions(const char *str)
 static int
 is_fused_param(char *str)
 {
-	if (!str || !*str || !*(str + 1))
+	if (!str || !*str || !str[1])
 		return FUNC_FAILURE;
 
 	char *p = str, *q = (char *)NULL;
 	int d = 0;
 
 	while (*p && *p != ' ') {
-		if (d == 0 && p != str && IS_DIGIT(*p) && IS_ALPHA(*(p - 1))) {
+		if (d == 0 && p != str && IS_DIGIT(*p) && IS_ALPHA_LOW(*(p - 1))) {
 			q = p;
 			d = 1;
 		}
-		if (d == 1 && IS_ALPHA(*p))
+		if (d == 1 && IS_ALPHA_LOW(*p))
 			return FUNC_FAILURE;
 		p++;
 	}
@@ -1322,16 +1302,13 @@ is_fused_param(char *str)
 	if (!q)
 		return FUNC_FAILURE;
 
-	char c = *q;
+	const char c = *q;
 	*q = '\0';
 
-	const int ret = is_internal_f(str);
+	const int ret = is_internal_cmd(str, PARAM_FNAME_NUM, 0, 0);
 	*q = c;
 
-	if (ret == 0)
-		return FUNC_FAILURE;
-
-	return FUNC_SUCCESS;
+	return (ret == 0 ? FUNC_FAILURE : FUNC_SUCCESS);
 }
 
 #ifndef _NO_TAGS
@@ -1379,7 +1356,7 @@ expand_tag(char ***args, const int tag_index)
 	}
 	p[j] = (char *)NULL;
 
-	/* Append the file names pointed to by the tag expression */
+	/* Append all filenames pointed to by the tag expression */
 	for (i = 0; i < (size_t)n; i++) {
 		if (SELFORPARENT(t[i]->d_name))
 			continue;
@@ -1454,7 +1431,7 @@ expand_tags(char ***substr)
 
 	for (i = 0; i < ntags; i++) {
 		const size_t tn = expand_tag(substr, tag_index[i]);
-		/* TN is the amount of files tagged as SUBSTR[TAG_INDEX[I]]
+		/* TN is the number of files tagged as SUBSTR[TAG_INDEX[I]]
 		 * Let's update the index of the next tag expression using this
 		 * value: if the next tag expression was at index 2, and if
 		 * the current tag expression was expanded to 3 files,
@@ -1512,112 +1489,70 @@ expand_mime_type_filter(const char *pattern)
 }
 #endif /* !_NO_MAGIC */
 
+/* Return an array of filenames matching the file type T or NULL if no
+ * match is found. */
 static char **
 expand_file_type_filter(const char t)
 {
 	if (files == 0)
 		return (char **)NULL;
 
-	filesn_t i = 0, n = 0;
+	filesn_t i = 0, c = 0;
 
 	char **f = xnmalloc((size_t)files + 1, sizeof(char *));
 	char buf[PATH_MAX + 1];
 
 	while (i < files) {
-		char *name = file_info[i].name;
+		char *n = file_info[i].name;
 		if (virtual_dir == 1) {
 			*buf = '\0';
 			if (xreadlink(XAT_FDCWD, file_info[i].name, buf, sizeof(buf)) == -1
 			|| !*buf)
 				continue;
-			name = buf;
+			n = buf;
 		}
 
-		if (!name || !*name)
+		if (!n || !*n)
 			continue;
 
 		switch (t) {
-		case 'b':
-			if (file_info[i].type == DT_BLK)
-				f[n++] = strdup(name);
-			break;
-		case 'c':
-			if (file_info[i].type == DT_CHR)
-				f[n++] = strdup(name);
-			break;
-		case 'C':
-			if (file_info[i].color == ca_c)
-				f[n++] = strdup(name);
-			break;
-		case 'd':
-			if (file_info[i].dir == 1)
-				f[n++] = strdup(name);
-			break;
+		case 'b': if (file_info[i].type == DT_BLK) f[c++] = strdup(n); break;
+		case 'c': if (file_info[i].type == DT_CHR) f[c++] = strdup(n); break;
+		case 'C': if (file_info[i].color == ca_c) f[c++] = strdup(n); break;
+		case 'd': if (file_info[i].dir == 1) f[c++] = strdup(n); break;
+		case 'D': if (file_info[i].color == ed_c) f[c++] = strdup(n); break;
 #ifdef SOLARIS_DOORS
-		case 'D':
-			if (file_info[i].type == DT_DOOR)
-				f[n++] = strdup(name);
-			break;
-		case 'P':
-			if (file_info[i].type == DT_PORT)
-				f[n++] = strdup(name);
-			break;
+		case 'O': if (file_info[i].type == DT_DOOR) f[c++] = strdup(n); break;
+		case 'P': if (file_info[i].type == DT_PORT) f[c++] = strdup(n); break;
 #endif /* SOLARIS_DOORS */
-		case 'f':
-			if (file_info[i].type == DT_REG)
-				f[n++] = strdup(name);
-			break;
-		case 'h':
+		case 'f': if (file_info[i].type == DT_REG) f[c++] = strdup(n); break;
+		case 'F': if (file_info[i].color == ef_c) f[c++] = strdup(n); break;
+		case 'h': 
 			if (file_info[i].dir == 0 && file_info[i].linkn > 1)
-				f[n++] = strdup(name);
-			break;
-		case 'l':
-			if (file_info[i].type == DT_LNK)
-				f[n++] = strdup(name);
-			break;
-
-		case 'o':
-			if (file_info[i].color == tw_c || file_info[i].color == ow_c)
-				f[n++] = strdup(name);
-			break;
-		case 't':
-			if (file_info[i].color == tw_c || file_info[i].color == st_c)
-				f[n++] = strdup(name);
-			break;
-
-		case 'p':
-			if (file_info[i].type == DT_FIFO)
-				f[n++] = strdup(name);
-			break;
-		case 's':
-			if (file_info[i].type == DT_SOCK)
-				f[n++] = strdup(name);
-			break;
-		case 'x':
-			if (file_info[i].exec == 1)
-				f[n++] = strdup(name);
-			break;
-		case 'u':
-			if (file_info[i].color == su_c)
-				f[n++] = strdup(name);
-			break;
-		case 'g':
-			if (file_info[i].color == sg_c)
-				f[n++] = strdup(name);
+				f[c++] = strdup(n);
 			break;
+		case 'l': if (file_info[i].type == DT_LNK) f[c++] = strdup(n); break;
+		case 'L': if (file_info[i].color == or_c) f[c++] = strdup(n); break;
+		case 'o': if (file_info[i].mode & S_IWOTH) f[c++] = strdup(n); break;
+		case 't': if (file_info[i].mode & S_ISVTX) f[c++] = strdup(n); break;
+		case 'p': if (file_info[i].type == DT_FIFO) f[c++] = strdup(n); break;
+		case 's': if (file_info[i].type == DT_SOCK) f[c++] = strdup(n); break;
+		case 'x': if (file_info[i].exec == 1) f[c++] = strdup(n); break;
+		case 'u': if (file_info[i].mode & S_ISUID) f[c++] = strdup(n); break;
+		case 'g': if (file_info[i].mode & S_ISGID) f[c++] = strdup(n); break;
 		default: break;
 		}
 
 		i++;
 	}
 
-	if (n == 0) {
+	if (c == 0) {
 		free(f);
 		return (char **)NULL;
 	}
 
-	f[n] = (char *)NULL;
-	f = xnrealloc(f, (size_t)n + 1, sizeof(char *));
+	f[c] = (char *)NULL;
+	f = xnrealloc(f, (size_t)c + 1, sizeof(char *));
 
 	return f;
 }
@@ -1648,7 +1583,7 @@ insert_fields(char ***dst, char ***src, const size_t i, size_t *num)
 
 	char **s = *src;
 
-	/* 1. Get amount of fields in SRC */
+	/* 1. Get number of fields in SRC */
 	size_t sn;
 	for (sn = 0; s[sn]; sn++);
 
@@ -1694,7 +1629,7 @@ insert_fields(char ***dst, char ***src, const size_t i, size_t *num)
 	return d;
 }
 
-/* Expand the ELN at SUBSTR[I] into the corresponding file name, which is
+/* Expand the ELN at SUBSTR[I] into the corresponding filename, which is
  * quoted, if necessary, according to the value of conf.quoting_style. */
 static void
 eln_expand(char ***substr, const size_t i)
@@ -1705,21 +1640,21 @@ eln_expand(char ***substr, const size_t i)
 
 	/* Because of should_expand_eln(), which is called immediately before this
 	 * function, it is guaranteed that NUM won't over/under-flow:
-	 * NUM is > 0 and <= the amount of listed files (and this latter is
+	 * NUM is > 0 and <= the number of listed files (and this latter is
 	 * never bigger than FILESN_MAX). */
 	const filesn_t j = num - 1;
 
-	/* If file name starts with a dash, and the command is external,
-	 * use the absolute path to the file name, to prevent the command from
-	 * taking the file name as a command option. */
+	/* If filename starts with a dash, and the command is external,
+	 * use the absolute path to the filename, to prevent the command from
+	 * taking the filename as a command option. */
 	char *abs_path = (char *)NULL;
 	if (file_info[j].name && *file_info[j].name == '-'
-	&& !is_internal_c((*substr)[0]))
+	&& !is_internal_cmd((*substr)[0], ALL_CMDS, 1, 1))
 		abs_path = xrealpath(file_info[j].name, NULL);
 
 	char *esc_str = (char *)NULL;
 	if (conf.quoting_style == QUOTING_STYLE_BACKSLASH
-	|| is_internal_c((*substr)[0]) || is_number((*substr)[0]))
+	|| is_internal_cmd((*substr)[0], ALL_CMDS, 1, 1) || is_number((*substr)[0]))
 		esc_str = escape_str(abs_path ? abs_path : file_info[j].name);
 	else
 		esc_str = quote_str(abs_path ? abs_path : file_info[j].name);
@@ -1732,7 +1667,7 @@ eln_expand(char ***substr, const size_t i)
 	if (i == 0)
 		flags |= FIRST_WORD_IS_ELN;
 
-	/* Replace the ELN by the corresponding escaped file name */
+	/* Replace the ELN by the corresponding escaped filename */
 	struct stat a;
 	if (file_info[j].type == DT_DIR && file_info[j].name[file_info[j].len > 0
 	? file_info[j].len - 1 : 0] != '/'
@@ -1774,7 +1709,7 @@ expand_sel(char ***substr)
 
 	/* 2. Add all selected files (in place of 'sel') */
 	for (i = 0; i < sel_n; i++) {
-		/* Escape selected file names and copy them into tmp array */
+		/* Escape selected filenames and copy them into tmp array */
 		char *esc_str = escape_str(sel_elements[i].name);
 		if (!esc_str)
 			continue;
@@ -2113,6 +2048,9 @@ expand_glob(char ***substr, const int *glob_array, const size_t glob_n)
 	for (g = 0; g < (size_t)glob_n; g++) {
 		glob_t globbuf;
 
+		if (is_quoted_word((size_t)glob_array[g] + old_pathc))
+			continue;
+
 		if (glob((*substr)[glob_array[g] + (int)old_pathc],
 			GLOB_BRACE | GLOB_TILDE, NULL, &globbuf) != FUNC_SUCCESS) {
 			globfree(&globbuf);
@@ -2136,7 +2074,7 @@ expand_glob(char ***substr, const int *glob_array, const size_t glob_n)
 				continue;
 
 			char *esc_str = (char *)NULL;
-			/* Escape the globbed file name and copy it */
+			/* Escape the globbed filename and copy it */
 			if (virtual_dir == 1 && is_file_in_cwd(globbuf.gl_pathv[i])) {
 				char buf[PATH_MAX + 1]; *buf = '\0';
 				if (xreadlink(XAT_FDCWD, globbuf.gl_pathv[i], buf,
@@ -2151,7 +2089,7 @@ expand_glob(char ***substr, const int *glob_array, const size_t glob_n)
 				glob_cmd[j] = esc_str;
 				j++;
 			} else {
-				xerror(_("%s: '%s': Error quoting file name\n"),
+				xerror(_("%s: '%s': Error quoting filename\n"),
 					PROGRAM_NAME, globbuf.gl_pathv[i]);
 				continue;
 			}
@@ -2219,13 +2157,13 @@ expand_word(char ***substr, const int *word_array, const size_t word_n)
 			}
 
 			for (i = 0; i < wordbuf.we_wordc; i++) {
-				/* Escape the globbed file name and copy it */
+				/* Escape the globbed filename and copy it */
 				char *esc_str = escape_str(wordbuf.we_wordv[i]);
 				if (esc_str) {
 					word_cmd[j] = esc_str;
 					j++;
 				} else {
-					xerror(_("%s: '%s': Error quoting file name\n"),
+					xerror(_("%s: '%s': Error quoting filename\n"),
 						PROGRAM_NAME, wordbuf.we_wordv[i]);
 
 					size_t k = 0;
@@ -2300,7 +2238,7 @@ check_ranges(char ***substr, int **range_array)
 
 /* Expand a range of numbers given by STR. It will expand the range
  * provided that both extremes are numbers, bigger than zero, equal or
- * smaller than the amount of files currently listed on the screen, and
+ * smaller than the number of files currently listed on the screen, and
  * the second (right) extreme is bigger than the first (left). Returns
  * an array of int's with the expanded range or NULL if one of the
  * above conditions is not met. */
@@ -2449,15 +2387,18 @@ expand_regex(char ***substr)
 			break;
 
 		/* Ignore the first string of the search function: it will be
-		 * expanded by the search function itself. */
-		if (*(*substr)[0] == '/') {
+		 * expanded by the search function itself.
+		 * Also, ignore quoted words and existent filenames. */
+		struct stat a;
+		if (*(*substr)[0] == '/' || is_quoted_word((size_t)i)
+		|| lstat((*substr)[i], &a) != -1) {
 			tmp[n] = (*substr)[i];
 			n++;
 			continue;
 		}
 
-		/* At this point, all file names are escaped. But check_regex()
-		 * needs unescaped file names. So, let's deescape it. */
+		/* At this point, all filenames are escaped. But check_regex()
+		 * needs unescaped filenames. So, let's deescape it. */
 		char *p = strchr((*substr)[i], '\\');
 		char *dstr = (char *)NULL;
 		if (p)
@@ -2474,7 +2415,7 @@ expand_regex(char ***substr)
 		snprintf(rstr, l, "^%s$", t);
 		free(dstr);
 
-		int ret = check_regex(rstr);
+		const int ret = check_regex(rstr);
 
 		if (ret != FUNC_SUCCESS
 		|| regcomp(&regex, rstr, reg_flags) != FUNC_SUCCESS) {
@@ -2493,7 +2434,7 @@ expand_regex(char ***substr)
 			if (regexec(&regex, file_info[j].name, 0, NULL, 0) != FUNC_SUCCESS)
 				continue;
 
-			/* Make sure the matching file name is not already in the tmp array */
+			/* Make sure the matching filename is not already in the tmp array */
 			filesn_t m = (filesn_t)n, found = 0;
 			while (--m >= 0) {
 				if (*file_info[j].name == *tmp[m]
@@ -2601,14 +2542,22 @@ glob_expand(char **cmd)
 	if (!cmd || !cmd[0] || !*cmd[0])
 		return 0;
 
-	/* Do not expand if command is deselect, sel or untrash, just to
-	 * allow the use of "*" for desel and untrash ("ds *" and "u *"),
-	 * and to let the sel function handle glob patterns itself. */
+	/* In case of 'desel', allow glob expansion only if the first parameter
+	 * is not "*", in which case the deselect function itself takes care of
+	 * deselecting all files. */
+	if (sel_n > 0 && cmd[1] && *cmd[1]
+	&& (strcmp(cmd[0], "ds") == 0 || strcmp(cmd[0], "desel") == 0)) {
+		if (*cmd[1] == '*' && !cmd[1][1])
+			return 0;
+		return (check_glob_char(cmd[1], GLOB_REGEX));
+	}
+
+	/* Do not expand if command is sel or untrash, just to allow the use
+	 * of "*" for desel and untrash ("ds *" and "u *"), and to let the
+	 * sel function handle glob patterns itself. */
 
 	if (strcmp(cmd[0], "s") != 0 && strcmp(cmd[0], "sel") != 0
 
-	&& strcmp(cmd[0], "ds") != 0 && strcmp(cmd[0], "desel") != 0
-
 	&& strcmp(cmd[0], "u") != 0 && strcmp(cmd[0], "undel") != 0
 	&& strcmp(cmd[0], "untrash") != 0
 
@@ -2692,7 +2641,7 @@ static int
 check_chained_cmds(char *str)
 {
 	/* If the user wants to create a file containing either '|' or ';' in the
-	 * name, they should be allowed. The file name will be later validated by
+	 * name, they should be allowed. The filename will be later validated by
 	 * the 'new' function itself. */
 	if (str && (strncmp(str, "n ", 2) == 0 || strncmp(str, "new ", 4) == 0))
 		return 0;
@@ -2720,7 +2669,7 @@ check_chained_cmds(char *str)
 		buf[len] = '\0';
 
 		if (strcmp(buf, "&&") != 0) {
-			if (is_internal_c(buf)) {
+			if (is_internal_cmd(buf, ALL_CMDS, 1, 1)) {
 				internal_ok = 1;
 				break;
 			}
@@ -2741,7 +2690,7 @@ check_chained_cmds(char *str)
 }
 
 /*
- * This function is one of the keys of CliFM. It will perform a series of
+ * This function is one of the keys of clifm. It will perform a series of
  * actions:
  * 1) Take the string stored by readline and get its substrings without
  * leading and trailing spaces (dequoting/unescaping if necessary).
@@ -2751,7 +2700,7 @@ check_chained_cmds(char *str)
  * 3) If the input string begins with ';' or ':' the whole string is
  * sent to exec_cmd(), where it will be directly executed by the system
  * shell.
- * 4) The following expansions (especific to CLiFM) are performed here:
+ * 4) The following expansions (especific to clifm) are performed here:
  * ELN's, "sel" keyword, ranges of numbers (ELN's), tags, pinned dir,
  * bookmark names, environment variables, file types (=x), mime types (@...),
  * path normalization, fastback, and, for internal commands only, tilde,
@@ -2769,7 +2718,7 @@ parse_input_str(char *str)
 	flags &= ~FIRST_WORD_IS_ELN;
 	flags &= ~IS_USRVAR_DEF;
 
-	/* If internal command plus fused parameter, split it */
+	/* If internal command plus fused parameter, split it. */
 	if (is_fused_param(str) == FUNC_SUCCESS) {
 		char *p = split_fused_param(str);
 		if (p) {
@@ -2785,13 +2734,12 @@ parse_input_str(char *str)
 	int chaining = 0, cond_cmd = 0, send_shell = 0;
 
 				/* ###########################
-				 * #  0.a) RUN AS EXTERNAL   #
+				 * #  0.1) RUN AS EXTERNAL   #
 				 * ###########################*/
 
-	/* If invoking a command via ';' or ':' set the send_shell flag to
+	/* If invoking a command via ';' or ':', set the send_shell flag to
 	 * true and send the whole string to exec_cmd(), in which case no
-	 * expansion is made: the command is send to the system shell as
-	 * is. */
+	 * expansion is made: the command is send to the system shell as is. */
 	if (*str == ';' || *str == ':' || check_shell_functions(str) == 1)
 		send_shell = 1;
 
@@ -2799,43 +2747,39 @@ parse_input_str(char *str)
 		for (i = 0; str[i]; i++) {
 
 				/* ##################################
-				 * #   0.b) CONDITIONAL EXECUTION   #
+				 * #   0.2) CONDITIONAL EXECUTION   #
 				 * ##################################*/
 
-			/* Check for chained commands (cmd1;cmd2) */
+			/* Check for chained commands (cmd1;cmd2). */
 			if (chaining == 0 && str[i] == ';' && i > 0 && str[i - 1] != '\\')
 				chaining = 1;
 
-			/* Check for conditional execution (cmd1 && cmd 2)*/
+			/* Check for conditional execution (cmd1 && cmd 2). */
 			if (cond_cmd == 0 && str[i] == '&' && i > 0 && str[i - 1] != '\\'
 			&& str[i + 1] == '&')
 				cond_cmd = 1;
 
 				/* ##################################
-				 * #   0.c) USER DEFINED VARIABLE   #
+				 * #   0.3) USER DEFINED VARIABLE   #
 				 * ##################################*/
 
-			/* If user defined variable send the whole string to
-			 * exec_cmd(), which will take care of storing the
-			 * variable. */
+			/* If user defined variable, send the whole string to
+			 * exec_cmd(), which will take care of storing the variable. */
 			if (!(flags & IS_USRVAR_DEF) && conf.int_vars == 1 && str[i] == '='
-			&& i > 0 && str[i - 1] != '\\' && str[0] != '=') {
-				if (check_int_var(str) == 1)
-					flags |= IS_USRVAR_DEF;
-			}
+			&& i > 0 && str[i - 1] != '\\' && str[0] != '='
+			&& check_int_var(str) == 1)
+				flags |= IS_USRVAR_DEF;
 		}
 	}
 
 	/* If chained commands, check each of them. If at least one of them
 	 * is internal, take care of the job (the system shell does not know
 	 * our internal commands and therefore cannot execute them); else,
-	 * if no internal command is found, let it to the system shell */
-	if (chaining == 1 || cond_cmd == 1) {
-		if (check_chained_cmds(str) == 1) {
-			if (fusedcmd_ok == 1)
-				free(str);
-			return (char **)NULL;
-		}
+	 * if no internal command is found, leave it to the system shell. */
+	if ((chaining == 1 || cond_cmd == 1) && check_chained_cmds(str) == 1) {
+		if (fusedcmd_ok == 1)
+			free(str);
+		return (char **)NULL;
 	}
 
 	if ((flags & IS_USRVAR_DEF) || send_shell == 1)
@@ -2849,17 +2793,17 @@ parse_input_str(char *str)
 	 * terminating and double spaces. */
 	char **substr = split_str(str, UPDATE_ARGS);
 
-	if (fusedcmd_ok == 1) /* Just in case split_fusedcmd returned NULL */
+	if (fusedcmd_ok == 1) /* Just in case split_fusedcmd returned NULL. */
 		free(str);
 
 	if (!substr)
 		return (char **)NULL;
 
-	/* Do not perform expansions for the 'n/new' command */
+	/* Do not perform expansions for the 'n/new' command. */
 	if (*substr[0] == 'n' && (!substr[0][1] || strcmp(substr[0], "new") == 0))
 		return substr;
 
-	/* Handle background/foreground process */
+	/* Handle background/foreground process. */
 	bg_proc = 0;
 
 	if (args_n > 0 && *substr[args_n] == '&' && !*(substr[args_n] + 1)) {
@@ -2879,11 +2823,8 @@ parse_input_str(char *str)
 					 * #     TRASH AS RM    #
 					 * ###################### */
 #ifndef _NO_TRASH
-	if (conf.tr_as_rm && substr[0] && *substr[0] == 'r' && !substr[0][1]) {
-		substr[0] = xnrealloc(substr[0], 2, sizeof(char));
+	if (conf.tr_as_rm == 1 && substr[0] && *substr[0] == 'r' && !substr[0][1])
 		*substr[0] = 't';
-		substr[0][1] = '\0';
-	}
 #endif /* !_NO_TRASH*/
 
 				/* ##############################
@@ -2891,19 +2832,19 @@ parse_input_str(char *str)
 				 * ##############################
 
 	 * Ranges, sel, ELN, pinned dirs, bookmarks, and internal variables.
-	 * These expansions are specific to CliFM. To be able to use them
+	 * These expansions are specific to clifm. To be able to use them
 	 * even with external commands, they must be expanded here, before
-	 * sending the input string, in case the command is external, to
+	 * sending the input string (in case the command is external) to
 	 * the system shell. */
 	is_sel = 0; sel_is_last = 0;
 
 	const int stdin_dir_ok = (stdin_tmp_dir
 		&& strcmp(workspaces[cur_ws].path, stdin_tmp_dir) == 0);
 
-	const int is_internal_cmd = is_internal(substr[0]);
+	const int is_int_cmd = is_internal_cmd(substr[0], PARAM_FNAME, 0, 1);
 
 	/* Let's expand ranges first: the numbers resulting from the expanded range
-	 * will be expanded into the corresponding file names by eln_expand() below. */
+	 * will be expanded into the corresponding filenames by eln_expand() below. */
 	expand_ranges(&substr);
 
 	for (i = 0; i <= args_n; i++) {
@@ -2911,10 +2852,18 @@ parse_input_str(char *str)
 		&& (virtual_dir == 0 || is_file_in_cwd(substr[i]) == 0)))
 			continue;
 
-		/* The following expansions expand into a SINGLE field */
+		/* The following expansions expand to a SINGLE field. */
+
+			/* ###################################
+			 * #   2.1) USER DEFINED VARIABLES   #
+			 * ###################################*/
+
+		if (conf.int_vars == 1 && usrvar_n > 0 && substr[i][0] == '$'
+		&& substr[i][1] && substr[i][1] != '(' && substr[i][1] != '{')
+			expand_int_var(&substr[i]);
 
 				/* ##########################
-				 * #   2.1) ELN EXPANSION   #
+				 * #   2.2) ELN EXPANSION   #
 				 * ########################## */
 
 		/* should_expand_eln() will check rl_line_buffer looking for the
@@ -2930,20 +2879,19 @@ parse_input_str(char *str)
 		rl_line_buffer = lb_tmp;
 
 				/* ################################
-				 * #  2.2) ENVIRONEMNT VARIABLES  #
+				 * #  2.3) ENVIRONEMNT VARIABLES  #
 				 * ###############################*/
 
 		if (*substr[i] == '$') {
-			char *p = getenv(substr[i] + 1);
+			char *p = xgetenv(substr[i] + 1, 1);
 			if (p) {
-				const size_t plen = strlen(p) + 1;
-				substr[i] = xnrealloc(substr[i], plen, sizeof(char));
-				xstrsncpy(substr[i], p, plen);
+				free(substr[i]);
+				substr[i] = p;
 			}
 		}
 
 				/* ################################
-				 * #  2.3) TILDE: ~user and home  #
+				 * #  2.4) TILDE: ~user and home  #
 				 * ################################ */
 
 		if (*substr[i] == '~') {
@@ -2955,25 +2903,26 @@ parse_input_str(char *str)
 		}
 
 			/* ##################################
-			 * #     2.4) URI file scheme       #
+			 * #     2.5) URI file scheme       #
 			 * ################################## */
 			/* file:///some/file -> /some/file */
 
 		size_t slen = strlen(substr[i]);
-		if (slen > FILE_URI_PREFIX_LEN && IS_FILE_URI(substr[i])) {
-			char tmp[PATH_MAX + 1];
-			xstrsncpy(tmp, substr[i], sizeof(tmp));
-			xstrsncpy(substr[i], tmp + FILE_URI_PREFIX_LEN, slen + 1);
+		if (IS_FILE_URI(substr[i], slen)) {
+			char *ptr = url_decode(substr[i] + FILE_URI_PREFIX_LEN);
+			if (ptr && *ptr)
+				xstrsncpy(substr[i], ptr, slen + 1);
+			free(ptr);
 		}
 
 			/* ###############################
-			 * #     2.5) "." and ".."       #
+			 * #     2.6) "." and ".."       #
 			 * ############################### */
 
 		if ((*substr[i] == '.' && (!substr[i][1] || (substr[i][1] == '.'
 		&& (!substr[i][2] || substr[i][2] == '/'))))
 		|| strstr(substr[i], "/..")) {
-			char *tmp = (is_internal_cmd == 1
+			char *tmp = (is_int_cmd == 1
 			&& (*substr[0] != 'l' || substr[0][1])) /* Exclude 'l' command. */
 				? normalize_path(substr[i], strlen(substr[i])) : NULL;
 			if (tmp) {
@@ -2983,7 +2932,7 @@ parse_input_str(char *str)
 		}
 
 			/* ######################################
-			 * #     2.6) FASTBACK EXPANSION        #
+			 * #     2.7) FASTBACK EXPANSION        #
 			 * ###################################### */
 
 		if (*substr[i] == '.' && substr[i][1] == '.' && substr[i][2] == '.') {
@@ -2995,7 +2944,7 @@ parse_input_str(char *str)
 		}
 
 			/* ######################################
-			 * #     2.7) PINNED DIR EXPANSION      #
+			 * #     2.8) PINNED DIR EXPANSION      #
 			 * ###################################### */
 
 		if (*substr[i] == ',' && !substr[i][1] && pinned_dir) {
@@ -3005,39 +2954,27 @@ parse_input_str(char *str)
 		}
 
 			/* ######################################
-			 * #   2.8) BOOKMARK NAMES EXPANSION    #
+			 * #   2.9) BOOKMARK NAMES EXPANSION    #
 			 * ###################################### */
 
-		/* Expand bookmark name (b:NAME) into the corresponding path */
-		if (*substr[i] == 'b' && substr[i][1] == ':' && substr[i][2]) {
-			if (expand_bm_name(&substr[i]) == FUNC_SUCCESS)
-				continue;
-		}
+		/* Expand bookmark name (b:NAME) into the corresponding path. */
+		if (*substr[i] == 'b' && substr[i][1] == ':' && substr[i][2]
+		&& expand_bm_name(&substr[i]) == FUNC_SUCCESS)
+			continue;
 
 			/* ######################################
-			 * #     2.9) WORKSPACE EXPANSION       #
+			 * #     2.10) WORKSPACE EXPANSION      #
 			 * ###################################### */
 
-		if (*substr[i] == 'w' && substr[i][1] == ':' && substr[i][2]) {
-			if (expand_workspace(&substr[i]) == FUNC_SUCCESS)
-				continue;
-		}
-
-			/* ###################################
-			 * #   2.10) USER DEFINED VARIABLES   #
-			 * ###################################*/
-
-		if (conf.int_vars == 1 && usrvar_n > 0) {
-			if (substr[i][0] == '$' && substr[i][1] && substr[i][1] != '('
-			&& substr[i][1] != '{')
-				expand_int_var(&substr[i]);
-		}
+		if (*substr[i] == 'w' && substr[i][1] == ':' && substr[i][2]
+		&& expand_workspace(&substr[i]) == FUNC_SUCCESS)
+			continue;
 
 			/* ###################################
 			 * #  2.11) SYMLINKS IN VIRTUAL DIR  #
 			 * ################################### */
 
-		/* We are in STDIN_TMP_DIR: Expand symlinks to target */
+		/* We are in STDIN_TMP_DIR: Expand symlinks to target. */
 		if (stdin_dir_ok == 1 && expand_symlink(&substr[i]) == -1) {
 			for (i = 0; i <= args_n; i++)
 				free(substr[i]);
@@ -3046,7 +2983,7 @@ parse_input_str(char *str)
 		}
 	}
 
-	/* The following expansions expand into MULTIPLE fields */
+	/* The following expansions expand to MULTIPLE fields. */
 
 				/* ###########################
 				 * #   2.12) SEL EXPANSION   #
@@ -3087,11 +3024,11 @@ parse_input_str(char *str)
 	substr[args_n + 1] = (char *)NULL;
 
 	const int is_action = is_action_name(substr[0]);
-	if (is_internal_cmd == 0 && is_action == 0)
+	if (is_int_cmd == 0 && is_action == 0)
 		return substr;
 
 		/* ####################################################
-		 * #          3) SHELL-LIKE EXPANSIONS                #
+		 * #            3) SHELL-LIKE EXPANSIONS              #
 		 * #      Only for internal commands and plugins      #
 		 * #################################################### */
 
@@ -3117,11 +3054,11 @@ parse_input_str(char *str)
 		if ((is_action == 1 && i == 0) || is_quoted_word(i))
 			continue;
 		/* Do not perform any of the expansions below for selected
-		 * elements: they are full path file names that, as such, do not
+		 * elements: they are full path filenames that, as such, do not
 		 * need any expansion. */
 		if (is_sel > 0) { /* is_sel is true only for the current input and if
 			there was some "sel" keyword in it. */
-			/* Strings between is_sel and sel_n are selected file names.
+			/* Strings between is_sel and sel_n are selected filenames.
 			 * Skip them. */
 			if (i >= (size_t)is_sel && i <= sel_n)
 				continue;
@@ -3132,9 +3069,14 @@ parse_input_str(char *str)
 		if (substr[0][0] == '/' && i == 0)
 			continue;
 
+		/* Ignore existent filenames as well. */
+		struct stat a;
+		if (lstat(substr[i], &a) != -1)
+			continue;
+
 #ifdef HAVE_WORDEXP
 		/* Let's make wordexp(3) ignore escaped words. */
-		int is_escaped = strchr(substr[i], '\\') ? 1 : 0;
+		const int is_escaped = strchr(substr[i], '\\') ? 1 : 0;
 #endif /* HAVE_WORDEXP */
 
 		size_t j = 0;
@@ -3151,10 +3093,11 @@ parse_input_str(char *str)
 
 #ifdef HAVE_WORDEXP
 			/* Command substitution, tilde, and environment variables
-			 * expansion is made by wordexp(3) */
+			 * expansion is made by wordexp(3). */
 			if (is_escaped == 0 && IS_WORD(substr[i][j], substr[i][j + 1])) {
 				/* Unlike glob() and tilde_expand(), wordexp() can expand tilde
-				 * and env vars even in the middle of a string. E.g.: $HOME/Downloads */
+				 * and env vars even in the middle of a string. E.g.:
+				 * '$HOME/Downloads'. */
 				if (word_n < INT_ARRAY_MAX) {
 					word_array[word_n] = (int)i;
 					word_n++;
@@ -3168,10 +3111,9 @@ parse_input_str(char *str)
 			 * #   3.1) WILDCARDS AND BRACE EXPANSION   #
 			 * ########################################## */
 
-	if (glob_n > 0 && glob_expand(substr) == 1) {
-		if (expand_glob(&substr, glob_array, glob_n) == -1)
-			return (char **)NULL;
-	}
+	if (glob_n > 0 && glob_expand(substr) == 1
+	&& expand_glob(&substr, glob_array, glob_n) == -1)
+		return (char **)NULL;
 
 	free(glob_array);
 
@@ -3180,10 +3122,8 @@ parse_input_str(char *str)
 		 * ############################################### */
 
 #ifdef HAVE_WORDEXP
-	if (word_n > 0) {
-		if (expand_word(&substr, word_array, word_n) == -1)
-			return (char **)NULL;
-	}
+	if (word_n > 0 && expand_word(&substr, word_array, word_n) == -1)
+		return (char **)NULL;
 
 	free(word_array);
 #endif /* HAVE_WORDEXP */
@@ -3243,7 +3183,7 @@ home_tilde(char *new_path, int *free_buf)
 /* Returns a pointer to a copy of the string STR, malloc'ed with size SIZE, or
  * NULL on error. */
 char *
-savestring(const char *restrict str, size_t size)
+savestring(const char *restrict str, const size_t size)
 {
 	if (!str)
 		return (char *)NULL;
