@@ -1,17 +1,15 @@
-/* strings.h */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,6 +20,8 @@
  * MA 02110-1301, USA.
 */
 
+/* strings.h */
+
 #ifndef STRINGS_H
 #define STRINGS_H
 
@@ -45,7 +45,7 @@ char *remove_quotes(char *str);
 char *replace_invalid_chars(const char *name);
 char *replace_slashes(char *str, const char c);
 char *replace_substr(const char *haystack, const char *needle, char *rep);
-char *savestring(const char *restrict str, size_t size);
+char *savestring(const char *restrict str, const size_t size);
 char **split_str(char *str, const int update_args);
 char *strbfrlst(char *str, const char c);
 char *strbtw(char *str, const char a, const char b);
