@@ -1,17 +1,15 @@
-/* suggestions.c -- functions to manage the suggestions system */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -21,11 +19,13 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
  * MA 02110-1301, USA.
 */
+
+/* suggestions.c -- functions to manage the suggestions system */
+
 #ifndef _NO_SUGGESTIONS
 
 #include "helpers.h"
 
-#include <stdio.h>
 #include <string.h>
 #include <strings.h> /* str(n)casecmp() */
 #include <unistd.h>
@@ -65,11 +65,18 @@ typedef char *rl_cpvfunc_t;
 
 #define BAEJ_OFFSET 1
 
+#define IS_BAEJ_SUG(t) ((t) == BOOKMARK_SUG || (t) == ALIAS_SUG \
+	|| (t) == ELN_SUG || (t) == JCMD_SUG || (t) == BACKDIR_SUG  \
+	|| (t) == SORT_SUG || (t) == WS_NUM_SUG                     \
+	|| (t) == FUZZY_FILENAME || (t) == DIRHIST_SUG              \
+	|| (t) == FASTBACK_SUG || (t) == WS_NUM_PREFIX_SUG          \
+	|| ((t) == COMP_SUG && (flags & BAEJ_SUGGESTION)))
+
 static char *last_word = (char *)NULL;
 static int last_word_offset = 0;
 static int point_is_first_word = 0;
 
-/* A big enough buffer to hold temporary data. */
+/* A buffer big enough to hold temporary data. */
 static char tmp_buf[PATH_MAX + 2];
 
 /*
@@ -91,6 +98,7 @@ change_word_color(const char *_last_word, const int offset, const char *color)
 }
 #endif // !_NO_HIGHLIGHT */
 
+/* Return a brief description for the command S, whose length is L. */
 static char *
 check_int_cmd_desc(const char *s, const size_t l)
 {
@@ -98,155 +106,156 @@ check_int_cmd_desc(const char *s, const size_t l)
 		return (char *)NULL;
 
 	if (l == 1) {
-		if (*s == 'b') return B_DESC;
-		if (*s == 'c') return C_DESC;
-		if (*s == 'd') return D_DESC;
-		if (*s == 'f') return F_DESC;
-		if (*s == 'j') return J_DESC;
-		if (*s == 'k') return K_DESC;
-		if (*s == 'l') return L_DESC;
-		if (*s == 'm') return M_DESC;
-		if (*s == 'n') return N_DESC;
-		if (*s == 'o') return O_DESC;
-		if (*s == 'p') return P_DESC;
-		if (*s == 'q') return Q_DESC;
-		if (*s == 'Q') return QU_DESC;
-		if (*s == 'r') return R_DESC;
-		if (*s == 's') return SEL_DESC;
-		if (*s == 't') return TRASH_DESC;
-		if (*s == 'u') return U_DESC;
-		if (*s == 'x') return X_DESC;
-		if (*s == 'X') return XU_DESC;
+		switch (*s) {
+		case 'b': return B_DESC;
+		case 'c': return C_DESC;
+		case 'd': return D_DESC;
+		case 'f': return F_DESC;
+		case 'j': return J_DESC;
+		case 'k': return K_DESC;
+		case 'l': return L_DESC;
+		case 'm': return M_DESC;
+		case 'n': return N_DESC;
+		case 'o': return O_DESC;
+		case 'p': return P_DESC;
+		case 'q': return Q_DESC;
+		case 'r': return R_DESC;
+		case 's': return SEL_DESC;
+		case 't': return TRASH_DESC;
+		case 'u': return U_DESC;
+		case 'x': return X_DESC;
+		case 'X': return XU_DESC;
+		default: return (char *)NULL;
+		}
 	}
 
 	else if (l == 2) {
 		if (*s == 'a') {
-			if (*(s + 1) == 'c') return AC_DESC;
-			if (*(s + 1) == 'd') return AD_DESC;
-			if (*(s + 1) == 'o') return AO_DESC;
+			if (s[1] == 'c') return AC_DESC;
+			if (s[1] == 'd') return AD_DESC;
+			if (s[1] == 'o') return AO_DESC;
 		}
 
 		else if (*s == 'b') {
-			if (*(s + 1) == 'b') return BB_DESC;
-			if (*(s + 1) == 'd') return BD_DESC;
-			if (*(s + 1) == 'l') return BL_DESC;
-			if (*(s + 1) == 'm') return BM_DESC;
-			if (*(s + 1) == 'r') return BR_DESC;
+			if (s[1] == 'b') return BB_DESC;
+			if (s[1] == 'd') return BD_DESC;
+			if (s[1] == 'l') return BL_DESC;
+			if (s[1] == 'm') return BM_DESC;
+			if (s[1] == 'r') return BR_DESC;
 		}
 
 		else if (*s == 'c') {
-			if (*(s + 1) == 'd') return CD_DESC;
-			if (*(s + 1) == 'l') return CL_DESC;
-			if (*(s + 1) == 's') return CS_DESC;
+			if (s[1] == 'd') return CD_DESC;
+			if (s[1] == 'l') return CL_DESC;
+			if (s[1] == 's') return CS_DESC;
 		}
 
 		else if (*s == 'd') {
-			if (*(s + 1) == 's') return DS_DESC;
-			if (*(s + 1) == 'h') return DH_DESC;
+			if (s[1] == 's') return DS_DESC;
+			if (s[1] == 'h') return DH_DESC;
 		}
 
 		else if (*s == 'f') {
-			if (*(s + 1) == 'c') return FC_DESC;
-			if (*(s + 1) == 'f') return FF_DESC;
-			if (*(s + 1) == 't') return FT_DESC;
-			if (*(s + 1) == 'z') return FZ_DESC;
+			if (s[1] == 'c') return FC_DESC;
+			if (s[1] == 'f') return FF_DESC;
+			if (s[1] == 't') return FT_DESC;
+			if (s[1] == 'z') return FZ_DESC;
 		}
 
-		else if (*s == 'h' && (*(s + 1) == 'f' || *(s + 1) == 'h'))
+		else if (*s == 'h' && (s[1] == 'f' || s[1] == 'h'))
 			return HF_DESC;
 
 		else if (*s == 'k') {
-			if (*(s + 1) == 'b') return KB_DESC;
-			if (*(s + 1) == 'k') return KK_DESC;
+			if (s[1] == 'b') return KB_DESC;
+			if (s[1] == 'k') return KK_DESC;
 		}
 
 		else if (*s == 'l') {
-			if (*(s + 1) == 'l' || *(s + 1) == 'v') return LL_DESC;
-			if (*(s + 1) == 'm') return LM_DESC;
+			if (s[1] == 'e') return LE_DESC;
+			if (s[1] == 'l' || s[1] == 'v') return LL_DESC;
+			if (s[1] == 'm') return LM_DESC;
 		}
 
 		else if (*s == 'm') {
-			if (*(s + 1) == 'd') return MD_DESC;
-			if (*(s + 1) == 'f') return MF_DESC;
-			if (*(s + 1) == 'm') return MM_DESC;
-			if (*(s + 1) == 'p') return MP_DESC;
-		}
-
-		else if (*s == 'l') {
-			if (*(s + 1) == 'e') return LE_DESC;
+			if (s[1] == 'd') return MD_DESC;
+			if (s[1] == 'f') return MF_DESC;
+			if (s[1] == 'm') return MM_DESC;
+			if (s[1] == 'p') return MP_DESC;
 		}
 
 		else if (*s == 'o') {
-			if (*(s + 1) == 'c') return OC_DESC;
-			if (*(s + 1) == 'w') return OW_DESC;
+			if (s[1] == 'c') return OC_DESC;
+			if (s[1] == 'w') return OW_DESC;
 		}
 
 		else if (*s == 'p') {
-			if (*(s + 1) == 'c') return PC_DESC;
-			if (*(s + 1) == 'f') return PF_DESC;
-			if (*(s + 1) == 'g') return PG_DESC;
-			if (*(s + 1) == 'p') return PP_DESC;
+			if (s[1] == 'c') return PC_DESC;
+			if (s[1] == 'f') return PF_DESC;
+			if (s[1] == 'g') return PG_DESC;
+			if (s[1] == 'p') return PP_DESC;
 		}
 
 		else if (*s == 'r') {
-			if (*(s + 1) == 'f') return RF_DESC;
-			if (*(s + 1) == 'l') return RL_DESC;
-			if (*(s + 1) == 'r') return RR_DESC;
+			if (s[1] == 'f') return RF_DESC;
+			if (s[1] == 'l') return RL_DESC;
+			if (s[1] == 'r') return RR_DESC;
 		}
 
 		else if (*s == 's') {
-			if (*(s + 1) == 'b') return SB_DESC;
-			if (*(s + 1) == 't') return ST_DESC;
+			if (s[1] == 'b') return SB_DESC;
+			if (s[1] == 't') return ST_DESC;
 		}
 
 		else if (*s == 't') {
-			if (*(s + 1) == 'a') return TA_DESC;
-			if (*(s + 1) == 'd') return TD_DESC;
-			if (*(s + 1) == 'e') return TE_DESC;
-			if (*(s + 1) == 'l') return TL_DESC;
-			if (*(s + 1) == 'm') return TM_DESC;
-			if (*(s + 1) == 'n') return TN_DESC;
-			if (*(s + 1) == 'u') return TU_DESC;
-			if (*(s + 1) == 'y') return TY_DESC;
+			if (s[1] == 'a') return TA_DESC;
+			if (s[1] == 'd') return TD_DESC;
+			if (s[1] == 'e') return TE_DESC;
+			if (s[1] == 'l') return TL_DESC;
+			if (s[1] == 'm') return TM_DESC;
+			if (s[1] == 'n') return TN_DESC;
+			if (s[1] == 'u') return TU_DESC;
+			if (s[1] == 'y') return TY_DESC;
 		}
 
-		else if (*s == 'v' && *(s + 1) == 'v')
+		else if (*s == 'v' && s[1] == 'v')
 			return VV_DESC;
 
 		else if (*s == 'w') {
-			if (*(s + 1) == 's') return WS_DESC;
+			if (s[1] == 's') return WS_DESC;
 		}
 
 	} else if (l == 3) {
-		if (*s == 'a' && *(s + 1) == 'c' && *(s + 2) == 'd')
+		if (*s == 'a' && s[1] == 'c' && s[2] == 'd')
 			return ACD_DESC;
-		if (*s == 'c' && *(s + 1) == 'm' && *(s + 2) == 'd')
+		if (*s == 'c' && s[1] == 'm' && s[2] == 'd')
 			return CMD_DESC;
-		if (*s == 'd' && *(s + 1) == 'u' && *(s + 2) == 'p')
+		if (*s == 'd' && s[1] == 'u' && s[2] == 'p')
 			return D_DESC;
-		if (*s == 'e' && *(s + 1) == 'x') {
-			if (*(s + 2) == 'p') return EXP_DESC;
-			if (*(s + 2) == 't') return EXT_DESC;
+		if (*s == 'e' && s[1] == 'x') {
+			if (s[2] == 'p') return EXP_DESC;
+			if (s[2] == 't') return EXT_DESC;
 		}
-		if (*s == 'l' && *(s + 1) == 'o' && *(s + 2) == 'g')
+		if (*s == 'l' && s[1] == 'o' && s[2] == 'g')
 			return LOG_DESC;
-		if (*s == 'm' && *(s + 1) == 's' && *(s + 2) == 'g')
+		if (*s == 'm' && s[1] == 's' && s[2] == 'g')
 			return MSG_DESC;
-		if (*s == 'n' && *(s + 1) == 'e') {
-			if (*(s + 2) == 't') return NET_DESC;
-			if (*(s + 2) == 'w') return N_DESC;
+		if (*s == 'n' && s[1] == 'e') {
+			if (s[2] == 't') return NET_DESC;
+			if (s[2] == 'w') return N_DESC;
 		}
-		if (*s == 'p' && *(s + 1) == 'i' && *(s + 2) == 'n')
+		if (*s == 'p' && s[1] == 'i' && s[2] == 'n')
 			return PIN_DESC;
-		if (*s == 's' && *(s + 1) == 'e' && *(s + 2) == 'l')
+		if (*s == 's' && s[1] == 'e' && s[2] == 'l')
 			return SEL_DESC;
-		if (*s == 't' && *(s + 1) == 'a' && *(s + 2) == 'g')
+		if (*s == 't' && s[1] == 'a' && s[2] == 'g')
 			return TAG_DESC;
-		if (*s == 'v' && *(s + 1) == 'e' && *(s + 2) == 'r')
+		if (*s == 'v' && s[1] == 'e' && s[2] == 'r')
 			return VER_DESC;
 	}
 
 	else if (l == 4) {
+		if (*s == 'a' && strcmp(s + 1, "uto") == 0)
+			return AUTO_DESC;
 		if (*s == 'b' && strcmp(s + 1, "ack") == 0)
 			return B_DESC;
 		if (*s == 'b' && strcmp(s + 1, "ulk") == 0)
@@ -309,8 +318,6 @@ check_int_cmd_desc(const char *s, const size_t l)
 			return RL_DESC;
 		if (*s == 's' && strcmp(s + 1, "elbox") == 0)
 			return SB_DESC;
-		if (*s == 's' && strcmp(s + 1, "plash") == 0)
-			return SPLASH_DESC;
 	}
 
 	else if (l == 7) {
@@ -366,7 +373,7 @@ recover_from_wrong_cmd(void)
 	if (rl_line_buffer && (rl_dispatching == 0
 	|| (words_num > 1 && point_is_first_word == 0))) {
 		const char *p = (strrchr(rl_line_buffer, ' '));
-		if (p && p != rl_line_buffer && *(p - 1) != '\\' && *(p + 1) != ' ')
+		if (p && p != rl_line_buffer && *(p - 1) != '\\' && p[1] != ' ')
 			return FUNC_FAILURE;
 	}
 
@@ -440,15 +447,8 @@ calculate_suggestion_lines(int *baej, const size_t suggestion_len)
 {
 	size_t cuc = (size_t)curcol; /* Current cursor column position. */
 
-	if (suggestion.type == BOOKMARK_SUG || suggestion.type == ALIAS_SUG
-	|| suggestion.type == ELN_SUG || suggestion.type == JCMD_SUG
-	|| suggestion.type == JCMD_SUG_NOACD || suggestion.type == BACKDIR_SUG
-	|| suggestion.type == SORT_SUG || suggestion.type == WS_NUM_SUG
-	|| suggestion.type == FUZZY_FILENAME || suggestion.type == DIRHIST_SUG
-	|| suggestion.type == FASTBACK_SUG || suggestion.type == WS_NUM_PREFIX_SUG
-	|| (suggestion.type == COMP_SUG && (flags & BAEJ_SUGGESTION))) {
-		/* 3 = 1 (one char forward) + 2 (" >") */
-		cuc += 3;
+	if (IS_BAEJ_SUG(suggestion.type)) {
+		cuc += 3; /* 3 = 1 (one char forward) + 2 (" >") */
 		flags |= BAEJ_SUGGESTION;
 		*baej = 1;
 	}
@@ -457,13 +457,13 @@ calculate_suggestion_lines(int *baej, const size_t suggestion_len)
 	if (conf.highlight == 0 && rl_point < rl_end)
 		cucs += (size_t)(rl_end - rl_point - 1);
 
-	/* slines: amount of lines we need to print the suggestion, including
+	/* slines: number of lines we need to print the suggestion, including
 	 * the current line. */
 	size_t slines = 1;
 
 	if (cucs > term_cols && term_cols > 0) {
 		slines = cucs / (size_t)term_cols;
-		const int cucs_rem = (int)cucs % term_cols;
+		const size_t cucs_rem = cucs % (size_t)term_cols;
 		if (cucs_rem > 0)
 			slines++;
 	}
@@ -624,7 +624,7 @@ print_suggestion(const char *str, size_t offset, char *color)
 
 	restore_cursor_position(slines);
 
-	/* Store the amount of lines taken by the current command line (plus the
+	/* Store the number of lines taken by the current command line (plus the
 	 * suggestion's length) to be able to correctly remove it later (via the
 	 * clear_suggestion function). */
 	suggestion.nlines = slines;
@@ -639,7 +639,7 @@ get_reg_file_color(const char *filename, const struct stat *attr,
 	int *free_color)
 {
 	if (conf.light_mode == 1) return fi_c;
-	if (access(filename, R_OK) == -1) return nf_c;
+	if (*nf_c && access(filename, R_OK) == -1) return nf_c;
 	if (attr->st_mode & S_ISUID) return su_c;
 	if (attr->st_mode & S_ISGID) return sg_c;
 
@@ -674,24 +674,18 @@ get_reg_file_color(const char *filename, const struct stat *attr,
 	return ext_color;
 }
 
-/* Used by the check_completions function to get file names color
+/* Used by the check_completions function to get filenames color
  * according to file type. */
 static char *
 get_comp_color(const char *filename, const struct stat *attr, int *free_color)
 {
-	char *color = (char *)NULL;
-
 	switch (attr->st_mode & S_IFMT) {
 	case S_IFDIR:
-		if (conf.light_mode == 1) return di_c;
-		if (access(filename, R_OK | X_OK) != 0)
-			return nd_c;
-		color = get_dir_color(filename, attr->st_mode, attr->st_nlink, -1);
-		break;
+		return conf.light_mode == 1 ? di_c
+			: get_dir_color(filename, attr, -1);
 
 	case S_IFREG:
-		color = get_reg_file_color(filename, attr, free_color);
-		break;
+		return get_reg_file_color(filename, attr, free_color);
 
 	case S_IFLNK: {
 		if (conf.light_mode == 1) return ln_c;
@@ -709,8 +703,6 @@ get_comp_color(const char *filename, const struct stat *attr, int *free_color)
 	case S_IFIFO: return pi_c;
 	default: return no_c;
 	}
-
-	return color;
 }
 
 static inline int
@@ -767,7 +759,6 @@ skip_leading_backslashes(char **str, size_t *len)
 static void
 match_print(char *match, const size_t len, char *color, const int append_slash)
 {
-//	char t[PATH_MAX + 2];
 	*tmp_buf = '\0';
 
 	if (append_slash == 1)
@@ -780,7 +771,7 @@ match_print(char *match, const size_t len, char *color, const int append_slash)
 	}
 
 	char *q;
-	if (cur_comp_type == TCMP_PATH && *tmp == '\\' && *(tmp + 1) == '~')
+	if (cur_comp_type == TCMP_PATH && *tmp == '\\' && tmp[1] == '~')
 		q = tmp + 1;
 	else
 		q = tmp;
@@ -794,7 +785,7 @@ print_match(char *match, const size_t len)
 {
 	int append_slash = 0, free_color = 0;
 
-	char *p = (char *)NULL, *_color = (char *)NULL;
+	char *p = (char *)NULL, *temp_color = (char *)NULL;
 	char *color = (conf.suggest_filetype_color == 1) ? no_c : sf_c;
 
 	if (*match == '~')
@@ -805,14 +796,14 @@ print_match(char *match, const size_t len)
 		if (S_ISDIR(attr.st_mode)
 		|| (S_ISLNK(attr.st_mode) && get_link_ref(p ? p : match) == S_IFDIR)) {
 			/* Do not append slash if suggesting the root dir. */
-			append_slash = (*match == '/' && !*(match + 1)) ? 0 : 1;
+			append_slash = (*match == '/' && !match[1]) ? 0 : 1;
 			suggestion.filetype = DT_DIR;
 		}
 
 		if (conf.suggest_filetype_color == 1) {
-			_color = get_comp_color(p ? p : match, &attr, &free_color);
-			if (_color)
-				color = _color;
+			temp_color = get_comp_color(p ? p : match, &attr, &free_color);
+			if (temp_color)
+				color = temp_color;
 			else
 				free_color = 0;
 		}
@@ -824,7 +815,7 @@ print_match(char *match, const size_t len)
 	suggestion.type = COMP_SUG;
 	match_print(match, len, color, append_slash);
 
-	if (free_color)
+	if (free_color == 1)
 		free(color);
 	return PARTIAL_MATCH;
 }
@@ -854,7 +845,7 @@ check_completions(char *str, size_t len, const int print)
 		return NO_MATCH;
 
 	if (conf.fuzzy_match != 0 && words_num == 1 && *str != '/'
-	&& is_internal_c(str))
+	&& is_internal_cmd(str, ALL_CMDS, 1, 1))
 		return NO_MATCH;
 
 	int printed = NO_MATCH;
@@ -903,7 +894,6 @@ print_directory_suggestion(const filesn_t i, const size_t len, char *color)
 
 	suggestion.filetype = DT_DIR;
 
-//	char name[NAME_MAX + 2];
 	*tmp_buf = '\0';
 	snprintf(tmp_buf, sizeof(tmp_buf), "%s/", file_info[i].name);
 
@@ -935,8 +925,7 @@ print_reg_file_suggestion(char *str, const filesn_t i, size_t len,
 			s++;
 		}
 
-		if (dot_slash) { /* Reinsert './', removed to check file name. */
-//			char t[NAME_MAX + 2];
+		if (dot_slash == 1) { /* Reinsert './', removed to check filename. */
 			*tmp_buf = '\0';
 			snprintf(tmp_buf, sizeof(tmp_buf), "./%s", tmp);
 			print_suggestion(tmp_buf, len + 2, color);
@@ -948,8 +937,7 @@ print_reg_file_suggestion(char *str, const filesn_t i, size_t len,
 		return;
 	}
 
-	if (dot_slash) {
-//		char t[NAME_MAX + 2];
+	if (dot_slash == 1) {
 		*tmp_buf = '\0';
 		snprintf(tmp_buf, sizeof(tmp_buf), "./%s", file_info[i].name);
 		print_suggestion(tmp_buf, len + 2, color);
@@ -966,12 +954,12 @@ check_filenames(char *str, size_t len, const int first_word,
 	char *color = (conf.suggest_filetype_color == 1) ? no_c : sf_c;
 
 	skip_leading_backslashes(&str, &len);
-	int dot_slash = skip_leading_dot_slash(&str, &len);
+	const int dot_slash = skip_leading_dot_slash(&str, &len);
 	skip_trailing_spaces(&str, &len);
-	int removed_slash = remove_trailing_slash(&str, &len);
+	const int removed_slash = remove_trailing_slash(&str, &len);
 
 	filesn_t fuzzy_index = -1;
-	int fuzzy_str_type = (conf.fuzzy_match == 1 && contains_utf8(str) == 1)
+	const int fuzzy_str_type = (conf.fuzzy_match == 1 && contains_utf8(str) == 1)
 		? FUZZY_FILES_UTF8 : FUZZY_FILES_ASCII;
 	int best_fz_score = 0;
 
@@ -984,7 +972,7 @@ check_filenames(char *str, size_t len, const int first_word,
 		|| len != file_info[i].len))
 			continue;
 
-		if (full_word) {
+		if (full_word == 1) {
 			if ((conf.case_sens_path_comp ? strcmp(str, file_info[i].name)
 			: strcasecmp(str, file_info[i].name)) == 0)
 				return FULL_MATCH;
@@ -1022,7 +1010,7 @@ check_filenames(char *str, size_t len, const int first_word,
 
 		/* ############### FUZZY MATCHING ################## */
 		else {
-			int s = fuzzy_match(str, file_info[i].name, len, fuzzy_str_type);
+			const int s = fuzzy_match(str, file_info[i].name, len, fuzzy_str_type);
 			if (s > best_fz_score) {
 				fuzzy_index = i;
 				if (s == TARGET_BEGINNING_BONUS)
@@ -1041,8 +1029,7 @@ check_filenames(char *str, size_t len, const int first_word,
 		if (file_info[fuzzy_index].dir)
 			print_directory_suggestion(fuzzy_index, len, color);
 		else
-			print_reg_file_suggestion(str, fuzzy_index, len, color,
-				dot_slash);
+			print_reg_file_suggestion(str, fuzzy_index, len, color, dot_slash);
 
 		return PARTIAL_MATCH;
 	}
@@ -1064,8 +1051,8 @@ check_history(const char *str, const size_t len)
 		if (!history[i].cmd || TOUPPER(*str) != TOUPPER(*history[i].cmd))
 			continue;
 
-		if (len > 1 && *(history[i].cmd + 1)
-		&& TOUPPER(*(str + 1)) != TOUPPER(*(history[i].cmd + 1)))
+		if (len > 1 && history[i].cmd[1]
+		&& TOUPPER(str[1]) != TOUPPER(history[i].cmd[1]))
 			continue;
 
 		if ((conf.case_sens_path_comp ? strncmp(str, history[i].cmd, len)
@@ -1126,7 +1113,7 @@ check_builtins(const char *str, const size_t len, const int print)
 static inline int
 print_cmd_suggestion(const size_t i, const size_t len)
 {
-	if (is_internal_c(bin_commands[i])) {
+	if (is_internal_cmd(bin_commands[i], ALL_CMDS, 1, 1)) {
 		if (strlen(bin_commands[i]) > len) {
 			suggestion.type = CMD_SUG;
 			print_suggestion(bin_commands[i], len, sx_c);
@@ -1135,7 +1122,7 @@ print_cmd_suggestion(const size_t i, const size_t len)
 		return FULL_MATCH;
 	}
 
-	if (conf.ext_cmd_ok) {
+	if (conf.ext_cmd_ok == 1) {
 		if (strlen(bin_commands[i]) > len) {
 			suggestion.type = CMD_SUG;
 			print_suggestion(bin_commands[i], len, sc_c);
@@ -1164,7 +1151,7 @@ print_internal_cmd_suggestion(char *str, const size_t len, const int print)
 		return check_builtins(str, len, print);
 
 	*p = '\0';
-	if (!is_internal_c(str))
+	if (!is_internal_cmd(str, ALL_CMDS, 1, 1))
 		return NO_MATCH;
 
 	return FULL_MATCH;
@@ -1178,7 +1165,7 @@ check_cmds(char *str, size_t len, const int print)
 		return NO_MATCH;
 
 	char *cmd = str;
-	if (cmd && *cmd == '\\' && *(cmd + 1)) {
+	if (*cmd == '\\' && cmd[1]) {
 		cmd++;
 		len--;
 	}
@@ -1188,15 +1175,15 @@ check_cmds(char *str, size_t len, const int print)
 		if (!bin_commands[i] || *cmd != *bin_commands[i])
 			continue;
 
-		if (!print) {
+		if (print == 0) {
 			if (strcmp(cmd, bin_commands[i]) == 0)
 				return FULL_MATCH;
 			continue;
 		}
 
 		/* Let's check the 2nd char as well before calling strcmp(). */
-		if (len > 1 && *(bin_commands[i] + 1)
-		&& *(cmd + 1) != *(bin_commands[i] + 1))
+		if (len > 1 && bin_commands[i][1]
+		&& cmd[1] != bin_commands[i][1])
 			continue;
 
 		if (strncmp(cmd, bin_commands[i], len) != 0)
@@ -1221,17 +1208,17 @@ check_jumpdb(const char *str, const size_t len, const int print)
 		if (!jump_db[i].path || TOUPPER(*str) != TOUPPER(*jump_db[i].path)
 		|| jump_db[i].rank == JUMP_ENTRY_PURGED)
 			continue;
-		if (len > 1 && *(jump_db[i].path + 1)
-		&& TOUPPER(*(str + 1)) != TOUPPER(*(jump_db[i].path + 1)))
+		if (len > 1 && jump_db[i].path[1]
+		&& TOUPPER(str[1]) != TOUPPER(jump_db[i].path[1]))
 			continue;
-		if (!print) {
+		if (print == 0) {
 			if ((conf.case_sens_path_comp ? strcmp(str, jump_db[i].path)
 			: strcasecmp(str, jump_db[i].path)) == 0)
 				return FULL_MATCH;
 			continue;
 		}
 
-		if (len && (conf.case_sens_path_comp
+		if (len > 0 && (conf.case_sens_path_comp
 		? strncmp(str, jump_db[i].path, len)
 		: strncasecmp(str, jump_db[i].path, len)) == 0) {
 			if (jump_db[i].len <= len)
@@ -1239,7 +1226,6 @@ check_jumpdb(const char *str, const size_t len, const int print)
 
 			suggestion.type = FILE_SUG;
 			suggestion.filetype = DT_DIR;
-//			char tmp[PATH_MAX + 2];
 			*tmp_buf = '\0';
 
 			if (jump_db[i].len > 0
@@ -1290,13 +1276,12 @@ check_eln(const char *str, const int print)
 
 	n--;
 	char *color = sf_c;
-	suggestion.type = ELN_SUG;
-	if (conf.suggest_filetype_color)
+	if (conf.suggest_filetype_color == 1)
 		color = file_info[n].color;
+	suggestion.type = ELN_SUG;
 
-//	char tmp[NAME_MAX + 1];
 	*tmp_buf = '\0';
-	if (file_info[n].dir) {
+	if (file_info[n].dir == 1) {
 		snprintf(tmp_buf, sizeof(tmp_buf), "%s/", file_info[n].name);
 		suggestion.filetype = DT_DIR;
 	} else {
@@ -1324,7 +1309,7 @@ check_aliases(const char *str, const size_t len, const int print)
 		if (TOUPPER(*p) != TOUPPER(*str))
 			continue;
 
-		if (!print) {
+		if (print == 0) {
 			if ((conf.case_sens_path_comp ? strcmp(p, str)
 			: strcasecmp(p, str)) == 0)
 				return FULL_MATCH;
@@ -1376,9 +1361,6 @@ check_jcmd(char *line)
 	print_suggestion(jump_suggestion, 0,
 		conf.suggest_filetype_color ? di_c : sf_c);
 
-	if (conf.autocd == 0)
-		suggestion.type = JCMD_SUG_NOACD;
-
 	free(jump_suggestion);
 	jump_suggestion = (char *)NULL;
 	return PARTIAL_MATCH;
@@ -1397,10 +1379,10 @@ check_help(char *full_line, const char *_last_word)
 		return NO_MATCH;
 
 	*ret = '\0';
-	int retval = is_internal_c(full_line);
+	const int retval = is_internal_cmd(full_line, ALL_CMDS, 1, 1);
 	*ret = ' ';
 
-	if (!retval)
+	if (retval == 0)
 		return NO_MATCH;
 
 	suggestion.type = INT_HELP_SUG;
@@ -1421,7 +1403,6 @@ check_users(const char *str, const size_t len)
 		if (len == 0 || (*str == *p->pw_name
 		&& strncmp(str, p->pw_name, len) == 0)) {
 			suggestion.type = USER_SUG;
-//			char t[NAME_MAX + 1];
 			*tmp_buf = '\0';
 			snprintf(tmp_buf, sizeof(tmp_buf), "~%s", p->pw_name);
 			print_suggestion(tmp_buf, len + 1, sf_c);
@@ -1450,7 +1431,6 @@ check_variables(const char *str, const size_t len)
 
 		*ret = '\0';
 		suggestion.type = VAR_SUG;
-//		char t[NAME_MAX + 1];
 		*tmp_buf = '\0';
 		snprintf(tmp_buf, sizeof(tmp_buf), "$%s", environ[i]);
 		print_suggestion(tmp_buf, len + 1, sh_c);
@@ -1467,7 +1447,6 @@ check_variables(const char *str, const size_t len)
 			continue;
 
 		suggestion.type = CMD_SUG;
-//		char t[NAME_MAX + 1];
 		*tmp_buf = '\0';
 		snprintf(tmp_buf, sizeof(tmp_buf), "$%s", usr_var[i].name);
 		print_suggestion(tmp_buf, len + 1, sh_c);
@@ -1539,9 +1518,13 @@ print_warning_prompt(const char fc, const unsigned char lc)
 	rl_set_prompt(decoded_prompt);
 	free(decoded_prompt);
 
-	if (conf.highlight == 1
+/*	if (conf.highlight == 1
 	&& ( (rl_point < rl_end && words_num > 1)
 	|| (lc == ' ' && words_num == 1) ) )
+		turn_it_wrong(); */
+	UNUSED(lc);
+	if (conf.highlight == 1
+	&& ((rl_point < rl_end && words_num > 1) || words_num == 1))
 		turn_it_wrong();
 }
 
@@ -1574,7 +1557,7 @@ check_sort_methods(const char *str, const size_t len)
 		return NO_MATCH;
 	}
 
-	int a = atoi(str);
+	const int a = atoi(str);
 	if (a < 0 || a > SORT_TYPES
 	|| (conf.light_mode == 1 && !ST_IN_LIGHT_MODE(a))) {
 		if (suggestion.printed)
@@ -1625,7 +1608,7 @@ check_prompts(char *word, const size_t len)
 static void
 get_last_word(const char *last_space)
 {
-	const char *tmp = (last_space && *(last_space + 1)) ? last_space + 1
+	const char *tmp = (last_space && last_space[1]) ? last_space + 1
 			: (rl_line_buffer ? rl_line_buffer : (char *)NULL);
 	if (tmp) {
 		const size_t len = strlen(tmp);
@@ -1636,7 +1619,7 @@ get_last_word(const char *last_space)
 		*last_word = '\0';
 	}
 
-	last_word_offset = (last_space && *(last_space + 1) && rl_line_buffer)
+	last_word_offset = (last_space && last_space[1] && rl_line_buffer)
 		? (int)((last_space + 1) - rl_line_buffer) : 0;
 }
 
@@ -1646,7 +1629,7 @@ check_workspaces(char *word, size_t wlen, const int type)
 	if (!word || !*word || !workspaces)
 		return NO_MATCH;
 
-	if (*word >= '1' && *word <= MAX_WS + '0' && !*(word + 1)) {
+	if (*word >= '1' && *word <= MAX_WS + '0' && !word[1]) {
 		const int a = atoi(word);
 		if (a > 0 && workspaces[a - 1].name) {
 			suggestion.type = type == WS_PREFIX_SUG
@@ -1688,7 +1671,7 @@ check_workspaces(char *word, size_t wlen, const int type)
 }
 
 static int
-check_fastback(char *w)
+check_fastback(const char *w)
 {
 	if (!w || !*w)
 		return NO_MATCH;
@@ -1750,6 +1733,25 @@ check_profiles(char *word, const size_t len)
 }
 #endif /* !_NO_PROFILES */
 
+static int
+check_kb_func_names(char *word, const size_t len)
+{
+	if (!word || !*word || len == 0 || kbinds_n == 0 || !kbinds)
+		return NO_MATCH;
+
+	size_t i;
+	for (i = 0; i < kbinds_n; i++) {
+		if (!kbinds[i].function || *word != *kbinds[i].function
+		|| strncmp(word, kbinds[i].function, len) != 0)
+			continue;
+		suggestion.type = NET_SUG; /* Same behavior */
+		print_suggestion(kbinds[i].function, len, sx_c);
+		return PARTIAL_MATCH;
+	}
+
+	return NO_MATCH;
+}
+
 static int
 check_remotes(char *word, const size_t len)
 {
@@ -1826,7 +1828,7 @@ check_bookmark_names(char *word, const size_t len)
 	if (!word || !*word || !bookmarks)
 		return NO_MATCH;
 
-	size_t prefix = (*word == 'b' && *(word + 1) == ':') ? 2 : 0;
+	const size_t prefix = (*word == 'b' && word[1] == ':') ? 2 : 0;
 	char *q = (char *)NULL, *w = word + prefix;
 	size_t l = len - prefix;
 
@@ -1906,7 +1908,7 @@ check_backdir(char *start)
 static int
 check_dirhist(char *word, const size_t len)
 {
-	int fuzzy_str_type = (conf.fuzzy_match == 1 && contains_utf8(word) == 1)
+	const int fuzzy_str_type = (conf.fuzzy_match == 1 && contains_utf8(word) == 1)
 		? FUZZY_FILES_UTF8 : FUZZY_FILES_ASCII;
 	int best_fz_score = 0, fuzzy_index = -1;
 
@@ -1979,6 +1981,7 @@ rl_suggestions(const unsigned char c)
 
 	/* We need a copy of the complete line. */
 	char *full_line = rl_line_buffer;
+	const char *lb = rl_line_buffer;
 
 	/* A copy of the last entered word. */
 	get_last_word(last_space);
@@ -2012,7 +2015,7 @@ rl_suggestions(const unsigned char c)
 	}
 
 	/* If not on the first word and not at the end of the last word, do nothing. */
-	int lw = is_last_word();
+	const int lw = is_last_word();
 	if (!lw) {
 		if (suggestion.printed == 1 && suggestion.nlines > 1)
 			clear_suggestion(CS_FREEBUF);
@@ -2077,7 +2080,7 @@ rl_suggestions(const unsigned char c)
 		/* An alias name could be the same as the beginning of the alias
 		 * defintion, so that this test must always be skipped in case
 		 * of aliases. */
-		} else if (suggestion.type != ALIAS_SUG && c != ' ' && word
+		} else if (suggestion.type != ALIAS_SUG && c != ' '
 		&& (conf.case_sens_path_comp ? (*word == *suggestion_buf
 		&& strncmp(word, suggestion_buf, wlen) == 0)
 		: (TOUPPER(*word) == TOUPPER(*suggestion_buf)
@@ -2111,21 +2114,20 @@ rl_suggestions(const unsigned char c)
 		}
 	}
 
-	char *lb = rl_line_buffer;
 	/* 3.d) Let's suggest non-fixed parameters for internal commands
 	 * Only if second word or more (first word is the command name). */
 
 	switch (words_num > 1 ? *s : '\0') {
 	case 'b': /* Bookmark names */
 		if (bm_n > 0 && s[1] == 'm' && s[2] == ' ') {
-			if (!(*(s + 3) == 'a' && *(s + 4) == ' ')
+			if (!(s[3] == 'a' && s[4] == ' ')
 			&& strncmp(s + 3, "add", 3) != 0) {
 				if ((printed = check_bookmark_names(word, wlen)) != NO_MATCH) {
 					goto SUCCESS;
 				} else {
 					if (suggestion.printed)
 						clear_suggestion(CS_FREEBUF);
-					if (*(s + 3) != '-') /* Might be --help, let it continue. */
+					if (s[3] != '-') /* Might be --help, let it continue. */
 						goto FAIL;
 				}
 			} else if (words_num > 5) {
@@ -2136,10 +2138,9 @@ rl_suggestions(const unsigned char c)
 
 		/* Backdir function (bd) */
 		else if (s[1] == 'd' && s[2] == ' ' && s[3]) {
-				if (*(s + 3) == '/' && !*(s + 4)) {
+				if (s[3] == '/' && !s[4])
 					/* The query string is a single slash: do nothing. */
 					goto FAIL;
-				}
 				if ((printed = check_backdir(s)) != NO_MATCH)
 					goto SUCCESS;
 		}
@@ -2165,13 +2166,17 @@ rl_suggestions(const unsigned char c)
 
 	case 'j': /* j command */
 		if (s[1] == ' ' || ((s[1] == 'c' || s[1] == 'p') && s[2] == ' ')) {
-//			if ((printed = check_jcmd(full_line)) != NO_MATCH) {
-			if ((printed = check_jcmd(s)) != NO_MATCH) {
-				zero_offset = 1;
+			if ((printed = check_jcmd(s)) != NO_MATCH)
 				goto SUCCESS;
-			} else {
+			else
 				goto FAIL;
-			}
+		}
+		break;
+
+	case 'k': /* 'kb bind' command */
+		if (s[1] == 'b' && s[2] == ' ' && s[3] && strncmp(s + 3, "bind ", 5) == 0) {
+			if ((printed = check_kb_func_names(word, wlen)) != NO_MATCH)
+				goto SUCCESS;
 		}
 		break;
 
@@ -2195,6 +2200,7 @@ rl_suggestions(const unsigned char c)
 #endif /* !_NO_PROFILES */
 
 		if (s[1] == 'r' && strncmp(s, "prompt set ", 11) == 0) {
+			if (words_num != 3) goto FAIL;
 			printed = check_prompts(word, wlen);
 			if (prompts_n > 0 && printed != NO_MATCH)
 				goto SUCCESS;
@@ -2204,8 +2210,7 @@ rl_suggestions(const unsigned char c)
 	case 's': /* Sort */
 		if (((s[1] == 't' && s[2] == ' ') || strncmp(s, "sort ", 5) == 0)
 		&& is_number(word)) {
-			if (words_num > 2)
-				goto FAIL;
+			if (words_num > 2) goto FAIL;
 			if ((printed = check_sort_methods(word, wlen)) != NO_MATCH)
 				goto SUCCESS;
 			goto FAIL;
@@ -2215,7 +2220,7 @@ rl_suggestions(const unsigned char c)
 #ifndef _NO_TAGS
 	case 't': /* Tag command */
 		if ((s[1] == 'a' || s[1] == 'u') && s[2] == ' ') {
-			if (*word == ':' && *(word + 1)
+			if (*word == ':' && word[1]
 			&& (printed = check_tags(word + 1, wlen - 1, TAGC_SUG)) != NO_MATCH)
 				goto SUCCESS;
 		} else if ((s[1] == 'l' || s[1] == 'm' || s[1] == 'n'
@@ -2229,8 +2234,7 @@ rl_suggestions(const unsigned char c)
 
 	case 'w': /* Workspaces */
 		if (s[1] == 's' && s[2] == ' ') {
-			if (words_num > 2)
-				goto FAIL;
+			if (words_num > 2) goto FAIL;
 			if ((printed = check_workspaces(word, wlen, WS_NAME_SUG)) != NO_MATCH)
 				goto SUCCESS;
 		}
@@ -2246,13 +2250,13 @@ rl_suggestions(const unsigned char c)
 	}
 
 	/* 3.d.2) ~usernames */
-	if (*word == '~' && *(word + 1) != '/') {
+	if (*word == '~' && word[1] != '/') {
 		if ((printed = check_users(word + 1, wlen - 1)) != NO_MATCH)
 			goto SUCCESS;
 	}
 
 	/* 3.d.3) Bookmark names (b:) */
-	if (*word == 'b' && *(word + 1) == ':' && *(word + 2)) {
+	if (*word == 'b' && word[1] == ':' && word[2]) {
 		if ((printed = check_bookmark_names(word, wlen)) != NO_MATCH)
 			goto SUCCESS;
 	}
@@ -2265,8 +2269,8 @@ rl_suggestions(const unsigned char c)
 
 #ifndef _NO_TAGS
 	/* 3.d.4) Tag names (t:) */
-	if (*lb != ';' && *lb != ':' && *word == 't' && *(word + 1) == ':'
-	&& *(word + 2)) {
+	if (*lb != ';' && *lb != ':' && *word == 't' && word[1] == ':'
+	&& word[2]) {
 		if ((printed = check_tags(word + 2, wlen - 2, TAGT_SUG)) != NO_MATCH)
 			goto SUCCESS;
 	}
@@ -2278,7 +2282,7 @@ rl_suggestions(const unsigned char c)
 	int flag = 0;
 
 	/* Let's find out whether the last entered character is escaped. */
-	int escaped = (wlen > 1 && word[wlen - 2] == '\\') ? 1 : 0;
+	const int escaped = (wlen > 1 && word[wlen - 2] == '\\');
 
 	for (st = 0; conf.suggestion_strategy[st]; st++) {
 		switch (conf.suggestion_strategy[st]) {
@@ -2299,10 +2303,10 @@ rl_suggestions(const unsigned char c)
 			if (!last_space && conf.autocd == 0 && conf.auto_open == 0)
 				break;
 
-			/* Skip internal commands not dealing with file names. */
+			/* Skip internal commands not dealing with filenames. */
 			if (first_word) {
 				flags |= STATE_COMPLETING;
-				if (is_internal_c(first_word) && !is_internal_f(first_word)) {
+				if (is_internal_cmd(first_word, NO_FNAME_NUM, 1, 1)) {
 					flags &= ~STATE_COMPLETING;
 					goto NO_SUGGESTION;
 				}
@@ -2320,7 +2324,7 @@ rl_suggestions(const unsigned char c)
 			flag = (c == ' ' && escaped == 0) ? CHECK_MATCH : PRINT_MATCH;
 
 			char *d = word;
-			if (wlen > FILE_URI_PREFIX_LEN && IS_FILE_URI(word)) {
+			if (IS_FILE_URI(word, wlen)) {
 				d += FILE_URI_PREFIX_LEN;
 				wlen -= FILE_URI_PREFIX_LEN;
 				last_word_offset += FILE_URI_PREFIX_LEN;
@@ -2371,7 +2375,7 @@ rl_suggestions(const unsigned char c)
 			}
 			break;
 
-		case 'f': /* 3.e.4) File names in CWD */
+		case 'f': /* 3.e.4) Filenames in CWD */
 			/* Do not check dirs and filenames if first word and
 			 * neither autocd nor auto-open are enabled. */
 			if (!last_space && conf.autocd == 0 && conf.auto_open == 0)
@@ -2388,7 +2392,7 @@ rl_suggestions(const unsigned char c)
 			}
 
 			/* If "./FILE" check only "FILE" */
-			if (wlen > 2 && *word == '.' && *(word + 1) == '/') {
+			if (wlen > 2 && *word == '.' && word[1] == '/') {
 				word += 2;
 				wlen -= 2;
 				last_word_offset += 2;
@@ -2396,13 +2400,13 @@ rl_suggestions(const unsigned char c)
 
 			/* If we have a slash, we're not looking for files in the CWD. */
 			char *p = strchr(word, '/');
-			if (p && *(p + 1))
+			if (p && p[1])
 				break;
 
-			/* Skip internal commands not dealing with file names. */
+			/* Skip internal commands not dealing with filenames. */
 			if (first_word) {
 				flags |= STATE_COMPLETING;
-				if (is_internal_c(first_word) && !is_internal_f(first_word)) {
+				if (is_internal_cmd(first_word, NO_FNAME_NUM, 1, 1)) {
 					flags &= ~STATE_COMPLETING;
 					goto NO_SUGGESTION;
 				}
@@ -2472,11 +2476,10 @@ CHECK_FIRST_WORD:
 	/* Skip 'b:' (bookmarks), 's:' (sel files) 't:' (tags) and 'w:'
 	 * (workspaces) constructs. */
 	if ((*word == 'b' || *word == 's' || *word == 't' || *word == 'w')
-	&& *(word + 1) == ':')
+	&& word[1] == ':')
 		goto NO_SUGGESTION;
 
-	if (!word || !*word
-
+	if (!*word
 	|| (c == ' ' && (*word == '\''
 	|| *word == '"' || *word == '$' || *word == '#'))
 	|| *word == '<' || *word == '>' || *word == '!' || *word == '{'
@@ -2502,7 +2505,7 @@ CHECK_FIRST_WORD:
 	} else if (point_is_first_word && rl_point < rl_end
 	&& (printed = check_completions(word, wlen, CHECK_MATCH)) != NO_MATCH) {
 		if (c == ' ' && printed != FULL_MATCH)
-			/* We have a partial match for a file name. If not a command
+			/* We have a partial match for a filename. If not a command
 			 * name, let's return NO_MATCH. */
 			printed = check_cmds(word, wlen, CHECK_MATCH);
 
