@@ -1,17 +1,15 @@
-/* tags.c -- functions to handle the files tag system */
-
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,11 +20,12 @@
  * MA 02110-1301, USA.
 */
 
+/* tags.c -- functions to handle the file tagging system */
+
 #ifndef _NO_TAGS
 
 #include "helpers.h"
 
-#include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <unistd.h>
@@ -62,8 +61,8 @@ print_symlink_error(const char *name)
 static int
 print_no_tags(void)
 {
-	printf(_("%s: No tags found, Use 'tag new' to create new "
-		"tags\n"), PROGRAM_NAME);
+	printf(_("%s: No tags found. Use 'tag new' to create new "
+		"tags.\n"), PROGRAM_NAME);
 	return FUNC_SUCCESS;
 }
 
@@ -98,7 +97,7 @@ check_tagged_file(const char *tag, const char *name)
 }
 #endif /* _DIRENT_HAVE_D_TPYE */
 
-/* Print the tagged file NAME tagged as TAG. */
+/* Print the tagged file named NAME tagged as TAG. */
 static void
 print_tagged_file(char *name, const char *tag)
 {
@@ -523,7 +522,7 @@ get_tags(char **args)
 	return t;
 }
 
-/* Tag file names found in ARGS as all tags (:TAG) found in ARGS. */
+/* Tag filenames found in ARGS as all tags (:TAG) found in ARGS. */
 static int
 tag_files(char **args)
 {
@@ -565,7 +564,7 @@ tag_files(char **args)
 	return FUNC_SUCCESS;
 }
 
-/* Untag file names found in ARGS tagged as ARGS[N]. */
+/* Untag filenames found in ARGS tagged as ARGS[N]. */
 static int
 untag(char **args, const size_t n, size_t *t)
 {
@@ -619,7 +618,7 @@ untag(char **args, const size_t n, size_t *t)
 	return exit_status;
 }
 
-/* Untag file names found in ARGS as all tags (:TAG) found in ARGS. */
+/* Untag filenames found in ARGS as all tags (:TAG) found in ARGS. */
 static int
 untag_files(char **args)
 {
@@ -650,7 +649,7 @@ rename_tag(char **args)
 		return print_no_such_tag(old);
 
 	if (*old == *new && strcmp(old, new) == 0) {
-		xerror("%s\n", "tag: New and old file names are the same");
+		xerror("%s\n", _("tag: New and old filenames are the same"));
 		return FUNC_FAILURE;
 	}
 
@@ -665,7 +664,7 @@ rename_tag(char **args)
 		return errno;
 	}
 
-	puts("Successfully renamed tag");
+	puts(_("Successfully renamed tag"));
 	reload_tags();
 	return FUNC_SUCCESS;
 }
@@ -722,7 +721,7 @@ merge_tags(char **args)
 	char *src = args[2], *dst = args[3];
 
 	if (strcmp(src, dst) == 0) {
-		xerror("%s\n", _("tag: Source and destiny are the same tag"));
+		xerror("%s\n", _("tag: Source and destination are the same tag"));
 		return FUNC_FAILURE;
 	}
 
@@ -828,8 +827,7 @@ tags_function(char **args)
 	if (strcmp(a[0], "tag") != 0 && strspn(a[0] + 1, b))
 		{ a = reconstruct_input(args); free_args = 1; }
 
-	if (*a[1] == 'l' && (strcmp(a[1], "ls") == 0
-	|| strcmp(a[1], "list") == 0))
+	if (*a[1] == 'l' && strcmp(a[1], "list") == 0)
 		{ exit_status = list_tags(a); goto END; }
 
 	if (*a[1] == 'l' && strcmp(a[1], "list-full") == 0)
@@ -841,13 +839,10 @@ tags_function(char **args)
 	if (*a[1] == 'n' && strcmp(a[1], "new") == 0)
 		{ exit_status = create_tags(a); goto END; }
 
-	if ((*a[1] == 'r' && (strcmp(a[1], "rm") == 0
-	|| strcmp(a[1], "remove") == 0))
-	|| (*a[1] == 'd' && strcmp(a[1], "del") == 0))
+	if (*a[1] == 'd' && strcmp(a[1], "del") == 0)
 		{ exit_status = remove_tags(a); goto END; }
 
-	if ((*a[1] == 'r' && strcmp(a[1], "rename") == 0)
-	|| (*a[1] == 'm' && strcmp(a[1], "mv") == 0))
+	if (*a[1] == 'r' && strcmp(a[1], "rename") == 0)
 		{ exit_status = rename_tag(a); goto END; }
 
 	if (*a[1] == 'u' && strcmp(a[1], "untag") == 0)
