@@ -1,17 +1,15 @@
-/* trash.c -- functions to manage the trash system */
-
 /*
- * This file is part of CliFM
- * 
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * This file is part of Clifm
+ *
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -22,12 +20,13 @@
  * MA 02110-1301, USA.
 */
 
+/* trash.c -- functions to manage the trash system */
+
 #ifndef _NO_TRASH
 
 #include "helpers.h"
 
 #include <errno.h>
-#include <stdio.h>
 #include <string.h>
 #include <unistd.h> /* access, unlinkat */
 /* <time.h>, required by time(2) and localtime_r(2), is included by "aux.h" */
@@ -49,17 +48,14 @@ xatoi, url_encode, xnmalloc, print_file_name */
 #include "spawn.h"      /* launch_execv */
 #include "xdu.h"        /* dir_size */
 
-/* Return the amount of currently trashed files. */
+/* Return the number of currently trashed files. */
 static size_t
 count_trashed_files(void)
 {
 	size_t n = 0;
 	if (trash_ok == 1 && trash_files_dir != NULL) {
 		filesn_t ret = count_dir(trash_files_dir, NO_CPOP);
-		if (ret <= 2)
-			n = 0;
-		else
-			n = (size_t)ret - 2;
+		n = ret <= 2 ? 0 : (size_t)ret - 2;
 	}
 
 	return n;
@@ -74,9 +70,9 @@ confirm_removal(const size_t n)
 		return 1; /* Yes */
 
 	char msg[256]; /* Big enough, in case of translations. */
-	snprintf(msg, sizeof(msg), _("Remove %zu file(s)? [y/n] "), n);
+	snprintf(msg, sizeof(msg), _("Remove %zu file(s)?"), n);
 
-	return rl_get_y_or_n(msg);
+	return rl_get_y_or_n(msg, conf.default_answer.remove);
 }
 
 /* We have removed N files from the trash can. Print the results. */
@@ -89,16 +85,18 @@ print_removal_result(const size_t n)
 	const size_t cur = (n <= trash_n ? (trash_n - n) : 0);
 
 	if (cur == 0) {
-		print_reload_msg(_("Trash can emptied: %zu file(s) removed\n"), n);
+		print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+			_("Trash can emptied: %zu file(s) removed\n"), n);
 	} else {
-		print_reload_msg(_("%zu file(s) removed from the trash can\n"), n);
-		print_reload_msg(_("%zu total trashed file(s)\n"), cur);
+		print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+			_("%zu file(s) removed from the trash can\n"), n);
+		print_reload_msg(NULL, NULL, _("%zu total trashed file(s)\n"), cur);
 	}
 }
 
 /* Remove the file named NAME and the corresponding .trashinfo file from
  * the trash can.
- * Return 0 on success or >0 on error. */
+ * Returns 0 on success or >0 on error. */
 static int
 remove_file_from_trash(const char *name)
 {
@@ -177,7 +175,7 @@ static int
 gen_trashinfo_file(char *file, const char *suffix, const struct tm *tm)
 {
 	/* Encode path to URL format (RF 2396) */
-	char *url_str = url_encode(file);
+	char *url_str = url_encode(file, 0);
 	if (!url_str) {
 		xerror(_("trash: '%s': Error encoding path\n"), file);
 		return FUNC_FAILURE;
@@ -225,7 +223,7 @@ remove_trashinfo_file(const char *name)
 	free(info_file);
 }
 
-/* Create the trashed file name: orig_filename.suffix, where SUFFIX is
+/* Create the trashed filename: orig_filename.suffix, where SUFFIX is
  * the current date and time (plus an integer in case of dups).
  * Returns the absolute path to this file and updates FILE_SUFFIX to
  * its basename. */
@@ -238,7 +236,7 @@ gen_dest_file(const char *file, const char *suffix, char **file_suffix)
 		return (char *)NULL;
 	}
 
-	/* If the length of the trashed file name (orig_filename.suffix) is
+	/* If the length of the trashed filename (orig_filename.suffix) is
 	 * longer than NAME_MAX (255), trim the original filename, so that
 	 * (original_filename_len + 1 (dot) + suffix_len) won't be longer
 	 * than NAME_MAX. */
@@ -249,9 +247,9 @@ gen_dest_file(const char *file, const char *suffix, char **file_suffix)
 
 	if (size > 0) {
 		/* THIS IS NOT UNICODE AWARE */
-		/* If SIZE is a positive value, that is, the trashed file name
-		 * exceeds NAME_MAX by SIZE bytes, reduce the original file name
-		 * SIZE bytes. Terminate the original file name (FILENAME) with
+		/* If SIZE is a positive value, that is, the trashed filename
+		 * exceeds NAME_MAX by SIZE bytes, reduce the original filename
+		 * SIZE bytes. Terminate the original filename (FILENAME) with
 		 * a tilde (~), to let the user know it was trimmed. */
 		filename[filename_len - (size_t)size - 1] = '~';
 		filename[filename_len - (size_t)size] = '\0';
@@ -267,8 +265,8 @@ gen_dest_file(const char *file, const char *suffix, char **file_suffix)
 	char *dest = xnmalloc(dlen, sizeof(char));
 	snprintf(dest, dlen, "%s/%s", trash_files_dir, *file_suffix);
 
-	/* If destiny file exists (there's already a trashed file with this name),
-	 * append an integer until it is made unique. */
+	/* If the destination file exists (there's already a trashed file with
+	 * this name), append an integer until it is made unique. */
 	struct stat a;
 	int inc = 1;
 	while (lstat(dest, &a) == 0 && inc > 0) {
@@ -321,8 +319,8 @@ trash_file(const char *suffix, const struct tm *tm, char *file)
 	int mvcmd = 0;
 	int ret = renameat(XAT_FDCWD, file, XAT_FDCWD, dest);
 	if (ret != FUNC_SUCCESS && errno == EXDEV) {
-		/* Destination file is on a different file system, which is why
-		 * renameat(2) doesn't work: let's try with mv(1). */
+		/* Destination file is on a different filesystem, which is why
+		 * renameat(2) fails: let's try with mv(1). */
 		char *tmp_cmd[] = {"mv", "--", file, dest, NULL};
 		ret = launch_execv(tmp_cmd, FOREGROUND, E_NOFLAG);
 		mvcmd = 1;
@@ -422,7 +420,7 @@ print_trashfiles(struct dirent ***ent, const int files_n)
 
 static char **
 list_and_get_input(struct dirent ***trash_files, const int files_n,
-	const int is_undel)
+	const int is_untrash)
 {
 	if (conf.clear_screen > 0)
 		CLEAR;
@@ -434,14 +432,14 @@ list_and_get_input(struct dirent ***trash_files, const int files_n,
 	/* Get input */
 	printf(_("\n%sEnter 'q' to quit\n"
 		"File(s) to be %s (e.g.: 1 2-6, or *):\n"), df_c,
-		is_undel == 1 ? _("restored") : _("removed"));
+		is_untrash == 1 ? _("restored") : _("removed"));
 
 	char *line = (char *)NULL;
 	char tprompt[(MAX_COLOR * 2) + 7];
 	snprintf(tprompt, sizeof(tprompt), "\001%s\002>\001%s\002 ", mi_c, tx_c);
 
 	while (!line)
-		line = rl_no_hist(tprompt);
+		line = rl_no_hist(tprompt, 0);
 
 	const filesn_t tfiles = files;
 	files = (filesn_t)trash_n;
@@ -613,7 +611,7 @@ read_original_path(const char *file, const char *src, int *status)
 	int fd = 0;
 	FILE *fp = open_fread(file, &fd);
 	if (!fp) {
-		xerror(_("undel: Info file for '%s' not found. Try restoring "
+		xerror(_("untrash: Info file for '%s' not found. Try restoring "
 			"the file manually.\n"), src);
 		*status = errno;
 		return (char *)NULL;
@@ -651,7 +649,7 @@ read_original_path(const char *file, const char *src, int *status)
 	char *url_decoded = url_decode(orig_path);
 
 	if (!url_decoded) {
-		xerror(_("undel: '%s': Error decoding original path\n"), orig_path);
+		xerror(_("untrash: '%s': Error decoding original path\n"), orig_path);
 		free(orig_path);
 		*status = FUNC_FAILURE;
 		return (char *)NULL;
@@ -673,39 +671,39 @@ static int
 check_untrash_dest(char *file)
 {
 	if (!file || !*file) {
-		xerror(_("undel: File name is NULL or empty\n"));
+		xerror(_("untrash: Filename is NULL or empty\n"));
 		return FUNC_FAILURE;
 	}
 
 	char *p = strrchr(file, '/');
 	if (!p) {
-		xerror(_("undel: '%s': No directory specified\n"), file);
+		xerror(_("untrash: '%s': No directory specified\n"), file);
 		return FUNC_FAILURE;
 	}
 
-	char c = *(p + 1);
-	*(p + 1) = '\0';
+	char c = p[1];
+	p[1] = '\0';
 	char *parent_dir = file;
 
 	const int ret = access(parent_dir, F_OK | X_OK | W_OK);
 	if (ret != 0) {
 		if (errno == ENOENT) {
 			if (create_untrash_parent(parent_dir) != FUNC_SUCCESS) {
-				*(p + 1) = c;
+				p[1] = c;
 				return FUNC_FAILURE;
 			}
 		} else {
-			xerror("undel: '%s': %s\n", parent_dir, strerror(errno));
-			*(p + 1) = c;
+			xerror("untrash: '%s': %s\n", parent_dir, strerror(errno));
+			p[1] = c;
 			return errno;
 		}
 	}
 
-	*(p + 1) = c;
+	p[1] = c;
 
 	struct stat a;
 	if (stat(file, &a) != -1) {
-		xerror(_("undel: '%s': Destination file exists\n"), file);
+		xerror(_("untrash: '%s': Destination file exists\n"), file);
 		return EEXIST;
 	}
 
@@ -718,14 +716,14 @@ untrash_file(char *file)
 	if (!file)
 		return FUNC_FAILURE;
 
-	char undel_file[PATH_MAX + 1];
-	char undel_info[PATH_MAX + 1];
-	snprintf(undel_file, sizeof(undel_file), "%s/%s", trash_files_dir, file);
-	snprintf(undel_info, sizeof(undel_info), "%s/%s.trashinfo",
+	char untrash_file[PATH_MAX + 1];
+	char untrash_info[PATH_MAX + 1];
+	snprintf(untrash_file, sizeof(untrash_file), "%s/%s", trash_files_dir, file);
+	snprintf(untrash_info, sizeof(untrash_info), "%s/%s.trashinfo",
 		trash_info_dir, file);
 
 	int ret = FUNC_SUCCESS;
-	char *orig_path = read_original_path(undel_info, file, &ret);
+	char *orig_path = read_original_path(untrash_info, file, &ret);
 	if (!orig_path)
 		return ret;
 
@@ -737,12 +735,12 @@ untrash_file(char *file)
 		return ret;
 	}
 
-	ret = renameat(XAT_FDCWD, undel_file, XAT_FDCWD, orig_path);
+	ret = renameat(XAT_FDCWD, untrash_file, XAT_FDCWD, orig_path);
 	if (ret == -1) {
 		if (errno == EXDEV) {
-			/* Destination file is on a different file system, which is why
+			/* Destination file is on a different filesystem, which is why
 			 * rename(3) doesn't work: let's try with mv(1). */
-			char *cmd[] = {"mv", "--", undel_file, orig_path, NULL};
+			char *cmd[] = {"mv", "--", untrash_file, orig_path, NULL};
 			ret = launch_execv(cmd, FOREGROUND, E_NOFLAG);
 			if (ret != FUNC_SUCCESS) {
 				if (conf.autols == 1)
@@ -751,7 +749,7 @@ untrash_file(char *file)
 				return ret;
 			}
 		} else {
-			xerror("undel: '%s': %s\n", undel_file, strerror(errno));
+			xerror("untrash: '%s': %s\n", untrash_file, strerror(errno));
 			if (conf.autols == 1)
 				press_any_key_to_continue(0);
 			free(orig_path);
@@ -761,8 +759,8 @@ untrash_file(char *file)
 
 	free(orig_path);
 
-	if (unlinkat(XAT_FDCWD, undel_info, 0) == -1) {
-		xerror(_("undel: '%s': %s\n"), undel_info, strerror(errno));
+	if (unlinkat(XAT_FDCWD, untrash_info, 0) == -1) {
+		xerror(_("untrash: '%s': %s\n"), untrash_info, strerror(errno));
 		return errno;
 	}
 
@@ -793,8 +791,9 @@ untrash_all(struct dirent ***tfiles, const int tfiles_n, const int free_files)
 		if (conf.autols == 1) reload_dirlist();
 
 		const size_t n = count_trashed_files();
-		print_reload_msg(_("%zu file(s) restored\n"), untrashed);
-		print_reload_msg(_("%zu total trashed file(s)\n"), n);
+		print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+			_("%zu file(s) restored\n"), untrashed);
+		print_reload_msg(NULL, NULL, _("%zu total trashed file(s)\n"), n);
 	}
 
 	return status;
@@ -825,8 +824,9 @@ untrash_files(char **args)
 		if (conf.autols == 1) reload_dirlist();
 
 		const size_t n = count_trashed_files();
-		print_reload_msg(_("%zu file(s) restored\n"), untrashed);
-		print_reload_msg(_("%zu total trashed file(s)\n"), n);
+		print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+			_("%zu file(s) restored\n"), untrashed);
+		print_reload_msg(NULL, NULL, _("%zu total trashed file(s)\n"), n);
 	}
 
 	return status;
@@ -856,7 +856,7 @@ untrash_function(char **args)
 	if (files_n <= 0)
 		return exit_status;
 
-	/* if "undel all" (or "u a" or "u *") */
+	/* if "untrash all" (or "u a" or "u *") */
 	if (args[1] && (strcmp(args[1], "*") == 0 || strcmp(args[1], "a") == 0
 	|| strcmp(args[1], "all") == 0))
 		return untrash_all(&trash_files, files_n, 1);
@@ -890,7 +890,7 @@ untrash_function(char **args)
 
 		int num = atoi(input[i]);
 		if (!is_number(input[i]) || num <= 0 || num > files_n) {
-			xerror(_("undel: %s: Invalid ELN\n"), input[i]);
+			xerror(_("untrash: %s: Invalid ELN\n"), input[i]);
 			exit_status = FUNC_FAILURE;
 			free_and_return = 1;
 		}
@@ -903,7 +903,7 @@ untrash_function(char **args)
 		return exit_status;
 	}
 
-	/* Undelete trashed files */
+	/* Untrash files */
 	for (i = 0; input[i]; i++) {
 		int num = atoi(input[i]);
 		if (untrash_file(trash_files[num - 1]->d_name) != FUNC_SUCCESS)
@@ -912,14 +912,15 @@ untrash_function(char **args)
 
 	free_files_and_input(&input, &trash_files, files_n);
 
-	/* If some trashed file still remains, reload the undel screen */
+	/* If some trashed file still remains, reload the untrash screen */
 	const size_t n = count_trashed_files();
 	if (n > 0) {
 		if (conf.clear_screen > 0) CLEAR;
 		untrash_function(args);
 	} else {
-		if (conf.autols == 1) reload_dirlist();
-		print_reload_msg(_("%zu trashed file(s)\n"), n);
+		if (conf.autols == 1)
+			reload_dirlist();
+		print_reload_msg(NULL, NULL, _("%zu trashed file(s)\n"), n);
 	}
 
 	return exit_status;
@@ -928,14 +929,13 @@ untrash_function(char **args)
 static void
 print_trashdir_size(void)
 {
-	const int base = xargs.si == 1 ? 1000 : 1024;
 	int status = 0;
 
 	printf(_("\n%sTotal size: "), df_c);
 	if (term_caps.suggestions == 1)
 		{fputs("Calculating...", stdout); fflush(stdout);}
 
-	const off_t full_size = dir_size(trash_files_dir, 0, &status) * base;
+	const off_t full_size = dir_size(trash_files_dir, 0, &status);
 	char *human_size = construct_human_size(full_size);
 
 	char err[sizeof(xf_cb) + 6]; *err = '\0';
@@ -1018,8 +1018,9 @@ check_trash_file(char *file)
 	}
 
 	/* Do no trash TRASH_DIR itself nor anything inside it (trashed files) */
-	if (strncmp(tmp_file, trash_dir, strlen(trash_dir)) == 0) {
-		fputs(_("trash: Use 'trash del' to remove trashed files\n"), stderr);
+	const size_t tlen = strlen(trash_dir);
+	if (strncmp(tmp_file, trash_dir, tlen) == 0 && tmp_file[tlen] == '/') {
+		xerror(_("trash: Use 'trash del' to remove trashed files\n"));
 		return FUNC_FAILURE;
 	}
 
@@ -1059,14 +1060,14 @@ list_ok_trashed_files(char **args, const int *trashed, const size_t trashed_n)
 		char *p = args[trashed[i]];
 		if (strchr(args[trashed[i]], '\\')
 		&& !(p = unescape_str(args[trashed[i]], 0)) ) {
-			xerror(_("trash: '%s': Error unescaping file name\n"),
+			xerror(_("trash: '%s': Error unescaping filename\n"),
 				args[trashed[i]]);
 			continue;
 		}
 
 		char *tmp = abbreviate_file_name(p);
 		if (!tmp) {
-			xerror(_("trash: '%s': Error abbreviating file name\n"), p);
+			xerror(_("trash: '%s': Error abbreviating filename\n"), p);
 			if (p && p != args[trashed[i]])
 				free(p);
 			continue;
@@ -1076,14 +1077,14 @@ list_ok_trashed_files(char **args, const int *trashed, const size_t trashed_n)
 
 		print_file_name(name, 0);
 
-		if (tmp && tmp != p)
+		if (tmp != p)
 			free(tmp);
 		if (p && p != args[trashed[i]])
 			free(p);
 	}
 }
 
-/* Print file names in ARGS and ask for confirmation.
+/* Print filenames in ARGS and ask for confirmation.
  * Return the number of files to be trashed if the answer is afirmative,
  * or zero otherwise. */
 static size_t
@@ -1107,7 +1108,7 @@ ask_for_confirmation(char **args)
 		free(name);
 	}
 
-	if (rl_get_y_or_n("Continue? [y/n] ") == 0)
+	if (rl_get_y_or_n(_("Continue?"), conf.default_answer.trash) == 0)
 		return 0;
 
 	return i;
@@ -1133,7 +1134,7 @@ trash_files_args(char **args)
 
 	time_t rawtime = time(NULL);
 	struct tm t;
-	char *suffix = localtime_r(&rawtime, &t) ? gen_date_suffix(t) : (char *)NULL;
+	char *suffix = localtime_r(&rawtime, &t) ? gen_date_suffix(t, 0) : NULL;
 	if (!suffix)
 		return FUNC_FAILURE;
 
@@ -1148,7 +1149,7 @@ trash_files_args(char **args)
 
 		char *deq_file = unescape_str(args[i], 0);
 		if (!deq_file) {
-			xerror(_("trash: '%s': Error unescaping file name\n"), args[i]);
+			xerror(_("trash: '%s': Error unescaping filename\n"), args[i]);
 			continue;
 		}
 
@@ -1203,8 +1204,9 @@ trash_files_args(char **args)
 
 PRINT_TRASHED:
 	list_ok_trashed_files(args, successfully_trashed, n);
-	print_reload_msg(_("%zu file(s) trashed\n"), trashed_files);
-	print_reload_msg(_("%zu total trashed file(s)\n"),
+	print_reload_msg(SET_SUCCESS_PTR, xs_cb,
+		_("%zu file(s) trashed\n"), trashed_files);
+	print_reload_msg(NULL, NULL, _("%zu total trashed file(s)\n"),
 		trash_n + trashed_files);
 
 	free(successfully_trashed);
@@ -1222,9 +1224,8 @@ trash_function(char **args)
 		return FUNC_FAILURE;
 	}
 
-	/* List trashed files ('tr' or 'tr ls') */
-	if (!args[1] || (*args[1] == 'l'
-	&& (strcmp(args[1], "ls") == 0 || strcmp(args[1], "list") == 0)))
+	/* List trashed files ('tr' or 'tr list') */
+	if (!args[1] || (*args[1] == 'l' && strcmp(args[1], "list") == 0))
 		return list_trashed_files();
 
 	trash_n = count_trashed_files();
@@ -1232,8 +1233,7 @@ trash_function(char **args)
 	if (*args[1] == 'd' && strcmp(args[1], "del") == 0)
 		return remove_from_trash(args);
 
-	if ((*args[1] == 'c' && strcmp(args[1], "clear") == 0)
-	|| (*args[1] == 'e' && strcmp(args[1], "empty") == 0))
+	if (*args[1] == 'e' && strcmp(args[1], "empty") == 0)
 		return trash_clear();
 
 	return trash_files_args(args);
