@@ -5,17 +5,17 @@
  * Modified code is licensed GPL2+. */
 
 /*
- * This file is part of CliFM
+ * This file is part of Clifm
  *
- * Copyright (C) 2016-2024, L. Abramovich <leo.clifm@outlook.com>
- * All rights reserved.
-
- * CliFM is free software; you can redistribute it and/or modify
+ * SPDX-License-Identifier: GPL-2.0-or-later
+ * Copyright (C) 2016-2025, L. Abramovich <leo.clifm@outlook.com>
+ *
+ * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
  * (at your option) any later version.
  *
- * CliFM is distributed in the hope that it will be useful,
+ * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  * GNU General Public License for more details.
@@ -43,9 +43,6 @@
 #pragma warning(disable : 4820)
 #endif
 
-//#include <stddef.h>
-//#include <stdlib.h>
-
 #if defined(_MSC_VER)
 # pragma warning(pop)
 #endif
@@ -166,7 +163,7 @@ utf8chr(const utf8_int8_t *src, utf8_int32_t chr)
 		c[0] = (utf8_int8_t)(0xe0 | (utf8_int8_t)(chr >> 12));
 		c[1] = (utf8_int8_t)(0x80 | (utf8_int8_t)((chr >> 6) & 0x3f));
 		c[2] = (utf8_int8_t)(0x80 | (utf8_int8_t)(chr & 0x3f));
-	} else { /* if (0 == ((int)0xffe00000 & chr)) { */
+	} else { /* ((int)0xffe00000 & chr) == 0 { */
 		/* 4-byte/21-bit utf8 code point
 		* (0b11110xxx 0b10xxxxxx 0b10xxxxxx 0b10xxxxxx) */
 		c[0] = (utf8_int8_t)(0xf0 | (utf8_int8_t)(chr >> 18));
@@ -329,132 +326,48 @@ utf8uprcodepoint(utf8_int32_t cp)
 	} else {
 		switch (cp) {
 		default: break;
-		case 0x00ff:
-			cp = 0x0178;
-			break;
-		case 0x0180:
-			cp = 0x0243;
-			break;
-		case 0x01dd:
-			cp = 0x018e;
-			break;
-		case 0x019a:
-			cp = 0x023d;
-			break;
-		case 0x019e:
-			cp = 0x0220;
-			break;
-		case 0x0292:
-			cp = 0x01b7;
-			break;
-		case 0x01c6:
-			cp = 0x01c4;
-			break;
-		case 0x01c9:
-			cp = 0x01c7;
-			break;
-		case 0x01cc:
-			cp = 0x01ca;
-			break;
-		case 0x01f3:
-			cp = 0x01f1;
-			break;
-		case 0x01bf:
-			cp = 0x01f7;
-			break;
-		case 0x0188:
-			cp = 0x0187;
-			break;
-		case 0x018c:
-			cp = 0x018b;
-			break;
-		case 0x0192:
-			cp = 0x0191;
-			break;
-		case 0x0199:
-			cp = 0x0198;
-			break;
-		case 0x01a8:
-			cp = 0x01a7;
-			break;
-		case 0x01ad:
-			cp = 0x01ac;
-			break;
-		case 0x01b0:
-			cp = 0x01af;
-			break;
-		case 0x01b9:
-			cp = 0x01b8;
-			break;
-		case 0x01bd:
-			cp = 0x01bc;
-			break;
-		case 0x01f5:
-			cp = 0x01f4;
-			break;
-		case 0x023c:
-			cp = 0x023b;
-			break;
-		case 0x0242:
-			cp = 0x0241;
-			break;
-		case 0x037b:
-			cp = 0x03fd;
-			break;
-		case 0x037c:
-			cp = 0x03fe;
-			break;
-		case 0x037d:
-			cp = 0x03ff;
-			break;
-		case 0x03f3:
-			cp = 0x037f;
-			break;
-		case 0x03ac:
-			cp = 0x0386;
-			break;
-		case 0x03ad:
-			cp = 0x0388;
-			break;
-		case 0x03ae:
-			cp = 0x0389;
-			break;
-		case 0x03af:
-			cp = 0x038a;
-			break;
-		case 0x03cc:
-			cp = 0x038c;
-			break;
-		case 0x03cd:
-			cp = 0x038e;
-			break;
-		case 0x03ce:
-			cp = 0x038f;
-			break;
-		case 0x0371:
-			cp = 0x0370;
-			break;
-		case 0x0373:
-			cp = 0x0372;
-			break;
-		case 0x0377:
-			cp = 0x0376;
-			break;
-		case 0x03d1:
-			cp = 0x0398;
-			break;
-		case 0x03d7:
-			cp = 0x03cf;
-			break;
-		case 0x03f2:
-			cp = 0x03f9;
-			break;
-		case 0x03f8:
-			cp = 0x03f7;
-			break;
-		case 0x03fb:
-			cp = 0x03fa;
-			break;
+		case 0x00ff: cp = 0x0178; break;
+		case 0x0180: cp = 0x0243; break;
+		case 0x01dd: cp = 0x018e; break;
+		case 0x019a: cp = 0x023d; break;
+		case 0x019e: cp = 0x0220; break;
+		case 0x0292: cp = 0x01b7; break;
+		case 0x01c6: cp = 0x01c4; break;
+		case 0x01c9: cp = 0x01c7; break;
+		case 0x01cc: cp = 0x01ca; break;
+		case 0x01f3: cp = 0x01f1; break;
+		case 0x01bf: cp = 0x01f7; break;
+		case 0x0188: cp = 0x0187; break;
+		case 0x018c: cp = 0x018b; break;
+		case 0x0192: cp = 0x0191; break;
+		case 0x0199: cp = 0x0198; break;
+		case 0x01a8: cp = 0x01a7; break;
+		case 0x01ad: cp = 0x01ac; break;
+		case 0x01b0: cp = 0x01af; break;
+		case 0x01b9: cp = 0x01b8; break;
+		case 0x01bd: cp = 0x01bc; break;
+		case 0x01f5: cp = 0x01f4; break;
+		case 0x023c: cp = 0x023b; break;
+		case 0x0242: cp = 0x0241; break;
+		case 0x037b: cp = 0x03fd; break;
+		case 0x037c: cp = 0x03fe; break;
+		case 0x037d: cp = 0x03ff; break;
+		case 0x03f3: cp = 0x037f; break;
+		case 0x03ac: cp = 0x0386; break;
+		case 0x03ad: cp = 0x0388; break;
+		case 0x03ae: cp = 0x0389; break;
+		case 0x03af: cp = 0x038a; break;
+		case 0x03cc: cp = 0x038c; break;
+		case 0x03cd: cp = 0x038e; break;
+		case 0x03ce: cp = 0x038f; break;
+		case 0x0371: cp = 0x0370; break;
+		case 0x0373: cp = 0x0372; break;
+		case 0x0377: cp = 0x0376; break;
+		case 0x03d1: cp = 0x0398; break;
+		case 0x03d7: cp = 0x03cf; break;
+		case 0x03f2: cp = 0x03f9; break;
+		case 0x03f8: cp = 0x03f7; break;
+		case 0x03fb: cp = 0x03fa; break;
 		}
 	}
 
@@ -496,132 +409,48 @@ utf8lwrcodepoint(utf8_int32_t cp)
 	} else {
 		switch (cp) {
 		default: break;
-		case 0x0178:
-			cp = 0x00ff;
-			break;
-		case 0x0243:
-			cp = 0x0180;
-			break;
-		case 0x018e:
-			cp = 0x01dd;
-			break;
-		case 0x023d:
-			cp = 0x019a;
-			break;
-		case 0x0220:
-			cp = 0x019e;
-			break;
-		case 0x01b7:
-			cp = 0x0292;
-			break;
-		case 0x01c4:
-			cp = 0x01c6;
-			break;
-		case 0x01c7:
-			cp = 0x01c9;
-			break;
-		case 0x01ca:
-			cp = 0x01cc;
-			break;
-		case 0x01f1:
-			cp = 0x01f3;
-			break;
-		case 0x01f7:
-			cp = 0x01bf;
-			break;
-		case 0x0187:
-			cp = 0x0188;
-			break;
-		case 0x018b:
-			cp = 0x018c;
-			break;
-		case 0x0191:
-			cp = 0x0192;
-			break;
-		case 0x0198:
-			cp = 0x0199;
-			break;
-		case 0x01a7:
-			cp = 0x01a8;
-			break;
-		case 0x01ac:
-			cp = 0x01ad;
-			break;
-		case 0x01af:
-			cp = 0x01b0;
-			break;
-		case 0x01b8:
-			cp = 0x01b9;
-			break;
-		case 0x01bc:
-			cp = 0x01bd;
-			break;
-		case 0x01f4:
-			cp = 0x01f5;
-			break;
-		case 0x023b:
-			cp = 0x023c;
-			break;
-		case 0x0241:
-			cp = 0x0242;
-		  break;
-		case 0x03fd:
-			cp = 0x037b;
-			break;
-		case 0x03fe:
-			cp = 0x037c;
-			break;
-		case 0x03ff:
-			cp = 0x037d;
-			break;
-		case 0x037f:
-			cp = 0x03f3;
-			break;
-		case 0x0386:
-			cp = 0x03ac;
-			break;
-		case 0x0388:
-			cp = 0x03ad;
-			break;
-		case 0x0389:
-			cp = 0x03ae;
-			break;
-		case 0x038a:
-			cp = 0x03af;
-			break;
-		case 0x038c:
-			cp = 0x03cc;
-			break;
-		case 0x038e:
-			cp = 0x03cd;
-			break;
-		case 0x038f:
-			cp = 0x03ce;
-			break;
-		case 0x0370:
-			cp = 0x0371;
-			break;
-		case 0x0372:
-			cp = 0x0373;
-			break;
-		case 0x0376:
-			cp = 0x0377;
-			break;
-		case 0x03f4:
-			cp = 0x03b8;
-			break;
-		case 0x03cf:
-			cp = 0x03d7;
-			break;
-		case 0x03f9:
-			cp = 0x03f2;
-			break;
-		case 0x03f7:
-			cp = 0x03f8;
-			break;
-		case 0x03fa:
-			cp = 0x03fb;
-			break;
+		case 0x0178: cp = 0x00ff; break;
+		case 0x0243: cp = 0x0180; break;
+		case 0x018e: cp = 0x01dd; break;
+		case 0x023d: cp = 0x019a; break;
+		case 0x0220: cp = 0x019e; break;
+		case 0x01b7: cp = 0x0292; break;
+		case 0x01c4: cp = 0x01c6; break;
+		case 0x01c7: cp = 0x01c9; break;
+		case 0x01ca: cp = 0x01cc; break;
+		case 0x01f1: cp = 0x01f3; break;
+		case 0x01f7: cp = 0x01bf; break;
+		case 0x0187: cp = 0x0188; break;
+		case 0x018b: cp = 0x018c; break;
+		case 0x0191: cp = 0x0192; break;
+		case 0x0198: cp = 0x0199; break;
+		case 0x01a7: cp = 0x01a8; break;
+		case 0x01ac: cp = 0x01ad; break;
+		case 0x01af: cp = 0x01b0; break;
+		case 0x01b8: cp = 0x01b9; break;
+		case 0x01bc: cp = 0x01bd; break;
+		case 0x01f4: cp = 0x01f5; break;
+		case 0x023b: cp = 0x023c; break;
+		case 0x0241: cp = 0x0242; break;
+		case 0x03fd: cp = 0x037b; break;
+		case 0x03fe: cp = 0x037c; break;
+		case 0x03ff: cp = 0x037d; break;
+		case 0x037f: cp = 0x03f3; break;
+		case 0x0386: cp = 0x03ac; break;
+		case 0x0388: cp = 0x03ad; break;
+		case 0x0389: cp = 0x03ae; break;
+		case 0x038a: cp = 0x03af; break;
+		case 0x038c: cp = 0x03cc; break;
+		case 0x038e: cp = 0x03cd; break;
+		case 0x038f: cp = 0x03ce; break;
+		case 0x0370: cp = 0x0371; break;
+		case 0x0372: cp = 0x0373; break;
+		case 0x0376: cp = 0x0377; break;
+		case 0x03f4: cp = 0x03b8; break;
+		case 0x03cf: cp = 0x03d7; break;
+		case 0x03f9: cp = 0x03f2; break;
+		case 0x03f7: cp = 0x03f8; break;
+		case 0x03fa: cp = 0x03fb; break;
 		}
 	}
 
